@@ -1,5 +1,5 @@
 /*
- * Quick DNR 0.7
+ * Quick DNR 0.8
  * (C)opyright 2003, Debabrata Banerjee
  * GNU GPL 2 or later
  * 
@@ -10,43 +10,6 @@
  * 
  */
 
-//Regular filter
-#define LUMA_THRESHOLD_DEFAULT 15
-#define CHROMA_THRESHOLD_DEFAULT 25
-
-//Double thresholded filter
-#define LUMA_THRESHOLD1_DEFAULT 10
-#define LUMA_THRESHOLD2_DEFAULT 2
-#define CHROMA_THRESHOLD1_DEFAULT 20
-#define CHROMA_THRESHOLD2_DEFAULT 4
-
-//#define QUICKDNR_DEBUG
-//#define TIME_FILTER
-
-//From linearblend
-#define cpuid(index,eax,ebx,ecx,edx)\
-    __asm __volatile\
-        ("movl %%ebx, %%esi\n\t"\
-         "cpuid\n\t"\
-         "xchgl %%ebx, %%esi"\
-         : "=a" (eax), "=S" (ebx),\
-           "=c" (ecx), "=d" (edx)\
-         : "0" (index));
-
-#define MM_MMX    0x0001 /* standard MMX */
-#define MM_3DNOW  0x0004 /* AMD 3DNOW */
-#define MM_MMXEXT 0x0002 /* SSE integer functions or AMD MMX ext */
-#define MM_SSE    0x0008 /* SSE functions */
-#define MM_SSE2   0x0010 /* PIV SSE2 functions */
-
-#ifdef TIME_FILTER
-#include <sys/time.h>
-#ifndef TIME_INTERVAL
-#define TIME_INTERVAL 300
-#endif /* undef TIME_INTERVAL */
-#endif /* TIME_FILTER */
-
-
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -55,6 +18,18 @@
 #include "filter.h"
 #include "frame.h"
 
+//Regular filter
+#define LUMA_THRESHOLD_DEFAULT 15
+#define CHROMA_THRESHOLD_DEFAULT 25
+
+//Double thresholded filter
+#define LUMA_THRESHOLD1_DEFAULT 10
+#define LUMA_THRESHOLD2_DEFAULT 1
+#define CHROMA_THRESHOLD1_DEFAULT 20
+#define CHROMA_THRESHOLD2_DEFAULT 2
+
+//#define QUICKDNR_DEBUG
+
 static const char FILTER_NAME[] = "quickdnr";
 
 typedef struct ThisFilter
@@ -70,13 +45,28 @@ typedef struct ThisFilter
   uint8_t Chroma_threshold1, Chroma_threshold2;
   uint8_t *average;
 
-#ifdef TIME_FILTER
-    int frames;
-    double seconds;
-#endif /* TIME_FILTER */
+  TF_STRUCT;
 
 } ThisFilter;
 
+#define MM_MMX    0x0001 /* standard MMX */
+#define MM_3DNOW  0x0004 /* AMD 3DNOW */
+#define MM_MMXEXT 0x0002 /* SSE integer functions or AMD MMX ext */
+#define MM_SSE    0x0008 /* SSE functions */
+#define MM_SSE2   0x0010 /* PIV SSE2 functions */
+
+#ifdef i386
+
+//From linearblend
+#define cpuid(index,eax,ebx,ecx,edx)\
+    __asm __volatile\
+        ("movl %%ebx, %%esi\n\t"\
+         "cpuid\n\t"\
+         "xchgl %%ebx, %%esi"\
+         : "=a" (eax), "=S" (ebx),\
+           "=c" (ecx), "=d" (edx)\
+         : "0" (index));
+
 int mm_support(void) // From linearblend
 {
   int rval;
@@ -180,6 +170,9 @@ int mm_support(void) // From linearblend
     return 0;
   }
 }
+#else // i386
+int mm_support(void) { return 0; }
+#endif
 
 int quickdnr(VideoFilter *f, VideoFrame *frame)
 {  
@@ -213,18 +206,15 @@ int quickdnr2(VideoFilter *f, VideoFrame *frame)
 {  
   ThisFilter *tf = (ThisFilter *)f; 
   int y,t; 
+  TF_VARS;
  
+  TF_START;
   if (tf->first)
   {
     memcpy (tf->average, frame->buf, frame->size);
     tf->first = 0;
   }
 
-#ifdef TIME_FILTER
-    struct timeval t1;
-    gettimeofday (&t1, NULL);
-#endif /* TIME_FILTER */
-
   for(y = 0; y < tf->Luma_size; y++) {
     t = abs(tf->average[y] - frame->buf[y]);
     if(t < tf->Luma_threshold1) {
@@ -243,26 +233,15 @@ int quickdnr2(VideoFilter *f, VideoFrame *frame)
       frame->buf[y] = tf->average[y];
     }
     else tf->average[y] = frame->buf[y];
-  }
+ }
 
-#ifdef TIME_FILTER
-    struct timeval t2;
-    gettimeofday (&t2, NULL);
-    tf->seconds +=
-        (t2.tv_sec - t1.tv_sec) + (t2.tv_usec - t1.tv_usec) * .000001;
-    tf->frames = (tf->frames + 1) % TIME_INTERVAL;
-    if (tf->frames == 0)
-    {
-        fprintf (stderr,
-                 "QuickDNR: filter timed at %3f frames/sec\n",
-                 TIME_INTERVAL / tf->seconds);
-        tf->seconds = 0;
-    }
-#endif /* TIME_FILTER */
+ TF_END(tf, "QuickDNR: ");
  
-  return 0;
+ return 0;
 }
 
+#ifdef i386
+
 int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
 {
   ThisFilter *tf = (ThisFilter *)f;
@@ -270,18 +249,15 @@ int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
   uint64_t *buf = (uint64_t *)frame->buf;
   uint64_t *av_p = (uint64_t *)tf->average;
   const uint64_t sign_convert = 0x8080808080808080LL;
+  TF_VARS;
 
+  TF_START;
   if (tf->first)
   {
     memcpy (tf->average, frame->buf, frame->size);
     tf->first = 0;
   }
 
-#ifdef TIME_FILTER
-    struct timeval t1;
-    gettimeofday (&t1, NULL);
-#endif /* TIME_FILTER */
-
   asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
 	       "prefetch 64(%1)     \n\t"
 	       "prefetch 64(%2)     \n\t"
@@ -357,20 +333,7 @@ int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
 
   asm volatile("emms\n\t");
 
-#ifdef TIME_FILTER
-    struct timeval t2;
-    gettimeofday (&t2, NULL);
-    tf->seconds +=
-        (t2.tv_sec - t1.tv_sec) + (t2.tv_usec - t1.tv_usec) * .000001;
-    tf->frames = (tf->frames + 1) % TIME_INTERVAL;
-    if (tf->frames == 0)
-    {
-        fprintf (stderr,
-                 "QuickDNR: filter timed at %3f frames/sec\n",
-                 TIME_INTERVAL / tf->seconds);
-        tf->seconds = 0;
-    }
-#endif /* TIME_FILTER */
+  TF_END(tf, "QuickDNR: ");
 
   return 0;
 }
@@ -383,7 +346,9 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
   uint64_t *buf = (uint64_t *)frame->buf;
   uint64_t *av_p = (uint64_t *)tf->average;
   const uint64_t sign_convert = 0x8080808080808080LL;
-
+  TF_VARS;
+  
+  TF_START;
  
   if (tf->first)
   {
@@ -391,18 +356,11 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
     tf->first = 0;
   }
 
-#ifdef TIME_FILTER
-    struct timeval t1;
-    gettimeofday (&t1, NULL);
-#endif /* TIME_FILTER */
-
   asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
 	       "prefetch 64(%1)     \n\t"
-	       "prefetch 64(%2)     \n\t"
 	       "movq (%0), %%mm4    \n\t"
 	       "movq (%1), %%mm5    \n\t"
-	       "movq (%2), %%mm6    \n\t"
-	       : : "r" (&sign_convert), "r" (&tf->Luma_threshold_mask1), "r" (&tf->Chroma_threshold_mask1)
+	       : : "r" (&sign_convert), "r" (&tf->Luma_threshold_mask1)
 	       );
 
   for(y = 0;y < (tf->Luma_size);y += 8) { //Luma
@@ -413,28 +371,7 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 		 "movq (%1), %%mm1     \n\t" //buf
 		 "movq %%mm0, %%mm2    \n\t"
 		 "movq %%mm1, %%mm3    \n\t"
-		 "movq %%mm1, %%mm7    \n\t"
-
-		 "pcmpgtb %%mm0, %%mm1 \n\t" //Secondary threshold
-		 "psubb %%mm0, %%mm3   \n\t" 
-		 "psubb %%mm7, %%mm0   \n\t" 
-		 "pand %%mm1, %%mm3    \n\t" 
-		 "pandn %%mm0,%%mm1    \n\t"
-		 "por %%mm1, %%mm3     \n\t"
-
-		 "paddb %%mm4, %%mm3   \n\t"
-		 "pcmpgtb (%2) , %%mm3 \n\t"
-
-		 "pavgb %%mm2, %%mm7   \n\t"
-
-		 "pand %%mm3, %%mm7    \n\t"
-		 "pandn %%mm2,%%mm3    \n\t"
-		 "por %%mm7, %%mm3     \n\t" //mm3 becomes av
-
-		 "movq (%1), %%mm1     \n\t" //reload registers
-		 "movq %%mm3, %%mm0    \n\t" 
-		 "movq %%mm3, %%mm2    \n\t"
-		 "movq %%mm1, %%mm3    \n\t"
+		 "movq %%mm1, %%mm6    \n\t"
 		 "movq %%mm1, %%mm7    \n\t"
 
 		 "pcmpgtb %%mm0, %%mm1 \n\t" //1 if av greater
@@ -442,14 +379,36 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 		 "psubb %%mm7, %%mm0   \n\t" //mm0=av-buf
 		 "pand %%mm1, %%mm3    \n\t" //select buf
 		 "pandn %%mm0,%%mm1    \n\t" //select av
-		 "por %%mm1, %%mm3     \n\t" //mm3=abs()
+		 "por %%mm1, %%mm3     \n\t" //mm3=abs(buf-av)
 
 		 "paddb %%mm4, %%mm3   \n\t" //hack! No proper unsigned mmx compares!
 		 "pcmpgtb %%mm5, %%mm3 \n\t" //compare diff with mask
-	       
+		 
+		 "movq %%mm2, %%mm0    \n\t" //reload registers
+		 "movq %%mm7, %%mm1    \n\t"
+
+		 "pcmpgtb %%mm0, %%mm1 \n\t" //Secondary threshold
+		 "psubb %%mm0, %%mm6   \n\t"
+		 "psubb %%mm7, %%mm0   \n\t"
+		 "pand %%mm1, %%mm6    \n\t"
+		 "pandn %%mm0,%%mm1    \n\t"
+		 "por %%mm1, %%mm6     \n\t"
+
+		 "paddb %%mm4, %%mm6   \n\t"
+		 "pcmpgtb (%2), %%mm6  \n\t"
+
+		 "movq %%mm2, %%mm0    \n\t"
+
+ 		 "pavgb %%mm7, %%mm2   \n\t"
+
+		 "pand %%mm6, %%mm2    \n\t"
+		 "pandn %%mm0,%%mm6    \n\t"
+		 "por %%mm2, %%mm6     \n\t" // Combined new/keep average
+
 		 "pand %%mm3, %%mm7    \n\t"
-		 "pandn %%mm2,%%mm3    \n\t"
-		 "por %%mm7, %%mm3     \n\t"
+		 "pandn %%mm6,%%mm3    \n\t"
+		 "por %%mm7, %%mm3     \n\t" // Combined new/keep average
+
 		 "movq %%mm3, (%0)     \n\t"
 		 "movq %%mm3, (%1)     \n\t"
 		 : : "r" (av_p), "r" (buf), "r" (&tf->Luma_threshold_mask2)
@@ -458,6 +417,11 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
     av_p++;
   }
 
+  asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
+	       "movq (%1), %%mm5    \n\t"
+	       : : "r" (&sign_convert), "r" (&tf->Chroma_threshold_mask1)
+	       );
+
   for(y = tf->Luma_size;y < tf->UV_size;y += 8) { //Chroma
     asm volatile("prefetchw 384(%0)    \n\t" //Experimental values for athlon
 		 "prefetch 384(%1)     \n\t"
@@ -466,28 +430,7 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 		 "movq (%1), %%mm1     \n\t" //buf
 		 "movq %%mm0, %%mm2    \n\t"
 		 "movq %%mm1, %%mm3    \n\t"
-		 "movq %%mm1, %%mm7    \n\t"
-
-		 "pcmpgtb %%mm0, %%mm1 \n\t" //Secondary threshold
-		 "psubb %%mm0, %%mm3   \n\t" 
-		 "psubb %%mm7, %%mm0   \n\t" 
-		 "pand %%mm1, %%mm3    \n\t" 
-		 "pandn %%mm0,%%mm1    \n\t"
-		 "por %%mm1, %%mm3     \n\t"
-
-		 "paddb %%mm4, %%mm3   \n\t"
-		 "pcmpgtb (%2) , %%mm3 \n\t"
-
-		 "pavgb %%mm2, %%mm7   \n\t"
-
-		 "pand %%mm3, %%mm7    \n\t"
-		 "pandn %%mm2,%%mm3    \n\t"
-		 "por %%mm7, %%mm3     \n\t" //mm3 becomes av
-
-		 "movq (%1), %%mm1     \n\t" //reload registers
-		 "movq %%mm3, %%mm0    \n\t" 
-		 "movq %%mm3, %%mm2    \n\t"
-		 "movq %%mm1, %%mm3    \n\t"
+		 "movq %%mm1, %%mm6    \n\t"
 		 "movq %%mm1, %%mm7    \n\t"
 
 		 "pcmpgtb %%mm0, %%mm1 \n\t" //1 if av greater
@@ -495,14 +438,36 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 		 "psubb %%mm7, %%mm0   \n\t" //mm0=av-buf
 		 "pand %%mm1, %%mm3    \n\t" //select buf
 		 "pandn %%mm0,%%mm1    \n\t" //select av
-		 "por %%mm1, %%mm3     \n\t" //mm3=abs()
+		 "por %%mm1, %%mm3     \n\t" //mm3=abs(buf-av)
 
 		 "paddb %%mm4, %%mm3   \n\t" //hack! No proper unsigned mmx compares!
-		 "pcmpgtb %%mm6, %%mm3 \n\t" //compare diff with mask
-	       
+		 "pcmpgtb %%mm5, %%mm3 \n\t" //compare diff with mask
+		 
+		 "movq %%mm2, %%mm0    \n\t" //reload registers
+		 "movq %%mm7, %%mm1    \n\t"
+
+		 "pcmpgtb %%mm0, %%mm1 \n\t" //Secondary threshold
+		 "psubb %%mm0, %%mm6   \n\t"
+		 "psubb %%mm7, %%mm0   \n\t"
+		 "pand %%mm1, %%mm6    \n\t"
+		 "pandn %%mm0,%%mm1    \n\t"
+		 "por %%mm1, %%mm6     \n\t"
+
+		 "paddb %%mm4, %%mm6   \n\t"
+		 "pcmpgtb (%2), %%mm6  \n\t"
+
+		 "movq %%mm2, %%mm0    \n\t"
+
+ 		 "pavgb %%mm7, %%mm2   \n\t"
+
+		 "pand %%mm6, %%mm2    \n\t"
+		 "pandn %%mm0,%%mm6    \n\t"
+		 "por %%mm2, %%mm6     \n\t" //Combined new/keep average
+
 		 "pand %%mm3, %%mm7    \n\t"
-		 "pandn %%mm2,%%mm3    \n\t"
-		 "por %%mm7, %%mm3     \n\t"
+		 "pandn %%mm6,%%mm3    \n\t"
+		 "por %%mm7, %%mm3     \n\t" //Combined new/keep average
+
 		 "movq %%mm3, (%0)     \n\t"
 		 "movq %%mm3, (%1)     \n\t"
 		 : : "r" (av_p), "r" (buf), "r" (&tf->Chroma_threshold_mask2)
@@ -513,23 +478,20 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 
   asm volatile("emms\n\t");
 
-#ifdef TIME_FILTER
-    struct timeval t2;
-    gettimeofday (&t2, NULL);
-    tf->seconds +=
-        (t2.tv_sec - t1.tv_sec) + (t2.tv_usec - t1.tv_usec) * .000001;
-    tf->frames = (tf->frames + 1) % TIME_INTERVAL;
-    if (tf->frames == 0)
-    {
-        fprintf (stderr,
-                 "QuickDNR: filter timed at %3f frames/sec\n",
-                 TIME_INTERVAL / tf->seconds);
-        tf->seconds = 0;
-    }
-#endif /* TIME_FILTER */
+  TF_END(tf, "QuickDNR: ");
 
   return 0;
 }
+#else // i386
+int quickdnrMMX(VideoFilter *f, VideoFrame *frame) {
+     (void)f; (void)frame;
+     return 0;
+};
+int quickdnr2MMX(VideoFilter *f, VideoFrame *frame) {
+     (void)f; (void)frame;
+     return 0;
+};
+#endif // i386
 
 void cleanup(VideoFilter *vf)
 {
@@ -558,10 +520,6 @@ VideoFilter *new_filter(VideoFrameType inpixfmt, VideoFrameType outpixfmt,
       fprintf(stderr,"Couldn't allocate memory for filter\n");
       return NULL;
     }
-#ifdef TIME_FILTER
-    struct timeval t1;
-    gettimeofday (&t1, NULL);
-#endif /* TIME_FILTER */
 
   filter->average=malloc(sizeof(uint8_t) * (*width) * 3 / 2 * (*height));
   if (filter->average == NULL)
@@ -578,15 +536,15 @@ VideoFilter *new_filter(VideoFrameType inpixfmt, VideoFrameType outpixfmt,
     case 1:
       //These might be better as logarithmic if this gets used a lot.
       filter->Luma_threshold1 = ((uint8_t) Param1) * 40 / 255;
-      filter->Luma_threshold2 = ((uint8_t) Param1) * 8 / 255;
+      filter->Luma_threshold2 = ((uint8_t)  Param1) * 4/255 > 2 ? 2 : ((uint8_t)  Param1) * 4/255;
       filter->Chroma_threshold1 = ((uint8_t) Param1) * 80 / 255;
-      filter->Chroma_threshold2 = ((uint8_t) Param1) * 16 / 255;
-      double_threshold = 1;
+      filter->Chroma_threshold2 = ((uint8_t)  Param1) * 8/255 > 4 ? 4 : ((uint8_t)  Param1) * 8/255;
+      double_threshold = 1; 
       break;
     case 2:
       filter->Luma_threshold1 = (uint8_t) Param1;
       filter->Chroma_threshold1 = (uint8_t) Param2;
-      double_threshold = -1;
+      double_threshold = 0;
       break;
     case 4:
       filter->Luma_threshold1 = (uint8_t) Param1;
@@ -641,6 +599,7 @@ VideoFilter *new_filter(VideoFrameType inpixfmt, VideoFrameType outpixfmt,
   else filter->vf.filter = &quickdnr;
 
   filter->first = 1;
+  TF_INIT(filter);
   
 #ifdef QUICKDNR_DEBUG
   fprintf(stderr,"DNR Loaded:%X Params: %u %u Luma1: %d %X%X Luma2: %X%X Chroma1: %d %X%X Chroma2: %X%X\n",
@@ -672,7 +631,7 @@ FilterInfo filter_table[] =
   {
     symbol:     "new_filter",
     name:       "quickdnr",
-    descript:   "removes noise with a fast thresholded average filter",
+    descript:   "removes noise with a fast single/double thresholded average filter",
     formats:    FmtList,
     libname:    NULL
   },
