@@ -9,6 +9,8 @@ typedef struct {
 
     int code;
     int maxcode;
+    int slicecount;
+    int lastcode;
 } via_slice_state_t;
 
 #endif
