@@ -5,7 +5,7 @@
 extern "C" {
 #endif
 
-#define LIBAVFORMAT_BUILD       4611
+#define LIBAVFORMAT_BUILD       4612
 
 #define LIBAVFORMAT_VERSION_INT FFMPEG_VERSION_INT
 #define LIBAVFORMAT_VERSION     FFMPEG_VERSION
@@ -27,9 +27,6 @@ extern "C" {
 #define MININT64 int64_t_C(0x8000000000000000)
 #endif
 
-#define AV_NOPTS_VALUE MININT64
-#define AV_TIME_BASE 1000000
-
 typedef struct AVPacket {
     int64_t pts; /* presentation time stamp in AV_TIME_BASE units (or
                     pts_den units in muxers or demuxers) */
@@ -172,6 +169,11 @@ typedef struct AVInputFormat {
        units) relative to the frames in stream component stream_index */
     int (*read_seek)(struct AVFormatContext *, 
                      int stream_index, int64_t timestamp);
+    /**
+     * gets the next timestamp in AV_TIME_BASE units.
+     */
+    int64_t (*read_timestamp)(struct AVFormatContext *s, int stream_index,
+                              int64_t *pos, int64_t pos_limit);
     /* can use flags: AVFMT_NOFILE, AVFMT_NEEDNUMBER */
     int flags;
     /* if extensions are defined, then no probe is done. You should
@@ -199,8 +201,7 @@ typedef struct AVIndexEntry {
 #define AVINDEX_KEYFRAME 0x0001
 /* the following 2 flags indicate that the next/prev keyframe is known, and scaning for it isnt needed */
     int flags;
-    int min_distance;         /* min distance between this and the previous key
-frame, used to avoid unneeded searching */
+    int min_distance;         /* min distance between this and the previous keyframe, used to avoid unneeded searching */
 } AVIndexEntry;
 
 typedef struct AVStream {
@@ -250,6 +251,7 @@ typedef struct AVStream {
 
 /* format I/O context */
 typedef struct AVFormatContext {
+    AVClass *av_class; /* set by av_alloc_format_context */
     /* can only be iformat or oformat, not both at the same time */
     struct AVInputFormat *iformat;
     struct AVOutputFormat *oformat;
@@ -259,6 +261,7 @@ typedef struct AVFormatContext {
     AVStream *streams[MAX_STREAMS];
     char filename[1024]; /* input or output filename */
     /* stream info */
+    int64_t timestamp;
     char title[512];
     char author[512];
     char copyright[512];
@@ -309,6 +312,8 @@ typedef struct AVFormatContext {
     /* av_seek_frame() support */
     int64_t data_offset; /* offset of the first packet */
     int index_built;
+
+    int build_index;
 } AVFormatContext;
 
 typedef struct AVPacketList {
@@ -374,6 +379,7 @@ extern AVImageFormat png_image_format;
 #endif
 extern AVImageFormat jpeg_image_format;
 extern AVImageFormat gif_image_format;
+extern AVImageFormat sgi_image_format;
 
 /* XXX: use automatic init with either ELF sections or C file parser */
 /* modules */
@@ -432,7 +438,7 @@ int au_init(void);
 int amr_init(void);
 
 /* wav.c */
-int wav_init(void);
+int ff_wav_init(void);
 
 /* raw.c */
 int pcm_read_seek(AVFormatContext *s, 
@@ -449,7 +455,7 @@ int yuv4mpeg_init(void);
 int ogg_init(void);
 
 /* dv.c */
-int dv_init(void);
+int ff_dv_init(void);
 
 /* ffm.c */
 int ffm_init(void);
@@ -491,6 +497,9 @@ int flic_init(void);
 /* sierravmd.c */
 int vmd_init(void);
 
+/* matroska.c */
+int matroska_init(void);
+
 //#include "rtp.h"
 
 //#include "rtsp.h"
@@ -532,6 +541,8 @@ int av_open_input_file(AVFormatContext **ic_ptr, const char *filename,
                        AVInputFormat *fmt,
                        int buf_size,
                        AVFormatParameters *ap);
+/* no av_open for output, so applications will need this: */
+AVFormatContext *av_alloc_format_context(void);
 
 #define AVERROR_UNKNOWN     (-1)  /* unknown error */
 #define AVERROR_IO          (-2)  /* i/o error */
@@ -558,7 +569,7 @@ int av_find_default_stream_index(AVFormatContext *s);
 int av_index_search_timestamp(AVStream *st, int timestamp);
 int av_add_index_entry(AVStream *st,
                        int64_t pos, int64_t timestamp, int distance, int flags);
-
+int av_seek_frame_binary(AVFormatContext *s, int stream_index, int64_t target_ts);
 
 /* media file output */
 int av_set_parameters(AVFormatContext *s, AVFormatParameters *ap);
