@@ -16,6 +16,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+#include <limits.h>
+ 
 #include "avformat.h"
 #include "avi.h"
 
@@ -105,6 +108,7 @@ static const CodecTag mov_video_tags[] = {
     { CODEC_ID_CINEPAK, MKTAG('c', 'v', 'i', 'd') }, /* Cinepak */
     { CODEC_ID_8BPS, MKTAG('8', 'B', 'P', 'S') }, /* Planar RGB (8BPS) */
     { CODEC_ID_SMC, MKTAG('s', 'm', 'c', ' ') }, /* Apple Graphics (SMC) */
+    { CODEC_ID_QTRLE, MKTAG('r', 'l', 'e', ' ') }, /* Apple Animation (RLE) */
     { CODEC_ID_NONE, 0 },
 };
 
@@ -219,6 +223,8 @@ typedef struct MOVStreamContext {
     long sample_size;
     long sample_count;
     long *sample_sizes;
+    long keyframe_count;
+    long *keyframes;
     int time_scale;
     long current_sample;
     long left_in_chunk; /* how many samples before next chunk */
@@ -897,11 +903,11 @@ static int mov_read_stsd(MOVContext *c, ByteIOContext *pb, MOV_atom_t atom)
                     /* if flag bit 3 is set, use the default palette */
                     color_count = 1 << color_depth;
                     if (color_depth == 2)
-                        color_table = qt_default_palette_4;
+                        color_table = ff_qt_default_palette_4;
                     else if (color_depth == 4)
-                        color_table = qt_default_palette_16;
+                        color_table = ff_qt_default_palette_16;
                     else
-                        color_table = qt_default_palette_256;
+                        color_table = ff_qt_default_palette_256;
 
                     for (j = 0; j < color_count; j++) {
                         r = color_table[j * 4 + 0];
@@ -989,6 +995,34 @@ static int mov_read_stsd(MOVContext *c, ByteIOContext *pb, MOV_atom_t atom)
                 size-=(16);
                 url_fskip(pb, size); /* The mp4s atom also contians a esds atom that we can skip*/
             }
+            else if( st->codec.codec_tag == MKTAG( 'm', 'p', '4', 'a' ))
+            {
+                /* Handle mp4 audio tag */
+                get_be32(pb); /* version */
+                get_be32(pb);
+                st->codec.channels = get_be16(pb); /* channels */
+                st->codec.bits_per_sample = get_be16(pb); /* bits per sample */
+                get_be32(pb);
+                st->codec.sample_rate = get_be16(pb); /* sample rate, not always correct */
+                get_be16(pb);
+                c->mp4=1;
+		{
+                MOV_atom_t a = { format, url_ftell(pb), size - (20 + 20 + 8) };
+                mov_read_default(c, pb, a);
+		}
+                /* Get correct sample rate from extradata */
+                if(st->codec.extradata_size) {
+                   const int samplerate_table[] = {
+                     96000, 88200, 64000, 48000, 44100, 32000, 
+                     24000, 22050, 16000, 12000, 11025, 8000,
+                     7350, 0, 0, 0
+                   };
+                   unsigned char *px = st->codec.extradata;
+                   // 5 bits objectTypeIndex, 4 bits sampleRateIndex, 4 bits channels
+                   int samplerate_index = ((px[0] & 7) << 1) + ((px[1] >> 7) & 1);
+                   st->codec.sample_rate = samplerate_table[samplerate_index];
+                }
+            }
 	    else if(size>=(16+20))
 	    {//16 bytes read, reading atleast 20 more
 #ifdef DEBUG
@@ -1101,6 +1135,34 @@ printf("track[%i].stsc.entries = %i\n", c->fc->nb_streams-1, entries);
     return 0;
 }
 
+static int mov_read_stss(MOVContext *c, ByteIOContext *pb, MOV_atom_t atom)
+{
+    AVStream *st = c->fc->streams[c->fc->nb_streams-1];
+    MOVStreamContext *sc = (MOVStreamContext *)st->priv_data;
+    int entries, i;
+
+    print_atom("stss", atom);
+
+    get_byte(pb); /* version */
+    get_byte(pb); get_byte(pb); get_byte(pb); /* flags */
+
+    entries = get_be32(pb);
+    sc->keyframe_count = entries;
+#ifdef DEBUG
+    av_log(NULL, AV_LOG_DEBUG, "keyframe_count = %ld\n", sc->keyframe_count);
+#endif
+    sc->keyframes = (long*) av_malloc(entries * sizeof(long));
+    if (!sc->keyframes)
+        return -1;
+    for(i=0; i<entries; i++) {
+        sc->keyframes[i] = get_be32(pb);
+#ifdef DEBUG
+/*        av_log(NULL, AV_LOG_DEBUG, "keyframes[]=%ld\n", sc->keyframes[i]); */
+#endif
+    }
+    return 0;
+}
+
 static int mov_read_stsz(MOVContext *c, ByteIOContext *pb, MOV_atom_t atom)
 {
     AVStream *st = c->fc->streams[c->fc->nb_streams-1];
@@ -1137,8 +1199,8 @@ static int mov_read_stts(MOVContext *c, ByteIOContext *pb, MOV_atom_t atom)
     AVStream *st = c->fc->streams[c->fc->nb_streams-1];
     //MOVStreamContext *sc = (MOVStreamContext *)st->priv_data;
     int entries, i;
-    int duration=0;
-    int total_sample_count=0;
+    int64_t duration=0;
+    int64_t total_sample_count=0;
 
     print_atom("stts", atom);
 
@@ -1174,31 +1236,16 @@ printf("track[%i].stts.entries = %i\n", c->fc->nb_streams-1, entries);
 #endif
     }
 
-#define MAX(a,b) a>b?a:b
-#define MIN(a,b) a>b?b:a
     /*The stsd atom which contain codec type sometimes comes after the stts so we cannot check for codec_type*/
     if(duration>0)
     {
-        //Find greatest common divisor to avoid overflow using the Euclidean Algorithm...
-        uint32_t max=MAX(duration,total_sample_count);
-        uint32_t min=MIN(duration,total_sample_count);
-        uint32_t spare=max%min;
-
-        while(spare!=0)
-        {
-            max=min;
-            min=spare;
-            spare=max%min;
-        }
-        
-        duration/=min;
-        total_sample_count/=min;
-
-        //Only support constant frame rate. But lets calculate the average. Needed by .mp4-files created with nec e606 3g phone.
-        //To get better precision, we use the duration as frame_rate_base
-
-        st->codec.frame_rate_base=duration;
-        st->codec.frame_rate = c->streams[c->total_streams]->time_scale * total_sample_count;
+        av_reduce(
+            &st->codec.frame_rate, 
+            &st->codec.frame_rate_base, 
+            c->streams[c->total_streams]->time_scale * total_sample_count,
+            duration,
+            INT_MAX
+        );
 
 #ifdef DEBUG
         printf("FRAME RATE average (video or audio)= %f (tot sample count= %i ,tot dur= %i timescale=%d)\n", (float)st->codec.frame_rate/st->codec.frame_rate_base,total_sample_count,duration,c->streams[c->total_streams]->time_scale);
@@ -1209,8 +1256,6 @@ printf("track[%i].stts.entries = %i\n", c->fc->nb_streams-1, entries);
         st->codec.frame_rate_base = 1;
         st->codec.frame_rate = c->streams[c->total_streams]->time_scale;
     }
-#undef MAX
-#undef MIN
     return 0;
 }
 
@@ -1408,7 +1453,7 @@ static const MOVParseTableEntry mov_default_parse_table[] = {
 { MKTAG( 's', 't', 's', 'c' ), mov_read_stsc },
 { MKTAG( 's', 't', 's', 'd' ), mov_read_stsd }, /* sample description */
 { MKTAG( 's', 't', 's', 'h' ), mov_read_default },
-{ MKTAG( 's', 't', 's', 's' ), mov_read_leaf }, /* sync sample */
+{ MKTAG( 's', 't', 's', 's' ), mov_read_stss }, /* sync sample */
 { MKTAG( 's', 't', 's', 'z' ), mov_read_stsz }, /* sample size */
 { MKTAG( 's', 't', 't', 's' ), mov_read_stts },
 { MKTAG( 't', 'k', 'h', 'd' ), mov_read_tkhd }, /* track header */
@@ -1454,6 +1499,7 @@ static void mov_free_stream_context(MOVStreamContext *sc)
         av_free(sc->chunk_offsets);
         av_free(sc->sample_to_chunk);
         av_free(sc->sample_sizes);
+        av_free(sc->keyframes);
         av_free(sc->header_data);
         av_free(sc);
     }
@@ -1524,7 +1570,7 @@ static int mov_read_header(AVFormatContext *s, AVFormatParameters *ap)
 	atom.size = 0x7FFFFFFFFFFFFFFFLL;
 
     if (atom.size == 0)
-         atom.size = 0x7FFFFFFFFFFFFFFFLL;
+        atom.size = 0x7FFFFFFFFFFFFFFFLL;
 
 #ifdef DEBUG
     printf("filesz=%Ld\n", atom.size);
@@ -1533,7 +1579,7 @@ static int mov_read_header(AVFormatContext *s, AVFormatParameters *ap)
     /* check MOV header */
     err = mov_read_default(mov, pb, atom);
     if (err<0 || (!mov->found_moov && !mov->found_mdat)) {
-	fprintf(stderr, "mov: header not found !!! (err:%d, moov:%d, mdat:%d) pos:%lld\n",
+	av_log(s, AV_LOG_ERROR, "mov: header not found !!! (err:%d, moov:%d, mdat:%d) pos:%lld\n",
 		err, mov->found_moov, mov->found_mdat, url_ftell(pb));
 	return -1;
     }
@@ -1585,7 +1631,7 @@ static int mov_read_packet(AVFormatContext *s, AVPacket *pkt)
     MOVContext *mov = (MOVContext *) s->priv_data;
     MOVStreamContext *sc;
     int64_t offset = 0x0FFFFFFFFFFFFFFFLL;
-    int i;
+    int i, a, b, m;
     int size;
     size = 0x0FFFFFFF;
 
@@ -1661,7 +1707,13 @@ again:
         for(i=0; i<(sc->sample_to_chunk_sz); i++) {
             if( (sc->sample_to_chunk[i].first)<=(sc->next_chunk) && (sc->sample_size>0) )
             {
-                foundsize=sc->sample_to_chunk[i].count*sc->sample_size;
+		// I can't figure out why for PCM audio sample_size is always 1
+		// (it should actually be channels*bits_per_second/8) but it is.
+		AVCodecContext* cod = &s->streams[sc->ffindex]->codec;
+                if (sc->sample_size == 1 && (cod->codec_id == CODEC_ID_PCM_S16BE || cod->codec_id == CODEC_ID_PCM_S16LE))
+		    foundsize=(sc->sample_to_chunk[i].count*cod->channels*cod->bits_per_sample)/8;
+		else
+		    foundsize=sc->sample_to_chunk[i].count*sc->sample_size;
             }
 #ifdef DEBUG
             /*printf("sample_to_chunk first=%ld count=%ld, id=%ld\n", sc->sample_to_chunk[i].first, sc->sample_to_chunk[i].count, sc->sample_to_chunk[i].id);*/
@@ -1719,6 +1771,27 @@ readchunk:
         get_buffer(&s->pb, pkt->data, pkt->size);
     }
     pkt->stream_index = sc->ffindex;
+    
+    // If the keyframes table exists, mark any samples that are in the table as key frames.
+    // If no table exists, treat very sample as a key frame.
+    if (sc->keyframes) {        
+        a = 0;
+        b = sc->keyframe_count - 1;
+        
+        while (a < b) {
+            m = (a + b + 1) >> 1;
+            if (sc->keyframes[m] > sc->current_sample) {
+                b = m - 1;
+            } else {
+                a = m;
+            }    
+        }
+        
+        if (sc->keyframes[a] == sc->current_sample)
+            pkt->flags |= PKT_FLAG_KEY;
+    }
+    else
+        pkt->flags |= PKT_FLAG_KEY;
 
 #ifdef DEBUG
 /*
