@@ -379,7 +379,7 @@ static void pmt_cb(void *opaque, const uint8_t *section, int section_len)
     
 #ifdef DEBUG_SI
     printf("PMT:\n");
-    av_hex_dump((uint8_t *)section, section_len);
+    av_hex_dump(stdout, (uint8_t *)section, section_len);
 #endif
     p_end = section + section_len - 4;
     p = section;
@@ -454,7 +454,7 @@ static void pat_cb(void *opaque, const uint8_t *section, int section_len)
 
 #ifdef DEBUG_SI
     printf("PAT:\n");
-    av_hex_dump((uint8_t *)section, section_len);
+    av_hex_dump(stdout, (uint8_t *)section, section_len);
 #endif
     p_end = section + section_len - 4;
     p = section;
@@ -503,7 +503,7 @@ static void pat_scan_cb(void *opaque, const uint8_t *section, int section_len)
 
 #ifdef DEBUG_SI
     printf("PAT:\n");
-    av_hex_dump((uint8_t *)section, section_len);
+    av_hex_dump(stdout, (uint8_t *)section, section_len);
 #endif
     p_end = section + section_len - 4;
     p = section;
@@ -564,7 +564,7 @@ static void sdt_cb(void *opaque, const uint8_t *section, int section_len)
 
 #ifdef DEBUG_SI
     printf("SDT:\n");
-    av_hex_dump((uint8_t *)section, section_len);
+    av_hex_dump(stdout, (uint8_t *)section, section_len);
 #endif
 
     p_end = section + section_len - 4;
@@ -840,6 +840,7 @@ static void mpegts_push_data(void *opaque,
                     memcpy(pkt->data, p, len);
                     pkt->stream_index = pes->st->index;
                     pkt->pts = pes->pts;
+                    pkt->dts = pes->dts;
                     pkt->startpos = pes->startpos;
                     /* reset pts values */
                     pes->pts = AV_NOPTS_VALUE;
@@ -1132,8 +1133,12 @@ static int mpegts_read_header(AVFormatContext *s,
                 handle_packets(ts, MAX_SCAN_PACKETS);
             }
             
-            if (ts->nb_services <= 0)
-                return -1;
+            if (ts->nb_services <= 0) {
+               /* raw transport stream */
+               ts->auto_guess = 1;
+               s->ctx_flags |= AVFMTCTX_NOHEADER;
+               goto do_pcr;
+           }
             
             /* tune to first service found */
             service = ts->services[0];
@@ -1170,7 +1175,8 @@ static int mpegts_read_header(AVFormatContext *s,
 
         s->pts_num = 1;
         s->pts_den = 27000000;
-        
+       
+    do_pcr: 
         st = av_new_stream(s, 0);
         if (!st)
             goto fail;
@@ -1469,7 +1475,7 @@ int mpegts_parse_packet(MpegTSContext *ts, AVPacket *pkt,
         if (len < TS_PACKET_SIZE)
             return -1;
         if (buf[0] != 0x47) {
-            buf--;
+            buf++;
             len--;
         } else {
             handle_packet(ts, buf, position);
