@@ -19,6 +19,7 @@ using namespace std;
 
 AudioOutputOSS::AudioOutputOSS(QString audiodevice, int laudio_bits, 
                                int laudio_channels, int laudio_samplerate)
+              : AudioOutput()
 {
     pthread_mutex_init(&audio_buflock, NULL);
     pthread_mutex_init(&avsync_lock, NULL);
@@ -64,7 +65,7 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     audio_samplerate = laudio_samplerate;
     if(audio_bits != 8 && audio_bits != 16)
     {
-        cerr << "AudioOutDSP only supports 8 or 16bit audio ";
+        Error("AudioOutputOSS only supports 8 or 16bit audio.");
         return;
     }
     audio_bytes_per_sample = audio_channels * audio_bits / 8;
@@ -88,11 +89,12 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
         {
             if (errno == EBUSY)
             {
-                cerr << "ERROR: something is currently using: " << audiodevice
-                     << "\nFix this, then run mythfrontend again\n";
-                exit(1);
+                Error(QString("WARNING: something is currently"
+                              " using: %1, retrying.").arg(audiodevice));
+                return;
             }
-            cerr << "Error opening audio device (" << audiodevice << "):\n";
+            VERBOSE(VB_IMPORTANT, QString("Error opening audio device (%1), the"
+                    " error was: %2").arg(audiodevice).arg(strerror(errno)));
             perror(audiodevice.ascii());
         }
         if (audiofd < 0)
@@ -101,12 +103,15 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
 
     if (audiofd == -1)
     {
-        cerr << "player: Can't open audio device: " << audiodevice << endl;
+        Error(QString("Error opening audio device (%1), the error was: %2")
+              .arg(audiodevice).arg(strerror(errno)));
         return;
     }
 
     fcntl(audiofd, F_SETFL, fcntl(audiofd, F_GETFL) & ~O_NONBLOCK);
 
+    SetFragSize();
+
     bool err = false;
 
     if (audio_channels > 2)
@@ -127,11 +132,9 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
 
     if (err)
     {
-        cerr << "player: " << audiodevice 
-             << ": error setting audio output device to "
-             << audio_samplerate << "kHz/" 
-             << audio_bits << "bits/"
-             << audio_channels << "channel\n";
+        Error(QString("Unable to set audio device (%1) to %2 kHz / %3 bits"
+                      " / %4 channels").arg(audiodevice).arg(audio_samplerate)
+                      .arg(audio_bits).arg(audio_channels));
         close(audiofd);
         audiofd = -1;
         return;
@@ -141,9 +144,12 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     
     audio_buf_info info;
     ioctl(audiofd, SNDCTL_DSP_GETOSPACE, &info);
+    fragment_size = info.fragsize;
 
-    audio_buffer_unused = info.bytes - audio_bytes_per_sample * 
-                          audio_samplerate / 10;
+    VERBOSE(VB_GENERAL, QString("Audio fragment size: %1")
+                                 .arg(fragment_size));
+
+    audio_buffer_unused = info.bytes - (fragment_size * 4);
     if(audio_buffer_unused < 0)
        audio_buffer_unused = 0;
 
@@ -156,11 +162,12 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     {
         if (!(caps & DSP_CAP_REALTIME))
         {
-            cerr << "audio device cannot report buffer state accurately!\n"
-                 << "audio/video sync will be bad, continuing anyway...\n";
+            VERBOSE(VB_IMPORTANT, "The audio device cannot report buffer state"
+                    " accurately! audio/video sync will be bad, continuing...");
         }
     } else {
-        perror("ioctl(SNDCTL_DSP_GETCAPS)");
+        VERBOSE(VB_IMPORTANT, QString("Unable to get audio card capabilities,"
+                " the error was: %1").arg(strerror(errno)));
     }
 
     audbuf_timecode = 0;
@@ -172,14 +179,48 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     
     pthread_mutex_unlock(&avsync_lock);
     pthread_mutex_unlock(&audio_buflock);
-    //printf("Ending reconfigure\n");
+    VERBOSE(VB_AUDIO, "Ending reconfigure");
+}
+
+/**
+ * Set the fragsize to something slightly smaller than the number of bytes of
+ * audio for one frame of video.
+ */
+void AudioOutputOSS::SetFragSize()
+{
+    // I think video_frame_rate isn't necessary. Someone clearly thought it was
+    // useful but I don't see why. Let's just hardcode 30 for now...
+    // if there's a problem, it can be added back.
+    const int video_frame_rate = 30;
+    const int bits_per_byte = 8;
+
+    // get rough measurement of audio bytes per frame of video
+    int fbytes = (audio_bits * audio_channels * audio_samplerate) / 
+                        (bits_per_byte * video_frame_rate);
+
+    // find the next smaller number that's a power of 2 
+    // there's probably a better way to do this
+    int count = 0;
+    while ( fbytes >> 1 )
+    {
+        fbytes >>= 1;
+        count++;
+    }
+
+    if (count > 4)
+    {
+        // High order word is the max number of fragments
+        int frag = 0x7fff0000 + count;
+        ioctl(audiofd, SNDCTL_DSP_SETFRAGMENT, &frag);
+        // ignore failure, since we check the actual fragsize before use
+    }
 }
 
 void AudioOutputOSS::KillAudio()
 {
     killAudioLock.lock();
 
-    //printf("Killing AudioOutputDSP\n");
+    VERBOSE(VB_AUDIO, "Killing AudioOutputDSP");
     if (output_audio)
     {
         killaudio = true;
@@ -237,7 +278,9 @@ void AudioOutputOSS::WriteAudio(unsigned char *aubuf, int size)
 
     if (lw < 0)
     {
-        perror("Writing to audio device");
+        Error(QString("Error writing to audio device (%1), unable to"
+              " continue. The error was: %2").arg(audiodevice)
+              .arg(strerror(errno)));
         close(audiofd);
         audiofd = -1;
         return;
@@ -253,7 +296,7 @@ void AudioOutputOSS::SetTimecode(long long timecode)
 
 void AudioOutputOSS::SetEffDsp(int dsprate)
 {
-    //printf("SetEffDsp: %d\n", dsprate);
+    VERBOSE(VB_AUDIO, QString("SetEffDsp: %1").arg(dsprate));
     effdsp = dsprate;
 }
 
@@ -361,7 +404,8 @@ void AudioOutputOSS::SetAudiotime(void)
 void AudioOutputOSS::AddSamples(char *buffers[], int samples, 
                                 long long timecode)
 {
-    //printf("AddSamples[] %d\n", samples * audio_bytes_per_sample);
+    VERBOSE(VB_AUDIO, QString("AddSamples[] %1")
+                              .arg(samples * audio_bytes_per_sample));
     pthread_mutex_lock(&audio_buflock);
 
     int audio_bytes = audio_bits / 8;
@@ -371,14 +415,14 @@ void AudioOutputOSS::AddSamples(char *buffers[], int samples,
     {
         if (blocking)
         {
-            //printf("Waiting for free space\n");
+            VERBOSE(VB_AUDIO, "Waiting for free space");
             // wait for more space
             pthread_cond_wait(&audio_bufsig, &audio_buflock);
             afree = audiofree(false);
         }
         else
         {
-            cout << "Audio buffer overflow, audio data lost!\n";
+            VERBOSE(VB_IMPORTANT, "Audio buffer overflow, audio data lost!");
             samples = afree / audio_bytes_per_sample;
         }
     }
@@ -409,7 +453,8 @@ void AudioOutputOSS::AddSamples(char *buffers[], int samples,
 
 void AudioOutputOSS::AddSamples(char *buffer, int samples, long long timecode)
 {
-    //printf("AddSamples %d\n", samples * audio_bytes_per_sample);
+    VERBOSE(VB_AUDIO, QString("AddSamples %1")
+                              .arg(samples * audio_bytes_per_sample));
     pthread_mutex_lock(&audio_buflock);
 
     int afree = audiofree(false);
@@ -420,14 +465,14 @@ void AudioOutputOSS::AddSamples(char *buffer, int samples, long long timecode)
     {
         if (blocking)
         {
-            //printf("Waiting for free space\n");
+            VERBOSE(VB_AUDIO, "Waiting for free space");
             // wait for more space
             pthread_cond_wait(&audio_bufsig, &audio_buflock);
             afree = audiofree(false);
         }
         else
         {
-            cout << "Audio buffer overflow, audio data lost!\n";
+            VERBOSE(VB_IMPORTANT, "Audio buffer overflow, audio data lost!");
             len = afree;
         }
     }
@@ -468,8 +513,8 @@ inline int AudioOutputOSS::getSpaceOnSoundcard(void)
         numbadioctls++;
         if (numbadioctls > 2 || space < -5000)
         {
-            cerr << "Your soundcard is not reporting free space correctly.\n"
-                 << "Falling back to old method...\n";
+            VERBOSE(VB_IMPORTANT, "Your soundcard is not reporting free space"
+                    " correctly. Falling back to old method...");
             audio_buffer_unused = 0;
             space = info.bytes;
         }
@@ -482,11 +527,10 @@ inline int AudioOutputOSS::getSpaceOnSoundcard(void)
 
 void AudioOutputOSS::OutputAudioLoop(void)
 {
-    int bytesperframe;
     int space_on_soundcard;
-    unsigned char zeros[1024];
+    unsigned char zeros[fragment_size];
  
-    bzero(zeros, 1024);
+    bzero(zeros, fragment_size);
 
     while (!killaudio)
     {
@@ -503,13 +547,15 @@ void AudioOutputOSS::OutputAudioLoop(void)
             // should this use ioctl(audio_fd, SNDCTL_DSP_POST, 0) instead ?
             
             space_on_soundcard = getSpaceOnSoundcard();
-            if (1024 < space_on_soundcard)
+            if (fragment_size < space_on_soundcard)
             {
-                WriteAudio(zeros, 1024);
+                WriteAudio(zeros, fragment_size);
             }
             else
             {
-                //printf("waiting for space to write 1024 zeros on soundcard which has %d bytes free\n",space_on_soundcard);
+                VERBOSE(VB_AUDIO, QString("waiting for space to write 1024 "
+                        "zeros on soundcard which has %1 bytes free")
+                        .arg(space_on_soundcard));
                 usleep(50);
             }
 
@@ -520,18 +566,12 @@ void AudioOutputOSS::OutputAudioLoop(void)
 
         /* do audio output */
         
-        /* approximate # of audio bytes for each frame. */
-// I think video_frame_rate isn't necessary. Someone clearly thought it was
-// useful but I don't see why. Let's just hardcode 30 for now...
-// if there's a problem, it can be added back.
-#define video_frame_rate 30
-        bytesperframe = audio_bytes_per_sample * 
-                        (int)(effdsp / 100.0 / video_frame_rate + 0.5);
-        
         // wait for the buffer to fill with enough to play
-        if (bytesperframe >= audiolen(true))
-        { 
-            //printf("audio thread waiting for buffer to fill (bytesperframe=%d,audiolen=%d\n", bytesperframe, audiolen(true));
+        if (fragment_size >= audiolen(true))
+        {
+            VERBOSE(VB_AUDIO, QString("audio thread waiting for buffer to fill"
+                                      " fragment_size=%1, audiolen=%2")
+                                      .arg(fragment_size).arg(audiolen(true)));
             usleep(200);
             continue;
         }
@@ -540,13 +580,14 @@ void AudioOutputOSS::OutputAudioLoop(void)
         // without blocking.  We don't want to block while holding audio_buflock
         
         space_on_soundcard = getSpaceOnSoundcard();
-        if (bytesperframe > space_on_soundcard)
+        if (fragment_size > space_on_soundcard)
         {
-            //printf("waiting for space to write %d bytes on soundcard whish has %d bytes free\n", bytesperframe, space_on_soundcard);
+            VERBOSE(VB_AUDIO, QString("Waiting for space on soundcard: "
+                                 "space=%1").arg(space_on_soundcard));
             numlowbuffer++;
             if (numlowbuffer > 5 && audio_buffer_unused)
             {
-                cerr << "dropping back audio_buffer_unused\n";
+                VERBOSE(VB_IMPORTANT, "dropping back audio_buffer_unused");
                 audio_buffer_unused /= 2;
             }
 
@@ -560,22 +601,25 @@ void AudioOutputOSS::OutputAudioLoop(void)
 
         // re-check audiolen() in case things changed.
         // for example, ClearAfterSeek() might have run
-        if (bytesperframe < audiolen(false))
+        if (fragment_size < audiolen(false))
         {
             int bdiff = AUDBUFSIZE - raud;
-            if (bytesperframe > bdiff)
+            if (fragment_size > bdiff)
             {
-                WriteAudio(audiobuffer + raud, bdiff);
-                WriteAudio(audiobuffer, bytesperframe - bdiff);
+                // always want to write whole fragments
+                unsigned char fragment[fragment_size];
+                memcpy(fragment, audiobuffer + raud, bdiff);
+                memcpy(fragment + bdiff, audiobuffer, fragment_size - bdiff);
+                WriteAudio(fragment, fragment_size);
             }
             else
             {
-                WriteAudio(audiobuffer + raud, bytesperframe);
+                WriteAudio(audiobuffer + raud, fragment_size);
             }
 
             /* update raud */
-            raud = (raud + bytesperframe) % AUDBUFSIZE;
-            //printf("Broadcasting free space avail\n");
+            raud = (raud + fragment_size) % AUDBUFSIZE;
+            VERBOSE(VB_AUDIO, "Broadcasting free space avail");
             pthread_cond_broadcast(&audio_bufsig);
         }
         pthread_mutex_unlock(&audio_buflock); // end critical section
@@ -585,9 +629,10 @@ void AudioOutputOSS::OutputAudioLoop(void)
 
 void *AudioOutputOSS::kickoffOutputAudioLoop(void *player)
 {
-    //printf("kickoffOutputAudioLoop: pid = %d\n", getpid());
+    VERBOSE(VB_AUDIO, QString("kickoffOutputAudioLoop: pid = %1")
+                              .arg(getpid()));
     ((AudioOutputOSS *)player)->OutputAudioLoop();
-    //printf("kickoffOutputAudioLoop exiting\n");
+    VERBOSE(VB_AUDIO, "kickoffOutputAudioLoop exiting");
     return NULL;
 }
 
