@@ -23,6 +23,7 @@
 #include "dialogbox.h"
 #include "mythdialogs.h"
 #include "mythplugin.h"
+#include "screensaver.h"
 
 MythContext *gContext = NULL;
 
@@ -36,6 +37,8 @@ class MythContextPrivate
 
     void Init(bool gui, bool lcd);
 
+    void LoadLogSettings(void);
+
     MythContext *parent;
 
     Settings *m_settings;
@@ -44,6 +47,7 @@ class MythContextPrivate
     QString m_installprefix;
 
     bool m_themeloaded;
+    QString m_menuthemepathname;
     QString m_themepathname;
     QPixmap *m_backgroundimage;
     QPalette m_palette;
@@ -81,6 +85,12 @@ class MythContextPrivate
     bool disablelibrarypopup;
 
     MythPluginManager *pluginmanager;
+
+    ScreenSaverControl *screensaver;
+
+    int m_logenable, m_logmaxcount, m_logprintlevel;
+    QMap<QString,int> lastLogCounts;
+    QMap<QString,QString> lastLogStrings;
 };
 
 MythContextPrivate::MythContextPrivate(MythContext *lparent)
@@ -103,6 +113,7 @@ MythContextPrivate::MythContextPrivate(MythContext *lparent)
     m_backgroundimage = NULL;
 
     m_db = QSqlDatabase::addDatabase("QMYSQL3", "MythContext");
+    screensaver = new ScreenSaverControl();
 }
 
 void MythContextPrivate::Init(bool gui, bool lcd)
@@ -145,6 +156,10 @@ void MythContextPrivate::Init(bool gui, bool lcd)
     serverSock = NULL;
     eventSock = new QSocket(0);
 
+    m_logenable = -1;
+    m_logmaxcount = -1;
+    m_logprintlevel = -1;
+
     mainWindow = NULL;
 
     if (lcd)
@@ -171,6 +186,13 @@ MythContextPrivate::~MythContextPrivate()
         delete lcd_device;
 }
 
+void MythContextPrivate::LoadLogSettings(void)
+{
+    m_logenable = parent->GetNumSetting("LogEnabled", 0);
+    m_logmaxcount = parent->GetNumSetting("LogMaxCount", 0);
+    m_logprintlevel = parent->GetNumSetting("LogPrintLevel", LP_ERROR);
+}
+
 MythContext::MythContext(const QString &binversion, bool gui, bool lcd)
            : QObject()
 {
@@ -361,6 +383,10 @@ void MythContext::LoadQtConfig(void)
 
     d->m_qtThemeSettings = new Settings;
 
+    QString style = GetSetting("Style", "");
+    if (style != "")
+        qApp->setStyle(style);
+
     QString themename = GetSetting("Theme");
     QString themedir = FindThemeDir(themename);
     
@@ -374,9 +400,17 @@ void MythContext::LoadQtConfig(void)
         delete d->m_backgroundimage;
     d->m_backgroundimage = NULL;
 
+    themename = GetSetting("MenuTheme");
+    d->m_menuthemepathname = FindThemeDir(themename) +"/";
+    
     InitializeScreenSettings();
 }
 
+void MythContext::RefreshBackendConfig(void)
+{
+    d->LoadLogSettings();
+}
+
 void MythContext::UpdateImageCache(void)
 {
     d->imageCache.clear();
@@ -663,9 +697,23 @@ QString MythContext::FindThemeDir(const QString &themename)
     if (dir.exists())
         return testdir;
 
+
+    // Don't complain about the "default" theme being missing
+    if(themename == QObject::tr("Default"))
+    {
+        testdir = d->m_installprefix + "/share/mythtv/";
+        dir.setPath(testdir);
+        if (dir.exists())
+            return testdir;
+    }
+    
     cerr << "Could not find theme: " << themename << endl;
     return "";
 }
+QString MythContext::GetMenuThemeDir(void)
+{
+    return d->m_menuthemepathname;
+}
 
 QString MythContext::GetThemeDir(void)
 {
@@ -772,8 +820,8 @@ void MythContext::SaveSetting(const QString &key, const QString &newValue)
     {
         KickDatabase(d->m_db);
 
-        QString querystr = QString("DELETE FROM settings WHERE value = '%1' "
-                                   "AND hostname = '%2';")
+        QString querystr = QString("DELETE FROM settings WHERE value = \"%1\" "
+                                   "AND hostname = \"%2\";")
                                   .arg(key).arg(d->m_localhostname);
 
         QSqlQuery result = d->m_db->exec(querystr);
@@ -781,7 +829,7 @@ void MythContext::SaveSetting(const QString &key, const QString &newValue)
             MythContext::DBError("Clear setting", querystr);
 
         querystr = QString("INSERT settings ( value, data, hostname ) "
-                           "VALUES ( '%1', '%2', '%3' );")
+                           "VALUES ( \"%1\", \"%2\", \"%3\" );")
                            .arg(key).arg(newValue).arg(d->m_localhostname);
 
         result = d->m_db->exec(querystr);
@@ -1238,7 +1286,6 @@ void MythContext::EventSocketRead(void)
 
         QString prefix = strlist[0];
         QString message = strlist[1];
-        QString extra = strlist[2];
         
         if (prefix != "BACKEND_MESSAGE")
         {
@@ -1247,7 +1294,9 @@ void MythContext::EventSocketRead(void)
         }
         else
         {
-            MythEvent me(message, extra);
+            strlist.pop_front();
+            strlist.pop_front();
+            MythEvent me(message, strlist);
             dispatch(me);
         }
     }
@@ -1272,12 +1321,12 @@ bool MythContext::CheckProtoVersion(QSocketDevice* socket)
             MythPopupBox::showOkPopup(d->mainWindow, 
                                       "Connection failure",
                                       tr(QString("The server uses network "
-                                                 "protocol version %1,"
+                                                 "protocol version %1, "
                                                  "but this client only "
                                                  "understands version %2.  "
                                                  "Make sure you are running "
-                                                 "compatible versions of"
-                                                 "the backend and frontend"))
+                                                 "compatible versions of "
+                                                 "the backend and frontend."))
                                                  .arg(strlist[1])
                                                  .arg(MYTH_PROTO_VERSION));
             qApp->unlock();
@@ -1428,3 +1477,118 @@ MythPluginManager *MythContext::getPluginManager(void)
     return d->pluginmanager;
 }
 
+
+void MythContext::DisableScreensaver(void)
+{
+    QApplication::postEvent(GetMainWindow(),
+            new ScreenSaverEvent(ScreenSaverEvent::ssetDisable));
+}
+
+void MythContext::RestoreScreensaver(void)
+{
+    QApplication::postEvent(GetMainWindow(),
+            new ScreenSaverEvent(ScreenSaverEvent::ssetRestore));
+}
+
+void MythContext::ResetScreensaver(void)
+{
+    QApplication::postEvent(GetMainWindow(),
+            new ScreenSaverEvent(ScreenSaverEvent::ssetReset));
+}
+
+void MythContext::DoDisableScreensaver(void)
+{
+    d->screensaver->Disable();
+}
+
+void MythContext::DoRestoreScreensaver(void)
+{
+    d->screensaver->Restore();
+}
+
+void MythContext::DoResetScreensaver(void)
+{
+    d->screensaver->Reset();
+}
+
+void MythContext::LogEntry(const QString &module, int priority,
+                           const QString &message, const QString &details)
+{
+    unsigned int logid;
+    int howmany;
+
+    if (d->m_logenable == -1) // Haven't grabbed the settings yet
+        d->LoadLogSettings();
+    if (d->m_logenable == 1)
+    {
+        if (message.left(21) != "Last message repeated")
+        {
+            if (message == d->lastLogStrings[module])
+            {
+                d->lastLogCounts[module] += 1;
+                return;
+            }
+            else
+            {
+                if (0 < d->lastLogCounts[module])
+                {
+                    LogEntry(module, priority, 
+                             QString("Last message repeated %1 times")
+                                    .arg(d->lastLogCounts[module]),
+                             d->lastLogStrings[module]);
+                }
+
+                d->lastLogCounts[module] = 0;
+                d->lastLogStrings[module] = message;
+            }
+        }
+
+        d->dbLock.lock();
+    
+        if (d->m_db->isOpen())
+        {
+            KickDatabase(d->m_db);
+    
+            QString querystr = QString("INSERT INTO mythlog (module, priority, "
+                                       "logdate, host, message, details) "
+                                       "values ( '%1', %2, now(), '%3', "
+                                       "'%4','%5' );") 
+                                       .arg(module) 
+                                       .arg(priority)
+                                       .arg(d->m_localhostname)
+                                       .arg(message).arg(details);
+    
+            QSqlQuery result = d->m_db->exec(querystr);
+            if (!result.isActive())
+                MythContext::DBError("LogEntry", querystr);
+
+            if (d->m_logmaxcount > 0)
+            {
+                querystr = QString("SELECT logid FROM mythlog WHERE "
+                                   "module='%1' ORDER BY logdate ASC") 
+                                   .arg(module);
+                result = d->m_db->exec(querystr);
+                howmany = result.size();
+                if (howmany > d->m_logmaxcount)
+                {
+                    while (howmany > d->m_logmaxcount)
+                    {
+                        result.next();
+                        logid = result.value(0).toUInt();
+                        querystr = QString("DELETE FROM mythlog WHERE "
+                                           "logid=%1").arg(logid);
+                        d->m_db->exec(querystr);
+                        howmany--;
+                    }
+                }
+            }
+        }
+    
+        d->dbLock.unlock();
+
+	if (priority <= d->m_logprintlevel)
+            VERBOSE(VB_ALL, module + ": " + message);
+    }
+}
+
+
