@@ -10,8 +10,10 @@
 #include <qevent.h>
 #include <qmutex.h>
 #include <qsocketdevice.h>
+#include <qstringlist.h>
 #include <qnetwork.h> 
 
+#include <iostream>
 #include <vector>
 using namespace std;
 
@@ -38,10 +40,23 @@ enum VerboseMask {
     VB_FILE      = 0x0040,
     VB_SCHEDULE  = 0x0080,
     VB_NETWORK   = 0x0100,
+    VB_COMMFLAG  = 0x0200,
+    VB_AUDIO     = 0x0400,
     VB_NONE      = 0x0000,
     VB_ALL       = 0xffff
 };
 
+enum LogPriorities {
+    LP_EMERG     = 0,
+    LP_ALERT     = 1,
+    LP_CRITICAL  = 2,
+    LP_ERROR     = 3,
+    LP_WARNING   = 4,
+    LP_NOTICE    = 5,
+    LP_INFO      = 6,
+    LP_DEBUG     = 7
+};
+
 #define VERBOSE(mask,args...) \
 do { \
 if ((print_verbose_messages & mask) != 0) \
@@ -59,7 +74,7 @@ class MythEvent : public QCustomEvent
         message = lmessage;
         extradata = "empty";
     }
-    MythEvent(const QString &lmessage, const QString &lextradata)
+    MythEvent(const QString &lmessage, const QStringList &lextradata)
            : QCustomEvent(MythEventMessage)
     {
         message = lmessage;
@@ -68,16 +83,18 @@ class MythEvent : public QCustomEvent
     
     ~MythEvent() {}
 
-    const QString Message() const { return message; }
-    const QString ExtraData() const { return extradata; } 
+    const QString& Message() const { return message; }
+    const QString& ExtraData(int idx = 0) const { return extradata[idx]; } 
+    const QStringList& ExtraDataList() const { return extradata; } 
+    int ExtraDataCount() const { return extradata.size(); }
 
   private:
     QString message;
-    QString extradata;
+    QStringList extradata;
 };
 
-#define MYTH_BINARY_VERSION "0.14.20040123-1"
-#define MYTH_PROTO_VERSION "1"
+#define MYTH_BINARY_VERSION "0.15.20040525-1"
+#define MYTH_PROTO_VERSION "8"
 
 extern int print_verbose_messages;
 
@@ -107,6 +124,8 @@ class MythContext : public QObject
     void LoadQtConfig(void);
     void UpdateImageCache(void);
 
+    void RefreshBackendConfig(void);
+
     void GetScreenSettings(float &wmult, float &hmult);
     void GetScreenSettings(int &width, float &wmult,
                            int &height, float &hmult);
@@ -116,11 +135,16 @@ class MythContext : public QObject
     QString FindThemeDir(const QString &themename);
     QString GetThemeDir(void);
 
+    QString GetMenuThemeDir(void);
+
     int OpenDatabase(QSqlDatabase *db);
     static void KickDatabase(QSqlDatabase *db);
     static void DBError(const QString &where, const QSqlQuery& query);
     static QString DBErrorMessage(const QSqlError& err);
 
+    void LogEntry(const QString &module, int priority,
+                  const QString &message, const QString &details);
+
     Settings *settings(void);
     Settings *qtconfig(void);
 
@@ -171,7 +195,19 @@ class MythContext : public QObject
 
     bool CheckProtoVersion(QSocketDevice* socket);
     bool CheckProtoVersion(QSocket* socket);
-    
+
+    // event wrappers
+    void DisableScreensaver(void);
+    void RestoreScreensaver(void);
+    // Reset screensaver idle time, for input events that X doesn't see
+    // (e.g., lirc)
+    void ResetScreensaver(void);
+
+    // actually do it
+    void DoDisableScreensaver(void);
+    void DoRestoreScreensaver(void);
+    void DoResetScreensaver(void);
+
   private slots:
     void EventSocketRead();
     void EventSocketConnected();
