@@ -11,6 +11,9 @@
 #include <qfocusdata.h>
 #include <qdict.h>
 #include <qsqldatabase.h>
+#ifdef QWS
+#include <qwindowsystem_qws.h>
+#endif
 
 #include <iostream>
 using namespace std;
@@ -22,10 +25,12 @@ using namespace std;
 #endif
 
 #include "uitypes.h"
+#include "uilistbtntype.h"
 #include "xmlparse.h"
 #include "mythdialogs.h"
 #include "lcddevice.h"
 #include "mythmediamonitor.h"
+#include "screensaver.h"
 
 #ifdef USE_LIRC
 static void *SpawnLirc(void *param)
@@ -98,7 +103,8 @@ class MythMainWindowPrivate
     bool exitingtomain;
 
     QDict<KeyContext> keyContexts;
-    QMap<int, JumpData> jumpMap;
+    QMap<int, JumpData*> jumpMap;
+    QMap<QString, JumpData> destinationMap;
     QMap<QString, MHData> mediaHandlerMap;
     QMap<QString, MPData> mediaPluginMap;
 
@@ -168,6 +174,8 @@ MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal)
     RegisterKey("Global", "PREVVIEW", "Previous View", "Home");
     RegisterKey("Global", "NEXTVIEW", "Next View", "End");
 
+    RegisterKey("Global", "HELP", "Help", "F1");
+
     RegisterKey("Global", "0", "0", "0");
     RegisterKey("Global", "1", "1", "1");
     RegisterKey("Global", "2", "2", "2");
@@ -178,13 +186,22 @@ MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal)
     RegisterKey("Global", "7", "7", "7");
     RegisterKey("Global", "8", "8", "8");
     RegisterKey("Global", "9", "9", "9");
+
+    qApp->setOverrideCursor(QCursor(Qt::BlankCursor));
 }
 
 MythMainWindow::~MythMainWindow()
 {
+    qApp->restoreOverrideCursor();
     delete d;
 }
 
+void MythMainWindow::closeEvent(QCloseEvent *e)
+{
+    (void)e;
+    qApp->quit();
+}
+
 void MythMainWindow::Init(void)
 {
     gContext->GetScreenSettings(d->xbase, d->screenwidth, d->wmult,
@@ -194,11 +211,17 @@ void MythMainWindow::Init(void)
 
     setFont(gContext->GetMediumFont());
     setCursor(QCursor(Qt::BlankCursor));
-    qApp->setOverrideCursor(QCursor(Qt::BlankCursor));
+
+#ifdef QWS
+#if QT_VERSION >= 0x030300
+    QWSServer::setCursorVisible(false);
+#endif
+#endif
 
     gContext->ThemeWidget(this);
 
     Show();
+    move(d->xbase, d->ybase);
 }
 
 void MythMainWindow::Show(void)
@@ -224,7 +247,7 @@ void MythMainWindow::detach(QWidget *child)
 {
     if (d->widgetList.back() != child)
     {
-        cerr << "Not removing top most widget, error\n";
+        cerr << "Not removing top-most widget\n";
         return;
     }
 
@@ -295,7 +318,7 @@ bool MythMainWindow::TranslateKeyPress(const QString &context,
     if (d->jumpMap.count(keynum) > 0 && d->exitmenucallback == NULL)
     {
         d->exitingtomain = true;
-        d->exitmenucallback = d->jumpMap[keynum].callback;
+        d->exitmenucallback = d->jumpMap[keynum]->callback;
         QApplication::postEvent(this, new ExitToMainMenuEvent());
         return false;
     }
@@ -427,7 +450,10 @@ void MythMainWindow::RegisterJump(const QString &destination,
         if (!query.isActive())
             MythContext::DBError("Insert Jump Point", query);
     }
-    
+   
+    JumpData jd = { callback, destination, description };
+    d->destinationMap[destination] = jd;
+ 
     QKeySequence keyseq(keybind);
 
     if (!keyseq.isEmpty())
@@ -437,12 +463,10 @@ void MythMainWindow::RegisterJump(const QString &destination,
 
         if (d->jumpMap.count(keynum) == 0)
         {
-            JumpData jd = { callback, destination, description };
-
             //VERBOSE(VB_GENERAL, QString("Binding: %1 to JumpPoint: %2")
             //                           .arg(keybind).arg(destination));
 
-            d->jumpMap[keynum] = jd;
+            d->jumpMap[keynum] = &d->destinationMap[destination];
         }
         else
         {
@@ -455,6 +479,22 @@ void MythMainWindow::RegisterJump(const QString &destination,
     //                               .arg(destination));
 }
 
+void MythMainWindow::JumpTo(const QString& destination)
+{
+    if (d->destinationMap.count(destination) > 0 && d->exitmenucallback == NULL)
+    {
+        d->exitingtomain = true;
+        d->exitmenucallback = d->destinationMap[destination].callback;
+        QApplication::postEvent(this, new ExitToMainMenuEvent());
+        return;
+    }
+}
+
+bool MythMainWindow::DestinationExists(const QString& destination) const
+{
+    return (d->destinationMap.count(destination) > 0) ? true : false;
+}
+
 void MythMainWindow::RegisterMediaHandler(const QString &destination,
                                           const QString &description,
                                           const QString &key, 
@@ -572,6 +612,8 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
 
         if (keycode) 
         {
+            gContext->ResetScreensaver();
+
             int mod = keycode & MODIFIER_MASK;
             int k = keycode & ~MODIFIER_MASK; /* trim off the mod */
             int ascii = 0;
@@ -604,6 +646,33 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
         d->ignore_lirc_keys = lme->eventsMuted();
     }
 #endif
+    else if (ce->type() == ScreenSaverEvent::kScreenSaverEventType)
+    {
+        ScreenSaverEvent *sse = (ScreenSaverEvent *)ce;
+        switch (sse->getSSEventType())
+        {
+            case ScreenSaverEvent::ssetDisable:
+            {
+                gContext->DoDisableScreensaver();
+                break;
+            }
+            case ScreenSaverEvent::ssetRestore:
+            {
+                gContext->DoRestoreScreensaver();
+                break;
+            }
+            case ScreenSaverEvent::ssetReset:
+            {
+                gContext->DoResetScreensaver();
+                break;
+            }
+            default:
+            {
+                cerr << "Unknown ScreenSaverEvent type: " <<
+                        sse->getSSEventType() << std::endl;
+            }
+        }
+    }
 }
 
 QObject *MythMainWindow::getTarget(QKeyEvent &key)
@@ -785,7 +854,7 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, const char *name)
             : MythDialog(parent, name, false)
 {
     float wmult, hmult;
-
+    arrowAccel = true;
     gContext->GetScreenSettings(wmult, hmult);
 
     setLineWidth(3);
@@ -793,11 +862,12 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, const char *name)
     setFrameShape(QFrame::Panel);
     setFrameShadow(QFrame::Raised);
     setPalette(parent->palette());
+    popupForegroundColor = foregroundColor ();
     setFont(parent->font());
     setCursor(QCursor(Qt::BlankCursor));
 
-    hpadding = 110;
-    wpadding = 80;
+    hpadding = gContext->GetNumSetting("PopupHeightPadding", 120);
+    wpadding = gContext->GetNumSetting("PopupWidthPadding", 80);
 
     vbox = new QVBoxLayout(this, (int)(10 * hmult));
 }
@@ -808,7 +878,7 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, bool graphicPopup,
             : MythDialog(parent, name, false)
 {
     float wmult, hmult;
-
+    arrowAccel = true;
     gContext->GetScreenSettings(wmult, hmult);
 
     setLineWidth(3);
@@ -820,8 +890,8 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, bool graphicPopup,
     setFont(parent->font());
     setCursor(QCursor(Qt::BlankCursor));
 
-    hpadding = 110;
-    wpadding = 80;
+    hpadding = gContext->GetNumSetting("PopupHeightPadding", 120);
+    wpadding = gContext->GetNumSetting("PopupWidthPadding", 80);
 
     vbox = new QVBoxLayout(this, (int)(10 * hmult));
 
@@ -892,11 +962,11 @@ QLabel *MythPopupBox::addLabel(QString caption, LabelSize size, bool wrap)
         case Medium: label->setFont(defaultMediumFont); break;
         case Small: label->setFont(defaultSmallFont); break;
     }
-
+    
     label->setMaximumWidth((int)m_parent->width() / 2);
     if (wrap)
         label->setAlignment(Qt::WordBreak | Qt::AlignLeft);
-
+    
     addWidget(label, false);
     return label;
 }
@@ -910,7 +980,7 @@ QButton *MythPopupBox::addButton(QString caption, QObject *target,
         slot = SLOT(defaultButtonPressedHandler());
     }
 
-    MythPushButton *button = new MythPushButton(caption, this);
+    MythPushButton *button = new MythPushButton(caption, this, arrowAccel);
     m_parent->connect(button, SIGNAL(pressed()), target, slot);
     addWidget(button, false);
     return button;
@@ -1013,7 +1083,7 @@ void MythPopupBox::keyPressEvent(QKeyEvent *e)
     {
         QString action = actions[i];
 
-        if (action == "ESCAPE")
+        if((action == "ESCAPE") || (arrowAccel && action == "LEFT"))
         {
             emit popupDone();
             handled = true;
@@ -1220,6 +1290,8 @@ MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
     my_containers.clear();
     widget_with_current_focus = NULL;
 
+    redrawRect = QRect(0, 0, 0, 0);
+
     //
     //  Load the theme. Crap out if we can't find it.
     //
@@ -1230,6 +1302,7 @@ MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
     if(!theme->LoadTheme(xmldata, window_name, theme_filename))
     {
         cerr << "dialogbox.o: Couldn't find your theme. I'm outta here" << endl;
+        cerr << window_name << " - " <<  theme_filename << endl;
         exit(0);
     }
 
@@ -1254,6 +1327,8 @@ MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
                     SLOT(updateForeground()));
             connect(type, SIGNAL(requestUpdate(const QRect &)), this, 
                     SLOT(updateForeground(const QRect &)));
+            connect(type, SIGNAL(requestRegionUpdate(const QRect &)), this,
+                    SLOT(updateForegroundRegion(const QRect &)));
         }
         ++an_it;
     }
@@ -1431,21 +1506,34 @@ void MythThemedDialog::updateForeground()
 
 void MythThemedDialog::updateForeground(const QRect &r)
 {
-    //
-    //  Debugging
-    //
+    QRect rect_to_update = r;
+    if(r.width() == 0 || r.height() == 0)
+    {
+        cerr << "MythThemedDialog.o: something is requesting a screen update of zero size. "
+             << "A widget probably has not done a calculateScreeArea(). Will redraw "
+             << "the whole screen (inefficient!)." 
+             << endl;
+             
+        rect_to_update = this->geometry();
+    }
 
-    /*
-    cout << "I am updating the foreground from "
-         << r.left()
-         << ","
-         << r.top()
-         << " to "
-         << r.left() + r.width()
-         << ","
-         << r.top() + r.height()
-         << endl;
-    */
+    redrawRect = redrawRect.unite(r);
+
+    update(redrawRect);
+}
+
+void MythThemedDialog::ReallyUpdateForeground(const QRect &r)
+{
+    QRect rect_to_update = r;
+    if(r.width() == 0 || r.height() == 0)
+    {
+        cerr << "MythThemedDialog.o: something is requesting a screen update of zero size. "
+             << "A widget probably has not done a calculateScreeArea(). Will redraw "
+             << "the whole screen (inefficient!)."
+             << endl;
+
+        rect_to_update = this->geometry();
+    }
 
     //
     //  We paint offscreen onto a pixmap
@@ -1468,7 +1556,7 @@ void MythThemedDialog::updateForeground(const QRect &r)
         //
 
         if (container_area.isValid() &&
-            r.intersects(container_area) &&
+            rect_to_update.intersects(container_area) &&
             looper->GetName().lower() != "background")
         {
             //
@@ -1521,11 +1609,57 @@ void MythThemedDialog::updateForeground(const QRect &r)
         whole_dialog_painter.end();
     }
 
+    redrawRect = QRect(0, 0, 0, 0);
+}
+
+void MythThemedDialog::updateForegroundRegion(const QRect &r)
+{
+    QRect area = r;
+    QPainter whole_dialog_painter(&my_foreground);
+
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while ((looper = an_it.current()) != 0)
+    {
+        QRect container_area = looper->GetAreaRect();
+
+        if (container_area.isValid() &&
+            r.intersects(container_area) &&
+            looper->GetName().lower() != "background")
+        {
+            QPixmap container_picture(r.size());
+            QPainter offscreen_painter(&container_picture);
+            offscreen_painter.drawPixmap(0, 0, my_background,
+                                         r.left(), r.top());
+
+            for (int i = 0; i <= looper->getLayers(); i++)
+            {
+                looper->DrawRegion(&offscreen_painter, area, i, context);
+            }
+
+            if (offscreen_painter.isActive())
+            {
+                offscreen_painter.end();
+                whole_dialog_painter.drawPixmap(r.topLeft(),
+                                                container_picture);
+            }
+
+        }
+        ++an_it;
+    }
+
+    if (whole_dialog_painter.isActive())
+        whole_dialog_painter.end();
+
     update(r);
 }
 
 void MythThemedDialog::paintEvent(QPaintEvent *e)
 {
+    if (redrawRect.width() > 0 && redrawRect.height() > 0)
+        ReallyUpdateForeground(redrawRect);
+
     bitBlt(this, e->rect().left(), e->rect().top(),
            &my_foreground, e->rect().left(), e->rect().top(),
            e->rect().width(), e->rect().height());
@@ -1720,6 +1854,27 @@ UITextType* MythThemedDialog::getUITextType(const QString &name)
     return NULL;
 }
 
+UIMultiTextType* MythThemedDialog::getUIMultiTextType(const QString &name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        UIType *hunter = looper->GetType(name);
+        if(hunter)
+        {
+            UIMultiTextType *hunted;
+            if( (hunted = dynamic_cast<UIMultiTextType*>(hunter)) )
+            {
+                return hunted;
+            }
+        }
+        ++an_it;
+    }
+    return NULL;
+}
+
 UIPushButtonType* MythThemedDialog::getUIPushButtonType(const QString &name)
 {
     QPtrListIterator<LayerSet> an_it(my_containers);
@@ -1888,6 +2043,48 @@ UIStatusBarType* MythThemedDialog::getUIStatusBarType(const QString &name)
     return NULL;
 }
 
+UIListBtnType* MythThemedDialog::getUIListBtnType(const QString &name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        UIType *hunter = looper->GetType(name);
+        if (hunter)
+        {
+            UIListBtnType *hunted;
+            if ((hunted = dynamic_cast<UIListBtnType*>(hunter)) )
+            {
+                return hunted;
+            }
+        }
+        ++an_it;
+    }
+    return NULL;
+}
+
+UIListTreeType* MythThemedDialog::getUIListTreeType(const QString &name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        UIType *hunter = looper->GetType(name);
+        if (hunter)
+        {
+            UIListTreeType *hunted;
+            if ((hunted = dynamic_cast<UIListTreeType*>(hunter)) )
+            {
+                return hunted;
+            }
+        }
+        ++an_it;
+    }
+    return NULL;
+}
+
 /*
 ---------------------------------------------------------------------
 */
@@ -2051,7 +2248,8 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     {
         image_box->calculateScreenArea();
     }
-    
+    initialDir = top_directory;
+        
     image_files.clear();
     buildTree(top_directory);
 
@@ -2117,7 +2315,8 @@ void MythImageFileDialog::buildTree(QString starting_where)
         {
             make_active = true;
         }
-        QString prefix = gContext->GetSetting("VideoStartupDir");
+        QString prefix = initialDir;
+
         if(prefix.length() < 1)
         {
             cerr << "mythdialogs.o: Seems unlikely that this is going to work" << endl;
@@ -2166,15 +2365,11 @@ void MythImageFileDialog::buildFileList(QString directory)
 {
     QStringList imageExtensions = QImage::inputFormatList();
 
-    //
-    //  Expand the list Qt gives us, working off what we know was built into
-    //  Qt based on the list it gave us
-    // 
+    // Expand the list Qt gives us, working off what we know was built into
+    // Qt based on the list it gave us
 
-    if(imageExtensions.contains("jpeg"))
-    {
+    if (imageExtensions.contains("jpeg") || imageExtensions.contains("JPEG"))
         imageExtensions += "jpg";
-    }
 
     QDir d(directory);
        
@@ -2260,3 +2455,285 @@ MythImageFileDialog::~MythImageFileDialog()
     }
 }
 
+// -----------------------------------------------------------------------------
+
+MythScrollDialog::MythScrollDialog(MythMainWindow *parent,
+                                   MythScrollDialog::ScrollMode mode,
+                                   const char *name)
+    : QScrollView(parent, name)
+{
+    if (!parent) {
+        std::cerr << "MythScrollDialog: Trying to create a dialog without a parent"
+                  << std::endl;
+        exit(-1);
+    }
+
+    m_parent     = parent;
+    m_scrollMode = mode;
+        
+    m_resCode    = 0;
+    m_inLoop     = false;
+    
+    gContext->GetScreenSettings(m_xbase, m_screenWidth, m_wmult,
+                                m_ybase, m_screenHeight, m_hmult);
+
+    m_defaultBigFont = gContext->GetBigFont();
+    m_defaultMediumFont = gContext->GetMediumFont();
+    m_defaultSmallFont = gContext->GetSmallFont();
+
+    setFont(m_defaultMediumFont);
+    setCursor(QCursor(Qt::ArrowCursor));
+    
+    setFrameShape(QFrame::NoFrame);
+    setHScrollBarMode(QScrollView::AlwaysOff);
+    setVScrollBarMode(QScrollView::AlwaysOff);
+    setFixedSize(QSize(m_screenWidth, m_screenHeight));
+
+    gContext->ThemeWidget(viewport());
+    if (viewport()->paletteBackgroundPixmap())
+        m_bgPixmap = new QPixmap(*(viewport()->paletteBackgroundPixmap()));
+    else {
+        m_bgPixmap = new QPixmap(m_screenWidth, m_screenHeight);
+        m_bgPixmap->fill(viewport()->colorGroup().base());
+    }
+    viewport()->setBackgroundMode(Qt::NoBackground);
+
+    m_upArrowPix = gContext->LoadScalePixmap("scrollarrow-up.png");
+    m_dnArrowPix = gContext->LoadScalePixmap("scrollarrow-dn.png");
+    m_ltArrowPix = gContext->LoadScalePixmap("scrollarrow-left.png");
+    m_rtArrowPix = gContext->LoadScalePixmap("scrollarrow-right.png");
+
+    int wmargin = (int)(20*m_wmult);
+    int hmargin = (int)(20*m_hmult);
+    
+    if (m_upArrowPix)
+        m_upArrowRect = QRect(m_screenWidth - m_upArrowPix->width() - wmargin,
+                              hmargin,
+                              m_upArrowPix->width(), m_upArrowPix->height());
+    if (m_dnArrowPix)
+        m_dnArrowRect = QRect(m_screenWidth - m_dnArrowPix->width() - wmargin,
+                              m_screenHeight - m_dnArrowPix->height() - hmargin,
+                              m_dnArrowPix->width(), m_dnArrowPix->height());
+    if (m_rtArrowPix)
+        m_rtArrowRect = QRect(m_screenWidth - m_rtArrowPix->width() - wmargin,
+                              m_screenHeight - m_rtArrowPix->height() - hmargin,
+                              m_rtArrowPix->width(), m_rtArrowPix->height());
+    if (m_ltArrowPix)
+        m_ltArrowRect = QRect(wmargin,
+                              m_screenHeight - m_ltArrowPix->height() - hmargin,
+                              m_ltArrowPix->width(), m_ltArrowPix->height());
+    
+    m_showUpArrow  = true;
+    m_showDnArrow  = true;
+    m_showRtArrow  = false;
+    m_showLtArrow  = false;
+    
+    m_parent->attach(this);
+}
+
+MythScrollDialog::~MythScrollDialog()
+{
+    m_parent->detach(this);
+    delete m_bgPixmap;
+
+    if (m_upArrowPix)
+        delete m_upArrowPix;
+    if (m_dnArrowPix)
+        delete m_dnArrowPix;
+    if (m_ltArrowPix)
+        delete m_ltArrowPix;
+    if (m_rtArrowPix)
+        delete m_rtArrowPix;
+}
+
+void MythScrollDialog::setArea(int w, int h)
+{
+    resizeContents(w, h);
+}
+
+void MythScrollDialog::setAreaMultiplied(int areaWTimes, int areaHTimes)
+{
+    if (areaWTimes < 1 || areaHTimes < 1) {
+        std::cerr << "MythScrollDialog: invalid areaWTimes or areaHTimes"
+                  << std::endl;
+        exit(-1);
+    }
+
+    resizeContents(m_screenWidth*areaWTimes,
+                   m_screenHeight*areaHTimes);
+}
+
+int MythScrollDialog::result() const
+{
+    return m_resCode;    
+}
+
+void MythScrollDialog::show()
+{
+    QScrollView::show();    
+}
+
+void MythScrollDialog::hide()
+{
+    if (isHidden())
+        return;
+
+    // Reimplemented to exit a modal when the dialog is hidden.
+    QWidget::hide();
+    if (m_inLoop)  
+    {
+        m_inLoop = false;
+        qApp->exit_loop();
+    }
+}
+
+int MythScrollDialog::exec()
+{
+    if (m_inLoop) 
+    {
+        std::cerr << "MythScrollDialog::exec: Recursive call detected."
+                  << std::endl;
+        return -1;
+    }
+
+    setResult(0);
+
+    show();
+
+    m_inLoop = true;
+    qApp->enter_loop();
+
+    int res = result();
+
+    return res;
+}
+
+void MythScrollDialog::done(int r)
+{
+    hide();
+    setResult(r);
+    close();
+}
+
+void MythScrollDialog::accept()
+{
+    done(Accepted);
+}
+
+void MythScrollDialog::reject()
+{
+    done(Rejected);
+}
+
+void MythScrollDialog::setResult(int r)
+{
+    m_resCode = r;    
+}
+
+void MythScrollDialog::keyPressEvent(QKeyEvent *e)
+{
+    if (e->state() != 0)
+        return;
+
+    bool handled = false;
+    QStringList actions;
+
+    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+    {
+        for (unsigned int i = 0; i < actions.size() && !handled; i++)
+        {
+            QString action = actions[i];
+            handled = true;
+
+            if (action == "ESCAPE")
+                reject();
+            else if (action == "UP" || action == "LEFT")
+            {
+                if (focusWidget() &&
+                    (focusWidget()->focusPolicy() == QWidget::StrongFocus ||
+                     focusWidget()->focusPolicy() == QWidget::WheelFocus))
+                {
+                }
+                else
+                    focusNextPrevChild(false);
+            }
+            else if (action == "DOWN" || action == "RIGHT")
+            {
+                if (focusWidget() &&
+                    (focusWidget()->focusPolicy() == QWidget::StrongFocus ||
+                     focusWidget()->focusPolicy() == QWidget::WheelFocus)) 
+                {
+                }
+                else
+                    focusNextPrevChild(true);
+            }
+            else
+                handled = false;
+        }
+    }
+}
+
+void MythScrollDialog::viewportPaintEvent(QPaintEvent *pe)
+{
+    if (!pe)
+        return;
+
+    QRect   er(pe->rect());
+    QRegion reg(er);
+    
+    paintEvent(reg, er.x()+contentsX(), er.y()+contentsY(),
+               er.width(), er.height());
+
+    if (m_scrollMode == HScroll) {
+        if (m_ltArrowPix && m_showLtArrow) {
+            QPixmap pix(m_ltArrowRect.size());
+            bitBlt(&pix, 0, 0, m_bgPixmap, m_ltArrowRect.x(), m_ltArrowRect.y());
+            bitBlt(&pix, 0, 0, m_ltArrowPix);
+            bitBlt(viewport(), m_ltArrowRect.x(), m_ltArrowRect.y(), &pix);
+            reg -= m_ltArrowRect;
+        }
+        if (m_rtArrowPix && m_showRtArrow) {
+            QPixmap pix(m_rtArrowRect.size());
+            bitBlt(&pix, 0, 0, m_bgPixmap, m_rtArrowRect.x(), m_rtArrowRect.y());
+            bitBlt(&pix, 0, 0, m_rtArrowPix);
+            bitBlt(viewport(), m_rtArrowRect.x(), m_rtArrowRect.y(), &pix);
+            reg -= m_rtArrowRect;
+        }
+    }
+    else {
+        if (m_upArrowPix && m_showUpArrow) {
+            QPixmap pix(m_upArrowRect.size());
+            bitBlt(&pix, 0, 0, m_bgPixmap, m_upArrowRect.x(), m_upArrowRect.y());
+            bitBlt(&pix, 0, 0, m_upArrowPix);
+            bitBlt(viewport(), m_upArrowRect.x(), m_upArrowRect.y(), &pix);
+            reg -= m_upArrowRect;
+        }
+        if (m_dnArrowPix && m_showDnArrow) {
+            QPixmap pix(m_dnArrowRect.size());
+            bitBlt(&pix, 0, 0, m_bgPixmap, m_dnArrowRect.x(), m_dnArrowRect.y());
+            bitBlt(&pix, 0, 0, m_dnArrowPix);
+            bitBlt(viewport(), m_dnArrowRect.x(), m_dnArrowRect.y(), &pix);
+            reg -= m_dnArrowRect;
+        }
+    }
+
+    QPainter p(viewport());
+    p.setClipRegion(reg);
+    p.drawPixmap(0, 0, *m_bgPixmap, 0, 0, viewport()->width(),
+                 viewport()->height());
+    p.end();    
+}
+
+void MythScrollDialog::paintEvent(QRegion&, int , int , int , int )
+{
+}
+
+void MythScrollDialog::setContentsPos(int x, int y)
+{
+    viewport()->setUpdatesEnabled(false);
+    QScrollView::setContentsPos(x,y);
+    viewport()->setUpdatesEnabled(true);
+    updateContents();
+}
+    
+      
