@@ -10,6 +10,7 @@
 #include <qpushbutton.h>
 #include <qevent.h>
 #include <qvaluevector.h>
+#include <qscrollview.h>
 
 #include <vector>
 using namespace std;
@@ -21,6 +22,7 @@ class XMLParse;
 class UIType;
 class UIManagedTreeListType;
 class UITextType;
+class UIMultiTextType;
 class UIPushButtonType;
 class UITextButtonType;
 class UIRepeatedImageType;
@@ -29,6 +31,8 @@ class UISelectorType;
 class UIBlackHoleType;
 class UIImageType;
 class UIStatusBarType;
+class UIListBtnType;
+class UIListTreeType;
 class LayerSet;
 class GenericTree;
 class MythMediaDevice;
@@ -93,9 +97,14 @@ class MythMainWindow : public QDialog
                              MediaPlayCallback fn);
     bool HandleMedia(QString &handler, const QString &mrl);
 
+    void JumpTo(const QString &destination);
+    bool DestinationExists(const QString &destination) const;
+
+
   protected:
     void keyPressEvent(QKeyEvent *e);
     void customEvent(QCustomEvent *ce);
+    void closeEvent(QCloseEvent *e);
 
     void ExitToMainMenu();
 
@@ -199,6 +208,7 @@ class MythPopupBox : public MythDialog
     QVBoxLayout *vbox;
     QColor popupForegroundColor;
     int hpadding, wpadding;
+    bool arrowAccel;
 };
 
 class MythProgressDialog: public MythDialog
@@ -236,6 +246,7 @@ class MythThemedDialog : public MythDialog
     UIType *getCurrentFocusWidget();
     UIManagedTreeListType *getUIManagedTreeListType(const QString &name);
     UITextType *getUITextType(const QString &name);
+    UIMultiTextType *getUIMultiTextType(const QString &name);
     UIPushButtonType *getUIPushButtonType(const QString &name);
     UITextButtonType *getUITextButtonType(const QString &name);
     UIRepeatedImageType *getUIRepeatedImageType(const QString &name);
@@ -243,7 +254,9 @@ class MythThemedDialog : public MythDialog
     UISelectorType *getUISelectorType(const QString &name);
     UIBlackHoleType *getUIBlackHoleType(const QString &name);
     UIImageType *getUIImageType(const QString &name);
-    UIStatusBarType*        getUIStatusBarType(const QString &name);
+    UIStatusBarType *getUIStatusBarType(const QString &name);
+    UIListBtnType *getUIListBtnType(const QString &name);
+    UIListTreeType *getUIListTreeType(const QString &name);
 
     void setContext(int a_context) { context = a_context; }
     int  getContext(){return context;}
@@ -253,7 +266,8 @@ class MythThemedDialog : public MythDialog
     virtual void updateBackground();
     virtual void initForeground();
     virtual void updateForeground();
-    virtual void updateForeground(const QRect &);
+    virtual void updateForeground(const QRect &); // draws anything that intersects
+    virtual void updateForegroundRegion(const QRect &); // only draws the region
     virtual bool assignFirstFocus();
     virtual bool nextPrevWidgetFocus(bool up_or_down);
     virtual void activateCurrent();
@@ -261,24 +275,25 @@ class MythThemedDialog : public MythDialog
   protected:
 
     void paintEvent(QPaintEvent* e);
-    UIType              *widget_with_current_focus;
+    UIType *widget_with_current_focus;
 
-    //
-    //  These need to be just "protected"
-    //  so that subclasses can mess with them
-    //
+    // These need to be just "protected" so that subclasses can mess with them
 
     QPixmap my_background;
     QPixmap my_foreground;
 
   private:
 
+    void ReallyUpdateForeground(const QRect &);
+
     XMLParse *theme;
     QDomElement xmldata;
     int context;
 
     QPtrList<LayerSet>  my_containers;
     QPtrList<UIType>    focus_taking_widgets;
+
+    QRect redrawRect;
 };
 
 class MythPasswordDialog: public MythDialog
@@ -356,8 +371,88 @@ class MythImageFileDialog: public MythThemedDialog
     GenericTree           *initial_node;
     UIImageType           *image_box;
     QStringList           image_files;
+    QString               initialDir;
 };
 
+// ---------------------------------------------------------------------------
+
+class MythScrollDialog : public QScrollView
+{
+    Q_OBJECT
+    
+  public:
+
+    enum DialogCode {
+        Rejected,
+        Accepted
+    };
+
+    enum ScrollMode {
+        HScroll=0,
+        VScroll
+    };
+
+    MythScrollDialog(MythMainWindow *parent, ScrollMode mode=HScroll,
+                     const char *name = 0);
+    ~MythScrollDialog();
+
+    void setArea(int w, int h);
+    void setAreaMultiplied(int areaWTimes, int areaHTimes);
+
+    int  result() const;
+
+  public slots:
+
+    int  exec();
+    virtual void done(int);
+    virtual void show();
+    virtual void hide();
+    virtual void setContentsPos(int x, int y);
+    
+  protected slots:
+
+    virtual void accept();
+    virtual void reject();
+
+  protected:
+
+    void         keyPressEvent(QKeyEvent *e);
+    virtual void paintEvent(QRegion& region, int x, int y, int w, int h);
+
+    void setResult(int r);
+    void viewportPaintEvent(QPaintEvent *pe);
+
+    MythMainWindow *m_parent;
+    int             m_screenWidth, m_screenHeight;
+    int             m_xbase, m_ybase;
+    float           m_wmult, m_hmult;
+
+    ScrollMode      m_scrollMode;
+
+    QFont           m_defaultBigFont;
+    QFont           m_defaultMediumFont;
+    QFont           m_defaultSmallFont;
+
+    int             m_resCode;
+    bool            m_inLoop;
+    
+    QPixmap        *m_bgPixmap;
+    
+    QPixmap        *m_upArrowPix;
+    QPixmap        *m_dnArrowPix;
+    QPixmap        *m_rtArrowPix;
+    QPixmap        *m_ltArrowPix;
+
+    bool            m_showUpArrow;
+    bool            m_showDnArrow;
+    bool            m_showLtArrow;
+    bool            m_showRtArrow;
+
+    QRect           m_upArrowRect;
+    QRect           m_dnArrowRect;
+    QRect           m_rtArrowRect;
+    QRect           m_ltArrowRect;
+};
 
 #endif
 
