@@ -41,6 +41,8 @@ void MythComboBox::keyPressEvent(QKeyEvent *e)
                 if (count() > 0)
                     setCurrentItem((currentItem() + 1) % count());
             }
+            else if (action == "SELECT" && AcceptOnSelect)
+                emit accepted(currentItem());
             else
                 handled = false;
         }
@@ -75,8 +77,25 @@ void MythComboBox::focusOutEvent(QFocusEvent *e)
     this->unsetPalette();
 
     if (lineEdit())
+    {
         lineEdit()->unsetPalette();
 
+        // commit change if the user was editting an entry
+        QString curText = currentText();
+        int i;
+        bool foundItem = false;
+
+        for(i = 0; i < count(); i++)
+            if (curText == text(i))
+                foundItem = true;
+
+        if ( !foundItem )
+        {
+            insertItem(curText);
+            setCurrentItem(count()-1);
+        }
+    }
+
     QComboBox::focusOutEvent(e);
 }
 
@@ -856,6 +875,20 @@ void MythPushButton::keyPressEvent(QKeyEvent *e)
                 emit pressed();
                 handled = true;
             }
+            else if(arrowAccel)
+            {
+                if(action == "LEFT") 
+                {
+                    parent()->event(e);
+                    handled = true;
+                }
+                else if (action == "RIGHT")
+                {
+                    setDown(true);
+                    emit pressed();
+                    handled = true;
+                }
+            }
         }
     }
 
