@@ -232,13 +232,24 @@ void SelectSetting::addSelection(const QString& label, QString value, bool selec
     if (value == QString::null)
         value = label;
     
-    labels.push_back(label);
-    values.push_back(value);
+    bool found = false;
+    for(unsigned i = 0 ; i < values.size() ; ++i)
+        if ((values[i] == value) &&
+            (labels[i] == label)) {
+            found = true;
+            break;
+        }
+
+    if (!found)
+    {
+        labels.push_back(label);
+        values.push_back(value);
     
-    emit selectionAdded(label, value);
+        emit selectionAdded(label, value);
     
-    if (select || !isSet)
-        setValue(value);
+        if (select || !isSet)
+            setValue(value);
+    }
 }
 
 void SelectSetting::fillSelectionsFromDir(const QDir& dir, bool absPath) {
@@ -488,7 +499,7 @@ QWidget* ComboBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
         box->setStretchFactor(label, 0);
     }
 
-    MythComboBox* widget = new MythComboBox(rw, box);
+    widget = new MythComboBox(rw, box);
     widget->setHelpText(getHelpText());
     widget->setBackgroundOrigin(QWidget::WindowOrigin);
     box->setStretchFactor(widget, 1);
@@ -505,8 +516,9 @@ QWidget* ComboBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     else
         connect(widget, SIGNAL(highlighted(int)),
                 this, SLOT(setValue(int)));
-    connect(this, SIGNAL(selectionAdded(const QString&,QString)),
-            widget, SLOT(insertItem(const QString&)));
+
+    connect(widget, SIGNAL(destroyed()),
+            this, SLOT(widgetDestroyed()));
     connect(this, SIGNAL(selectionsCleared()),
             widget, SLOT(clear()));
 
@@ -757,6 +769,7 @@ QWidget* ListBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
 
     widget = new MythListBox(box);
     widget->setBackgroundOrigin(QWidget::WindowOrigin);
+    widget->setHelpText(getHelpText());
 
     for(unsigned int i = 0 ; i < labels.size() ; ++i) {
         widget->insertItem(labels[i]);
@@ -764,9 +777,6 @@ QWidget* ListBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
             widget->setCurrentItem(i);
     }
 
-    // xxx, needs to update the listbox when a selection is added
-//     connect(this, SIGNAL(selectionAdded(const QString&, QString)),
-//             widget, SLOT(insertItem(QString)));
     connect(widget, SIGNAL(destroyed()),
             this, SLOT(widgetDestroyed()));
     connect(this, SIGNAL(selectionsCleared()),
