@@ -527,8 +527,19 @@ void ThemedMenuPrivate::parseText(TextAttributes &attributes,
             else if (info.tagName() == "centered")
             {
                 if (getFirstText(info) == "yes")
-                    attributes.textflags = Qt::AlignTop | Qt::AlignHCenter | 
-                                           Qt::WordBreak;
+                {
+                    if (gContext->GetLanguage() == "ja")
+                    {
+                        attributes.textflags = Qt::AlignVCenter | 
+                                               Qt::AlignHCenter |
+                                               Qt::WordBreak;
+                    }
+                    else
+                    {
+                        attributes.textflags = Qt::AlignTop | Qt::AlignHCenter |
+                                               Qt::WordBreak;
+                    }
+                }
             } 
             else if (info.tagName() == "outline")
             {
@@ -1090,6 +1101,10 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
             {
                 addit = gContext->GetNumSetting(getFirstText(info));
             }
+            else if (info.tagName() == "dependjumppoint")
+            {
+                addit = gContext->GetMainWindow()->DestinationExists(getFirstText(info));
+            }
             else
             {
                 cerr << "Unknown tag " << info.tagName() << " in button\n";
@@ -1123,8 +1138,23 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     if (!f.open(IO_ReadOnly))
     {
+        
         cerr << "Couldn't read menu file " << menuname << endl;
-        exit(1);
+        if(menuname == "mainmenu.xml" )
+        {
+            exit(0);
+        }
+        else
+        {
+            MythPopupBox::showOkPopup(gContext->GetMainWindow(), QObject::tr("No Menu File"),
+                                      QObject::tr(QString("Myth could not locate the menu file %1.\n\n"
+                                      "We will now return to the main menu.").arg(menuname)));
+
+            parseMenu("mainmenu.xml");
+            return;
+        }
+        
+        
     }
 
     QString errorMsg;
@@ -1841,6 +1871,8 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         { 
             if (currentrow > 0)
                 currentrow--;
+            else if (columns == 1)
+                currentrow = buttonRows.size() - 1;
 
             if (currentcolumn >= buttonRows[currentrow].numitems)
                 currentcolumn = buttonRows[currentrow].numitems - 1;
@@ -1861,6 +1893,8 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         {
             if (currentrow < (int)buttonRows.size() - 1)
                 currentrow++;
+            else if (columns == 1)
+                currentrow = 0;
 
             if (currentcolumn >= buttonRows[currentrow].numitems)
                 currentcolumn = buttonRows[currentrow].numitems - 1;
@@ -1880,6 +1914,9 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         }
         else if (action == "SELECT")
         {
+            if (lcddev)
+                lcddev->stopAll();
+
             lastbutton = activebutton;
             activebutton = NULL;
             parent->repaint(lastbutton->posRect);
@@ -1942,10 +1979,20 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
 QString ThemedMenuPrivate::findMenuFile(const QString &menuname)
 {
     QString testdir = QDir::homeDirPath() + "/.mythtv/" + menuname;
-   
     QFile file(testdir);
     if (file.exists())
         return testdir;
+
+    testdir = gContext->GetMenuThemeDir() + "/" + menuname;
+    file.setName(testdir);
+    if (file.exists())
+        return testdir;
+
+        
+    testdir = gContext->GetThemeDir() + "/" + menuname;
+    file.setName(testdir);
+    if (file.exists())
+        return testdir;
         
     testdir = prefix + "/share/mythtv/" + menuname;
     file.setName(testdir);
@@ -2048,6 +2095,11 @@ void ThemedMenuPrivate::handleAction(const QString &action)
         if (menulevel == 1)
             parent->done(0);
     }
+    else if (action.left(5) == "JUMP ")
+    {
+        QString rest = action.right(action.length() - 5);
+        gContext->GetMainWindow()->JumpTo(rest);
+    }
     else
     {
         selection = action;
