@@ -18,6 +18,7 @@
 #include "mythwidgets.h"
 #include "util.h"
 #include "mythdialogs.h"
+#include "generictree.h"
 
 using namespace std;
 
@@ -38,6 +39,7 @@ class LayerSet
    ~LayerSet();
  
     void    Draw(QPainter *, int, int);
+    void    DrawRegion(QPainter *, QRect &, int, int);
 
     QString GetName() { return m_name; }
     void    SetName(const QString &name) { m_name = name; }
@@ -100,12 +102,15 @@ class UIType : public QObject
     bool    canTakeFocus(){ return takes_focus;}
     int     getOrder(){return m_order;}
     virtual void Draw(QPainter *, int, int);
+    virtual void DrawRegion(QPainter *, QRect &, int, int);
     virtual void calculateScreenArea();
     QRect   getScreenArea(){return screen_area;}
     QString cutDown(const QString &data, QFont *font, bool multiline = false, 
                     int overload_width = -1, int overload_height = -1);
     QString getName(){return m_name;}
 
+    bool    isShown(){return !hidden;}
+    bool    isHidden(){return hidden;}
     
   public slots:
   
@@ -113,6 +118,10 @@ class UIType : public QObject
     virtual void looseFocus();
     virtual void activate(){}
     virtual void refresh();
+    virtual void show();
+    virtual void hide();
+    virtual bool toggleShow();
+    
 
   signals:
   
@@ -122,6 +131,7 @@ class UIType : public QObject
 
     void requestUpdate();
     void requestUpdate(const QRect &);
+    void requestRegionUpdate(const QRect &);
     void takingFocus();
     void loosingFocus();
 
@@ -138,6 +148,7 @@ class UIType : public QObject
     bool     takes_focus;
     QRect    screen_area;   // The area, in real screen coordinates
     bool     drawFontShadow;
+    bool     hidden;        // Is this "widget" seen or hidden ?
 };
 
 class UIBarType : public UIType
@@ -364,7 +375,8 @@ class UIListType : public UIType
                        m_current = -1; m_columns = 0; }
  
     void Draw(QPainter *, int drawlayer, int);
-
+    bool ShowSelAlways() const { return m_showSelAlways; }
+    void ShowSelAlways(bool bnew) { m_showSelAlways = bnew; }
   private:
     //QString cutDown(QString, QFont *, int);
     int m_selheight;
@@ -376,6 +388,7 @@ class UIListType : public UIType
     int m_count;
     bool m_darrow;
     bool m_uarrow;
+    bool m_showSelAlways;
     QRect m_fill_area;
     QColor m_fill_color;
     int m_fill_type;
@@ -497,11 +510,13 @@ class UITextType : public UIType
     void SetCutDown(bool cut) { m_cutdown = cut; }
 
     QRect DisplayArea() { return m_displaysize; }
+    void SetDisplayArea(const QRect &rect) { m_displaysize = rect; }
     virtual void calculateScreenArea();
 
-    void Draw(QPainter *, int, int);
+    virtual void Draw(QPainter *, int, int);
+
+  protected:
 
-  private:
     //QString cutDown(QString, QFont *, int, int);
     int m_justification;
     QRect m_displaysize;
@@ -517,6 +532,63 @@ class UITextType : public UIType
 
 };
 
+class UIMultiTextType : public UITextType
+{
+
+  Q_OBJECT
+
+  public: 
+
+    enum AnimationStage
+    {
+        Animation_Drop = 0,
+        Animation_DropPause,
+        Animation_Scroll,
+        Animation_ScrollPause
+    };
+
+    UIMultiTextType(
+                    const QString &, 
+                    fontProp *, 
+                    int,
+                    QRect displayrect, 
+                    QRect altdisplayrect
+                   );
+
+    void setTexts(QStringList new_messagep);
+    void clearTexts();
+    void Draw(QPainter*, int, int);
+  
+    void setDropTimingLength(int x){drop_timing_length = x;}
+    void setDropTimingPause(int x){drop_timing_pause = x;}
+    void setScrollTimingLength(int x){scroll_timing_length = x;}
+    void setScrollTimingPause(int x){scroll_timing_pause = x;}
+    
+    void setMessageSpacePadding(int x){message_space_padding = x;}
+    
+  
+  private slots:
+
+    void        animate();
+
+  private:
+
+    QStringList     messages;  
+    int             current_text_index;
+    QTimer          transition_timer;
+    AnimationStage  animation_stage;
+    int             horizontal_transform;
+    int             max_horizontal_transform;
+    int             vertical_transform;
+    int             drop_timing_length;
+    int             drop_timing_pause;
+    int             scroll_timing_length;
+    int             scroll_timing_pause;
+    
+    int             message_space_padding;    
+
+};
+
 class UIStatusBarType : public UIType
 {
   public:
@@ -554,89 +626,6 @@ class UIStatusBarType : public UIType
     
 };
 
-class GenericTree
-{
-    //
-    //  This is used by the UIType below
-    //  (UIManagedTreeListType) to fill it
-    //  with arbitrary simple data types and
-    //  allow that object to fire SIGNALS
-    //  when nodes, leafnodes, etc. are reached
-    //  by the user
-    //
-    typedef QValueVector<int> IntVector;
-
-  public:
-
-    GenericTree();
-    GenericTree(const QString a_string);
-    GenericTree(QString a_string, int an_int);
-    GenericTree(QString a_string, int an_int, bool selectable_flag);
-    ~GenericTree();
-
-    GenericTree*  addNode(QString a_string);
-    GenericTree*  addNode(QString a_string, int an_int);
-    GenericTree*  addNode(QString a_string, int an_int, bool selectable_flag);
-    GenericTree*  findLeaf();
-    GenericTree*  findLeaf(int ordering_index);
-    GenericTree*  findNode(QValueList<int> route_of_branches);
-    GenericTree*  recursiveNodeFinder(QValueList<int> route_of_branches);
-    bool          checkNode(QValueList<int> route_of_branches);
-    GenericTree*  nextSibling(int number_down);
-    GenericTree*  nextSibling(int number_down, int ordering_index);
-    GenericTree*  prevSibling(int number_up);
-    GenericTree*  prevSibling(int number_up, int ordering_index);
-    GenericTree*  getParent();
-    GenericTree*  getChildAt(uint reference);
-    GenericTree*  getSelectedChild(int ordering_index);
-    GenericTree*  getChildAt(uint reference, int ordering_index);
-    int           getChildPosition(GenericTree *which_child);
-    int           getChildPosition(GenericTree *which_child, int ordering_index);
-    int           getPosition();
-    int           getPosition(int ordering_index);
-    void          init();
-    void          setInt(int an_int){my_int = an_int;}
-    int           getInt(){return my_int;}
-    void          setParent(GenericTree* a_parent){my_parent = a_parent;}
-    const QString getString(){return my_string;}
-    void          printTree(int margin);    // debugging
-    void          printTree(){printTree(0);}// debugging
-    int           calculateDepth(int start);
-    int           childCount(){return my_subnodes.count();}
-    int           siblingCount();
-    void          setSelectable(bool flag){selectable = flag;}
-    bool          isSelectable(){return selectable;}
-    void          setAttribute(uint attribute_position, int value_of_attribute);
-    int           getAttribute(uint which_one);
-    IntVector*    getAttributes(){return my_attributes;}
-    void          reorderSubnodes(int ordering_index);
-    void          setOrderingIndex(int ordering_index){current_ordering_index = ordering_index;}
-    int           getOrderingIndex(){return current_ordering_index;}
-    void          becomeSelectedChild();
-    void          setSelectedChild(GenericTree* a_node){my_selected_subnode = a_node;}
-    void          addYourselfIfSelectable(QPtrList<GenericTree> *flat_list);
-    void          buildFlatListOfSubnodes(int ordering_index, bool scrambled_parents);
-    GenericTree*  nextPrevFromFlatList(bool forward_or_back, bool wrap_around, GenericTree *active);
-    GenericTree*  getChildByName(QString a_name);
-    void          sortByString();
-    void          sortBySelectable();
-    void          deleteAllChildren();
-
-  private:
-
-    QString               my_string;
-    QStringList           my_stringlist;
-    int                   my_int;
-    QPtrList<GenericTree> my_subnodes;
-    QPtrList<GenericTree> my_ordered_subnodes;
-    QPtrList<GenericTree> my_flatened_subnodes;
-    GenericTree*          my_selected_subnode;
-    IntVector             *my_attributes;
-    GenericTree*          my_parent;
-    bool                  selectable;
-    int                   current_ordering_index;
-};
-
 class UIManagedTreeListType : public UIType
 {
     Q_OBJECT
@@ -665,8 +654,10 @@ class UIManagedTreeListType : public UIType
     void    assignTreeData(GenericTree *a_tree);
     void    moveToNode(QValueList<int> route_of_branches);
     void    moveToNodesFirstChild(QValueList<int> route_of_branchs);
-    QValueList <int> * getRouteToActive();
+    QValueList <int>* getRouteToActive();
+    QStringList getRouteToCurrent();
     bool    tryToSetActive(QValueList <int> route);
+    bool    tryToSetCurrent(QStringList route);
     void    setHighlightImage(QPixmap an_image){highlight_image = an_image;}
     void    setArrowImages(QPixmap up, QPixmap down, QPixmap left, QPixmap right)
                           {
@@ -693,6 +684,8 @@ class UIManagedTreeListType : public UIType
     void    sortTreeBySelectable(){if(my_tree_data) my_tree_data->sortBySelectable();}
     GenericTree *getCurrentNode() { return current_node; }
     void    setCurrentNode(GenericTree *a_node);
+    int     getActiveBin();
+    void    setActiveBin(int a_bin);
 
   public slots:
 
@@ -953,102 +946,68 @@ class UIBlackHoleType : public UIType
     QRect area;
 };
 
-// ********************************************************************
-
-class UIListBtnType : public UIType
+class UIKeyType : public UIType
 {
     Q_OBJECT
 
   public:
+    UIKeyType(const QString &);
+    ~UIKeyType();
 
-    typedef struct {
-        QString text;
-        bool    checked;
-    } UIListBtnTypeItem;
-    
-    UIListBtnType(const QString& name, const QRect& area, int order,
-                        bool showArrow=true, bool showScrollArrows=false,
-                        bool showChecked=false);
-    ~UIListBtnType();
+    void SetContainer(LayerSet *container) { m_container = container; }
+    UIType *GetType(const QString &name) { return m_container->GetType(name); }
 
-    void SetFontActive(fontProp *font);
-    void SetFontInactive(fontProp *font);
-    
-    void SetSpacing(int spacing) {m_itemSpacing = spacing;}
-    void SetMargin(int margin) {m_itemMargin = margin;}
-    void SetItemRegColor(const QColor& beg, const QColor& end, uint alpha);
-    void SetItemSelColor(const QColor& beg, const QColor& end, uint alpha);
-    void Draw(QPainter *, int order, int);
+    QString GetAction() { return m_action; }
+    QString GetChars() { return m_chars; }
+    QPoint GetPosition() { return m_pos; }
+    QString GetClicked() { return m_clicked; }
+    QString GetNotClicked() { return m_notclicked; }
+    QString GetSubtitle() { return m_subtitle; }
 
-    void SetActive(bool active) {m_active = active;}
+    void SetAction(QString action) { m_action = action; }
+    void SetClicked(QString filename) { m_clicked = filename; }
+    void SetNotClicked(QString filename) { m_notclicked = filename; }
+    void SetSubtitle(QString filename) { m_subtitle = filename; }
+    void SetChars(QString chars) { m_chars = chars; }
+    void SetFont(QString font) { m_font = font; }
+    void SetPosition(QPoint pos) { m_pos = pos; }
 
-    void Reset();
-    void AddItem(const QString& text, bool checked = false);
-    void SetItemCurrent(int current);
-    int  GetItemCurrent();
-    int  GetCount();
-    const UIListBtnTypeItem* GetItem(int itemPos);
-    void SetItemChecked(int itemPos, bool checked);
-
-    void MoveDown();
-    void MoveUp();
-    void Toggle();
+    virtual void Draw(QPainter *, int, int);
 
   private:
+    QString m_action;
+    QString m_clicked;
+    QString m_notclicked;
+    QString m_subtitle;
+    QString m_chars;
+    QString m_font;
+    QPoint m_pos;
 
-    void Init();
-    void LoadPixmap(QPixmap& pix, const QString& fileName);
+    LayerSet *m_container;
+};
 
-    int   m_order;
-    QRect m_rect;
-    QRect m_contentsRect;
-    QRect m_arrowsRect;
-    QRect m_itemTextRect;
-    QRect m_itemCheckRect;
-    QRect m_itemArrowRect;
-    
-    int   m_itemHeight;
-    int   m_itemSpacing;
-    int   m_itemMargin;
-    uint  m_itemsVisible;
+class UIKeyboardType : public UIType
+{
+    Q_OBJECT
 
-    bool  m_active;
-    bool  m_showScrollArrows;
-    bool  m_showArrow;
-    bool  m_showChecked;
-    bool  m_showUpArrow;
-    bool  m_showDnArrow;
-    
-    QPixmap   m_itemRegPix;
-    QPixmap   m_itemSelActPix;
-    QPixmap   m_itemSelInactPix;
-    QPixmap   m_upArrowRegPix;
-    QPixmap   m_dnArrowRegPix;
-    QPixmap   m_upArrowActPix;
-    QPixmap   m_dnArrowActPix;
-    QPixmap   m_arrowPix;
-    QPixmap   m_checkPix;
+  public:
+    UIKeyboardType(const QString &, int);
+    ~UIKeyboardType();
 
-    QColor    m_itemRegBeg;
-    QColor    m_itemRegEnd;
-    QColor    m_itemSelBeg;
-    QColor    m_itemSelEnd;
-    uint      m_itemRegAlpha;
-    uint      m_itemSelAlpha;
+    typedef QValueList <UIKeyType*> KeyList;
+    void SetContainer(LayerSet *container) { m_container = container; }
+    UIType *GetType(const QString &name) { return m_container->GetType(name); }
+    UIKeyType *GetKey(const QString &action);
+    KeyList GetKeys() { return m_keymap.values(); }
 
-    fontProp* m_fontActive;
-    fontProp* m_fontInactive;
+    void AddKey(const QString &action, UIKeyType *key) { m_keymap[action] = key; }
+    bool HasKey(const QString &action) { return m_keymap.keys().contains(action); }
 
-    bool      m_initialized;
+    virtual void Draw(QPainter *, int, int);
 
-    UIListBtnTypeItem* m_topItem;
-    UIListBtnTypeItem* m_selItem;
-    QPtrList<UIListBtnTypeItem> m_itemList;
-    
-  signals:
-
-    void itemSelected(int itemPos);
-    void itemChecked(int itemPos, bool checked);
+  private:
+    LayerSet *m_container;
+    QMap <QString, UIKeyType*> m_keymap;
 };
 
 #endif
