@@ -3,6 +3,7 @@ using namespace std;
 
 #include <cmath>
 
+#include "uilistbtntype.h"
 #include "xmlparse.h"
 
 
@@ -523,6 +524,10 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
                     inttype = 4;
                 else if (typ == "WeekslotRecord") 
                     inttype = 5;
+                else if (typ == "FindOneRecord")
+                    inttype = 6;
+                else if (typ == "OverrideRecord")
+                    inttype = 7;
 
                 recImgs[inttype] = img;
             }
@@ -573,7 +578,7 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
         concolor = reccolor;
     guide->SetRecordingColors(reccolor, concolor);
     guide->SetSelectorColor(selcolor);
-    for (int i = 1; i < 6; i++)
+    for (int i = 1; i <= 7; i++)
         guide->LoadImage(i, recImgs[i]);
     if (seltype.lower() == "box")
         guide->SetSelectorType(1);
@@ -597,6 +602,8 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
             guide->SetJustification(Qt::AlignHCenter | Qt::AlignVCenter | jst);
         else if (align.lower() == "vcenter")
             guide->SetJustification(Qt::AlignVCenter | jst);
+        else if (align.lower() == "hcenter")
+            guide->SetJustification(Qt::AlignHCenter | jst);
     }
     else 
         guide->SetJustification(jst);
@@ -720,6 +727,9 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
             bar->SetJustification(Qt::AlignHCenter | Qt::AlignVCenter);
         else if (align.lower() == "vcenter")
             bar->SetJustification(Qt::AlignVCenter);
+        else if (align.lower() == "hcenter")
+            bar->SetJustification(Qt::AlignHCenter);
+    
     }
     align = "";
 
@@ -826,10 +836,18 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
             {
                 parseListBtnArea(container, info);
             }
+            else if (info.tagName() == "listtreearea")
+            {
+                parseListTreeArea(container, info);
+            }
             else if (info.tagName() == "textarea")
             {
                 parseTextArea(container, info);
             }
+            else if (info.tagName() == "multitextarea")
+            {
+                parseMultiTextArea(container, info);
+            }
             else if (info.tagName() == "statusbar")
             {
                 parseStatusBar(container, info);
@@ -868,6 +886,10 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
             {
                 parseBar(container, info);
             }
+            else if (info.tagName() == "keyboard")
+            {
+                parseKeyboard(container, info);
+            }
             else if (info.tagName() == "guidegrid")
             {
                 parseGuideGrid(container, info);
@@ -967,7 +989,7 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
             }
             else
             {
-                cerr << "Unknown tag in listarea: " << info.tagName() << endl;
+                cerr << "Unknown tag in textarea: " << info.tagName() << endl;
                 return;
             }
         }
@@ -1010,6 +1032,8 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
             text->SetJustification(jst | Qt::AlignHCenter | Qt::AlignVCenter);
         else if (align.lower() == "vcenter")
             text->SetJustification(jst | Qt::AlignVCenter);
+        else if (align.lower() == "hcenter")
+            text->SetJustification(jst | Qt::AlignHCenter);
     }
     align = "";
     text->SetParent(container);
@@ -1017,6 +1041,144 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
     container->AddType(text);
 }
 
+void XMLParse::parseMultiTextArea(LayerSet *container, QDomElement &element)
+{
+    int context = -1;
+    QRect area = QRect(0, 0, 0, 0);
+    QRect altArea = QRect(0, 0, 0, 0);
+    QPoint shadowOffset = QPoint(0, 0);
+    QString font = "";
+    QString cutdown = "";
+    QString value = "";
+    QString statictext = "";
+    QString multiline = "";
+    int padding = -1;
+    int drop_delay = -1;
+    int drop_pause = -1;
+    int scroll_delay = -1;
+    int scroll_pause = -1;
+    int draworder = 0;
+
+    QString name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        cerr << "Multitext area needs a name\n";
+        return;
+    }
+
+    QString layerNum = element.attribute("draworder", "");
+    if (layerNum.isNull() && layerNum.isEmpty())
+    {
+        cerr << "Multitext area needs a draworder\n";
+        return;
+    }
+    draworder = layerNum.toInt();
+
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling())
+    {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "context")
+            {
+                context = getFirstText(info).toInt();
+            }
+            else if(info.tagName() == "padding")
+            {
+                padding = getFirstText(info).toInt();
+            }
+            else if(info.tagName() == "dropdelay")
+            {
+                drop_delay = getFirstText(info).toInt();
+            }
+            else if(info.tagName() == "droppause")
+            {
+                drop_pause = getFirstText(info).toInt();
+            }
+            else if(info.tagName() == "scrolldelay")
+            {
+                scroll_delay = getFirstText(info).toInt();
+            }
+            else if(info.tagName() == "scrollpause")
+            {
+                scroll_pause = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "area")
+            {
+                area = parseRect(getFirstText(info));
+                normalizeRect(area);
+            }
+            else if (info.tagName() == "altarea")
+            {
+                altArea = parseRect(getFirstText(info));
+                normalizeRect(altArea);
+            }
+            else if (info.tagName() == "font")
+            {
+                font = getFirstText(info);
+            }
+            else if (info.tagName() == "cutdown")
+            {
+                cutdown = getFirstText(info);
+            }
+            else if (info.tagName() == "shadow")
+            {
+                shadowOffset = parsePoint(getFirstText(info));
+                shadowOffset.setX((int)(shadowOffset.x() * wmult));
+                shadowOffset.setY((int)(shadowOffset.y() * hmult));
+            }
+            else
+            {
+                cerr << "Unknown tag in multitext area: " 
+                     << info.tagName() 
+                     << endl;
+                return;
+            }
+        }
+    }
+
+    fontProp *testfont = GetFont(font);
+    if (!testfont)
+    {
+        cerr << "Unknown font: " << font << " in multitextarea: " << name << endl;
+        return;
+    }
+
+    UIMultiTextType *multitext = new UIMultiTextType(name, testfont, draworder, 
+                                      area, altArea);
+    multitext->SetScreen(wmult, hmult);
+    if (context != -1)
+    {
+        multitext->SetContext(context);
+    }
+
+    if(padding > -1)
+    {
+        multitext->setMessageSpacePadding(padding);
+    }
+    if(drop_delay > -1)
+    {
+        multitext->setDropTimingLength(drop_delay);
+    }
+    if(drop_pause > -1)
+    {
+        multitext->setDropTimingPause(drop_pause);
+    }
+    if(scroll_delay > -1)
+    {
+        multitext->setScrollTimingLength(scroll_delay);
+    }
+    if(scroll_pause > -1)
+    {
+        multitext->setScrollTimingPause(scroll_pause);
+    }
+    
+    multitext->SetParent(container);
+    multitext->calculateScreenArea();
+    container->AddType(multitext);
+}
+
 void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
 {
     int context = -1;
@@ -2238,7 +2400,6 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
     QString fontInactive;
     bool    showArrow = true;
     bool    showScrollArrows = false;
-    bool    showChecked = false;
     int     draworder = 0;
     QColor  grUnselectedBeg(Qt::black);
     QColor  grUnselectedEnd(80,80,80);
@@ -2293,9 +2454,160 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
                 if (getFirstText(info).lower() == "no")
                     showArrow = false;
             }
-            else if (info.tagName() == "showchecks") {
+            else if (info.tagName() == "showscrollarrows") {
                 if (getFirstText(info).lower() == "yes")
-                    showChecked = true;
+                    showScrollArrows = true;
+            }
+            else if (info.tagName() == "gradient") {
+
+                if (info.attribute("type","").lower() == "selected") {
+                    grSelectedBeg = QColor(info.attribute("start"));
+                    grSelectedEnd = QColor(info.attribute("end"));
+                    grSelectedAlpha = info.attribute("alpha","255").toUInt();
+                }
+                else if (info.attribute("type","").lower() == "unselected") {
+                    grUnselectedBeg = QColor(info.attribute("start"));
+                    grUnselectedEnd = QColor(info.attribute("end"));
+                    grUnselectedAlpha = info.attribute("alpha","100").toUInt();
+                }
+                else {
+                    std::cerr << "Unknown type for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+
+                if (!grSelectedBeg.isValid() || !grSelectedEnd.isValid() ||
+                    !grUnselectedBeg.isValid() || !grUnselectedEnd.isValid()) {
+                    std::cerr << "Unknown color for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+
+                if (grSelectedAlpha > 255 || grUnselectedAlpha > 255) {
+                    std::cerr << "Incorrect alpha for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+            }
+            else if (info.tagName() == "spacing") {
+                spacing = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "margin") {
+                margin = getFirstText(info).toInt();
+            }
+            else
+            {
+                std::cerr << "Unknown tag in listbtn area: "
+                          << info.tagName() << endl;
+                return;
+            }
+            
+        }
+    }
+
+    fontProp *fpActive = GetFont(fontActive);
+    if (!fpActive)
+    {
+        cerr << "Unknown font: " << fontActive
+             << " in listbtn area: " << name << endl;
+        return;
+    }
+    
+    fontProp *fpInactive = GetFont(fontInactive);
+    if (!fpInactive)
+    {
+        cerr << "Unknown font: " << fontInactive
+             << " in listbtn area: " << name << endl;
+        return;
+    }
+    
+
+    UIListBtnType *l = new UIListBtnType(name, area, draworder, showArrow,
+                                         showScrollArrows);
+    l->SetScreen(wmult, hmult);
+    l->SetFontActive(fpActive);
+    l->SetFontInactive(fpInactive);
+    l->SetItemRegColor(grUnselectedBeg, grUnselectedEnd, grUnselectedAlpha);
+    l->SetItemSelColor(grSelectedBeg, grSelectedEnd, grSelectedAlpha);
+    l->SetSpacing((int)(spacing*hmult));
+    l->SetMargin((int)(margin*wmult));
+    l->SetParent(container);
+
+    container->AddType(l);
+    container->bumpUpLayers(0);
+}
+
+void XMLParse::parseListTreeArea(LayerSet *container, QDomElement &element)
+{
+    QRect   area = QRect(0,0,0,0);
+    QRect   listsize = QRect(0,0,0,0);
+    int     leveloffset = 0;
+    QString fontActive;
+    QString fontInactive;
+    bool    showArrow = true;
+    bool    showScrollArrows = false;
+    int     draworder = 0;
+    QColor  grUnselectedBeg(Qt::black);
+    QColor  grUnselectedEnd(80,80,80);
+    uint    grUnselectedAlpha(100);
+    QColor  grSelectedBeg(82,202,56);
+    QColor  grSelectedEnd(52,152,56);
+    uint    grSelectedAlpha(255);
+    int     spacing = 2;
+    int     margin = 3;
+
+    QString name = element.attribute("name", "");
+    if (name.isEmpty()) {
+        std::cerr << "ListBtn area needs a name" << std::endl;
+        return;
+    }
+
+    QString layerNum = element.attribute("draworder", "");
+    if (layerNum.isNull() || layerNum.isEmpty())
+    {
+        cerr << "ListBtn area needs a draworder\n";
+        return;
+    }
+
+    draworder = layerNum.toInt();
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling()) {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "area")
+            {
+                area = parseRect(getFirstText(info));
+                normalizeRect(area);
+            }
+            else if (info.tagName() == "listsize")
+            {
+                listsize = parseRect(getFirstText(info));
+                normalizeRect(listsize);
+            }
+            else if (info.tagName() == "leveloffset")
+            {
+                leveloffset = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "fcnfont")
+            {
+                QString fontName = info.attribute("name", "");
+                QString fontFcn  = info.attribute("function", "");
+
+                if (fontFcn.lower() == "active")
+                    fontActive = fontName;
+                else if (fontFcn.lower() == "inactive")
+                    fontInactive = fontFcn;
+                else {
+                    std::cerr << "Unknown font function for listbtn area: "
+                              << fontFcn
+                              << std::endl;
+                    return;
+                }
+            }
+            else if (info.tagName() == "showarrow") {
+                if (getFirstText(info).lower() == "no")
+                    showArrow = false;
             }
             else if (info.tagName() == "showscrollarrows") {
                 if (getFirstText(info).lower() == "yes")
@@ -2340,30 +2652,34 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
             }
             else
             {
-                std::cerr << "Unknown tag in listbtn area: " << info.tagName() << endl;
+                std::cerr << "Unknown tag in listbtn area: "
+                          << info.tagName() << endl;
                 return;
             }
-            
+
         }
     }
 
     fontProp *fpActive = GetFont(fontActive);
     if (!fpActive)
     {
-        cerr << "Unknown font: " << fontActive << " in listbtn area: " << name << endl;
+        cerr << "Unknown font: " << fontActive
+             << " in listbtn area: " << name << endl;
         return;
     }
-    
+
     fontProp *fpInactive = GetFont(fontInactive);
     if (!fpInactive)
     {
-        cerr << "Unknown font: " << fontInactive << " in listbtn area: " << name << endl;
+        cerr << "Unknown font: " << fontInactive
+             << " in listbtn area: " << name << endl;
         return;
     }
-    
 
-    UIListBtnType *l = new UIListBtnType(name, area, draworder, showArrow,
-                                         showScrollArrows, showChecked);
+
+    UIListTreeType *l = new UIListTreeType(name, area, listsize, leveloffset,
+                                           draworder);
+
     l->SetScreen(wmult, hmult);
     l->SetFontActive(fpActive);
     l->SetFontInactive(fpInactive);
@@ -2371,7 +2687,166 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
     l->SetItemSelColor(grSelectedBeg, grSelectedEnd, grSelectedAlpha);
     l->SetSpacing((int)(spacing*hmult));
     l->SetMargin((int)(margin*wmult));
+    l->SetParent(container);
+    l->calculateScreenArea();
     
     container->AddType(l);
-    
+    container->bumpUpLayers(0);
+}
+
+void XMLParse::parseKey(LayerSet *container, QDomElement &element)
+{
+    QString name, action, order, notclicked;
+    QString clicked, chars, font, subtitle;
+    QPoint pos;
+
+    notclicked = clicked = subtitle = chars = font = "";
+    pos = QPoint(0, 0);
+
+    name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        cerr << "key needs a name\n";
+        return;
+    }
+
+    action = element.attribute("action", "");
+    if (action.isNull() || action.isEmpty())
+    {
+        cerr << "key needs an action\n";
+        return;
+    }
+
+    order = element.attribute("draworder", "");
+    if (order.isNull() || order.isEmpty())
+    {
+        cerr << "keyboard needs an order\n";
+        return;
+    }
+
+    LayerSet *c = new LayerSet("_internal");
+
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling())
+    {
+        QDomElement e = child.toElement();
+        if (!e.isNull())
+        {
+            if (e.tagName() == "notclickedimg")
+            {
+                notclicked = getFirstText(e);
+            }
+            else if (e.tagName() == "clickedimg")
+            {
+                clicked = getFirstText(e);
+            }
+            else if (e.tagName() == "subtitleimg")
+            {
+                subtitle = getFirstText(e);
+            }
+            else if (e.tagName() == "position")
+            {
+                pos = parsePoint(getFirstText(e));
+                pos.setX((int)(pos.x() * wmult));
+                pos.setY((int)(pos.y() * hmult));
+
+            }
+            else if (e.tagName() == "chars")
+            {
+                chars = getFirstText(e);
+            }
+            else if (e.tagName() == "textarea")
+            {
+                parseTextArea(c, e);
+            }
+            else if (e.tagName() == "font")
+            {
+                font = getFirstText(e);
+            }
+            else
+            {
+                cerr << "Unknown: " << e.tagName() << " in key\n";
+                return;
+            }
+        }
+    }
+
+    vector<UIType *>::iterator i = c->getAllTypes()->begin();
+    for (; i != c->getAllTypes()->end(); i++)
+    {
+        UIType *type = (*i);
+        if (UITextType *text = dynamic_cast<UITextType*>(type))
+        {
+            QRect rect = text->DisplayArea();
+            rect.moveTopLeft(rect.topLeft() + pos);
+            text->SetDisplayArea(rect);
+        }
+    }
+
+    UIKeyType *key = new UIKeyType(name);
+    key->SetContainer(c);
+    key->SetOrder(order.toInt());
+    key->SetAction(action);
+    key->SetClicked(clicked);
+    key->SetNotClicked(notclicked);
+    key->SetSubtitle(subtitle);
+    key->SetChars(chars);
+    key->SetFont(font);
+    key->SetPosition(pos);
+    container->AddType(key);
+}
+
+void XMLParse::parseKeyboard(LayerSet *container, QDomElement &element)
+{
+    QString name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        cerr << "keyboard needs a name\n";
+        return;
+    }
+
+    QString order = element.attribute("draworder", "");
+    if (order.isNull() || order.isEmpty())
+    {
+        cerr << "keyboard needs an order\n";
+        return;
+    }
+
+    LayerSet *c = new LayerSet("_internal");
+
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling())
+    {
+        QDomElement e = child.toElement();
+        if (!e.isNull())
+        {
+            if (e.tagName() == "image")
+            {
+                parseImage(c, e);
+            }
+            else if (e.tagName() == "key")
+            {
+                parseKey(c, e);
+            }
+            else
+            {
+                cerr << "Unknown: " << e.tagName() << " in keyboard\n";
+                return;
+            }
+        }
+    }
+
+    UIKeyboardType *kbd = new UIKeyboardType(name, order.toInt());
+    kbd->SetContainer(c);
+    container->AddType(kbd);
+
+    vector<UIType *>::iterator i = c->getAllTypes()->begin();
+    for (; i != c->getAllTypes()->end(); i++)
+    {
+        UIType *type = (*i);
+        if (UIKeyType *keyt = dynamic_cast<UIKeyType*>(type))
+        {
+            kbd->AddKey(keyt->GetAction(), keyt);
+        }
+    }
 }
