@@ -34,6 +34,7 @@ class XMLParse
     void parseGuideGrid(LayerSet *, QDomElement &);
     void parseManagedTreeList(LayerSet *, QDomElement &);
     void parseTextArea(LayerSet *, QDomElement &);
+    void parseMultiTextArea(LayerSet *, QDomElement &);
     void parseStatusBar(LayerSet *, QDomElement &);
     void parseImage(LayerSet *, QDomElement &);
     void parseRepeatedImage(LayerSet *, QDomElement &);
@@ -43,6 +44,9 @@ class XMLParse
     void parseSelector(LayerSet *, QDomElement &);
     void parseBlackHole(LayerSet *, QDomElement &);
     void parseListBtnArea(LayerSet *, QDomElement &); 
+    void parseListTreeArea(LayerSet *, QDomElement &);
+    void parseKeyboard(LayerSet *, QDomElement &);
+    void parseKey(LayerSet *, QDomElement &);
  
   private:
     QMap<QString, fontProp> fontMap;
