@@ -11,7 +11,6 @@
 #include "osd.h"
 #include "jitterometer.h"
 #include "recordingprofile.h"
-#include "commercial_skip.h"
 #include "videooutbase.h"
 
 extern "C" {
@@ -24,7 +23,7 @@ using namespace std;
 class VideoOutput;
 class OSDSet;
 class RemoteEncoder;
-class QSqlDatabase;
+class MythSqlDatabase;
 class ProgramInfo;
 class DecoderBase;
 class AudioOutput;
@@ -42,9 +41,11 @@ struct TextContainer
 class NuppelVideoPlayer
 {
  public:
-    NuppelVideoPlayer(QSqlDatabase *ldb = NULL, ProgramInfo *info = NULL);
+    NuppelVideoPlayer(MythSqlDatabase *ldb = NULL, ProgramInfo *info = NULL);
    ~NuppelVideoPlayer();
 
+    friend class CommDetect;
+
     void SetParentWidget(QWidget *widget) { parentWidget = widget; }
 
     void SetAsPIP(void) { disableaudio = disablevideo = true; }
@@ -129,7 +130,7 @@ class NuppelVideoPlayer
     void ClearBookmark(void);
     long long GetBookmark(void);
 
-    void ToggleCC(void);
+    void ToggleCC(char mode, int arg);
 
     // edit mode stuff
     bool EnableEdit(void);
@@ -143,7 +144,8 @@ class NuppelVideoPlayer
     void ForceVideoOutputType(VideoOutputType type);
 
     void SetVideoParams(int width, int height, double fps, 
-                        int keyframedistance, float aspect = 1.33333);
+                        int keyframedistance, float aspect = 1.33333,
+                        FrameScanType scan = kScan_Ignore);
     void SetAudioParams(int bps, int channels, int samplerate);
     void SetEffDsp(int dsprate);
     void SetFileLength(int total, int frames);
@@ -174,7 +176,8 @@ class NuppelVideoPlayer
     void ReinitVideo(void);
     void ReinitAudio(void);
 
-    void ToggleLetterbox(void);
+    void ToggleLetterbox(int letterboxMode = -1);
+    void Zoom(int direction);
     int GetLetterbox(void);
 
     void ExposeEvent(void);
@@ -230,7 +233,9 @@ class NuppelVideoPlayer
 
     int UpdateDelay(struct timeval *nexttrigger);
 
-    void ShowText();
+    void ShowText(void);
+    void ResetCC(void);
+    void UpdateCC(unsigned char *inpos);
 
     void UpdateTimeDisplay(void);
     void UpdateSeekAmount(bool up);
@@ -252,6 +257,9 @@ class NuppelVideoPlayer
 
     float WarpFactor(void);
 
+    FrameScanType detectInterlace(FrameScanType newScan, FrameScanType scan,
+                                  float fps, int video_height);
+
     QString filename;
     
     /* rtjpeg_plugin stuff */
@@ -263,6 +271,7 @@ class NuppelVideoPlayer
     int video_size;
     double video_frame_rate;
     float video_aspect;
+    FrameScanType m_scan;
 
     int filesize;
     int startpos;
@@ -298,6 +307,7 @@ class NuppelVideoPlayer
     QWaitCondition decoderThreadPaused, videoThreadPaused;
  
     bool cc;
+    unsigned char ccmode;
 
     bool playing;
     bool decoder_thread_alive;
@@ -317,7 +327,7 @@ class NuppelVideoPlayer
     long long rewindtime, fftime;
     RemoteEncoder *nvr_enc;
 
-    CommDetect *commDetect;
+    class CommDetect *commDetect;
 
     int totalLength;
     long long totalFrames;
@@ -366,8 +376,7 @@ class NuppelVideoPlayer
     WId embedid;
     int embx, emby, embw, embh;
 
-    QSqlDatabase *m_db;
-    QMutex db_lock;
+    MythSqlDatabase *m_db;
     ProgramInfo *m_playbackinfo;
 
     long long bookmarkseek;
@@ -378,9 +387,9 @@ class NuppelVideoPlayer
     int commercialskipmethod;
     bool tryunflaggedskip;
 
-    QString cclines[4];
-    int ccindent[4];
-    int lastccrow;
+    QString ccline;
+    int cccol;
+    int ccrow;
 
     DecoderBase *decoder;
 
