@@ -118,7 +118,12 @@ class NuppelVideoRecorder : public RecorderBase
     void DoMJPEG(void);
 
     void FormatTeletextSubtitles(struct VBIData *vbidata);
-    void FormatCC(struct ccsubtitle *subtitle, struct cc *cc, int data);
+    void FormatCC(struct cc *cc);
+    void FormatCCField(struct cc *cc, int tc, int field);
+    QChar CharCC(int code);
+    void ResetCC(struct cc *cc, int mode);
+    void BufferCC(struct cc *cc, int mode, int len, int clr);
+    int NewRowCC(struct cc *cc, int mode, int len);
     
     bool encoding;
     
@@ -143,7 +148,6 @@ class NuppelVideoRecorder : public RecorderBase
     int quiet;
     int rawmode;
     int usebttv;
-    struct video_audio *origaudio;
 
     CommDetect *commDetect;
 
@@ -253,8 +257,6 @@ class NuppelVideoRecorder : public RecorderBase
 
     bool cleartimeonpause;
 
-    struct ccsubtitle subtitle;
-
     bool usingv4l2;
     int channelfd;
 
@@ -263,7 +265,11 @@ class NuppelVideoRecorder : public RecorderBase
 
     ChannelBase *channelObj;
 
-    bool setorigaudio;
+    bool skip_btaudio;
+
+    bool correct_bttv;
+
+    int volume;
 };
 
 #endif
