@@ -12,7 +12,6 @@
 #include <qfile.h>
 #include <qregexp.h>
 
-#include <iostream>
 using namespace std;
 
 #include "RingBuffer.h"
@@ -109,7 +108,7 @@ ThreadedFileWriter::ThreadedFileWriter(const char *filename,
     if (fd <= 0)
     {
         /* oops! */
-        cerr << "ERROR opening file in ThreadedFileWriter.\n";
+        VERBOSE(VB_IMPORTANT,"ERROR opening file in ThreadedFileWriter.");
         perror(filename);
         exit(1);
     }
@@ -150,7 +149,7 @@ unsigned ThreadedFileWriter::Write(const void *data, unsigned count)
     while (count > BufFree())
     {
         if (first)
-            cerr << "IOBOUND - blocking in ThreadedFileWriter::Write()\n";
+             VERBOSE(VB_IMPORTANT, "IOBOUND - blocking in ThreadedFileWriter::Write()");
         first = 0;
         usleep(5000);
     }
@@ -382,6 +381,9 @@ void RingBuffer::Init(void)
     stopreads = false;
     wanttoread = 0;
 
+    numfailures = 0;
+    commserror = false;
+
     pthread_rwlock_init(&rwlock, NULL);
 }
 
@@ -440,7 +442,10 @@ void RingBuffer::Reset(void)
         if (readaheadrunning)
             ResetReadAhead(0);
     }
-    
+   
+    numfailures = 0;
+    commserror = false;
+ 
     pthread_rwlock_unlock(&rwlock);
 }
 
@@ -458,6 +463,7 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
         {
             perror("ERROR: file I/O problem in 'safe_read()'");
             errcnt++;
+            numfailures++;
             if (errcnt == 3) 
                 break;
         }
@@ -496,6 +502,7 @@ int RingBuffer::safe_read(RemoteFile *rf, void *data, unsigned sz)
         VERBOSE(VB_IMPORTANT, "RemoteFile::Read() failed in RingBuffer::safe_read().");
         rf->Seek(internalreadpos, SEEK_SET);
         ret = 0;
+        numfailures++;
      }
 
     return ret;
@@ -679,7 +686,7 @@ void RingBuffer::ReadAheadThread(void)
         readaheadpaused = false;
 
         pthread_rwlock_rdlock(&rwlock);
-        if (totfree > readblocksize)
+        if (totfree > readblocksize && !commserror)
         {
             // limit the read size
             totfree = readblocksize;
@@ -753,10 +760,13 @@ void RingBuffer::ReadAheadThread(void)
             }
         }
 
+        if (numfailures > 5)
+            commserror = true;
+
         totfree = ReadBufFree();
         used = READ_AHEAD_SIZE - totfree;
 
-        if (ateof)
+        if (ateof || commserror)
         {
             readsallowed = true;
             totfree = 0;
@@ -776,8 +786,11 @@ void RingBuffer::ReadAheadThread(void)
             readsAllowedWait.wakeAll();
 
         availWaitMutex.lock();
-        if (ateof || stopreads || (wanttoread <= used && wanttoread > 0))
+        if (commserror || ateof || stopreads || 
+            (wanttoread <= used && wanttoread > 0))
+        {
             availWait.wakeAll();
+        }
         availWaitMutex.unlock();
             
         pthread_rwlock_unlock(&rwlock);
@@ -796,6 +809,9 @@ void RingBuffer::ReadAheadThread(void)
 
 int RingBuffer::ReadFromBuf(void *buf, int count)
 {
+    if (commserror)
+        return 0;
+
     bool readone = false;
 
     if (readaheadpaused && stopreads)
@@ -807,7 +823,7 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         while (!readsallowed && !stopreads)
         {
             if (!readsAllowedWait.wait(5000))
-                cerr << "taking too long to be allowed to read..\n";
+                 VERBOSE(VB_IMPORTANT, "taking too long to be allowed to read..");
         }
 
     int avail = ReadBufAvail();
@@ -820,13 +836,13 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         wanttoread = count;
         if (!availWait.wait(&availWaitMutex, 2000))
         {
-            cerr << "Waited 2 seconds for data to become available, waiting "
-                    "again...\n";
+            VERBOSE(VB_IMPORTANT,"Waited 2 seconds for data to become available, waiting "
+                    "again...");
             readErr++;
             if (readErr > 7)
             {
-                cerr << "Waited 14 seconds for data to become available, "
-                        "aborting\n";
+                VERBOSE(VB_IMPORTANT,"Waited 14 seconds for data to become available, "
+                        "aborting");
                 wanttoread = 0;
                 stopreads = true;
                 availWaitMutex.unlock();
@@ -840,6 +856,9 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         avail = ReadBufAvail();
         if (ateof && avail < count)
             count = avail;
+
+        if (commserror)
+            return 0;
     }
 
     if ((ateof || stopreads) && avail < count)
@@ -938,8 +957,8 @@ int RingBuffer::Read(void *buf, int count)
             {
                 if (!availWait.wait(&availWaitMutex, 15000))
                 {
-                    cerr << "Couldn't read data from the capture card in 15 "
-                            "seconds.  Game over, man.\n";
+                    VERBOSE(VB_IMPORTANT,"Couldn't read data from the capture card in 15 "
+                            "seconds.  Game over, man.");
                     StopReads();
                 }
 
