@@ -10,6 +10,7 @@ using namespace std;
 #include "remoteencoder.h"
 #include "programinfo.h"
 #include "mythcontext.h"
+#include "mythdbcon.h"
 
 extern "C" {
 #ifdef USING_XVMC
@@ -22,13 +23,10 @@ extern "C" {
 
 extern pthread_mutex_t avcodeclock;
 
-AvFormatDecoder::AvFormatDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
+AvFormatDecoder::AvFormatDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
                                  ProgramInfo *pginfo)
-               : DecoderBase(parent)
+               : DecoderBase(parent, db, pginfo)
 {
-    m_db = db;
-    m_playbackinfo = pginfo;
-
     ic = NULL;
     directrendering = false;
     video_last_P_pts = lastapts = lastvpts = 0;
@@ -241,6 +239,31 @@ void AvFormatDecoder::InitByteContext(void)
 
 #define ALIGN(x, a) (((x) + (a) - 1) & ~((a) - 1))
 
+static QMap<void*, enum PixelFormat> _PixelFormatsMap;
+
+static enum PixelFormat getFormat(struct AVCodecContext *cc,
+                                  const enum PixelFormat *pixfmts) 
+{
+    if (_PixelFormatsMap.end() != _PixelFormatsMap.find(cc))
+    {
+        return _PixelFormatsMap[cc];
+    }
+
+    VERBOSE(VB_IMPORTANT, "Pixel format not set, returning pixfmts[0]");
+    return pixfmts[0];
+}
+
+void AvFormatDecoder::SetPixelFormat(const int pixFormat) 
+{
+    PixelFormat pix = (PixelFormat)pixFormat;
+    for (int i = 0; i < ic->nb_streams; i++) 
+    {
+        AVCodecContext *cc = &ic->streams[i]->codec;
+        if (CODEC_TYPE_VIDEO == cc->codec_type)
+            _PixelFormatsMap[cc]=pix;
+    }
+}
+
 int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                               char testbuf[2048])
 {
@@ -263,7 +286,7 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
 
     fmt->flags |= AVFMT_NOFILE;
 
-    ic = (AVFormatContext *)av_mallocz(sizeof(AVFormatContext));
+    ic = av_alloc_format_context();
     if (!ic)
     {
         cerr << "Couldn't allocate context\n";
@@ -320,19 +343,23 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                                          ALIGN(enc->height, 16), fps, 
                                          keyframedist, aspect_ratio);
              
-                enc->error_resilience = 2;
+                enc->error_resilience = FF_ER_COMPLIANT;
                 enc->workaround_bugs = FF_BUG_AUTODETECT;
-                enc->error_concealment = 3;
-                enc->idct_algo = 0;
+                enc->error_concealment = FF_EC_GUESS_MVS | FF_EC_DEBLOCK;
+                enc->idct_algo = FF_IDCT_AUTO;
                 enc->debug = 0;
                 enc->rate_emu = 0;
+                enc->error_rate = 0;
 
                 if (!novideo && (enc->codec_id == CODEC_ID_MPEG1VIDEO || 
                     enc->codec_id == CODEC_ID_MPEG2VIDEO))
                 {
 #ifdef USING_XVMC
                     if (gContext->GetNumSetting("UseXVMC", 1))
+                    {
                         enc->codec_id = CODEC_ID_MPEG2VIDEO_XVMC;
+                        //enc->get_format = getFormat;
+                    }
 #endif            
 #ifdef USING_VIASLICE
                     if (gContext->GetNumSetting("UseViaSlice", 1))
@@ -456,6 +483,9 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
         keyframedist = 1;
     }
 
+    //if (livetv || watchingrecording)
+        ic->build_index = 0;
+    
     dump_format(ic, 0, filename, 0);
     if (hasFullPositionMap)
     {
@@ -859,6 +889,7 @@ void AvFormatDecoder::MpegPreProcessPkt(AVStream *stream, AVPacket *pkt)
 {
     AVCodecContext *context = &(stream->codec);
     unsigned char *bufptr = pkt->data;
+    //unsigned char *bufend = pkt->data + pkt->size;
     unsigned int state = 0xFFFFFFFF, v = 0;
     
     int prvcount = -1;
@@ -880,7 +911,26 @@ void AvFormatDecoder::MpegPreProcessPkt(AVStream *stream, AVPacket *pkt)
             {
                 case SEQ_START:
                 {
+                    //int size = 8;
+                    //if (bufptr + size > bufend) 
+                    //    return;
                     unsigned char *test = bufptr;
+                    /* XXX: Doesn't work properly.
+                    if (test[size - 1] & 0x40) 
+                        size += 64;
+                    if (bufptr + size > bufend) 
+                        return;
+                    if (test[size - 1] & 0x80) 
+                        size += 64;
+                    if ((bufptr + size + 4) > bufend) 
+                        return;
+                    test = bufptr + size;
+                    if (test[0] != 0x0 || test[1] != 0x0 || test[2] != 0x1) {
+                        cerr << "avfd: sequence header size mismatch" << endl;
+                        continue;
+                    }
+
+                    test = bufptr;*/
                     int width = (test[0] << 4) | (test[1] >> 4);
                     int height = ((test[1] & 0xff) << 8) | test[2];
 
@@ -889,11 +939,12 @@ void AvFormatDecoder::MpegPreProcessPkt(AVStream *stream, AVPacket *pkt)
                     float aspect = GetMpegAspect(context, aspectratioinfo,
                                                  width, height);
 
-                    if (CheckVideoParams(width, height) ||
-                        aspect != current_aspect)
+                    if (width < 2500 && height < 2000 &&
+                        (CheckVideoParams(width, height) ||
+                         aspect != current_aspect))
                     {
-                        m_parent->SetVideoParams(ALIGN(width,16),
-                                                 ALIGN(height,16), fps,
+                        m_parent->SetVideoParams(ALIGN(width, 16),
+                                                 ALIGN(height, 16), fps,
                                                  keyframedist, aspect);
                         m_parent->ReinitVideo();
                         current_width = width;
@@ -1436,10 +1487,13 @@ void AvFormatDecoder::SetPositionMap(void)
     
     if (m_playbackinfo && m_db) 
     {
+        m_db->lock();
         if (positionMapByFrame)
-            m_playbackinfo->SetPositionMap(posMap, MARK_GOP_BYFRAME, m_db);
+            m_playbackinfo->SetPositionMap(posMap, MARK_GOP_BYFRAME, 
+                                           m_db->db());
         else
-            m_playbackinfo->SetPositionMap(posMap, MARK_GOP_START, m_db);
+            m_playbackinfo->SetPositionMap(posMap, MARK_GOP_START, m_db->db());
+        m_db->unlock();
     }
 }
 
@@ -1447,7 +1501,9 @@ bool AvFormatDecoder::PosMapFromDb()
 {
     // Overwrites current positionmap with entire contents of database
     QMap<long long, long long> posMap;
-    m_playbackinfo->GetPositionMap(posMap, MARK_GOP_BYFRAME, m_db);
+
+    m_db->lock();
+    m_playbackinfo->GetPositionMap(posMap, MARK_GOP_BYFRAME, m_db->db());
     if (!posMap.empty())
     {
         positionMapByFrame = true;
@@ -1461,9 +1517,11 @@ bool AvFormatDecoder::PosMapFromDb()
     {
         // Fall back to map by keyframe number, which assumes an
         // even cadence of keyframes
-        m_playbackinfo->GetPositionMap(posMap, MARK_GOP_START, m_db);
+        m_playbackinfo->GetPositionMap(posMap, MARK_GOP_START, m_db->db());
     }
 
+    m_db->unlock();
+
     if (posMap.empty())
         return false; // no position map in recording
 
