@@ -3,7 +3,6 @@
 
 #include <qstring.h>
 #include <qmap.h>
-#include <qsqldatabase.h>
 
 #include "programinfo.h"
 #include "format.h"
@@ -16,11 +15,12 @@ extern "C" {
 }
 
 class ProgramInfo;
+class MythSqlDatabase;
 
 class AvFormatDecoder : public DecoderBase
 {
   public:
-    AvFormatDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
+    AvFormatDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
                     ProgramInfo *pginfo);
    ~AvFormatDecoder();
 
@@ -54,6 +54,8 @@ class AvFormatDecoder : public DecoderBase
 
     QString GetEncodingType(void) { return QString("MPEG-2"); }
 
+    void SetPixelFormat(const int);
+
   protected:
     RingBuffer *getRingBuf(void) { return ringBuffer; }
 
@@ -144,9 +146,6 @@ class AvFormatDecoder : public DecoderBase
     bool seen_gop;
     int seq_count;
 
-    QSqlDatabase *m_db;
-    ProgramInfo *m_playbackinfo;
-
     double fps;
 
     QPtrList<VideoFrame> inUseBuffers;
