@@ -17,7 +17,7 @@
 #include <iostream>
 using namespace std;
 
-Channel::Channel(TVRec *parent, const QString &videodevice)
+Channel::Channel(TVRec *parent, const QString &videodevice, bool strength)
        : ChannelBase(parent)
 {
     device = videodevice;
@@ -29,6 +29,7 @@ Channel::Channel(TVRec *parent, const QString &videodevice)
     usingv4l2 = false;
     videomode = VIDEO_MODE_NTSC;
     currentFormat = "";
+    usingstrength = strength;
 }
 
 Channel::~Channel(void)
@@ -46,7 +47,8 @@ bool Channel::Open(void)
         isopen = true;
     else
     {
-         cerr << "Channel::Open(): Can't open: " << device << endl;
+         VERBOSE(VB_IMPORTANT, QString("Channel::Open(): Can't open: %1")
+			              .arg(device));
          perror(device.ascii());
          return false;
     }
@@ -281,8 +283,10 @@ bool Channel::ChannelUp(void)
 
         if (chancount > totalChannels)
         {
-            cerr << "Error, couldn't find any available channels.\n";
-            cerr << "Your database is most likely setup incorrectly.\n";
+            VERBOSE(VB_IMPORTANT, "Error, couldn't find any available "
+                                  "channels.");
+            VERBOSE(VB_IMPORTANT, "Your database is most likely setup "
+                                  "incorrectly.");
             break;
         }
     }
@@ -314,8 +318,10 @@ bool Channel::ChannelDown(void)
 
         if (chancount > totalChannels)
         {
-            cerr << "Error, couldn't find any available channels.\n";
-            cerr << "Your database is most likely setup incorrectly.\n";
+            VERBOSE(VB_IMPORTANT, "Error, couldn't find any available "
+                                  "channels.");
+            VERBOSE(VB_IMPORTANT, "Your database is most likely setup "
+                                  "incorrectly.");
             break;
         }
     }
@@ -325,24 +331,36 @@ bool Channel::ChannelDown(void)
 
 bool Channel::SetChannelByString(const QString &chan)
 {
+    QString inputName;
+    
     if (!Open())
     {
-        cerr << "channel object wasn't open, can't change channels\n";
+        VERBOSE(VB_IMPORTANT, "Channel object wasn't open, can't change "
+                              "channels");
         return false;
     }
 
     QSqlDatabase *db_conn;
-    pthread_mutex_t db_lock;
+    pthread_mutex_t *db_lock;
 
-    if (!pParent->CheckChannel(this, chan, db_conn, db_lock))
+    if (!pParent->CheckChannel(this, chan, db_conn, db_lock, inputName))
     {
         VERBOSE(VB_IMPORTANT, QString("CheckChannel failed. Please verify "
                 "channel \"%1\" in the \"setup\" Channel Editor.").arg(chan));
         return false;
     }
 
-    pthread_mutex_lock(&db_lock);
-    QString thequery = QString("SELECT finetune, freqid, tvformat, freqtable "
+    // If CheckChannel filled in the inputName then we need to change inputs
+    // and return, since the act of changing inputs will change the channel as well.
+    if(!inputName.isEmpty())
+    {
+        ChannelBase::SwitchToInput(inputName, chan);
+        return true;
+    }
+
+    pthread_mutex_lock(db_lock);
+    QString thequery = QString("SELECT finetune, freqid, tvformat, "
+                                   "freqtable, commfree "
                                "FROM channel "
                                "LEFT JOIN videosource USING (sourceid) "
                                "WHERE channum = \"%1\" AND "
@@ -355,7 +373,7 @@ bool Channel::SetChannelByString(const QString &chan)
         MythContext::DBError("fetchtuningparams", query);
     if (query.numRowsAffected() <= 0)
     {
-        pthread_mutex_unlock(&db_lock);
+        pthread_mutex_unlock(db_lock);
         return false;
     }
     query.next();
@@ -364,6 +382,7 @@ bool Channel::SetChannelByString(const QString &chan)
     QString freqid = query.value(1).toString();
     QString tvformat = query.value(2).toString();
     QString freqtable = query.value(3).toString();
+    commfree = query.value(4).toBool();
 
     if (freqtable == "default" || freqtable.isNull() || freqtable.isEmpty())
         SetFreqTable(defaultFreqTable);
@@ -373,21 +392,37 @@ bool Channel::SetChannelByString(const QString &chan)
     if (tvformat.isNull() || tvformat.isEmpty())
         tvformat = "Default";
 
-    pthread_mutex_unlock(&db_lock);
+    pthread_mutex_unlock(db_lock);
+
+    // Determine if freqid is id or frequency
+    //  freqid is a frequency if and only if it is a valid integer and 
+    //  larger than 50 MHz
+    bool ok;
+    int frequency = freqid.toInt(&ok);
+    bool isFrequency = ok && frequency > 50000;
+
+    SetFormat(tvformat);
 
     // Tune
     if (externalChanger[currentcapchannel].isEmpty())
     {
-        if (!TuneTo(freqid, finetune))
-            return false;
+        if (isFrequency)
+        {
+            frequency = frequency * 16 / 1000 + finetune;
+            if (!TuneToFrequency(frequency))
+                return false;
+        }
+        else
+        {
+            if (!TuneTo(freqid, finetune))
+                return false;
+        }
     }
     else if (!ChangeExternalChannel(freqid))
         return false;
 
     curchannelname = chan;
 
-    SetFormat(tvformat);
-
     pParent->SetVideoFiltersForChannel(this, chan);
     SetContrast();
     SetColour();
@@ -399,6 +434,82 @@ bool Channel::SetChannelByString(const QString &chan)
     return true;
 }
 
+template<typename V>
+V clamp(V val, V minv, V maxv) { return std::min(maxv, std::max(minv, val)); }
+
+// Returns ATSC signal strength 0-100. >75 is good.
+int signalStrengthATSC(int device, int input) 
+{
+    struct video_signal vsig;
+
+    int ioctlval = ioctl(device,VIDIOCGSIGNAL,&vsig);
+    if (ioctlval == -1) 
+    {
+        perror("VIDIOCGSIGNAL problem in channel.h's signalStrengthATSC()");
+        return 0;
+    }
+
+    int signal = (input == 0) ? vsig.strength : vsig.aux;
+
+    if ((signal & 0xff) != 0x43) 
+        return 0;
+    else 
+        return clamp(101 - (signal >> 9), 0, 100);
+}
+
+// Returns ATSC signal strength 0-100. >75 is good. Using v4l2.
+int signalStrengthATSC_v4l2(int device, int input)
+{
+    struct v4l2_tuner vsig;
+    vsig.index = input;
+
+    int ioctlval = ioctl(device,VIDIOC_G_TUNER, &vsig);
+    if (ioctlval == -1)
+    {
+        perror("VIDIOC_G_TUNER problem in channel.h's signalStrengthATSC()");
+        return 0;
+    }
+    VERBOSE(VB_CHANNEL, QString("signal strength: %1").arg(vsig.signal));
+
+    return clamp(vsig.signal, 0, 100);
+}
+
+bool Channel::CheckSignal(int msecTotal, int reqSignal, int input) 
+{
+    int msecSleep = 500, maxSignal = 0, i = 0;
+
+    msecTotal = max(msecSleep, msecTotal);
+
+    if (usingstrength) 
+    {
+        VERBOSE(VB_CHANNEL, QString("CheckSignal(%1, %2, %3) usingv4l2(%4)")
+                .arg(msecTotal).arg(reqSignal).arg(input).arg(usingv4l2));
+
+        for (i = 0; i < (msecTotal / msecSleep) + 1; i++) 
+        {
+            if (i != 0) 
+                usleep(msecSleep);
+            if (usingv4l2)
+                maxSignal = max(maxSignal, signalStrengthATSC_v4l2(videofd, 
+                                input));
+            else
+                maxSignal = max(maxSignal, signalStrengthATSC(videofd, input));
+
+
+            if (maxSignal >= reqSignal) 
+                break;
+        }
+        VERBOSE(VB_IMPORTANT, QString("Maximum signal strength detected: %1\% "
+                                      "after %2 msec wait")
+                                      .arg(maxSignal).arg(i * msecSleep));
+
+        if (maxSignal < reqSignal) 
+            return false;
+    }
+
+    return true;
+}
+
 bool Channel::TuneTo(const QString &channum, int finetune)
 {
     int i = GetCurrentChannelNum(channum);
@@ -407,6 +518,41 @@ bool Channel::TuneTo(const QString &channum, int finetune)
 
     int frequency = curList[i].freq * 16 / 1000 + finetune;
 
+    VERBOSE(VB_CHANNEL, QString("TuneTo(%1) curList[i].freq(%2)")
+                                .arg(channum).arg(curList[i].freq));
+
+    return TuneToFrequency(frequency);
+}
+
+bool Channel::CheckSignalFull(void)
+{
+    int signalThresholdWait = 5000;
+    int signalThreshold = 65;
+    if (usingstrength)
+    {
+        signalThresholdWait = gContext->GetNumSetting("ATSCCheckSignalWait",
+                                                      5000);
+        signalThreshold = gContext->GetNumSetting("ATSCCheckSignalThreshold",
+                                                  65);
+    }
+
+    return CheckSignal(signalThresholdWait, signalThreshold, currentcapchannel);
+}
+
+bool Channel::TuneToFrequency(int frequency)
+{
+    VERBOSE(VB_CHANNEL, QString("TuneToFrequency(%1)").arg(frequency));
+
+    int signalThresholdWait = 5000;
+    int signalThreshold = 65;
+    if (usingstrength) 
+    {
+        signalThresholdWait = gContext->GetNumSetting("ATSCCheckSignalWait", 
+                                                      5000);
+        signalThreshold = gContext->GetNumSetting("ATSCCheckSignalThreshold", 
+                                                  65);
+    }
+
     if (usingv4l2)
     {
         struct v4l2_frequency vf;
@@ -419,7 +565,8 @@ bool Channel::TuneTo(const QString &channum, int finetune)
             perror("VIDIOC_S_FREQUENCY");
             return false;
         }
-        return true;
+        return CheckSignal(signalThresholdWait, signalThreshold,
+                           currentcapchannel);
     }
 
     if (ioctl(videofd, VIDIOCSFREQ, &frequency) == -1)
@@ -428,7 +575,7 @@ bool Channel::TuneTo(const QString &channum, int finetune)
         return false;
     }
 
-    return true;
+    return CheckSignal(signalThresholdWait, signalThreshold, currentcapchannel);
 }
 
 void Channel::SwitchToInput(int newcapchannel, bool setstarting)
