@@ -14,7 +14,7 @@ class TVRec;
 class Channel : public ChannelBase
 {
  public:
-    Channel(TVRec *parent, const QString &videodevice);
+    Channel(TVRec *parent, const QString &videodevice, bool strength = false);
     virtual ~Channel(void);
 
     bool Open(void);
@@ -27,6 +27,10 @@ class Channel : public ChannelBase
     bool ChannelUp(void);
     bool ChannelDown(void);
 
+    bool CheckSignalFull(void);
+    bool CheckSignal(int msecWait = 5000, 
+                     int requiredSignalPercentage = 65,
+                     int input = 0);
     unsigned short *Channel::GetV4L1Field(int attrib, 
                                           struct video_picture &vid_pic);
     int Channel::ChangeColourAttribute(int attrib, const char *name, bool up);
@@ -44,12 +48,15 @@ class Channel : public ChannelBase
 
     void SetFd(int fd); 
 
+    bool IsCommercialFree() { return commfree; }
+
   private:
     int GetCurrentChannelNum(const QString &channame);
 
     void SetFreqTable(const int index);
     int SetFreqTable(const QString &name);
     bool TuneTo(const QString &chan, int finetune);
+    bool TuneToFrequency(int frequency);
 
     QString device;
     bool isopen;  
@@ -64,6 +71,10 @@ class Channel : public ChannelBase
     QString currentFormat;
 
     int defaultFreqTable;
+
+    bool commfree;
+
+    bool usingstrength;
 };
 
 #endif
