@@ -8,7 +8,7 @@ using namespace std;
 
 #include "mythcontext.h"
 
-const QString currentDatabaseVersion = "1027";
+const QString currentDatabaseVersion = "1049";
 
 void UpdateDBVersionNumber(const QString &newnumber)
 {
@@ -33,6 +33,17 @@ void performActualUpdate(const QString updates[], QString version,
     while (thequery != "")
     {
         db_conn->exec(thequery);
+
+        if (db_conn->lastError().type() != QSqlError::None)
+        {
+            cerr << "DB Error (Performing database upgrade):" << endl;
+            cerr << "Query was:" << endl
+                 << thequery << endl
+                 << MythContext::DBErrorMessage(db_conn->lastError()) << endl;
+
+            exit(-1);
+        }
+
         counter++;
         thequery = updates[counter];
     }
@@ -46,10 +57,10 @@ void InitializeDatabase(void);
 void UpgradeTVDatabaseSchema(void)
 {
     QString dbver = gContext->GetSetting("DBSchemaVer");
-    
+
     if (dbver == currentDatabaseVersion)
         return;
-
+    
     if (dbver == "")
     {
         InitializeDatabase();        
@@ -369,7 +380,7 @@ void UpgradeTVDatabaseSchema(void)
     if (dbver == "1014")
     {
         const QString updates[] = {
-"ALTER TABLE dvb_sat CHANGE lnb_log_switch lnb_lof_switch INTEGER DEFAULT 11700000;",
+"ALTER TABLE dvb_sat CHANGE lnb_lof_switch lnb_lof_switch INTEGER DEFAULT 11700000;",
 "UPDATE settings SET value='RecPriorityOrder' WHERE value='RecPriorityingOrder';",
 ""
 };
@@ -558,6 +569,354 @@ QString("ALTER TABLE videosource ADD COLUMN freqtable VARCHAR(16) NOT NULL DEFAU
         performActualUpdate(updates, "1027", dbver);
     }
 
+    if (dbver == "1027")
+    {
+        const QString updates[] = {
+"CREATE TABLE IF NOT EXISTS dvb_signal_quality ("
+"    id INT UNSIGNED AUTO_INCREMENT NOT NULL PRIMARY KEY,"
+"    sampletime TIMESTAMP NOT NULL,"
+"    cardid INT UNSIGNED NOT NULL,"
+"    fe_snr INT UNSIGNED NOT NULL,"
+"    fe_ss  INT UNSIGNED NOT NULL,"
+"    fe_ber INT UNSIGNED NOT NULL,"
+"    fe_unc INT UNSIGNED NOT NULL,"
+"    myth_cont INT UNSIGNED NOT NULL,"
+"    myth_over INT UNSIGNED NOT NULL,"
+"    myth_pkts INT UNSIGNED NOT NULL,"
+"    FOREIGN KEY(cardid) REFERENCES capturecard(id),"
+"    INDEX (sampletime,cardid)"
+");",
+"ALTER TABLE capturecard ADD skipbtaudio TINYINT(1) DEFAULT 0;",
+""
+};
+        performActualUpdate(updates, "1028", dbver);
+    }
+
+    if (dbver == "1028") {
+        const QString updates[] = {
+"ALTER TABLE channel ADD COLUMN commfree TINYINT NOT NULL default '0';",
+"ALTER TABLE record ADD COLUMN recgroup VARCHAR(32) default 'Default';",
+"ALTER TABLE record ADD COLUMN dupmethod INT NOT NULL DEFAULT 6;",
+"ALTER TABLE record ADD COLUMN dupin INT NOT NULL DEFAULT 15;",
+"UPDATE record SET dupmethod = 1 WHERE recorddups = 2;",
+"UPDATE record SET dupin = 2 WHERE recorddups = 1;",
+"ALTER TABLE record DROP COLUMN recorddups;",
+"ALTER TABLE recorded ADD COLUMN recgroup VARCHAR(32) default 'Default';",
+"ALTER TABLE recorded ADD COLUMN recordid INT DEFAULT NULL;",
+"CREATE TABLE recgrouppassword ("
+"  recgroup VARCHAR(32) NOT NULL PRIMARY KEY, "
+"  password VARCHAR(10) NOT NULL, "
+"  UNIQUE(recgroup)"
+");",
+""
+};
+        performActualUpdate(updates, "1029", dbver);
+    }
+
+    if (dbver == "1029")
+    {
+        const QString updates[] = {
+"ALTER TABLE record CHANGE preroll startoffset INT DEFAULT 0 NOT NULL;",
+"ALTER TABLE record CHANGE postroll endoffset INT DEFAULT 0 NOT NULL;",
+""
+};
+        performActualUpdate(updates, "1030", dbver);
+    }
+
+    if (dbver == "1030") 
+    {
+        const QString updates[] = {
+"ALTER TABLE channel ADD COLUMN visible TINYINT(1) NOT NULL default '1';",
+"UPDATE channel SET visible = 1;",
+""
+};
+        performActualUpdate(updates, "1031", dbver);
+    }
+
+    if (dbver == "1031") {
+        const QString updates[] = {
+"ALTER TABLE capturecard ADD dvb_on_demand TINYINT NOT NULL DEFAULT 0;",
+""
+};
+        performActualUpdate(updates, "1032", dbver);
+    }
+
+    if (dbver == "1032")
+    {
+        const QString updates[] = {
+"UPDATE record SET dupmethod = 6 WHERE dupmethod = 22;",
+""
+};
+        performActualUpdate(updates, "1033", dbver);
+    }
+
+    if (dbver == "1033")
+    {
+        const QString updates[] = {
+"ALTER TABLE program ADD title_pronounce VARCHAR(128) NULL;",
+"ALTER TABLE program ADD INDEX (title_pronounce);",
+""
+};
+        performActualUpdate(updates, "1034", dbver);
+    }
+
+    if (dbver == "1034")
+    {
+        const QString updates[] = {
+"CREATE TABLE mythlog ("
+"  logid int(10) unsigned PRIMARY KEY NOT NULL auto_increment,"
+"  module char(32) NOT NULL,"
+"  priority int(11) NOT NULL,"
+"  acknowledged bool default 0,"
+"  logdate datetime,"
+"  host varchar(128),"
+"  message varchar(255) NOT NULL,"
+"  details text"
+");",
+"CREATE TABLE housekeeping ("
+"  tag varchar(64) PRIMARY KEY NOT NULL,"
+"  lastrun datetime"
+");",
+""
+};
+        performActualUpdate(updates, "1035", dbver);
+    }
+    
+    if (dbver == "1035")
+    {
+        const QString updates[] = {
+"ALTER TABLE dvb_sat CHANGE pos pos FLOAT;",
+"ALTER TABLE dvb_sat ADD diseqc_pos SMALLINT DEFAULT 0 AFTER diseqc_port;",
+""
+};
+        performActualUpdate(updates,"1036", dbver);
+    }
+
+    if (dbver == "1036")
+    {
+        const QString updates[] = {
+"UPDATE channel SET callsign=chanid WHERE callsign IS NULL OR callsign='';",
+"ALTER TABLE record ADD COLUMN station VARCHAR(20) NOT NULL DEFAULT '';",
+"ALTER TABLE recordoverride ADD COLUMN station VARCHAR(20) NOT NULL DEFAULT '';",
+""
+};
+        performActualUpdate(updates, "1037", dbver);
+    }
+
+    if (dbver == "1037")
+    {
+        const QString updates[] = {
+"ALTER TABLE videosource ADD lineupid VARCHAR(64) NULL;",
+"ALTER TABLE videosource ADD password VARCHAR(64) NULL;",
+"ALTER TABLE program ADD ( "
+"    stereo bool, "
+"    subtitled bool, "
+"    hdtv bool, "
+"    closecaptioned bool, "
+"    partnumber int, "
+"    parttotal int, "
+"    seriesid char(12), "
+"    originalairdate date, "
+"    showtype varchar(30), "
+"    colorcode varchar(20), "
+"    syndicatedepisodenumber varchar(20), "
+"    programid char(12) "
+");",
+"DELETE FROM credits;",
+"ALTER TABLE credits DROP INDEX chanid;",
+"ALTER TABLE credits ADD UNIQUE chanid (chanid, starttime, person, role);",
+"DELETE FROM people;",
+"ALTER TABLE people DROP INDEX name;",
+"ALTER TABLE people ADD UNIQUE name (name(41));",
+"CREATE TABLE programgenres ( "
+"    chanid int unsigned NOT NULL, "
+"    starttime timestamp NOT NULL, "
+"    relevance char(1) NOT NULL, "
+"    genre char(30), "
+"    PRIMARY KEY (chanid, starttime, relevance) "
+");",
+""
+};
+        performActualUpdate(updates, "1038", dbver);
+    }
+
+    if (dbver == "1038")
+    {
+        const QString updates[] = {
+"CREATE TABLE IF NOT EXISTS programgenres ( "
+"    chanid int unsigned NOT NULL, "
+"    starttime timestamp NOT NULL, "
+"    relevance char(1) NOT NULL, "
+"    genre char(30), "
+"    PRIMARY KEY (chanid, starttime, relevance) "
+");",
+""
+};
+        performActualUpdate(updates, "1039", dbver);
+    }
+
+    if (dbver == "1039")
+    {
+        const QString updates[] = {
+"ALTER TABLE channel CHANGE name name VARCHAR(64);",
+""
+};
+        performActualUpdate(updates, "1040", dbver);
+    }
+
+    if (dbver == "1040")
+    {
+        const QString updates[] = {
+"ALTER TABLE channel ADD outputfilters VARCHAR(255) NULL",
+""
+};
+        performActualUpdate(updates, "1041", dbver);
+    }
+
+    if (dbver == "1041")
+    {
+        const QString updates[] = {
+"ALTER TABLE record ADD seriesid varchar(12) NULL;",
+"ALTER TABLE record ADD programid varchar(12) NULL;",
+"ALTER TABLE recorded ADD seriesid varchar(12) NULL;",
+"ALTER TABLE recorded ADD programid varchar(12) NULL;",
+"ALTER TABLE oldrecorded ADD seriesid varchar(12) NULL;",
+"ALTER TABLE oldrecorded ADD programid varchar(12) NULL;",
+"ALTER TABLE program ADD INDEX (seriesid);",
+"ALTER TABLE program ADD INDEX (programid);",
+"ALTER TABLE record ADD INDEX (seriesid);",
+"ALTER TABLE record ADD INDEX (programid);",
+"ALTER TABLE recorded ADD INDEX (seriesid);",
+"ALTER TABLE recorded ADD INDEX (programid);",
+"ALTER TABLE oldrecorded ADD INDEX (seriesid);",
+"ALTER TABLE oldrecorded ADD INDEX (programid);",
+""
+};
+        performActualUpdate(updates, "1042", dbver);
+    }
+
+    if (dbver == "1042")
+    {
+        const QString updates[] = {
+"INSERT INTO settings SET value=\"DataDirectMessage\";",
+""
+};
+        performActualUpdate(updates, "1043", dbver);
+    }
+
+    if (dbver == "1043")
+    {
+        const QString updates[] = {
+"ALTER TABLE program CHANGE title title VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE subtitle subtitle VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE description description TEXT NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE category category VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE category_type category_type VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE title_pronounce title_pronounce VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE seriesid seriesid VARCHAR(12) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE showtype showtype VARCHAR(30) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE colorcode colorcode VARCHAR(20) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE syndicatedepisodenumber syndicatedepisodenumber VARCHAR(20) NOT NULL DEFAULT '';",
+"ALTER TABLE program CHANGE programid programid VARCHAR(12) NOT NULL DEFAULT '';",
+
+"ALTER TABLE channel CHANGE channum channum VARCHAR(5) NOT NULL DEFAULT '';",
+"ALTER TABLE channel CHANGE callsign callsign VARCHAR(20) NOT NULL DEFAULT '';",
+"ALTER TABLE channel CHANGE name name VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE channel CHANGE icon icon VARCHAR(255) NOT NULL DEFAULT 'none';",
+"ALTER TABLE channel CHANGE videofilters videofilters VARCHAR(255) NOT NULL DEFAULT '';",
+"ALTER TABLE channel CHANGE xmltvid xmltvid VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE channel CHANGE outputfilters outputfilters VARCHAR(255) NOT NULL DEFAULT '';",
+
+"ALTER TABLE record CHANGE title title VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE subtitle subtitle VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE description description TEXT NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE profile profile VARCHAR(128) NOT NULL DEFAULT 'Default';",
+"ALTER TABLE record CHANGE category category VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE recgroup recgroup VARCHAR(32) NOT NULL DEFAULT 'Default';",
+"ALTER TABLE record CHANGE station station VARCHAR(20) NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE seriesid seriesid VARCHAR(12) NOT NULL DEFAULT '';",
+"ALTER TABLE record CHANGE programid programid VARCHAR(12) NOT NULL DEFAULT '';",
+
+"ALTER TABLE recorded CHANGE title title VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE subtitle subtitle VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE description description TEXT NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE hostname hostname VARCHAR(255) NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE category category VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE recgroup recgroup VARCHAR(32) NOT NULL DEFAULT 'Default';",
+"ALTER TABLE recorded CHANGE seriesid seriesid VARCHAR(12) NOT NULL DEFAULT '';",
+"ALTER TABLE recorded CHANGE programid programid VARCHAR(12) NOT NULL DEFAULT '';",
+
+"ALTER TABLE oldrecorded CHANGE title title VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE oldrecorded CHANGE subtitle subtitle VARCHAR(128) NOT NULL DEFAULT '';",
+"ALTER TABLE oldrecorded CHANGE description description TEXT NOT NULL DEFAULT '';",
+"ALTER TABLE oldrecorded CHANGE category category VARCHAR(64) NOT NULL DEFAULT '';",
+"ALTER TABLE oldrecorded CHANGE seriesid seriesid VARCHAR(12) NOT NULL DEFAULT '';",
+"ALTER TABLE oldrecorded CHANGE programid programid VARCHAR(12) NOT NULL DEFAULT '';",
+""
+};
+        performActualUpdate(updates, "1044", dbver);
+    }
+
+    if (dbver == "1044")
+    {
+        const QString updates[] = {
+"UPDATE channel SET icon = 'none' WHERE icon = '';",
+"UPDATE record SET profile = 'Default' WHERE profile = '';",
+"UPDATE record SET recgroup = 'Default' WHERE recgroup = '';",
+"UPDATE recorded SET recgroup = 'Default', starttime = starttime WHERE recgroup = '';",
+""
+};
+        performActualUpdate(updates, "1045", dbver);
+    }
+
+    if (dbver == "1045")
+    {
+        const QString updates[] = {
+"UPDATE recorded SET recgroup = 'Default', starttime = starttime WHERE recgroup = '';",
+""
+};
+        performActualUpdate(updates, "1046", dbver);
+    }
+
+    if (dbver == "1046")
+    {
+        const QString updates[] = {
+"ALTER TABLE record ADD COLUMN search INT UNSIGNED NOT NULL DEFAULT 0;",
+"UPDATE record SET search = 0 WHERE search IS NULL;",
+""
+};
+        performActualUpdate(updates, "1047", dbver);
+    }
+
+    if (dbver == "1047")
+    {
+        const QString updates[] = {
+"CREATE TABLE networkiconmap ("
+"    id INTEGER NOT NULL AUTO_INCREMENT,"
+"    network VARCHAR(20) NOT NULL UNIQUE,"
+"    url VARCHAR(255) NOT NULL,"
+"    PRIMARY KEY(id)"
+");",
+"CREATE TABLE callsignnetworkmap ("
+"    id INTEGER NOT NULL AUTO_INCREMENT,"
+"    callsign VARCHAR(20) NOT NULL UNIQUE,"
+"    network VARCHAR(20) NOT NULL,"
+"    PRIMARY KEY(id)"
+");",
+""
+};
+        performActualUpdate(updates, "1048", dbver);
+    }
+
+    if (dbver == "1048")
+    {
+        const QString updates[] = {
+"ALTER TABLE cardinput CHANGE preference preference INT NOT NULL DEFAULT 0;",
+"UPDATE cardinput SET preference = 0 WHERE preference IS NULL;",
+""
+};
+        performActualUpdate(updates, "1049", dbver);
+    }
 }
 
 void InitializeDatabase(void)
