@@ -59,6 +59,7 @@ DVBChannel::DVBChannel(int aCardNum, TVRec *parent)
            : ChannelBase(parent), cardnum(aCardNum)
 {
     fd_frontend = 0;
+    force_channel_change = false;
     first_tune = true;
     isOpen = false;
     diseqc = NULL;
@@ -78,8 +79,19 @@ DVBChannel::~DVBChannel()
     if (dvbcam)
         delete dvbcam;
 
-    if (isOpen && (fd_frontend > 0))
+    CloseDVB();
+}
+
+void DVBChannel::CloseDVB()
+{
+    VERBOSE(VB_ALL, "Closing DVB channel");
+
+    if (fd_frontend > 0)
+    {
         close(fd_frontend);
+        fd_frontend = 0;
+        isOpen = false;
+    }
 }
 
 bool DVBChannel::Open()
@@ -132,17 +144,21 @@ bool DVBChannel::Open()
                 dvbcam, SLOT(ChannelChanged(dvb_channel_t&, uint8_t*, int)));
     }
 
+    force_channel_change = true;
+
     return isOpen = true;
 }
 
 bool DVBChannel::SetChannelByString(const QString &chan)
 {
-    if (curchannelname == chan)
+    if (curchannelname == chan && !force_channel_change)
         return true;
 
     if (!isOpen)
         return false;
 
+    force_channel_change = false;
+
     CHANNEL(QString("Trying to tune to channel %1.").arg(chan));
 
     if (GetChannelOptions(chan) == false)
@@ -189,10 +205,11 @@ bool DVBChannel::GetChannelOptions(QString channum)
     QString         thequery;
     QSqlQuery       query;
     QSqlDatabase    *db_conn;
-    pthread_mutex_t db_lock;
+    pthread_mutex_t *db_lock;
+    QString         inputName;
 
     if (!pParent->CheckChannel((ChannelBase*)this, channum,
-                               db_conn, db_lock))
+                               db_conn, db_lock, inputName))
     {
         ERROR("Failed to verify channel integrity.");
         return false;
@@ -200,11 +217,17 @@ bool DVBChannel::GetChannelOptions(QString channum)
 
     dvbcam->SetDatabase(db_conn, db_lock);
 
-    pthread_mutex_lock(&db_lock);
+    pthread_mutex_lock(db_lock);
     MythContext::KickDatabase(db_conn);
 
-    thequery = QString("SELECT chanid FROM channel WHERE channum='%1'")
-                       .arg(channum);
+    thequery = QString("SELECT channel.chanid "
+                       "FROM channel,cardinput,capturecard WHERE "
+                       "channel.channum='%1' AND "
+                       "cardinput.sourceid = channel.sourceid AND "
+                       "cardinput.cardid = '%2' AND "
+                       "capturecard.cardid = cardinput.cardid AND "
+                       "capturecard.cardtype = 'DVB'")
+                       .arg(channum).arg(pParent->GetCaptureCardNum());
     query = db_conn->exec(thequery);
 
     if (!query.isActive())
@@ -213,7 +236,7 @@ bool DVBChannel::GetChannelOptions(QString channum)
     if (query.numRowsAffected() <= 0)
     {
         ERROR("Unable to find channel in database.");
-        pthread_mutex_unlock(&db_lock);
+        pthread_mutex_unlock(db_lock);
         return false;
     }
 
@@ -251,7 +274,7 @@ bool DVBChannel::GetChannelOptions(QString channum)
 
     if (query.numRowsAffected() <= 0)
     {
-        thequery += QString("SELECT satid FROM dvb_channel WHERE chanid='%1'")
+        thequery = QString("SELECT satid FROM dvb_channel WHERE chanid='%1'")
                     .arg(chanid);
         query = db_conn->exec(thequery);
 
@@ -266,7 +289,7 @@ bool DVBChannel::GetChannelOptions(QString channum)
         else if (info.type == FE_QPSK)
         {
             int satid = query.value(0).toInt();
-            thequery += QString("SELECT * FROM dvb_sat WHERE satid='%1'")
+            thequery = QString("SELECT * FROM dvb_sat WHERE satid='%1'")
                         .arg(satid);
             query = db_conn->exec(thequery);
 
@@ -279,11 +302,11 @@ bool DVBChannel::GetChannelOptions(QString channum)
                               .arg(channum).arg(satid));
         }
 
-        pthread_mutex_unlock(&db_lock);
+        pthread_mutex_unlock(db_lock);
         return false;
     }
 
-    pthread_mutex_unlock(&db_lock);
+    pthread_mutex_unlock(db_lock);
 
     query.next();
     if (!ParseQuery(query))
@@ -295,7 +318,8 @@ bool DVBChannel::GetChannelOptions(QString channum)
     return true;
 }
 
-bool DVBChannel::GetChannelPids(QSqlDatabase*& db_conn, pthread_mutex_t& db_lock,
+bool DVBChannel::GetChannelPids(QSqlDatabase*& db_conn, 
+                                pthread_mutex_t*& db_lock,
                                 int chanid)
 {
     dvb_pids_t& pids = chan_opts.pids;
@@ -308,7 +332,7 @@ bool DVBChannel::GetChannelPids(QSqlDatabase*& db_conn, pthread_mutex_t& db_lock
     pids.other.clear();
 
     // TODO: Also fetch & store language field.
-    pthread_mutex_lock(&db_lock);
+    pthread_mutex_lock(db_lock);
     MythContext::KickDatabase(db_conn);
     QString thequery = QString("SELECT pid, type FROM dvb_pids WHERE chanid=%1")
                                .arg(chanid);
@@ -320,7 +344,7 @@ bool DVBChannel::GetChannelPids(QSqlDatabase*& db_conn, pthread_mutex_t& db_lock
     if (query.numRowsAffected() <= 0)
     {
         ERROR(QString("Unable to find any pids for channel id %1.").arg(chanid));
-        pthread_mutex_unlock(&db_lock);
+        pthread_mutex_unlock(db_lock);
         return false;
     }
 
@@ -342,7 +366,7 @@ bool DVBChannel::GetChannelPids(QSqlDatabase*& db_conn, pthread_mutex_t& db_lock
         }
     }
 
-    pthread_mutex_unlock(&db_lock);
+    pthread_mutex_unlock(db_lock);
     return true;
 }
 
@@ -567,7 +591,7 @@ void DVBChannel::CheckOptions()
        (symbol_rate < info.symbol_rate_min ||
         symbol_rate > info.symbol_rate_max))
         WARNING(QString("Symbol Rate setting (%1) is "
-                        "out of range (min/max:%1/%1)")
+                        "out of range (min/max:%2/%3)")
                         .arg(symbol_rate)
                         .arg(info.symbol_rate_min)
                         .arg(info.symbol_rate_max));
@@ -680,8 +704,14 @@ void DVBChannel::connectNotify(const char* signal)
     {
         monitorClients++;
         if (!monitorRunning)
-            pthread_create(&statusMonitorThread, NULL,
+        {
+            pthread_attr_t attr;
+            pthread_attr_init(&attr);
+            pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+            pthread_create(&statusMonitorThread, &attr,
                            StatusMonitorHelper, this);
+        }
     }
 }
 
@@ -1038,7 +1068,13 @@ bool DVBChannel::Tune(dvb_channel_t& channel, bool all)
         polls.events = POLLIN;
         polls.revents = 0;
 
-        poll_return = poll(&polls, 1, 150);
+        for (;;)
+        {
+            poll_return = poll(&polls, 1, 150);
+            if (poll_return == -1 && (errno == EAGAIN || errno == EINTR))
+                continue;
+            break;
+        }
 
         if (poll_return > 0)
         {
