@@ -21,6 +21,8 @@
  *      Dave Chapman (dave at dchapman.com)
  *          - The dvbstream library, which some code,
  *            in ::StartRecording, is based upon.
+ *      Martin Smith (martin at spamcop.net)
+ *          - The signal quality monitor
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -63,11 +65,20 @@ DVBRecorder::DVBRecorder(DVBChannel* advbchannel): RecorderBase()
     was_paused = true;
     channel_changed = true;
     dvbchannel = advbchannel;
+    dvb_on_demand = false;
 
     paused = false;
     mainpaused = false;
     recording = false;
 
+    cont_errors = 0;
+    stream_overflows = 0;
+    bad_packets = 0;
+
+    signal_monitor_interval = 0;
+    expire_data_days = 3;
+    signal_monitor_quit = false;
+
     wait_for_seqstart = true;
     wait_for_seqstart_enabled = true;
     dmx_buf_size = DEF_DMX_BUF_SIZE;
@@ -112,6 +123,24 @@ void DVBRecorder::SetOption(const QString &name, int value)
             pktbuffer = (unsigned char*)realloc(pktbuffer, pkt_buf_size);
         }
     }
+    else if (name == "signal_monitor_interval")
+    {
+        signal_monitor_interval = value;
+
+        if (signal_monitor_interval < 0)
+            signal_monitor_interval = 0; 
+    }
+    else if (name == "expire_data_days")
+    {
+        expire_data_days = value;
+
+        if (expire_data_days < 1)
+            expire_data_days = 1;
+    }
+    else if (name == "dvb_on_demand")
+    {
+        dvb_on_demand = value;
+    }
     else
         RecorderBase::SetOption(name, value);
 }
@@ -150,6 +179,12 @@ bool DVBRecorder::Open()
     if (isopen)
         return true;
 
+    if (dvb_on_demand && dvbchannel->Open())
+    {
+        // this is required to trigger a re-tune
+        dvbchannel->SetChannelByString(dvbchannel->GetCurrentName());
+    }
+
     fd_dvr = open(dvbdevice(DVB_DEV_DVR,cardnum), O_RDONLY | O_NONBLOCK);
     if(fd_dvr < 0)
     {
@@ -173,11 +208,16 @@ void DVBRecorder::Close()
     if (!isopen)
         return;
 
+    VERBOSE(VB_ALL, "Closing DVB recorder");
+
     CloseFilters();
 
     if (fd_dvr > 0)
         close(fd_dvr);
 
+    if (dvb_on_demand && dvbchannel)
+        dvbchannel->CloseDVB();
+
     isopen = false;
 }
 
@@ -211,7 +251,7 @@ void DVBRecorder::OpenFilters(dvb_pid_t& pids, dmx_pes_type_t type)
     {
         int this_pid = pids[i];
 
-        RECORD(QString("Adding pid %1, type %1").arg(this_pid).arg((int)type));
+        RECORD(QString("Adding pid %1, type %2").arg(this_pid).arg((int)type));
 
         if (this_pid < 0x10 || this_pid > 0x1fff)
             WARNING(QString("PID value (%1) is outside dvb specification.")
@@ -310,7 +350,10 @@ void DVBRecorder::SetDemuxFilters(dvb_pids_t& pids)
     OpenFilters(pids.other,       DMX_PES_OTHER);
 
     if (fd_demux.size() == 0 && pid_ipack.size() == 0)
+    {
         ERROR("No PIDS set, please correct your channel setup.");
+        return;
+    }
 
     pid_ipack[pids.audio[0]]->pv = (struct ipack_s *)pid_ipack[pids.video[0]];
     pid_ipack[pids.video[0]]->pa = (struct ipack_s *)pid_ipack[pids.audio[0]];
@@ -336,10 +379,32 @@ void DVBRecorder::StartRecording()
     receiving = false;
     struct pollfd polls;
 
+    pthread_t qualthread;
+    bool qthreadexists = false;
+    signal_monitor_quit = false;
+
     polls.fd = fd_dvr;
     polls.events = POLLIN;
     polls.revents = 0;
 
+    cont_errors = 0;
+    stream_overflows = 0;
+    bad_packets = 0;
+
+    // start the thread that logs signal data to the db
+
+    pthread_attr_t attr;
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+    if (signal_monitor_interval > 0 &&
+        pthread_create(&qualthread, &attr, QualityMonitorHelper, this) == 0)
+    {
+        qthreadexists = true;
+        RECORD("DVB Quality monitor is starting at " << 
+               signal_monitor_interval << "s for card " << cardnum);
+    }
+
     encoding = true;
     recording = true;
 
@@ -371,7 +436,13 @@ void DVBRecorder::StartRecording()
             emit Unpaused();
         }
 
-        ret = poll(&polls, 1, 1000);
+        for (;;)
+        {
+            ret = poll(&polls, 1, 1000);
+            if (ret == -1 && (errno == EAGAIN || errno == EINTR))
+                continue;
+            break;
+        }
 
         if (ret == 0 && --dataflow < 1)
             WARNING("No data from card in 1 second.");
@@ -393,6 +464,15 @@ void DVBRecorder::StartRecording()
 
     FinishRecording();
 
+    // stop collecting data if the thread was started successfully
+
+    if (qthreadexists)
+    {
+        signal_monitor_quit = true;
+    }
+
+    ExpireQualityData();
+
     recording = false;
 
     emit Stopped();
@@ -409,6 +489,13 @@ void DVBRecorder::ReadFromDMX()
 
         if (readsz < 0)
         {
+            if (errno == EOVERFLOW)
+            {
+                ++stream_overflows;
+                RECORD("DVB Buffer overflow error detected on read");
+                break;
+            }
+
             if (errno == EAGAIN)
                 break;
             ERRNO("Error reading from DVB device.");
@@ -437,6 +524,7 @@ void DVBRecorder::ReadFromDMX()
             if (pktbuf[1] & 0x80)
             {
                 WARNING("Uncorrectable error in packet, dropped.");
+                ++bad_packets;
                 continue;
             }
 
@@ -465,6 +553,7 @@ void DVBRecorder::ReadFromDMX()
                     RECORD(QString("PID %1 contcounter %2 cc %3")
                            .arg(pid).arg(contcounter[pid]).arg(cc));
                     contcounter[pid] = cc;
+                    ++cont_errors;
                 }            
             }
 
@@ -672,3 +761,107 @@ void DVBRecorder::GetBlankFrameMap(QMap<long long, int> &blank_frame_map)
 {
     (void)blank_frame_map;
 }
+
+// we need the capture card id and I can't see an easy way to get it
+// from this object
+
+int DVBRecorder::GetIDForCardNumber(int cardnum)
+{
+    pthread_mutex_lock(db_lock);
+    int cardid = -1;
+
+    QSqlQuery result = db_conn->exec(QString("SELECT cardid FROM capturecard "
+                                        "WHERE videodevice=\"%1\" AND "
+                                        " cardtype='DVB';").arg(cardnum));
+
+    if (result.isActive() && result.numRowsAffected() > 0)
+    {
+        result.next();
+
+        cardid = result.value(0).toInt();
+    }
+    else
+        RECORD("Could not get cardid for card number " << cardnum);
+
+    pthread_mutex_unlock(db_lock);
+
+    return cardid;
+}
+
+void DVBRecorder::QualityMonitorSample(int cardid,
+                                       dvb_stats_t& sample)
+{
+    QString sql = QString("INSERT INTO dvb_signal_quality("
+                          "sampletime, cardid, fe_snr, fe_ss, fe_ber, "
+                          "fe_unc, myth_cont, myth_over, myth_pkts) "
+                          "VALUES(NOW(), "
+                          "\"%1\",\"%2\",\"%3\",\"%4\","
+                          "\"%5\",\"%6\",\"%7\",\"%8\");")
+        .arg(cardid)
+        .arg(sample.snr & 0xffff)
+        .arg(sample.ss & 0xffff).arg(sample.ber).arg(sample.ub)
+        .arg(cont_errors).arg(stream_overflows).arg(bad_packets);
+
+    QSqlQuery result = db_conn->exec(sql);
+
+    if (!result.isActive())
+        MythContext::DBError("DVB quality sample insert failed", result);
+}
+
+void *DVBRecorder::QualityMonitorThread()
+{
+    dvb_stats_t fe_stats;
+
+    int cardid = GetIDForCardNumber(cardnum);
+
+    // loop until cancelled, wake at intervals and log data
+
+    while (!signal_monitor_quit)
+    {
+        sleep(signal_monitor_interval);
+
+        if (signal_monitor_quit)
+            break;
+
+        if (cardid >= 0 &&
+            db_conn != NULL && db_lock != NULL && dvbchannel != NULL &&
+            dvbchannel -> FillFrontendStats(fe_stats))
+        {
+            pthread_mutex_lock(db_lock);
+
+            QualityMonitorSample(cardid, fe_stats);
+
+            pthread_mutex_unlock(db_lock);
+        }
+    }
+
+    RECORD("DVB Quality monitor thread stopped for card " << cardnum);
+    return NULL;
+}
+
+void DVBRecorder::ExpireQualityData()
+{
+    RECORD("Expiring DVB quality data older than " << expire_data_days <<
+           " day(s)");
+
+    pthread_mutex_lock(db_lock);
+
+    QString sql = QString("DELETE FROM dvb_signal_quality "
+                          "WHERE sampletime < "
+                          "SUBDATE(NOW(), INTERVAL \"%1\" DAY);").
+        arg(expire_data_days);
+
+    QSqlQuery query = db_conn->exec(sql);
+
+    if (!query.isActive())
+        MythContext::DBError("Could not expire DVB signal data",
+                             query);
+
+    pthread_mutex_unlock(db_lock);
+}
+
+void *DVBRecorder::QualityMonitorHelper(void *self)
+{
+    return ((DVBRecorder *)self)->QualityMonitorThread();
+}
+
