@@ -81,6 +81,12 @@ private:
     void SetDemuxFilters(dvb_pids_t& pids);
     void CorrectStreamNumber(ipack* ip, int pid);
 
+    static void* QualityMonitorHelper(void*);
+    int GetIDForCardNumber(int cardnum);
+    void* QualityMonitorThread();
+    void QualityMonitorSample(int cardnum, dvb_stats_t& sample);
+    void ExpireQualityData();
+
     bool recording;
     bool encoding;
 
@@ -95,9 +101,12 @@ private:
     bool wait_for_seqstart;
     bool wait_for_seqstart_enabled;
 
+    bool signal_monitor_quit;
+
     QMap<long long, long long> positionMap;
     long long prev_gop_save_pos;
 
+    bool    dvb_on_demand;
     bool    isopen;
     int     cardnum;
     bool    swfilter;
@@ -110,6 +119,13 @@ private:
     int     pkt_buf_size;
     unsigned char* pktbuffer;
 
+    unsigned int cont_errors;
+    unsigned int stream_overflows;
+    unsigned int bad_packets;
+
+    int     signal_monitor_interval;
+    int     expire_data_days;
+
     int fd_dvr;
     vector<int> fd_demux;
     pid_ipack_t pid_ipack;
