@@ -117,7 +117,12 @@ void DVBSections::DelPid(int pid)
 void DVBSections::Start()
 {
     AddPid(0);
-    pthread_create(&thread, NULL, ThreadHelper, this);
+
+    pthread_attr_t attr;
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+    pthread_create(&thread, &attr, ThreadHelper, this);
 }
 
 void DVBSections::Stop()
@@ -196,8 +201,11 @@ void DVBSections::ThreadLoop()
 
         if (ret < 0)
         {
-            ERRNO("Poll failed while waiting for Section");
             pthread_mutex_unlock(&poll_lock);
+            if (errno == EAGAIN || errno == EINTR)
+                continue;
+
+            ERRNO("Poll failed while waiting for Section");
             continue;
         }
 
@@ -205,7 +213,6 @@ void DVBSections::ThreadLoop()
         {
             // FIXME: Handle POLLERR
             if (! (pollArray[i].revents & POLLIN || pollArray[i].revents & POLLPRI) )
-            // FIXME: does this jump to for or while? it should go to for
                 continue;
 
             int rsz = read(pollArray[i].fd, &buffer, MAX_SECTION_SIZE);
@@ -220,7 +227,7 @@ void DVBSections::ThreadLoop()
                 head.section_number = buffer[6];
                 head.section_last   = buffer[7];
 
-                ParseTable(&head, &buffer[8], rsz - sizeof(tablehead_t));
+                ParseTable(&head, &buffer[8], rsz - 8);
 
                 continue;
             }
@@ -312,9 +319,9 @@ void DVBSections::ParsePMT(tablehead_t* head, uint8_t* buffer, int size)
     if (curpmtsize == 0 && head->table_id_ext == curprogram)
     {
         pthread_mutex_lock(&pmap_lock);
-        curpmtbuf = (uint8_t*)realloc(curpmtbuf, size - 4);
-        memcpy(curpmtbuf, &buffer[4], size - 4);
-        curpmtsize = size - 4;
+        curpmtbuf = (uint8_t*)realloc(curpmtbuf, size);
+        memcpy(curpmtbuf, buffer, size);
+        curpmtsize = size;
         pthread_mutex_unlock(&pmap_lock);
 
         ChannelChanged(chan_opts, curpmtbuf, curpmtsize);
