@@ -11,7 +11,8 @@ typedef enum FrameType_
     FMT_VIA_HWSLICE,
     FMT_IA44,
     FMT_AI44,
-    FMT_ARGB32
+    FMT_ARGB32,
+    FMT_YUV422P
 } VideoFrameType;
 
 typedef struct VideoFrame_
@@ -28,6 +29,9 @@ typedef struct VideoFrame_
     long long timecode;
 
     unsigned char *priv[4]; // random empty storage
+
+    unsigned char *qscale_table;
+    int            qstride;
 } VideoFrame;
 
 #endif
