@@ -43,6 +43,16 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player,
     GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(), startchannel, 
                                   player, allowsecondaryepg, "guidegrid");
 
+    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
+    if (switchMode) 
+    {
+        // xrandr to gui size
+        char buf[128];
+        int screenwidth = gContext->GetNumSetting("GuiVidModeWidth", 0);
+        int screenheight = gContext->GetNumSetting("GuiVidModeHeight", 0);
+        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
+        system(buf);
+    }
     gg->Show();
 
     if (thread)
@@ -65,9 +75,6 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player,
     if (thread)
         qApp->unlock();
 
-    if (chanstr == QString::null)
-        chanstr = "";
-
     return chanstr;
 }
 
@@ -123,6 +130,9 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
 
     scrolltype = gContext->GetNumSetting("EPGScrollType", 1);
 
+    selectChangesChannel = gContext->GetNumSetting("SelectChangesChannel", 0);
+    selectRecThreshold = gContext->GetNumSetting("SelChangeRecThreshold", 16);
+    
     LayerSet *container = NULL;
     container = theme->GetSet("guide");
     if (container)
@@ -163,11 +173,12 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     }
 
     channelOrdering = gContext->GetSetting("ChannelOrdering", "channum + 0");
-    dateformat = gContext->GetSetting("DateFormat", "ddd MMMM d");
+    dateformat = gContext->GetSetting("ShortDateFormat", "ddd d");
     unknownTitle = gContext->GetSetting("UnknownTitle", "Unknown");
     unknownCategory = gContext->GetSetting("UnknownCategory", "Unknown");
     currentTimeColor = gContext->GetSetting("EPGCurrentTimeColor", "red");
-    displaychannum = gContext->GetNumSetting("DisplayChanNum");
+    channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
+    channelFormat.replace(" ", "\n");
 
     UIBarType *type = NULL;
     container = theme->GetSet("chanbar");
@@ -295,7 +306,8 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
         return; //check for Escape in case something goes wrong 
                 //with KeyRelease events, shouldn't happen.
 
-    if (e->key() != Key_Control)
+    if (e->key() != Key_Control && e->key() != Key_Shift && 
+        e->key() != Key_Meta && e->key() != Key_Alt)
         keyDown = true;    
 
     if (e->state() == Qt::ControlButton)
@@ -350,11 +362,29 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
             else if (action == "6")
                 showProgFinder();
             else if (action == "MENU")
-                enter();
+                enter();                    
             else if (action == "ESCAPE")
                 escape();
             else if (action == "SELECT")
-                editRecording();
+            {                         
+                if(m_player && selectChangesChannel)
+                {
+                    // See if this show is far enough into the future that it's probable
+                    // that the user wanted to schedule it to record instead of changing the channel.
+                    ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
+                    if (pginfo && (pginfo->title != unknownTitle) &&
+                        ((pginfo->SecsTillStart() / 60) >= selectRecThreshold))
+                    {
+                        editRecording();
+                    }
+                    else
+                    {
+                        enter();
+                    }
+                }
+                else
+                    editRecording();
+            }
             else if (action == "INFO")
                 editScheduled();
             else if (action == "TOGGLERECORD")
@@ -489,18 +519,19 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
     QSqlQuery query;
 
     QString queryall = "SELECT channel.channum, channel.callsign, "
-                       "channel.icon, channel.chanid, favorites.favid "
-                       "FROM channel LEFT JOIN favorites ON "
-                       "favorites.chanid = channel.chanid ORDER BY " +
-                       channelOrdering + ";";
+                       "channel.icon, channel.chanid, favorites.favid, "
+                       "channel.name FROM channel LEFT JOIN favorites ON "
+                       "favorites.chanid = channel.chanid WHERE visible = 1 "
+                       "GROUP BY channum, callsign "
+                       "ORDER BY " + channelOrdering + ";";
 
     if (showFavorites)
     {
         queryfav = "SELECT channel.channum, channel.callsign, "
-                   "channel.icon, channel.chanid, favorites.favid "
-                   "FROM favorites, channel WHERE "
-                   "channel.chanid = favorites.chanid ORDER BY " + 
-                   channelOrdering + ";";
+                   "channel.icon, channel.chanid, favorites.favid, "
+                   "channel.name FROM favorites, channel WHERE "
+                   "channel.chanid = favorites.chanid and visible = 1 "
+                   "ORDER BY " + channelOrdering + ";";
 
         query.exec(queryfav);   
 
@@ -525,21 +556,14 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
         {
             ChannelInfo val;
             val.chanstr = query.value(0).toString();
-            if ((val.chanstr != QString::null) && (val.chanstr != ""))
+            if (val.chanstr != "")
             {
                 val.callsign = query.value(1).toString();
-                if (val.callsign == QString::null)
-                    val.callsign = "";
                 val.iconpath = query.value(2).toString();
-                if (val.iconpath == QString::null)
-                    val.iconpath = "none";
-                if (val.iconpath.stripWhiteSpace().length() == 0)
-                    val.iconpath = "none";
                 val.chanstr = query.value(0).toString();
-                if (val.chanstr == QString::null)
-                val.chanstr = "";
                 val.chanid = query.value(3).toInt();
                 val.favid = query.value(4).toInt();
+                val.channame = query.value(5).toString();
                 val.icon = NULL;
         
                 if (gotostartchannel && val.chanstr == m_startChanStr && !set)
@@ -835,6 +859,13 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
                 case kWeekslotRecord:
                     recFlag = 5;
                     break;
+                case kFindOneRecord:
+                    recFlag = 6;
+                    break;
+                case kOverrideRecord:
+                case kDontRecord:
+                    recFlag = 7;
+                    break;
                 case kNotRecording:
                 default:
                     recFlag = 0;
@@ -842,12 +873,12 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
                 }
 
                 int recStat;
-                if (!proginfo->recording)
-                    recStat = 0;
-                else if (!proginfo->conflicting)
+                if (proginfo->recstatus == rsConflict)
+                    recStat = 2;
+                if (proginfo->recstatus <= rsWillRecord)
                     recStat = 1;
                 else
-                    recStat = 2;
+                    recStat = 0;
 
                 type->SetProgramInfo(row, cnt, tempRect, proginfo->title,
                                      proginfo->category, arrow, recFlag, 
@@ -913,7 +944,7 @@ void GuideGrid::paintDate(QPainter *p)
     {
         UITextType *type = (UITextType *)container->GetType("date");
         if (type)
-            type->SetText(m_currentStartTime.toString("ddd"));
+            type->SetText(m_currentStartTime.toString(dateformat));
     }
 
     if (container)
@@ -999,13 +1030,12 @@ void GuideGrid::paintChannels(QPainter *p)
             }
         }
 
-        QString favstr = "";
+        QString tmpChannelFormat = channelFormat;
         if (chinfo->favid > 0)
-            favstr = "*";
-
-        QString write = "";
-        QString callsignstr = favstr + " " + chinfo->callsign + " " +
-                              favstr;
+        {
+            tmpChannelFormat.insert(tmpChannelFormat.find('<'), "* ");
+            tmpChannelFormat.insert(tmpChannelFormat.find('>') + 1, " *");
+        }
 
         if (type)
         {
@@ -1023,11 +1053,7 @@ void GuideGrid::paintChannels(QPainter *p)
                 type->ResetImage(y);
             }
 
-            if (!displaychannum)
-                write = chinfo->chanstr + "\n";
-        
-            write = write + callsignstr;
-            type->SetText(y, write);
+            type->SetText(y, chinfo->Text(tmpChannelFormat));
         }
     }
 
@@ -1488,7 +1514,7 @@ void GuideGrid::pageUp()
  
 void GuideGrid::showProgFinder()
 {
-    RunProgramFind();
+    RunProgramFind(false, true);
 
     setActiveWindow();
     setFocus();
@@ -1509,6 +1535,16 @@ void GuideGrid::enter()
 
     unsetCursor();
     selectState = 1;
+    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
+    if (m_player && switchMode) 
+    {
+        // xrandr back to video size
+        char buf[128];
+        int screenwidth = gContext->GetNumSetting("TVVidModeWidth", 0);
+        int screenheight = gContext->GetNumSetting("TVVidModeHeight", 0);
+        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
+        system(buf);
+    }
     accept();
 }
 
@@ -1522,6 +1558,16 @@ void GuideGrid::escape()
     }
 
     unsetCursor();
+    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
+    if (m_player && switchMode) 
+    {
+        // xrandr back to video size
+        char buf[128];
+        int screenwidth = gContext->GetNumSetting("TVVidModeWidth", 0);
+        int screenheight = gContext->GetNumSetting("TVVidModeHeight", 0);
+        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
+        system(buf);
+    }
     accept();
 }
 
