@@ -100,6 +100,9 @@ class GuideGrid : public MythDialog
 
     int m_context;
 
+    bool selectChangesChannel;
+    int selectRecThreshold;
+    
     int gridfilltype;
     int scrolltype;
 
@@ -141,7 +144,7 @@ class GuideGrid : public MythDialog
 
     bool selectState;
     bool showFavorites;
-    bool displaychannum;
+    QString channelFormat;
 
     int desiredDisplayChans;
     int DISPLAY_CHANS;
