@@ -17,6 +17,11 @@ class HDTVRecorder : public RecorderBase
                        { RecorderBase::SetOption(name, value); }
     void SetVideoFilters(QString &filters);
 
+    void SetOptionsFromProfile(RecordingProfile *profile,
+                               const QString &videodev,
+                               const QString &audiodev,
+                               const QString &vbidev, int ispip);
+
     void Initialize(void);
     void StartRecording(void);
     void StopRecording(void);
@@ -27,6 +32,7 @@ class HDTVRecorder : public RecorderBase
     bool GetPause(void);
     void WaitForPause(void);
     bool IsRecording(void);
+    bool IsErrored(void) { return false; }
 
     long long GetFramesWritten(void);
 
@@ -60,6 +66,7 @@ class HDTVRecorder : public RecorderBase
     bool cleartimeonpause;
 
     long long framesWritten;
+    long long framesSeen;
 
     int width, height;
 
@@ -67,7 +74,10 @@ class HDTVRecorder : public RecorderBase
 
     int keyframedist;
     bool gopset;
-    bool pict_start_is_gop;
+    bool m_in_mpg_headers;
+    bool seq_start_is_gop;
+    int seq_count;
+    int m_header_sync;
 
     QMap<long long, long long> positionMap;
 
