@@ -17,12 +17,16 @@ class InfoDialog : public MythDialog
   public:
     InfoDialog(ProgramInfo *pginfo, MythMainWindow *parent, 
                const char *name = 0);
+   ~InfoDialog();           
 
   protected slots:
     void selected(QListViewItem *);
     void advancedEdit(QListViewItem *);
     void numberPress(QListViewItem *, int);
 
+  protected:
+    bool eventFilter(QObject *o, QEvent *e);
+
   private:
     QLabel *getDateLabel(ProgramInfo *pginfo);
 
