@@ -14,19 +14,17 @@ using namespace std;
 #include "remoteencoder.h"
 #include "programinfo.h"
 #include "mythcontext.h"
+#include "mythdbcon.h"
 
 #include "videoout_ivtv.h"
 #include "videodev_myth.h"
 
 bool IvtvDecoder::ntsc = true;
 
-IvtvDecoder::IvtvDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
+IvtvDecoder::IvtvDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
                          ProgramInfo *pginfo)
-           : DecoderBase(parent)
+           : DecoderBase(parent, db, pginfo)
 {
-    m_db = db;
-    m_playbackinfo = pginfo;
-
     framesRead = 0;
     framesPlayed = 0;
     lastStartFrame = 0;
@@ -77,8 +75,6 @@ void IvtvDecoder::SeekReset(long long newkey, int skipframes)
     videoout->Start(0, skipframes+5);
     videoout->Pause();
 
-    while (ReadWrite(1, 0))
-        ;
     while (!videoout->GetFramesPlayed() && !ateof)
         ReadWrite(1, 0);
 
@@ -180,7 +176,9 @@ int IvtvDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
 
     if (m_playbackinfo && m_db)
     {
-        m_playbackinfo->GetPositionMap(positionMap, MARK_GOP_START, m_db);
+        m_db->lock();
+        m_playbackinfo->GetPositionMap(positionMap, MARK_GOP_START, m_db->db());
+        m_db->unlock();
         if (positionMap.size() && !livetv && !watchingrecording)
         {
             long long totframes = positionMap.size() * keyframedist;
@@ -268,9 +266,13 @@ void IvtvDecoder::MpegPreProcessPkt(unsigned char *buf, int len,
                 }
                 case PICTURE_START:
                 {
-                    framesRead++;
-                    if (exitafterdecoded)
-                        gotvideo = 1;
+                    int type = (bufptr[1] >> 3) & 7;
+                    if (type >= 1 && type <= 3)
+                    {
+                        framesRead++;
+                        if (exitafterdecoded)
+                            gotvideo = 1;
+                    }
                     break;
                 }
                 default:
@@ -335,7 +337,7 @@ bool IvtvDecoder::ReadWrite(int onlyvideo, int delay)
     {
         if (vidread >= vidwrite)
         {
-            long long startpos = ringBuffer->GetReadPosition();
+            long long startpos = ringBuffer->GetTotalReadPosition();
             count = ringBuffer->Read(&vidbuf[vidread], vidmax - vidread);
             if (count > 0)
             {
@@ -349,7 +351,7 @@ bool IvtvDecoder::ReadWrite(int onlyvideo, int delay)
 
         if (vidread < vidwrite)
         {
-            long long startpos = ringBuffer->GetReadPosition();
+            long long startpos = ringBuffer->GetTotalReadPosition();
             count = ringBuffer->Read(&vidbuf[vidread], vidwrite - vidread);
             if (count > 0)
             {
@@ -546,7 +548,11 @@ bool IvtvDecoder::DoFastForward(long long desiredFrame)
 void IvtvDecoder::SetPositionMap(void)
 {
     if (m_playbackinfo && m_db)
-        m_playbackinfo->SetPositionMap(positionMap, MARK_GOP_START, m_db);
+    {
+        m_db->lock();
+        m_playbackinfo->SetPositionMap(positionMap, MARK_GOP_START, m_db->db());
+        m_db->unlock();
+    }
 }
 
 void IvtvDecoder::UpdateFramesPlayed(void)
