@@ -38,7 +38,7 @@ class RawDataList
 class NuppelDecoder : public DecoderBase
 {
   public:
-    NuppelDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
+    NuppelDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
                   ProgramInfo *pginfo);
    ~NuppelDecoder();
 
@@ -128,9 +128,6 @@ class NuppelDecoder : public DecoderBase
     long long lastKey;
     long long framesPlayed;
 
-    QSqlDatabase *m_db;
-    ProgramInfo *m_playbackinfo;
-
     QPtrList <RawDataList> StoredData;
     bool getrawframes;
     bool getrawvideo;
