@@ -22,6 +22,8 @@ using namespace std;
 #include "libmyth/oldsettings.h"
 #include "udpnotify.h"
 
+#include "osdlistbtntype.h"
+
 OSD::OSD(int width, int height, int frint, const QString &font, 
          const QString &ccfont, const QString &prefix, const QString &osdtheme,
          int dispx, int dispy, int dispw, int disph)
@@ -51,6 +53,8 @@ OSD::OSD(int width, int height, int frint, const QString &font,
     m_setsvisible = false;
     setList = NULL;
 
+    runningTreeMenu = NULL;
+
     setList = new vector<OSDSet *>;
 
     fontSizeType = gContext->GetSetting("OSDThemeFontSizeType", "default");
@@ -145,6 +149,56 @@ void OSD::SetDefaults(void)
                                           displaywidth, displayheight);
         container->AddType(ccpage);
     }
+
+    container = GetSet("menu");
+    if (!container)
+    {
+        QString name = "menu";
+        container = new OSDSet(name, true, vid_width, vid_height,
+                               wmult, hmult, frameint);
+        AddSet(container, name);
+ 
+        QRect area = QRect(20, 40, 620, 300);
+        QRect listarea = QRect(0, 0, 274, 260);
+
+        normalizeRect(area);
+        normalizeRect(listarea);
+        listarea.moveBy(-xoffset, -yoffset);
+
+        OSDListTreeType *lb = new OSDListTreeType("menu", area, listarea, 10,
+                                                  wmult, hmult);
+ 
+        lb->SetItemRegColor(QColor("#505050"), QColor("#000000"), 100);
+        lb->SetItemSelColor(QColor("#52CA38"), QColor("#349838"), 255);
+ 
+        lb->SetSpacing((int)(2 * hmult));
+        lb->SetMargin((int)(3 * wmult));
+
+        TTFFont *actfont = GetFont("infofont");
+        TTFFont *inactfont = GetFont("infofont");
+
+        if (!actfont)
+        {
+            actfont = LoadFont(fontname, 16);
+
+            if (actfont)
+                fontMap["treemenulistfont"] = actfont;
+        }
+
+        if (!actfont)
+        {
+            QMap<QString, TTFFont *>::Iterator it = fontMap.begin();
+            actfont = it.data();
+        }
+
+        if (!inactfont)
+            inactfont = actfont;
+
+        lb->SetFontActive(actfont);
+        lb->SetFontInactive(inactfont);
+
+        container->AddType(lb);
+    }
 }
 
 void OSD::Reinit(int width, int height, int frint, int dispx, int dispy, 
@@ -382,7 +436,7 @@ void OSD::parseBox(OSDSet *container, QDomElement &element)
             if (info.tagName() == "area")
             {
                 area = parseRect(getFirstText(info));
-                normalizeRect(&area);
+                normalizeRect(area);
             }
             else
             {
@@ -474,7 +528,7 @@ void OSD::parseTextArea(OSDSet *container, QDomElement &element)
             if (info.tagName() == "area")
             {
                 area = parseRect(getFirstText(info));
-                normalizeRect(&area);
+                normalizeRect(area);
             }
             else if (info.tagName() == "font")
             {
@@ -603,7 +657,7 @@ void OSD::parseSlider(OSDSet *container, QDomElement &element)
             if (info.tagName() == "area")
             {
                 area = parseRect(getFirstText(info));
-                normalizeRect(&area);            
+                normalizeRect(area);            
             }
             else if (info.tagName() == "filename")
             {
@@ -687,7 +741,7 @@ void OSD::parseEditArrow(OSDSet *container, QDomElement &element)
             if (info.tagName() == "area")
             {
                 area = parseRect(getFirstText(info));
-                normalizeRect(&area);
+                normalizeRect(area);
             }
             else if (info.tagName() == "filename")
             {
@@ -742,7 +796,7 @@ void OSD::parsePositionRects(OSDSet *container, QDomElement &element)
             if (info.tagName() == "area")
             {
                 QRect area = parseRect(getFirstText(info));
-                normalizeRect(&area);
+                normalizeRect(area);
 
                 rects->AddPosition(area);
             }
@@ -810,6 +864,151 @@ void OSD::parsePositionImage(OSDSet *container, QDomElement &element)
     container->AddType(image);
 }
 
+void OSD::parseListTree(OSDSet *container, QDomElement &element)
+{
+    QRect   area = QRect(0,0,0,0);
+    QRect   listsize = QRect(0,0,0,0);
+    int     leveloffset = 0;
+    QString fontActive;
+    QString fontInactive;
+    bool    showArrow = true;
+    bool    showScrollArrows = false;
+    QColor  grUnselectedBeg(Qt::black);
+    QColor  grUnselectedEnd(80,80,80);
+    uint    grUnselectedAlpha(100);
+    QColor  grSelectedBeg(82,202,56);
+    QColor  grSelectedEnd(52,152,56);
+    uint    grSelectedAlpha(255);
+    int     spacing = 2;
+    int     margin = 3;
+
+    QString name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        cerr << "listtree needs a name\n";
+        return;
+    }
+
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling()) 
+    {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "area")
+            {
+                area = parseRect(getFirstText(info));
+                normalizeRect(area);
+            }
+            else if (info.tagName() == "listsize")
+            {
+                listsize = parseRect(getFirstText(info));
+                normalizeRect(listsize);
+                listsize.moveBy(-xoffset, -yoffset);
+            }
+            else if (info.tagName() == "leveloffset")
+            {
+                leveloffset = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "fcnfont")
+            {
+                QString fontName = info.attribute("name", "");
+                QString fontFcn  = info.attribute("function", "");
+
+                if (fontFcn.lower() == "active")
+                    fontActive = fontName;
+                else if (fontFcn.lower() == "inactive")
+                    fontInactive = fontName;
+                else 
+                {
+                    cerr << "Unknown font function for listtree area: "
+                         << fontFcn << endl;
+                    return;
+                }
+            }
+            else if (info.tagName() == "showarrow") 
+            {
+                if (getFirstText(info).lower() == "no")
+                    showArrow = false;
+            }
+            else if (info.tagName() == "showscrollarrows") 
+            {
+                if (getFirstText(info).lower() == "yes")
+                    showScrollArrows = true;
+            }
+            else if (info.tagName() == "gradient") 
+            {
+                if (info.attribute("type","").lower() == "selected") 
+                {
+                    grSelectedBeg = QColor(info.attribute("start"));
+                    grSelectedEnd = QColor(info.attribute("end"));
+                    grSelectedAlpha = info.attribute("alpha","255").toUInt();
+                }
+                else if (info.attribute("type","").lower() == "unselected") 
+                {
+                    grUnselectedBeg = QColor(info.attribute("start"));
+                    grUnselectedEnd = QColor(info.attribute("end"));
+                    grUnselectedAlpha = info.attribute("alpha","100").toUInt();
+                }
+                else 
+                {
+                    cerr << "Unknown type for gradient in listtree area\n";
+                    return;
+                }
+
+                if (!grSelectedBeg.isValid() || !grSelectedEnd.isValid() ||
+                    !grUnselectedBeg.isValid() || !grUnselectedEnd.isValid()) 
+                {
+                    cerr << "Unknown color for gradient in listtree area\n";
+                    return;
+                }
+
+                if (grSelectedAlpha > 255 || grUnselectedAlpha > 255) {
+                    cerr << "Incorrect alpha for gradient in listtree area\n";
+                    return;
+                }
+            }
+            else if (info.tagName() == "spacing") {
+                spacing = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "margin") {
+                margin = getFirstText(info).toInt();
+            }
+            else
+            {
+                cerr << "Unknown tag in listtree area: "
+                     << info.tagName() << endl;
+                return;
+            }
+        }
+    }
+
+    TTFFont *fpActive = GetFont(fontActive);
+    if (!fpActive)
+    {
+        cerr << "Unknown font: " << fontActive << " in listtree\n";
+        return;
+    }
+
+    TTFFont *fpInactive = GetFont(fontInactive);
+    if (!fpInactive)
+    {
+        cerr << "Unknown font: " << fontInactive << " in listtree\n";
+        return;
+    }
+
+    OSDListTreeType *lb = new OSDListTreeType(name, area, listsize, leveloffset,
+                                              wmult, hmult);
+    lb->SetFontActive(fpActive);
+    lb->SetFontInactive(fpInactive);
+    lb->SetItemRegColor(grUnselectedBeg, grUnselectedEnd, grUnselectedAlpha);
+    lb->SetItemSelColor(grSelectedBeg, grSelectedEnd, grSelectedAlpha);
+    lb->SetSpacing((int)(spacing * hmult));
+    lb->SetMargin((int)(margin * wmult));
+
+    container->AddType(lb);
+}
+
 void OSD::parseContainer(QDomElement &element)
 {
     QString name = element.attribute("name", "");
@@ -879,6 +1078,10 @@ void OSD::parseContainer(QDomElement &element)
             {
                 parsePositionImage(container, info);
             }
+            else if (info.tagName() == "listtreemenu")
+            {
+                parseListTree(container, info);
+            }
             else
             {
                 cerr << "Unknown container child: " << info.tagName() << endl;
@@ -952,12 +1155,12 @@ bool OSD::LoadTheme(void)
     return true;
 }
 
-void OSD::normalizeRect(QRect *rect)
+void OSD::normalizeRect(QRect &rect)
 {   
-    rect->setWidth((int)(rect->width() * wmult));
-    rect->setHeight((int)(rect->height() * hmult));
-    rect->moveTopLeft(QPoint((int)(xoffset + rect->x() * wmult),
-                             (int)(yoffset + rect->y() * hmult)));
+    rect.setWidth((int)(rect.width() * wmult));
+    rect.setHeight((int)(rect.height() * hmult));
+    rect.moveTopLeft(QPoint((int)(xoffset + rect.x() * wmult),
+                            (int)(yoffset + rect.y() * hmult)));
 }       
 
 QPoint OSD::parsePoint(QString text)
@@ -1162,10 +1365,6 @@ void OSD::UpdatePause(int position, QString slidertext)
                 changed = true;
             }
         }
-
-        //test w/ this later.
-        //m_setsvisible = true; 
-        //changed = true;
     }
     osdlock.unlock();
 }
@@ -1238,6 +1437,29 @@ void OSD::ClearAllCCText()
     osdlock.unlock();
 }
 
+void OSD::UpdateCCText(vector<ccText*> *ccbuf,
+                       int replace, int scroll, bool scroll_prsv,
+                       int scroll_yoff, int scroll_ymax)
+{
+    osdlock.lock();
+    OSDSet *container = GetSet("cc_page");
+    if (container)
+    {
+        OSDTypeCC *ccpage = (OSDTypeCC *)container->GetType("cc_page");
+        int visible = 0;
+
+        if (ccpage)
+            visible = ccpage->UpdateCCText(ccbuf,
+                                           replace, scroll, scroll_prsv,
+                                           scroll_yoff, scroll_ymax);
+
+        container->Display(visible);
+        m_setsvisible = true;
+        changed = true;
+    }
+    osdlock.unlock();
+}
+
 void OSD::SetSettingsText(const QString &text, int length)
 {
     osdlock.lock();
@@ -1685,10 +1907,12 @@ OSDSurface *OSD::Display(void)
 
     drawSurface->SetChanged(false);
 
+    //actuallydraw = true;
+
     if (actuallydraw)
     {
         drawSurface->SetChanged(true);
-        drawSurface->Clear();
+        drawSurface->ClearUsed();
     }
 
     vector<OSDSet *> removeList;
@@ -1863,3 +2087,66 @@ void OSD::ClearNotify(UDPNotifyOSDSet *notifySet)
     osdlock.unlock();
 }
 
+OSDListTreeType *OSD::ShowTreeMenu(const QString &name, 
+                                   OSDGenericTree *treeToShow)
+{
+    if (runningTreeMenu || !treeToShow)
+        return NULL;
+
+    OSDListTreeType *rettree = NULL;
+
+    osdlock.lock();
+
+    OSDSet *container = GetSet(name);
+    if (container)
+    {
+        rettree = (OSDListTreeType *)container->GetType("menu");
+        if (rettree)
+        {
+            rettree->SetAsTree(treeToShow);
+            rettree->SetVisible(true);
+            runningTreeMenu = rettree;
+            treeMenuContainer = name;
+            container->Display();
+            m_setsvisible = true;
+            changed = true;
+        }
+    }
+
+    osdlock.unlock();
+
+    return rettree;
+}
+
+bool OSD::IsRunningTreeMenu(void)
+{
+    if (runningTreeMenu)
+        return true;
+    return false;
+}
+
+bool OSD::TreeMenuHandleKeypress(QKeyEvent *e)
+{
+    if (!runningTreeMenu)
+        return false;
+
+    bool ret = runningTreeMenu->HandleKeypress(e);
+
+    osdlock.lock();
+
+    if (!runningTreeMenu->IsVisible())
+    {
+        OSDSet *container = GetSet(treeMenuContainer);
+        if (container)
+            container->Hide();
+
+        runningTreeMenu = NULL;
+    }
+
+    changed = true;
+
+    osdlock.unlock();
+
+    return ret;
+}
+
