@@ -31,6 +31,10 @@ class OSDTypePositionIndicator;
 class OSDSurface;
 class TV;
 class UDPNotifyOSDSet;
+class OSDListTreeType;
+class QKeyEvent;
+class OSDGenericTree;
+class ccText;
 
 class OSD : public QObject
 {
@@ -56,6 +60,9 @@ class OSD : public QObject
     void AddCCText(const QString &text, int x, int y, int color, 
                    bool teletextmode = false);
     void ClearAllCCText();
+    void UpdateCCText(vector<ccText*> *ccbuf,
+                      int replace = 0, int scroll = 0, bool scroll_prsv = false,
+                      int scroll_yoff = 0, int scroll_ymax = 15);
     void SetSettingsText(const QString &text, int length);
 
     void NewDialogBox(const QString &name, const QString &message, 
@@ -106,6 +113,11 @@ class OSD : public QObject
     void StartNotify(UDPNotifyOSDSet *notifySet, int displaytime = 5);
     void ClearNotify(UDPNotifyOSDSet *notifySet);
 
+    bool IsRunningTreeMenu(void);
+    bool TreeMenuHandleKeypress(QKeyEvent *e);
+    OSDListTreeType *ShowTreeMenu(const QString &name, 
+                                  OSDGenericTree *treeToShow);
+
  private:
     void SetDefaults();
     TTFFont *LoadFont(QString name, int size); 
@@ -114,7 +126,7 @@ class OSD : public QObject
     void HighlightDialogSelection(OSDSet *container, int num);  
  
     bool LoadTheme();
-    void normalizeRect(QRect *rect);
+    void normalizeRect(QRect &rect);
     QPoint parsePoint(QString text);
     QRect parseRect(QString text);
 
@@ -130,6 +142,7 @@ class OSD : public QObject
     void parseEditArrow(OSDSet *container, QDomElement &element);
     void parsePositionRects(OSDSet *container, QDomElement &element);
     void parsePositionImage(OSDSet *container, QDomElement &element);
+    void parseListTree(OSDSet *container, QDomElement &element);
 
     QString fontname;
     QString ccfontname;
@@ -167,6 +180,9 @@ class OSD : public QObject
 
     OSDSurface *drawSurface;
     bool changed;
+
+    OSDListTreeType *runningTreeMenu;
+    QString treeMenuContainer;
 };
     
 #endif
