@@ -329,6 +329,7 @@ void OSDSet::Hide(void)
     m_timeleft = -1;
     m_fadetime = 0;
     m_notimeout = false;
+    m_displaying = false;
 
     if (currentOSDFunctionalType)
     {
@@ -728,6 +729,26 @@ OSDTypeImage::OSDTypeImage(const QString &name)
     m_drawwidth = -1;
     m_onlyusefirst = false;
 
+    m_displaypos = QPoint(0, 0);
+
+    m_yuv = NULL;
+    m_alpha = NULL;
+    m_ybuffer = NULL;
+    m_ubuffer = NULL;
+    m_vbuffer = NULL;
+    m_isvalid = false;
+    m_filename = "";
+}
+
+OSDTypeImage::OSDTypeImage(void)
+            : OSDType("")
+{
+    m_name = "";
+    m_drawwidth = -1;
+    m_onlyusefirst = false;
+
+    m_displaypos = QPoint(0, 0);
+
     m_yuv = NULL;
     m_alpha = NULL;
     m_ybuffer = NULL;
@@ -745,6 +766,11 @@ OSDTypeImage::~OSDTypeImage()
         delete [] m_alpha;
 }
 
+void OSDTypeImage::SetName(const QString &name)
+{
+    m_name = name;
+}
+
 void OSDTypeImage::Reinit(float wmult, float hmult)
 {
     int x = (int)(m_displaypos.x() * wmult / m_wmult);
@@ -816,6 +842,38 @@ void OSDTypeImage::LoadImage(const QString &filename, float wmult, float hmult,
     m_imagesize = QRect(0, 0, width, height);
 }
 
+void OSDTypeImage::LoadFromQImage(const QImage &img)
+{
+    if (m_isvalid)
+    {
+        if (m_yuv)
+            delete [] m_yuv;
+        if (m_alpha)
+            delete [] m_alpha;
+
+        m_isvalid = false;
+        m_yuv = NULL;
+        m_alpha = NULL;
+    }
+
+    m_isvalid = true;
+
+    int width = (img.width() / 2) * 2;
+    int height = (img.height() / 2) * 2;
+
+    m_yuv = new unsigned char[width * height * 3 / 2];
+    m_ybuffer = m_yuv;
+    m_ubuffer = m_yuv + (width * height);
+    m_vbuffer = m_yuv + (width * height * 5 / 4);
+
+    m_alpha = new unsigned char[width * height];
+
+    rgb32_to_yuv420p(m_ybuffer, m_ubuffer, m_vbuffer, m_alpha, img.bits(),
+                     width, height, img.width());
+
+    m_imagesize = QRect(0, 0, width, height);
+}
+
 void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff, 
                         int yoff)
 {
@@ -841,14 +899,6 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     ystart = (ystart / 2) * 2;
     xstart = ((xstart + 1) / 2) * 2;
 
-    if (height + ystart > surface->height)
-        height = surface->height - ystart - 1;
-    if (width + xstart > surface->width)
-        width = surface->width - xstart - 1;
-
-    if (width == 0 || height == 0)
-        return;
-
     int startline = 0;
     int startcol = 0;
 
@@ -864,6 +914,14 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         xstart = 0;
     }
 
+    if (height + ystart > surface->height)
+        height = surface->height - ystart - 1;
+    if (width + xstart > surface->width)
+        width = surface->width - xstart - 1;
+
+    if (width == 0 || height == 0)
+        return;
+
     QRect destRect = QRect(xstart, ystart, width, height);
     bool needblend = false;
 
@@ -878,9 +936,6 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     int uvdestwidth;
 
     int alphamod = 255;
-    bool transdest = false;
-
-    int newalpha = 0;
 
     if (maxfade > 0 && fade >= 0)
         alphamod = (int)((((float)(fade) / maxfade) * 256.0) + 0.5);
@@ -938,87 +993,36 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     if (m_onlyusefirst)
         startingx = 0;
 
-    // overlap with something we've already drawn..
-    for (int y = startline; y < height; y++)
-    {
-        ysrcwidth = y * iwidth;
-        ydestwidth = (y + ystart - startline) * surface->width;
+    ysrcwidth = startline * iwidth;
+    ydestwidth = ystart * surface->width;
 
-        dest = surface->y + xstart + ydestwidth;
-        destalpha = surface->alpha + xstart + ydestwidth;
-        src = m_ybuffer + ysrcwidth + startingx;
+    dest = surface->y + xstart + ydestwidth;
+    destalpha = surface->alpha + xstart + ydestwidth;
+    src = m_ybuffer + ysrcwidth + startingx;
 
-        srcalpha = m_alpha + ysrcwidth + startingx;
+    srcalpha = m_alpha + ysrcwidth + startingx;
 
-        uvsrcwidth = ysrcwidth / 4;
-        uvdestwidth = ydestwidth / 4;
+    uvsrcwidth = ysrcwidth / 4;
+    uvdestwidth = ydestwidth / 4;
 
-        udest = surface->u + xstart / 2 + uvdestwidth;
-        usrc = m_ubuffer + uvsrcwidth + startingx / 2;
+    udest = surface->u + xstart / 2 + uvdestwidth;
+    usrc = m_ubuffer + uvsrcwidth + startingx / 2;
 
-        vdest = surface->v + xstart / 2 + uvdestwidth;
-        vsrc = m_vbuffer + uvsrcwidth + startingx / 2;
+    vdest = surface->v + xstart / 2 + uvdestwidth;
+    vsrc = m_vbuffer + uvsrcwidth + startingx / 2;
 
-        for (int x = startcol; x < width; x++)
-        {
-            alpha = *srcalpha;
-
-            if (alpha == 0)
-                goto imagedrawend;
-
-            alpha = ((alpha * alphamod) + 0x80) >> 8;
-
-            newalpha = alpha;
-
-            if (*destalpha != 0)
-            {
-                transdest = false;
-                newalpha = surface->pow_lut[alpha][*destalpha];
-                *dest = blendColorsAlpha(*src, *dest, newalpha);
-                *destalpha = *destalpha + ((alpha * (255 - *destalpha)) / 255);
-            }
-            else
-            {
-                transdest = true;
-                *dest = *src;
-                *destalpha = alpha;
-            }
-
-            if ((y % 2 == 0) && (x % 2 == 0))
-            {
-                if (!transdest)
-                {
-                    *udest = blendColorsAlpha(*usrc, *udest, newalpha);
-                    *vdest = blendColorsAlpha(*vsrc, *vdest, newalpha);
-                }
-                else
-                {
-                    *udest = *usrc;
-                    *vdest = *vsrc;
-                }
-            }
-
-imagedrawend:
-            if ((y % 2 == 0) && (x % 2 == 0))
-            {
-                udest++;
-                vdest++;
-                if (!m_onlyusefirst)
-                {
-                    usrc++;
-                    vsrc++;
-                }
-            }
-
-            dest++;
-            destalpha++;
-            if (!m_onlyusefirst)
-            {
-                src++;
-                srcalpha++;
-            }
-        }
-    }
+    if (m_onlyusefirst)
+        (surface->blendcolumnfunc) (src, usrc, vsrc, srcalpha, iwidth, dest,
+                                    udest, vdest, destalpha, surface->width,
+                                    width - startcol, height - startline,
+                                    alphamod, 1, surface->rec_lut,
+                                    surface->pow_lut);
+    else
+        (surface->blendregionfunc) (src, usrc, vsrc, srcalpha, iwidth, dest,
+                                    udest, vdest, destalpha, surface->width,
+                                    width - startcol, height - startline,
+                                    alphamod, 1, surface->rec_lut,
+                                    surface->pow_lut);
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
@@ -1135,7 +1139,7 @@ OSDTypeEditSlider::OSDTypeEditSlider(const QString &name,
     m_displayrect = displayrect;
     m_drawwidth = displayrect.width();
 
-    m_drawMap = new int[m_drawwidth + 1];
+    m_drawMap = new unsigned char[m_drawwidth + 1];
     for (int i = 0; i < m_drawwidth; i++)
          m_drawMap[i] = 0;
 
@@ -1194,7 +1198,7 @@ void OSDTypeEditSlider::Reinit(float wchange, float hchange, float wmult,
 
     delete [] m_drawMap;
     
-    m_drawMap = new int[m_drawwidth + 1];
+    m_drawMap = new unsigned char[m_drawwidth + 1];
     for (int i = 0; i < m_drawwidth; i++)
          m_drawMap[i] = 0;
 
@@ -1265,7 +1269,6 @@ void OSDTypeEditSlider::Draw(OSDSurface *surface, int fade, int maxfade,
             
     unsigned char *dest, *destalpha, *src, *rsrc, *srcalpha, *rsrcalpha;
     unsigned char *udest, *vdest, *usrc, *rusrc, *vsrc, *rvsrc;
-    int a; 
             
     int iwidth, riwidth, width;
     iwidth = m_imagesize.width();
@@ -1317,107 +1320,39 @@ void OSDTypeEditSlider::Draw(OSDSurface *surface, int fade, int maxfade,
     int uvdestwidth;
  
     int alphamod = 255;
-    bool transdest = false;
-    int newalpha = 0;
     
     if (maxfade > 0 && fade >= 0)
         alphamod = (int)((((float)(fade) / maxfade) * 256.0) + 0.5);
 
-    unsigned char *alpha;
-    unsigned char *ybuf;
-    unsigned char *ubuf;
-    unsigned char *vbuf;
+    ysrcwidth = startline * iwidth; 
+    rysrcwidth = startline * riwidth;
+    ydestwidth = ystart * surface->width;
 
-    for (int y = startline; y < height; y++)
-    {
-        ysrcwidth = y * iwidth; 
-        rysrcwidth = y * riwidth;
-        ydestwidth = (y + ystart - startline) * surface->width;
+    dest = surface->y + xstart + ydestwidth;
+    destalpha = surface->alpha + xstart + ydestwidth;
+    src = m_ybuffer + ysrcwidth;
+    rsrc = m_rybuffer + rysrcwidth;
 
-        dest = surface->y + xstart + ydestwidth;
-        destalpha = surface->alpha + xstart + ydestwidth;
-        src = m_ybuffer + ysrcwidth;
-        rsrc = m_rybuffer + rysrcwidth;
+    srcalpha = m_alpha + ysrcwidth;
+    rsrcalpha = m_ralpha + rysrcwidth;
 
-        srcalpha = m_alpha + ysrcwidth;
-        rsrcalpha = m_ralpha + rysrcwidth;
+    uvdestwidth = ydestwidth / 4;
+    uvsrcwidth = ysrcwidth / 4;
+    ruvsrcwidth = rysrcwidth / 4;
 
-        uvdestwidth = ydestwidth / 4;
-        uvsrcwidth = ysrcwidth / 4;
-        ruvsrcwidth = rysrcwidth / 4;
+    udest = surface->u + xstart / 2 + uvdestwidth;
+    usrc = m_ubuffer + uvsrcwidth;
+    rusrc = m_rubuffer + ruvsrcwidth;
 
-        udest = surface->u + xstart / 2 + uvdestwidth;
-        usrc = m_ubuffer + uvsrcwidth;
-        rusrc = m_rubuffer + ruvsrcwidth;
-
-        vdest = surface->v + xstart / 2 + uvdestwidth;
-        vsrc = m_vbuffer + uvsrcwidth;
-        rvsrc = m_rvbuffer + ruvsrcwidth;
-
-        for (int x = startcol; x < width; x++)
-        {
-            if (m_drawMap[x] == 0)
-            {
-                alpha = srcalpha;
-                ybuf = src;
-                ubuf = usrc;
-                vbuf = vsrc; 
-            }
-            else
-            {
-                alpha = rsrcalpha;
-                ybuf = rsrc;
-                ubuf = rusrc;
-                vbuf = rvsrc;
-            }
-
-            if (*alpha == 0)
-                goto editsliderdrawend;
-
-            a = *alpha;
-
-            a = ((a * alphamod) + 0x80) >> 8;
-            newalpha = a;
-
-            if (*destalpha != 0)
-            {
-                transdest = false;
-                newalpha = surface->pow_lut[a][*destalpha];
-                *dest = blendColorsAlpha(*ybuf, *dest, newalpha);
-                *destalpha = *destalpha + ((a * (255 - *destalpha)) / 255);
-            }
-            else
-            {
-                transdest = true;
-                *dest = *ybuf;
-                *destalpha = a;
-            }
-
-            if ((y % 2 == 0) && (x % 2 == 0))
-            {
-                if (!transdest)
-                {
-                    *udest = blendColorsAlpha(*ubuf, *udest, newalpha);
-                    *vdest = blendColorsAlpha(*vbuf, *vdest, newalpha);
-                }
-                else
-                {
-                    *udest = *ubuf;
-                    *vdest = *vbuf;
-                }
-            }
-
-editsliderdrawend:
-            if ((y % 2 == 0) && (x % 2 == 0))
-            {
-                udest++;
-                vdest++;
-            }
-
-            dest++;
-            destalpha++;
-        }
-    }
+    vdest = surface->v + xstart / 2 + uvdestwidth;
+    vsrc = m_vbuffer + uvsrcwidth;
+    rvsrc = m_rvbuffer + ruvsrcwidth;
+    (surface->blendcolumn2func) (rsrc, rusrc, rvsrc, rsrcalpha, riwidth, src,
+                                 usrc, vsrc, srcalpha, iwidth,
+                                 m_drawMap + startcol, dest, udest, vdest,
+                                 destalpha, surface->width, width - startcol,
+                                 height - startline, alphamod, 1,
+                                 surface->rec_lut, surface->pow_lut);
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1484,9 +1419,9 @@ void OSDTypeBox::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     if (maxfade > 0 && fade >= 0)
         alphamod = (int)((((float)(fade) / maxfade) * 256.0) + 0.5);
 
-    alpha = ((alpha * alphamod) + 0x809) >> 8;
-    int newalpha, ydestwidth;
+    int ydestwidth;
 
+    alpha = ((alpha * alphamod) + 0x80) >> 8;
     if (!needblend)
     {
         for (int y = ystart; y < yend; y++)
@@ -1500,29 +1435,11 @@ void OSDTypeBox::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         return;
     }
 
-    for (int y = ystart; y < yend; y++)
-    {
-        dest = surface->y + y * surface->width + xstart;
-        destalpha = surface->alpha + y * surface->width + xstart;
-
-        for (int x = xstart; x < xend; x++)
-        {
-            if (*destalpha != 0)
-            {
-                newalpha = surface->pow_lut[alpha][*destalpha];
-                *dest = blendColorsAlpha(0, *dest, newalpha);
-                *destalpha = *destalpha + ((alpha * (255 - *destalpha)) / 255);
-            }
-            else
-            {
-                *dest = 0;
-                *destalpha = alpha;
-            }
-
-            dest++;
-            destalpha++;
-        }
-    }
+    dest = surface->y + ystart * surface->width + xstart;
+    destalpha = surface->alpha + ystart * surface->width + xstart;
+    (surface->blendconstfunc) (0, 0, 0, alpha, dest, NULL, NULL, destalpha,
+                               surface->width, width, height, 0,
+                               surface->rec_lut, surface->pow_lut);
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1803,6 +1720,100 @@ void OSDTypeCC::ClearAllCCText()
     }
 }
 
+int OSDTypeCC::UpdateCCText(vector<ccText*> *ccbuf,
+                            int replace, int scroll, bool scroll_prsv,
+                            int scroll_yoff, int scroll_ymax)
+// ccbuf      :  new text
+// replace    :  replace last lines
+// scroll     :  scroll amount
+// scroll_prsv:  preserve last lines and move into scroll window
+// scroll_yoff:  yoff < scroll window <= ymax
+// scroll_ymax:
+{
+    vector<ccText*>::iterator i;
+    int visible = 0;
+
+    if (m_textlist && (scroll || replace))
+    {
+        ccText *cc;
+
+        // get last row
+        int ylast = 0;
+        i = m_textlist->end() - 1;
+        cc = *i;
+        if (cc)
+            ylast = cc->y;
+
+        // calculate row positions to delete, keep
+        int ydel = scroll_yoff + scroll;
+        int ykeep = scroll_ymax;
+        int ymove = 0;
+        if (scroll_prsv && ylast)
+        {
+            ymove = ylast - scroll_ymax;
+            ydel += ymove;
+            ykeep += ymove;
+        }
+
+        i = m_textlist->begin();
+        while (i < m_textlist->end())
+        {
+            cc = (*i);
+            if (!cc)
+            {
+                i = m_textlist->erase(i);
+                continue;
+            }
+
+            if (cc->y > (ylast - replace))
+            {
+                // delete last lines
+                delete cc;
+                i = m_textlist->erase(i);
+            }
+            else if (scroll)
+            {
+                if (cc->y > ydel && cc->y <= ykeep)
+                {
+                    // scroll up
+                    cc->y -= (scroll + ymove);
+                    i++;
+                }
+                else
+                {
+                    // delete lines outside scroll window
+                    i = m_textlist->erase(i);
+                    delete cc;
+                }
+            }
+            else
+                i++;
+        }
+    }
+
+    if (m_textlist)
+        visible += m_textlist->size();
+
+    if (ccbuf)
+    {
+        // add new text
+        i = ccbuf->begin();
+        while (i < ccbuf->end())
+        {
+            if (*i)
+            {
+                visible++;
+                if (!m_textlist)
+                    m_textlist = new vector<ccText *>;
+                m_textlist->push_back(*i);
+            }
+            i++;
+        }
+    }
+
+    return visible;
+}
+
 void OSDTypeCC::Draw(OSDSurface *surface, int fade, int maxfade, int xoff, 
                      int yoff)
 {
@@ -1833,7 +1844,7 @@ void OSDTypeCC::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
             else
             {
                 x = (cc->x + 3) * displaywidth / 36 + xoffset;
-                y = cc->y * displayheight / 19 + yoffset;
+                y = cc->y * displayheight / 17 + yoffset;
             }
 
             int maxx = x + textlength;
