@@ -112,8 +112,9 @@ class OSDSet : public QObject
     bool m_draweveryframe;
 };
 
-class OSDType
+class OSDType : public QObject
 {
+    Q_OBJECT
   public:
     OSDType(const QString &name);
     virtual ~OSDType();
@@ -210,12 +211,15 @@ class OSDTypeImage : public OSDType
                  int scaleh = -1);
     OSDTypeImage(const OSDTypeImage &other);
     OSDTypeImage(const QString &name);    
+    OSDTypeImage(void);
     virtual ~OSDTypeImage();
 
+    void SetName(const QString &name);
     void Reinit(float wmult, float hmult);
 
     void LoadImage(const QString &filename, float wmult, float hmult, 
                    int scalew = -1, int scaleh = -1);
+    void LoadFromQImage(const QImage &img);
 
     void SetStaticSize(int scalew, int scaleh) { m_scalew = scalew;
                                                  m_scaleh = scaleh; }
@@ -225,6 +229,9 @@ class OSDTypeImage : public OSDType
 
     QRect ImageSize() { return m_imagesize; }
 
+    int width() { return m_imagesize.width(); }
+    int height() { return m_imagesize.height(); }
+
     virtual void Draw(OSDSurface *surface, int fade, int maxfade, int xoff, 
                       int yoff);
 
@@ -321,7 +328,7 @@ class OSDTypeEditSlider : public OSDTypeImage
     int m_curval;
     int m_drawwidth;
 
-    int *m_drawMap;
+    unsigned char *m_drawMap;
 
     bool m_risvalid;
     unsigned char *m_ryuv;
@@ -432,6 +439,9 @@ class OSDTypeCC : public OSDType
     void AddCCText(const QString &text, int x, int y, int color, 
                    bool teletextmode = false);
     void ClearAllCCText();
+    int UpdateCCText(vector<ccText*> *ccbuf,
+                     int replace = 0, int scroll = 0, bool scroll_prsv = false,
+                     int scroll_yoff = 0, int scroll_ymax = 15);
 
     void Draw(OSDSurface *surface, int fade, int maxfade, int xoff, int yoff);
 
