@@ -26,12 +26,13 @@
 #include "programinfo.h"
 #include "oldsettings.h"
 #include "tv.h"
+#include "guidegrid.h"
 
 using namespace std;
 
 #include "libmyth/mythcontext.h"
 
-void RunProgramFind(bool thread)
+void RunProgramFind(bool thread, bool ggActive)
 {
     if (thread)
         qApp->lock();
@@ -41,10 +42,10 @@ void RunProgramFind(bool thread)
     ProgFinder *programFind = NULL;
     if (gContext->GetLanguage() == "ja")
         programFind = new JaProgFinder(gContext->GetMainWindow(),
-                                       "program finder");
+                                       "program finder", ggActive);
     else
         programFind = new ProgFinder(gContext->GetMainWindow(),
-                                     "program finder");
+                                     "program finder", ggActive);
 
     programFind->Initialize();
     programFind->Show();
@@ -64,18 +65,22 @@ void RunProgramFind(bool thread)
     return;
 }
 
-ProgFinder::ProgFinder(MythMainWindow *parent, const char *name)
+ProgFinder::ProgFinder(MythMainWindow *parent, const char *name, bool gg)
           : MythDialog(parent, name)
 {
     curSearch = 10;
     searchCount = 37;
+    ggActive = gg;
+
+    channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
 }
 
 void ProgFinder::Initialize(void)
 {
     running = true;
     m_db = QSqlDatabase::database();
-
+    
+    
     allowkeypress = true;
 
     baseDir = gContext->GetInstallPrefix();
@@ -112,7 +117,6 @@ void ProgFinder::Initialize(void)
 
     inSearch = 0;
     pastInitial = false;
-    searchCount = 37;
     recordingCount = 0;
 
     initData = new QString[(int)(searchCount*showsPerListing)];
@@ -331,9 +335,15 @@ void ProgFinder::updateInfo(QPainter *p)
         QString channum = "";
         QString channame = "";
         QString recording = "";
+        QString seriesid = "";
+        QString programid = "";
 
-        channum = showData[curShow].channelNum;
-        channame = showData[curShow].channelCallsign;
+        if (channelFormat.contains("<num>"))
+            channum = showData[curShow].channelNum;
+        if (channelFormat.contains("<sign>"))
+            channame = showData[curShow].channelCallsign;
+        else if (channelFormat.contains("<name>"))
+            channame = showData[curShow].channelName;
         title = progData[curProgram];
         timedate = showData[curShow].startDisplay + " - " +
                    showData[curShow].endDisplay;
@@ -343,6 +353,8 @@ void ProgFinder::updateInfo(QPainter *p)
             subtitle = "";
         description = showData[curShow].description;
         recording = showData[curShow].recText;
+        seriesid = showData[curShow].seriesid;
+        programid = showData[curShow].programid;
 
         if (gotInitData[curSearch] == 1)
         {
@@ -381,6 +393,14 @@ void ProgFinder::updateInfo(QPainter *p)
             type = (UITextType *)container->GetType("recordingstatus");
             if (type)
                 type->SetText(recording);
+
+            type = (UITextType *)container->GetType("seriesid");
+            if (type)
+                type->SetText(seriesid);
+
+            type = (UITextType *)container->GetType("programid");
+            if (type)
+                type->SetText(programid);
         }
     }
 
@@ -402,6 +422,16 @@ void ProgFinder::escape()
 
 void ProgFinder::showGuide()
 {
+    if(!ggActive)
+    {
+        QString startchannel = gContext->GetSetting("DefaultTVChannel");
+        if (startchannel == "")
+            startchannel = "3";
+
+        RunProgramGuide(startchannel);
+    }
+
+                
     running = false;
     hide();
 
@@ -500,6 +530,12 @@ void ProgFinder::getInfo(bool toggle)
                 case kSingleRecord:
                     data = tr("Recording just this showing");
                     break;
+                case kOverrideRecord:
+                    data = tr("Recording this showing with override options");
+                    break;
+                case kFindOneRecord:
+                    data = tr("Recording one showing of this program");
+                    break;
                 case kTimeslotRecord:
                     data = tr("Recording every day when shown in this timeslot");
                     break;
@@ -512,6 +548,9 @@ void ProgFinder::getInfo(bool toggle)
                 case kAllRecord:
                     data = tr("Recording all showings");
                     break;
+                case kDontRecord:
+                    data = tr("Manually not recording this showing");
+                    break;
                 case kNotRecording:
                     data = tr("Not recording this showing");
                     break;
@@ -570,7 +609,7 @@ void ProgFinder::cursorLeft()
 {
     inSearch--;
     if (inSearch == -1)
-        inSearch = 0;
+        escape();
     else
     {
         if (inSearch == 0)
@@ -1115,7 +1154,9 @@ int ProgFinder::checkRecordingStatus(int showNum)
             if (showData[showNum].subtitle == curRecordings[j].subtitle &&
                 showData[showNum].description == curRecordings[j].description)
             {
-                if (curRecordings[j].type == kSingleRecord)
+                if (curRecordings[j].type == kSingleRecord ||
+                    curRecordings[j].type == kOverrideRecord ||
+                    curRecordings[j].type == kDontRecord)
                 {
                     if (showData[showNum].startdatetime ==
                         curRecordings[j].startdatetime)
@@ -1128,7 +1169,10 @@ int ProgFinder::checkRecordingStatus(int showNum)
             {
                 if ((showData[showNum].startdatetime).time() ==
                      (curRecordings[j].startdatetime).time()
-                    && showData[showNum].channelID == curRecordings[j].chanid)
+                    && (showData[showNum].channelID == curRecordings[j].chanid
+                      || (showData[showNum].channelCallsign != ""
+                         && showData[showNum].channelCallsign ==
+                            curRecordings[j].chansign)))
                 {
                     return curRecordings[j].type;
                 }
@@ -1139,14 +1183,20 @@ int ProgFinder::checkRecordingStatus(int showNum)
                      (curRecordings[j].startdatetime).time()
                     && (showData[showNum].startdatetime).toString("dddd") ==
                      (curRecordings[j].startdatetime).toString("dddd")
-                    && showData[showNum].channelID == curRecordings[j].chanid)
+                    && (showData[showNum].channelID == curRecordings[j].chanid
+                      || (showData[showNum].channelCallsign != ""
+                         && showData[showNum].channelCallsign ==
+                            curRecordings[j].chansign)))
                 {
                     return curRecordings[j].type;
                 }
             }
             if (curRecordings[j].type == kChannelRecord)
             {
-                if (showData[showNum].channelID == curRecordings[j].chanid)
+                if (showData[showNum].channelID == curRecordings[j].chanid
+                      || (showData[showNum].channelCallsign != ""
+                         && showData[showNum].channelCallsign ==
+                            curRecordings[j].chansign))
                 {
                     return curRecordings[j].type;
                 }
@@ -1155,6 +1205,10 @@ int ProgFinder::checkRecordingStatus(int showNum)
             {
                 return curRecordings[j].type;
             }
+            if (curRecordings[j].type == kFindOneRecord)
+            {
+                return curRecordings[j].type;
+            }
         }
     }
     return 0;
@@ -1167,8 +1221,12 @@ void ProgFinder::getRecordingInfo()
     QDateTime recDateTime;
     QString thequery;
     QString data;
-    thequery = QString("SELECT chanid,starttime,startdate,title,subtitle,"
-                       "description,type FROM record;");
+
+    thequery = QString("SELECT record.chanid,starttime,startdate,"
+                       "title,subtitle,description,type,callsign,"
+                       "seriesid,programid "
+                       "FROM record,channel "
+                       "WHERE record.chanid = channel.chanid;");
 
     QSqlQuery query = m_db->exec(thequery);
 
@@ -1177,7 +1235,7 @@ void ProgFinder::getRecordingInfo()
     if (rows == -1)
     {
         cerr << "MythProgFind: Error executing query! (getRecordingInfo)\n";
-        cerr << "MythProgFind: QUERY = " << thequery << endl;
+        cerr << "MythProgFind: QUERY = " << thequery.local8Bit() << endl;
         return;
     }
 
@@ -1201,13 +1259,9 @@ void ProgFinder::getRecordingInfo()
                 curRecordings[recordingCount].subtitle = QString::fromUtf8(query.value(4).toString());
                 curRecordings[recordingCount].description = QString::fromUtf8(query.value(5).toString());
                 curRecordings[recordingCount].type = query.value(6).toInt();
-
-                if (curRecordings[recordingCount].title == QString::null)
-                    curRecordings[recordingCount].title = "";
-                if (curRecordings[recordingCount].subtitle == QString::null)
-                    curRecordings[recordingCount].subtitle = "";
-                if (curRecordings[recordingCount].description == QString::null)
-                    curRecordings[recordingCount].description = "";
+                curRecordings[recordingCount].chansign = QString::fromUtf8(query.value(7).toString());
+                curRecordings[recordingCount].seriesid = QString::fromUtf8(query.value(8).toString());
+                curRecordings[recordingCount].programid = QString::fromUtf8(query.value(9).toString());
 
                 recordingCount++;
             }
@@ -1229,12 +1283,15 @@ void ProgFinder::selectShowData(QString progTitle)
 
     QSqlQuery query(QString::null, m_db);
     query.prepare("SELECT subtitle,starttime,channel.channum,"
-                  "channel.callsign,description,endtime,channel.chanid "
+                  "channel.callsign,description,endtime,channel.chanid,"
+                  "channel.sourceid,channel.name,seriesid,programid "
                   "FROM program,channel "
                   "WHERE program.title = :TITLE AND "
                   "program.chanid = channel.chanid "
                   "AND program.starttime > :STARTTIME "
-                  "ORDER BY program.starttime;");
+                  "GROUP BY starttime,endtime,channum,callsign "
+                  "ORDER BY starttime," +
+                  gContext->GetSetting("ChannelOrdering", "channum + 0") +";");
     query.bindValue(":TITLE", progTitle.utf8());
     query.bindValue(":STARTTIME", progStart.toString("yyyyMMddhhmm50"));
 
@@ -1289,20 +1346,25 @@ void ProgFinder::selectShowData(QString progTitle)
             showData[showCount].subtitle = QString::fromUtf8(query.value(0).toString());
             showData[showCount].description = QString::fromUtf8(query.value(4).toString());
             showData[showCount].channelID = query.value(6).toString();
+            showData[showCount].channelName = query.value(8).toString();
             showData[showCount].starttime = progStart.toString("yyyyMMddhhmm");
             showData[showCount].endtime = progEnd.toString("yyyyMMddhhmm");
             rectype = checkRecordingStatus(showCount);
             showData[showCount].recording = rectype;
+            showData[showCount].seriesid = query.value(9).toString();
+            showData[showCount].programid = query.value(10).toString();
 
-            if (showData[showCount].subtitle == QString::null)
-                showData[showCount].subtitle = "";
-            if (showData[showCount].description == QString::null)
-                showData[showCount].description = "";
             switch (rectype)
             {
                 case kSingleRecord:
                     data = tr("Recording just this showing");
                     break;
+                case kOverrideRecord:
+                    data = tr("Recording this showing with override options");
+                    break;
+                case kFindOneRecord:
+                    data = tr("Recording one showing of this program");
+                    break;
                 case kTimeslotRecord:
                     data = tr("Recording every day when shown in this timeslot");
                     break;
@@ -1315,6 +1377,9 @@ void ProgFinder::selectShowData(QString progTitle)
                 case kAllRecord:
                     data = tr("Recording all showings");
                     break;
+                case kDontRecord:
+                    data = tr("Manually not recording this showing");
+                    break;
                 case kNotRecording:
                     data = tr("Not recording this showing");
                     break;
@@ -1572,30 +1637,20 @@ void ProgFinder::getAllProgramData()
 
 // japanese HIRAGANA list and more
 const char* JaProgFinder::searchChars[] = {
-    "1", "2", "3", "4", "5", "6", "7", "8", "9", "0",
-    "あ", "い", "う", "え", "お",
-    "か", "き", "く", "け", "こ",
-    "さ", "し", "す", "せ", "そ",
-    "た", "ち", "つ", "て", "と",
-    "な", "に", "ぬ", "ね", "の",
-    "は", "ひ", "ふ", "へ", "ほ",
-    "ま", "み", "む", "め", "も",
-    "や",       "ゆ",       "よ",
-    "ら", "り", "る", "れ", "ろ",
-    "わ", "を", "ん",
-    "(ス)",  "(字)", "(多)",
+    "あ", "か", "さ", "た", "な", "は", "ま", "や", "ら", "わ",
+    "英", "数",
     0,
 };
 
-JaProgFinder::JaProgFinder(MythMainWindow *parent, const char *name)
-            : ProgFinder(parent, name)
+JaProgFinder::JaProgFinder(MythMainWindow *parent, const char *name, bool gg)
+            : ProgFinder(parent, name, gg)
 {
     for (numberOfSearchChars = 0; searchChars[numberOfSearchChars];
          numberOfSearchChars++)
          ;
 
     searchCount = numberOfSearchChars;
-    curSearch = 10;
+    curSearch = 0;
 }
 
 void JaProgFinder::fillSearchData()
@@ -1609,25 +1664,58 @@ void JaProgFinder::fillSearchData()
     }
 }
 
-// searching subtitle, i hope subtitle inserted japanese HIRAGANA
+// search title_pronounce
+// we hope japanese HIRAGANA and alphabets, numerics is inserted
+// these character must required ZENKAKU
+// because query work not fine, if mysql's default charset latin1
 QString JaProgFinder::whereClauseGetSearchData(int charNum)
 {
     QDateTime progStart = QDateTime::currentDateTime();
     QString thequery;
-    thequery = QString("SELECT DISTINCT title "
-                       "FROM program "
-                       "WHERE ( title LIKE '%1%' OR subtitle LIKE '(す)%2%' "
-                       "OR subtitle LIKE '(じ)%3%' "
-                       "OR subtitle LIKE '(じ)(す)%4%' "
-                       "OR subtitle LIKE '(た)%5%' ) "
-                       "AND starttime > %6 "
-                       "ORDER BY title;")
-                        .arg(searchData[charNum].utf8())
-                        .arg(searchData[charNum].utf8())
-                        .arg(searchData[charNum].utf8())
-                        .arg(searchData[charNum].utf8())
-                        .arg(searchData[charNum].utf8())
-                        .arg(progStart.toString("yyyyMMddhhmm50"));
+
+    thequery = QString("SELECT DISTINCT title FROM program ");
+
+    switch (charNum) {
+    case 0:
+        thequery += QString("WHERE ( title_pronounce >= 'あ' AND title_pronounce <= 'お') ");
+        break;
+    case 1:
+        thequery += QString("WHERE ( title_pronounce >= 'か' AND title_pronounce <= 'ご') ");
+        break;
+    case 2:
+        thequery += QString("WHERE ( title_pronounce >= 'さ' AND title_pronounce <= 'そ') ");
+        break;
+    case 3:
+        thequery += QString("WHERE ( title_pronounce >= 'た' AND title_pronounce <= 'ど') ");
+        break;
+    case 4:
+        thequery += QString("WHERE ( title_pronounce >= 'な' AND title_pronounce <= 'の') ");
+        break;
+    case 5:
+        thequery += QString("WHERE ( title_pronounce >= 'は' AND title_pronounce <= 'ぽ') ");
+        break;
+    case 6:
+        thequery += QString("WHERE ( title_pronounce >= 'ま' AND title_pronounce <= 'も') ");
+        break;
+    case 7:
+        thequery += QString("WHERE ( title_pronounce >= 'や' AND title_pronounce <= 'よ') ");
+        break;
+    case 8:
+        thequery += QString("WHERE ( title_pronounce >= 'ら' AND title_pronounce <= 'ろ') ");
+        break;
+    case 9:
+        thequery += QString("WHERE ( title_pronounce >= 'わ' AND title_pronounce <= 'ん') ");
+        break;
+    case 10:
+        thequery += QString("WHERE ( title_pronounce >= 'Ａ' AND title_pronounce <= 'ｚ') ");
+        break;
+    case 11:
+        thequery += QString("WHERE ( title_pronounce >= '０' AND title_pronounce <= '９') ");
+        break;
+    }
+
+    thequery += QString("AND starttime > %1 ORDER BY title_pronounce;")
+                       .arg(progStart.toString("yyyyMMddhhmm50"));
     return thequery;
 }
 
