@@ -25,7 +25,7 @@ class QSqlDatabase;
 class QWidget;
 class TV;
 
-void RunProgramFind(bool thread = false);
+void RunProgramFind(bool thread = false, bool ggActive = false);
 
 class ProgFinder : public MythDialog
 {
@@ -39,11 +39,14 @@ class ProgFinder : public MythDialog
         QString endDisplay;
         QString channelNum;
         QString channelCallsign;
+        QString channelName;
         QString starttime;
         QString endtime;
 	QDateTime startdatetime;
 	int recording;
 	QString recText;
+        QString seriesid;
+        QString programid;
     };
 
     struct recordingRecord 
@@ -54,11 +57,14 @@ class ProgFinder : public MythDialog
         QString subtitle;
         QString description;
         int type;
+        QString chansign;
+        QString seriesid;
+        QString programid;
     };
 
     Q_OBJECT
   public:
-    ProgFinder(MythMainWindow *parent, const char *name = 0);
+    ProgFinder(MythMainWindow *parent, const char *name = 0, bool gg = false);
     virtual ~ProgFinder();
 
     void Initialize(void);
@@ -110,6 +116,7 @@ class ProgFinder : public MythDialog
     bool pastInitial;
     bool running;
     int *gotInitData;
+    bool ggActive;
 
     QTimer *update_Timer;
 
@@ -144,6 +151,7 @@ class ProgFinder : public MythDialog
 
     QString dateFormat;
     QString timeFormat;
+    QString channelFormat;
 
     bool allowkeypress;
 };
@@ -151,7 +159,7 @@ class ProgFinder : public MythDialog
 class JaProgFinder : public ProgFinder
 {
   public:
-    JaProgFinder(MythMainWindow *parent, const char *name = 0);
+    JaProgFinder(MythMainWindow *parent, const char *name = 0, bool gg=false);
 
   protected:
     virtual void fillSearchData();
