@@ -32,7 +32,9 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     db = ldb;
     startTime = QDateTime::currentDateTime();
     timeFormat = gContext->GetSetting("ShortDateFormat") +
-	" " + gContext->GetSetting("TimeFormat");
+        " " + gContext->GetSetting("TimeFormat");
+    channelOrdering = gContext->GetSetting("ChannelOrdering", "channum + 0");
+    channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
 
     allowEvents = true;
     allowUpdates = true;
@@ -127,8 +129,10 @@ void ProgLister::keyPressEvent(QKeyEvent *e)
             nextView();
         else if (action == "MENU")
             chooseView();
-        else if (action == "SELECT" || action == "RIGHT" || action == "LEFT")
+        else if (action == "SELECT" || action == "RIGHT")
             select();
+        else if (action == "LEFT")
+            accept();
         else if (action == "INFO")
             edit();
         else if (action == "TOGGLERECORD")
@@ -170,15 +174,15 @@ void ProgLister::LoadWindow(QDomElement &element)
             if (e.tagName() == "font")
                 theme->parseFont(e);
             else if (e.tagName() == "container")
-	    {
-		theme->parseContainer(e, name, context, area);
-		if (name.lower() == "view")
-		    viewRect = area;
-		if (name.lower() == "selector")
-		    listRect = area;
-		if (name.lower() == "program_info")
-		    infoRect = area;
-	    }
+            {
+                theme->parseContainer(e, name, context, area);
+                if (name.lower() == "view")
+                    viewRect = area;
+                if (name.lower() == "selector")
+                    listRect = area;
+                if (name.lower() == "program_info")
+                    infoRect = area;
+            }
             else
             {
                 cerr << "Unknown element: " << e.tagName() << endl;
@@ -215,7 +219,7 @@ void ProgLister::updateBackground(void)
             }
             ltype->SetText(value);
         }
-	container->Draw(&tmp, 0, 0);
+        container->Draw(&tmp, 0, 0);
     }
 
     tmp.end();
@@ -364,14 +368,14 @@ void ProgLister::setViewFromEdit(void)
         {
             QString querystr = QString("DELETE FROM keyword "
                                        "WHERE phrase = '%1';")
-                                       .arg(viewList[oldview]);
+                                       .arg(viewList[oldview].utf8());
             QSqlQuery query;
             query.exec(querystr);
         }
         if (newview < 0)
         {
             QString querystr = QString("REPLACE INTO keyword "
-                                       "VALUES('%1');").arg(text);
+                                       "VALUES('%1');").arg(text.utf8());
             QSqlQuery query;
             query.exec(querystr);
         }
@@ -398,7 +402,7 @@ void ProgLister::deleteKeyword(void)
     QString text = viewList[view];
 
     QString querystr = QString("DELETE FROM keyword WHERE phrase = '%1';")
-                               .arg(text);
+                               .arg(text.utf8());
     QSqlQuery query;
     query.exec(querystr);
 
@@ -565,10 +569,11 @@ void ProgLister::fillViewList(const QString &view)
 
     if (type == plChannel) // list by channel
     {
-        QString channelOrdering = 
-            gContext->GetSetting("ChannelOrdering", "channum + 0");
         QString querystr = "SELECT channel.chanid, channel.channum, "
-            "channel.callsign FROM channel ORDER BY " + channelOrdering + ";";
+            "channel.callsign, channel.name FROM channel "
+            "WHERE channel.visible = 1 "
+            "GROUP BY channum, callsign "
+            "ORDER BY " + channelOrdering + ";";
         QSqlQuery query;
         query.exec(querystr);
         if (query.isActive() && query.numRowsAffected())
@@ -576,15 +581,15 @@ void ProgLister::fillViewList(const QString &view)
             while (query.next())
             {
                 QString chanid = query.value(0).toString();
-                QString chantext;
                 QString channum = query.value(1).toString();
-                if (channum != QString::null && channum != "")
-                    chantext = channum;
-                else
-                    chantext = "???";
                 QString chansign = query.value(2).toString();
-                if (chansign != QString::null && chansign != "")
-                    chantext = chantext + " " + chansign;
+                QString channame = query.value(3).toString();
+
+                QString chantext = channelFormat;
+                chantext.replace("<num>", channum)
+                    .replace("<sign>", chansign)
+                    .replace("<name>", channame);
+
                 viewList << chanid;
                 viewTextList << chantext;
                 viewCount++;
@@ -603,8 +608,9 @@ void ProgLister::fillViewList(const QString &view)
             while (query.next())
             {
                 QString category = query.value(0).toString();
-                if (category <= " ")
+                if (category <= " " || category == NULL)
                     continue;
+                category = QString::fromUtf8(query.value(0).toString());
                 viewList << category;
                 viewTextList << category;
                 viewCount++;
@@ -625,6 +631,7 @@ void ProgLister::fillViewList(const QString &view)
                 QString phrase = query.value(0).toString();
                 if (phrase <= " ")
                     continue;
+                phrase = QString::fromUtf8(query.value(0).toString());
                 viewList << phrase;
                 viewTextList << phrase;
                 viewCount++;
@@ -669,48 +676,62 @@ void ProgLister::fillItemList(void)
 
     if (type == plTitle) // per title listings
     {
-        where = QString("WHERE program.title = \"%1\" "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND program.title = \"%1\" "
                         "AND program.endtime > %2 "
                         "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime,channel.channum;")
+                        "GROUP BY starttime,endtime,channum,callsign "
+                        "ORDER BY program.starttime,%3;")
                         .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        .arg(startTime.toString("yyyyMMddhhmm50"))
+                        .arg(channelOrdering);
     }
     else if (type == plNewListings) // what's new list
     {
         where = QString("LEFT JOIN oldprogram ON title=oldtitle "
-                        "WHERE oldtitle IS NULL AND program.endtime > %1 "
+                        "WHERE channel.visible = 1 "
+                        "AND oldtitle IS NULL AND program.endtime > %1 "
                         "AND program.chanid = channel.chanid "
-                        "AND ( program.airdate = 0 OR "
-                        "program.airdate >= YEAR(NOW() - INTERVAL 1 YEAR)) "
+                        "AND (program.category_type <> 'movie' OR "
+                        "(program.category_type = 'movie' AND "
+                        "program.airdate >= YEAR(NOW() - INTERVAL 2 YEAR))) "
                         "GROUP BY title ORDER BY starttime LIMIT 500;")
                         .arg(startTime.toString("yyyyMMddhhmm50"));
     }
     else if (type == plTitleSearch) // keyword search
     {
-        where = QString("WHERE program.title LIKE \"\%%1\%\" "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND program.title LIKE \"\%%1\%\" "
                         "AND program.endtime > %2 "
                         "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime,channel.channum "
+                        "GROUP BY starttime,endtime,channum,callsign "
+                        "ORDER BY program.starttime,%3 "
                         "LIMIT 500;")
                         .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        .arg(startTime.toString("yyyyMMddhhmm50"))
+                        .arg(channelOrdering);
     }
     else if (type == plDescSearch) // description search
     {
-        where = QString("WHERE (program.title LIKE \"\%%1\%\" "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND (program.title LIKE \"\%%1\%\" "
                         "OR program.subtitle LIKE \"\%%2\%\" "
                         "OR program.description LIKE \"\%%3\%\") "
                         "AND program.endtime > %4 "
                         "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime,channel.channum "
+                        "GROUP BY starttime,endtime,channum,callsign "
+                        "ORDER BY program.starttime,%5 "
                         "LIMIT 500;")
-                        .arg(viewList[curView].utf8()).arg(viewList[curView].utf8()).arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        .arg(viewList[curView].utf8())
+                        .arg(viewList[curView].utf8())
+                        .arg(viewList[curView].utf8())
+                        .arg(startTime.toString("yyyyMMddhhmm50"))
+                        .arg(channelOrdering);
     }
     else if (type == plChannel) // list by channel
     {
-        where = QString("WHERE channel.chanid = \"%1\" "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND channel.chanid = \"%1\" "
                         "AND program.endtime > %2 "
                         "AND program.chanid = channel.chanid "
                         "ORDER BY program.starttime;")
@@ -719,23 +740,29 @@ void ProgLister::fillItemList(void)
     }
     else if (type == plCategory) // list by category
     {
-        where = QString("WHERE program.category = \"\%1\" "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND program.category = \"\%1\" "
                         "AND program.endtime > %2 "
                         "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime,channel.channum "
+                        "GROUP BY starttime,endtime,channum,callsign "
+                        "ORDER BY program.starttime,%3 "
                         "LIMIT 500;")
                         .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        .arg(startTime.toString("yyyyMMddhhmm50"))
+                        .arg(channelOrdering);
     }
     else if (type == plMovies) // list movies
     {
-        where = QString("WHERE program.category_type LIKE \"\%%1\%\" "
-                        "AND program.endtime > %2 "
+        where = QString("WHERE channel.visible = 1 "
+                        "AND (program.category_type = 'movie' "
+                        "OR programid LIKE \"MV\%\") "
+                        "AND program.endtime > %1 "
                         "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime,channel.channum "
+                        "GROUP BY starttime,endtime,channum,callsign "
+                        "ORDER BY program.starttime,%2 "
                         "LIMIT 500;")
-                        .arg(tr("Movie").utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        .arg(startTime.toString("yyyyMMddhhmm50"))
+                        .arg(channelOrdering);
     }
 
     ProgramInfo::GetProgramListByQuery(db, &itemList, where);
@@ -826,7 +853,7 @@ void ProgLister::updateList(QPainter *p)
                 ProgramInfo *pi = itemList.at(i+skip);
 
                 ltype->SetItemText(i, 1, pi->startts.toString(timeFormat));
-                ltype->SetItemText(i, 2, pi->chanstr + " " + pi->chansign);
+                ltype->SetItemText(i, 2, pi->ChannelText(channelFormat));
 
                 if (pi->subtitle == "")
                     tmptitle = pi->title;
@@ -842,9 +869,9 @@ void ProgLister::updateList(QPainter *p)
 
                 ltype->SetItemText(i, 3, tmptitle);
 
-                if (pi->conflicting)
+                if (pi->recstatus == rsConflict)
                     ltype->EnableForcedFont(i, "conflicting");
-                else if (pi->recording)
+                else if (pi->recstatus <= rsWillRecord)
                     ltype->EnableForcedFont(i, "recording");
 
                 if (i + skip == curItem)
