@@ -7,7 +7,7 @@
 #include <qmap.h>
 #include "scheduledrecording.h"
 
-#define NUMPROGRAMLINES 29
+#define NUMPROGRAMLINES 35
 
 enum MarkTypes {
     MARK_UPDATED_CUT = -3,
@@ -52,16 +52,16 @@ enum RecStatusType {
     rsRecording = -2,
     rsWillRecord = -1,
     rsUnknown = 0,
-    rsManualOverride = 1,
+    rsDontRecord = 1,
     rsPreviousRecording = 2,
     rsCurrentRecording = 3,
-    rsOtherShowing = 4,
+    rsEarlierShowing = 4,
     rsTooManyRecordings = 5,
     rsCancelled = 6,
-    rsLowerRecPriority = 7,
-    rsManualConflict = 8,
-    rsAutoConflict = 9,
-    rsOverlap = 10,
+    rsConflict = 7,
+    rsLaterShowing = 8,
+    //rsUnused = 9,
+    //rsUnused = 10,
     rsLowDiskSpace = 11,
     rsTunerBusy = 12
 };
@@ -80,7 +80,7 @@ class ProgramInfo
     // returns 0 for one-time, 1 for weekdaily, 2 for weekly
     int IsProgramRecurring(void);
 
-    // checks title, subtitle, description
+    // checks for duplicates according to dupmethod
     bool IsSameProgram(const ProgramInfo& other) const;
     // checks chanid, start/end times, sourceid, cardid, inputid.
     bool IsSameTimeslot(const ProgramInfo& other) const;
@@ -99,8 +99,9 @@ class ProgramInfo
     void ApplyRecordTimeChange(QSqlDatabase *db, 
                                const QDateTime &newstartts,
                                const QDateTime &newendts);
-    void ApplyRecordRecPriorityChange(QSqlDatabase *db,
-                               const QString &newrecpriority);
+    void ApplyRecordRecPriorityChange(QSqlDatabase *db, int);
+    void ApplyRecordRecGroupChange(QSqlDatabase *db,
+                               const QString &newrecgroup);
     void ToggleRecord(QSqlDatabase *dB);
 
     ScheduledRecording* GetScheduledRecording(QSqlDatabase *db) 
@@ -109,8 +110,15 @@ class ProgramInfo
         return record;
     };
 
+    int getRecordID(QSqlDatabase *db)
+    {
+        GetProgramRecordingStatus(db);
+        recordid = record->getRecordID();
+        return recordid;
+    }
+
     void StartedRecording(QSqlDatabase *db);
-    void FinishedRecording(QSqlDatabase* db);
+    void FinishedRecording(QSqlDatabase* db, bool prematurestop);
 
     QGridLayout* DisplayWidget(QWidget *parent = NULL);
 
@@ -171,6 +179,7 @@ class ProgramInfo
     void FillInRecordInfo(vector<ProgramInfo *> &reclist);
     void EditScheduled(QSqlDatabase *db);
     void EditRecording(QSqlDatabase *db);
+    QString ChannelText(QString);
 
     int getProgramFlags(QSqlDatabase *db);
 
@@ -194,6 +203,7 @@ class ProgramInfo
     static ProgramInfo *GetProgramFromRecorded(QSqlDatabase *db,
                                                const QString &channel, 
                                                QDateTime &dtime);
+    int SecsTillStart() const { return QDateTime::currentDateTime().secsTo(startts); }
 
     QString title;
     QString subtitle;
@@ -204,7 +214,11 @@ class ProgramInfo
     QString chanstr;
     QString chansign;
     QString channame;
-    QString recpriority;
+    
+    
+    int recpriority;
+    QString recgroup;
+    int chancommfree;
 
     QString pathname;
     long long filesize;
@@ -220,28 +234,37 @@ class ProgramInfo
     int spread;
     int startCol;
 
-    bool conflicting;
-    bool recording;
-    int override;
     RecStatusType recstatus;
-    unsigned recordid;
+    RecStatusType savedrecstatus;
+    int numconflicts;
+    int conflictpriority;
+    int reactivate;             // 0 = not requested
+                                // 1 = requested, pending
+                                // 2 = requested, replaced
+                                // -1 = reactivated
+    int recordid;
     RecordingType rectype;
-    RecordingDupsType recdups;
+    RecordingDupInType dupin;
+    RecordingDupMethodType dupmethod;
 
     int sourceid;
     int inputid;
     int cardid;
+    bool shareable;
     bool conflictfixed;
 
     QString schedulerid;
 
     int programflags;
+    QString chanOutputFilters;
+    
+    QString seriesid;
+    QString programid;
+
 
 private:
     void handleRecording(QSqlDatabase *db);
     void handleNotRecording(QSqlDatabase *db);
-    void handleConflicting(QSqlDatabase *db);
-    void setOverride(QSqlDatabase *db, int override);
 
     class ScheduledRecording* record;
 };
