@@ -6,6 +6,8 @@
 #include <qlayout.h>
 #include <iostream>
 
+#define TRANSCODER_GROUP 6
+
 QString RecordingProfileParam::whereClause(void) {
   return QString("id = %1").arg(parentProfile.getProfileNum());
 }
@@ -57,6 +59,17 @@ public:
     };
 };
 
+class BTTVVolume: public CodecParam, public SliderSetting {
+public:
+    BTTVVolume(const RecordingProfile& parent):
+       CodecParam(parent, "volume"),
+       SliderSetting(0,100,1) {
+       setLabel(QObject::tr("Volume (%)"));
+       setValue(90);
+       setHelpText(QObject::tr("Recording volume of the capture card"));
+    };
+};
+
 class SampleRate: public CodecParam, public ComboBoxSetting {
 public:
     SampleRate(const RecordingProfile& parent, bool analog = true):
@@ -200,6 +213,7 @@ public:
         params->setLabel("MP3");
         params->addChild(new SampleRate(parent));
         params->addChild(new MP3Quality(parent));
+        params->addChild(new BTTVVolume(parent));
         addTarget("MP3", params);
 
         params = new VerticalConfigurationGroup(false);
@@ -212,6 +226,7 @@ public:
         params = new VerticalConfigurationGroup(false);
         params->setLabel("Uncompressed");
         params->addChild(new SampleRate(parent));
+        params->addChild(new BTTVVolume(parent));
         addTarget("Uncompressed", params);
     };
 
@@ -344,6 +359,30 @@ public:
     };
 };
 
+class MPEG4OptionIDCT: public CodecParam, public CheckBoxSetting {
+public:
+    MPEG4OptionIDCT(const RecordingProfile& parent):
+        CodecParam(parent, "mpeg4optionidct") {
+        setLabel(QObject::tr("Enable interlaced DCT encoding"));
+        setValue(false);
+        setHelpText(QObject::tr("If set, the MPEG4 encoder will use interlaced "
+                    "DCT encoding.  You probably want this when encoding "
+                    "interlaced video."));
+    };
+};
+
+class MPEG4OptionIME: public CodecParam, public CheckBoxSetting {
+public:
+    MPEG4OptionIME(const RecordingProfile& parent):
+        CodecParam(parent, "mpeg4optionime") {
+        setLabel(QObject::tr("Enable interlaced motion estimation"));
+        setValue(false);
+        setHelpText(QObject::tr("If set, the MPEG4 encoder will use interlaced "
+                    "motion estimation.  You probably want this when encoding "
+                    "interlaced video."));
+    };
+};
+
 class MPEG4OptionVHQ: public CodecParam, public CheckBoxSetting {
 public:
     MPEG4OptionVHQ(const RecordingProfile& parent):
@@ -501,9 +540,18 @@ public:
         params->addChild(new MPEG4MinQuality(parent));
         params->addChild(new MPEG4QualDiff(parent));
         params->addChild(new MPEG4ScaleBitrate(parent));
-        params->addChild(new MPEG4OptionVHQ(parent));
-        params->addChild(new MPEG4Option4MV(parent));
 
+        HorizontalConfigurationGroup *hq;
+        hq = new HorizontalConfigurationGroup(false, false);
+        hq->addChild(new MPEG4OptionVHQ(parent));
+        hq->addChild(new MPEG4Option4MV(parent));
+        params->addChild(hq);
+
+        HorizontalConfigurationGroup *inter;
+        inter = new HorizontalConfigurationGroup(false, false);
+        inter->addChild(new MPEG4OptionIDCT(parent));
+        inter->addChild(new MPEG4OptionIME(parent));
+        params->addChild(inter);
         addTarget("MPEG-4", params);
 
         // We'll eventually want to add MPEG2 software encoding params here
@@ -761,11 +809,24 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
         QString query = QString("SELECT name, id FROM recordingprofiles "
                                 "WHERE profilegroup = %1 ORDER BY id;")
                                 .arg(group);
-       QSqlQuery result = db->exec(query);
+
+        QSqlQuery result = db->exec(query);
         if (result.isActive() && result.numRowsAffected() > 0)
+        {
             while (result.next())
-                setting->addSelection(result.value(0).toString(),
-                                      result.value(1).toString());
+            {
+                if(group == TRANSCODER_GROUP)
+                {
+                    setting->addSelection(QString( "From %1").arg(result.value(0).toString()),
+                                          result.value(1).toString());
+                }
+                else
+                {
+                    setting->addSelection(result.value(0).toString(),
+                                          result.value(1).toString());    
+                }
+            }
+        }
     }
 }
 
