@@ -107,6 +107,17 @@ void RemoteDeleteRecording(ProgramInfo *pginfo, bool forgetHistory)
     }
 }
 
+bool RemoteReactivateRecording(ProgramInfo *pginfo)
+{
+    QStringList strlist = QString("REACTIVATE_RECORDING");
+    pginfo->ToStringList(strlist);
+
+    if (!gContext->SendReceiveStringList(strlist))
+        return false;
+
+    return strlist[0].toInt();
+}
+
 bool RemoteGetAllPendingRecordings(vector<ProgramInfo *> &recordinglist)
 {
     QStringList strlist = QString("QUERY_GETALLPENDING");
@@ -164,10 +175,9 @@ void RemoteGetAllScheduledRecordings(vector<ProgramInfo *> &scheduledlist)
     }
 }
 
-vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo,
-                                             bool removenonplaying)
+vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo)
 {
-    QString cmd = QString("QUERY_GETCONFLICTING %1").arg(removenonplaying);
+    QString cmd = QString("QUERY_GETCONFLICTING");
     QStringList strlist = cmd;
     pginfo->ToStringList(strlist);
 
@@ -199,6 +209,21 @@ vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo,
     return retlist;
 }
 
+RemoteEncoder *RemoteRequestNextFreeRecorder(int curr)
+{
+    QStringList strlist = "GET_NEXT_FREE_RECORDER";
+    strlist << QString("%1").arg(curr);
+
+    if (!gContext->SendReceiveStringList(strlist, true))
+        return NULL;
+
+    int num = strlist[0].toInt();
+    QString hostname = strlist[1];
+    int port = strlist[2].toInt();
+
+    return new RemoteEncoder(num, hostname, port);
+}
+
 RemoteEncoder *RemoteRequestRecorder(void)
 {
     QStringList strlist = "GET_FREE_RECORDER";
