@@ -13,12 +13,13 @@ bool RemoteCheckFile(ProgramInfo *pginfo);
 void RemoteQueueTranscode(ProgramInfo *pginfo, int state);
 void RemoteStopRecording(ProgramInfo *pginfo);
 void RemoteDeleteRecording(ProgramInfo *pginfo, bool forgetHistory);
+bool RemoteReactivateRecording(ProgramInfo *pginfo);
 bool RemoteGetAllPendingRecordings(vector<ProgramInfo *> &recordinglist);
 void RemoteGetAllScheduledRecordings(vector<ProgramInfo *> &scheduledlist);
-vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo,
-                                             bool removenonplaying);
+vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo);
 void RemoteSendMessage(const QString &message);
 RemoteEncoder *RemoteRequestRecorder(void);
+RemoteEncoder *RemoteRequestNextFreeRecorder(int curr);
 RemoteEncoder *RemoteGetExistingRecorder(ProgramInfo *pginfo);
 RemoteEncoder *RemoteGetExistingRecorder(int recordernum);
 void RemoteGeneratePreviewPixmap(ProgramInfo *pginfo);
