@@ -13,11 +13,18 @@
 // be compared to another.  Lower number means more specific.
 int RecTypePriority(RecordingType rectype)
 {
-    if (rectype == kChannelRecord ||
-        rectype == kAllRecord)
-        return rectype+3;
-    else
-        return rectype;
+    switch (rectype) {
+    case kNotRecording:   return 0; break;
+    case kDontRecord:     return 1; break;
+    case kOverrideRecord: return 2; break;
+    case kSingleRecord:   return 3; break;
+    case kFindOneRecord:  return 4; break;
+    case kWeekslotRecord: return 5; break;
+    case kTimeslotRecord: return 6; break;
+    case kChannelRecord:  return 7; break;
+    case kAllRecord:      return 9; break;
+    default: return 9;
+    }
 }
 
 class SRSetting: public SimpleDBStorage {
@@ -45,23 +52,57 @@ protected:
     const ScheduledRecording& parent;
 };
 
+// NOTE: if this changes, you _MUST_ update the RecTypePriority function above.
 class SRRecordingType: public ComboBoxSetting, public SRSetting {
 public:
     SRRecordingType(const ScheduledRecording& parent):
         SRSetting(parent, "type") {
         setLabel(QObject::tr("Schedule"));
+        addSelection(QString::number(kNotRecording));
+        addSelection(QString::number(kSingleRecord));
+        addSelection(QString::number(kFindOneRecord));
+        addSelection(QString::number(kWeekslotRecord));
+        addSelection(QString::number(kTimeslotRecord));
+        addSelection(QString::number(kChannelRecord));
+        addSelection(QString::number(kAllRecord));
+        addSelection(QString::number(kOverrideRecord));
+        addSelection(QString::number(kDontRecord));
+    };
+    void addNormalSelections(void) {
         addSelection(QObject::tr("Do not record this program"),
                      QString::number(kNotRecording));
         addSelection(QObject::tr("Record only this showing"),
                      QString::number(kSingleRecord));
+        addSelection(QObject::tr("Record one showing of this program"),
+                     QString::number(kFindOneRecord));
+        addSelection(QObject::tr("Record in this timeslot every week"),
+                     QString::number(kWeekslotRecord));
         addSelection(QObject::tr("Record in this timeslot every day"),
                      QString::number(kTimeslotRecord));
         addSelection(QObject::tr("Record at any time on this channel"),
                      QString::number(kChannelRecord));
         addSelection(QObject::tr("Record at any time on any channel"),
                      QString::number(kAllRecord));
-        addSelection(QObject::tr("Record in this timeslot every week"),
-                     QString::number(kWeekslotRecord));
+    };
+    void addOverrideSelections(void) {
+        addSelection(QObject::tr("Record this showing with normal options"),
+                     QString::number(kNotRecording));
+        addSelection(QObject::tr("Record this showing with override options"),
+                     QString::number(kOverrideRecord));
+        addSelection(QObject::tr("Do not record this showing"),
+                     QString::number(kDontRecord));
+    };
+    virtual void load(QSqlDatabase* db) {
+        SimpleDBStorage::load(db);
+        QString tempval = getValue();
+        clearSelections();
+        if (tempval.toInt() == kOverrideRecord ||
+            tempval.toInt() == kDontRecord)
+            addOverrideSelections();
+        else
+            addNormalSelections();
+        setValue(getValueIndex(tempval));
+        setUnchanged();
     };
 };
 
@@ -83,15 +124,33 @@ public:
     };
 };
 
-class SRRecordDups: public ComboBoxSetting, public SRSetting {
+class SRDupIn: public ComboBoxSetting, public SRSetting {
 public:
-    SRRecordDups(const ScheduledRecording& parent):
-        SRSetting(parent, "recorddups") {
-        setLabel(QObject::tr("Record duplicates"));
-        addSelection(QObject::tr("Never"), QString::number(kRecordDupsNever));
-        addSelection(QObject::tr("If deleted"),
-                     QString::number(kRecordDupsIfDeleted));
-        addSelection(QObject::tr("Always"), QString::number(kRecordDupsAlways));
+    SRDupIn(const ScheduledRecording& parent):
+        SRSetting(parent, "dupin") {
+        setLabel(QObject::tr("Duplicate Location"));
+        addSelection(QObject::tr("All places"),
+                 QString::number(kDupsInAll));
+        addSelection(QObject::tr("Current Recs"),
+                 QString::number(kDupsInRecorded));
+        addSelection(QObject::tr("Previous Recs"),
+                 QString::number(kDupsInOldRecorded));
+    };
+};
+
+class SRDupMethod: public ComboBoxSetting, public SRSetting {
+public:
+    SRDupMethod(const ScheduledRecording& parent):
+        SRSetting(parent, "dupmethod") {
+        setLabel(QObject::tr("Duplicate Check"));
+        addSelection(QObject::tr("Sub & Desc"),
+                 QString::number(kDupCheckSubDesc));
+        addSelection(QObject::tr("Subtitle"),
+                 QString::number(kDupCheckSub));
+        addSelection(QObject::tr("Description"),
+                 QString::number(kDupCheckDesc));
+        addSelection(QObject::tr("None"),
+                 QString::number(kDupCheckNone));
     };
 };
 
@@ -103,20 +162,20 @@ public:
     };
 };
 
-class SRPreRoll: public SpinBoxSetting, public SRSetting {
+class SRStartOffset: public SpinBoxSetting, public SRSetting {
 public:
-    SRPreRoll(const ScheduledRecording& parent):
-        SpinBoxSetting(-60, 60, 10, true),
-        SRSetting(parent, "preroll") {
+    SRStartOffset(const ScheduledRecording& parent):
+        SpinBoxSetting(-120, 120, 10, true),
+        SRSetting(parent, "startoffset") {
         setLabel(QObject::tr("Start Early (minutes)"));
     };
 };
 
-class SRPostRoll: public SpinBoxSetting, public SRSetting {
+class SREndOffset: public SpinBoxSetting, public SRSetting {
 public:
-    SRPostRoll(const ScheduledRecording& parent):
-        SpinBoxSetting(-60, 240, 10, true),
-        SRSetting(parent, "postroll") {
+    SREndOffset(const ScheduledRecording& parent):
+        SpinBoxSetting(-120, 240, 10, true),
+        SRSetting(parent, "endoffset") {
         setLabel(QObject::tr("End Late   (minutes)"));
     };
 };
@@ -126,8 +185,7 @@ public:
     SRMaxEpisodes(const ScheduledRecording& parent):
         SpinBoxSetting(0, 60, 1),
         SRSetting(parent, "maxepisodes") {
-        setLabel(QObject::tr("Maximum number of episodes to keep "
-                             "(set to 0 for ALL)"));
+        setLabel(QObject::tr("Max episodes"));
     };
 };
 
@@ -135,8 +193,7 @@ class SRMaxNewest: public CheckBoxSetting, public SRSetting {
 public:
     SRMaxNewest(const ScheduledRecording& parent):
         SRSetting(parent, "maxnewest") {
-        setLabel(QObject::tr("Record new and delete oldest once "
-                             "maximum count is reached"));
+        setLabel(QObject::tr("Delete oldest over Max"));
         setValue(false);
     };
 };
@@ -148,6 +205,13 @@ public:
     };
 };
 
+class SRStation: public LineEditSetting, public SRSetting {
+public:
+    SRStation(const ScheduledRecording& parent): SRSetting(parent, "station") {
+        setVisible(false);
+    };
+};
+
 class SRTitle: public LineEditSetting, public SRSetting {
 public:
     SRTitle(const ScheduledRecording& parent):
@@ -207,13 +271,51 @@ public:
 class SRRecPriority: public SpinBoxSetting, public SRSetting {
 public:
     SRRecPriority(const ScheduledRecording& parent): 
-        SpinBoxSetting(-99, 99, 1),
+        SpinBoxSetting(-99, 99, 5, true),
         SRSetting(parent, "recpriority") {
         setLabel(QObject::tr("Priority"));
         setValue(0);
     };
 };
 
+class SRRecGroup: public ComboBoxSetting, public SRSetting {
+public:
+    SRRecGroup(const ScheduledRecording& parent):
+        ComboBoxSetting(true),
+        SRSetting(parent, "recgroup") {
+        setLabel(QObject::tr("Recording Group"));
+    };
+
+    virtual void load(QSqlDatabase *db) {
+        fillSelections(db);
+        SRSetting::load(db);
+    };
+
+    virtual void fillSelections(QSqlDatabase *db) {
+        addSelection(QString("Default"), QString("Default"));
+
+        QString thequery = QString("SELECT DISTINCT recgroup from recorded "
+                                   "WHERE recgroup <> '%1'")
+                                   .arg(QString("Default"));
+        QSqlQuery query = db->exec(thequery);
+
+        if (query.isActive() && query.numRowsAffected() > 0)
+            while (query.next())
+                addSelection(query.value(0).toString(),
+                             query.value(0).toString());
+
+        thequery = QString("SELECT DISTINCT recgroup from record "
+                           "WHERE recgroup <> '%1'")
+                           .arg(QString("Default"));
+        query = db->exec(thequery);
+
+        if (query.isActive() && query.numRowsAffected() > 0)
+            while (query.next())
+                addSelection(query.value(0).toString(),
+                             query.value(0).toString());
+    };
+};
+
 class SRCategory: public LineEditSetting, public SRSetting {
 public:
     SRCategory(const ScheduledRecording& parent):
@@ -222,17 +324,35 @@ public:
     };
 };
 
+class SRSeriesid: public LineEditSetting, public SRSetting {
+public:
+    SRSeriesid(const ScheduledRecording& parent):
+        SRSetting(parent, "seriesid") {
+        setVisible(false);
+    };
+};
+
+class SRProgramid: public LineEditSetting, public SRSetting {
+public:
+    SRProgramid(const ScheduledRecording& parent):
+        SRSetting(parent, "programid") {
+        setVisible(false);
+    };
+};
+
 ScheduledRecording::ScheduledRecording() {
     addChild(id = new ID());
     addChild(type = new SRRecordingType(*this));
     addChild(profile = new SRProfileSelector(*this));
-    addChild(recorddups = new SRRecordDups(*this));
+    addChild(dupin = new SRDupIn(*this));
+    addChild(dupmethod = new SRDupMethod(*this));
     addChild(autoexpire = new SRAutoExpire(*this));
     addChild(maxepisodes = new SRMaxEpisodes(*this));
-    addChild(preroll = new SRPreRoll(*this));
-    addChild(postroll = new SRPostRoll(*this));
+    addChild(startoffset = new SRStartOffset(*this));
+    addChild(endoffset = new SREndOffset(*this));
     addChild(maxnewest = new SRMaxNewest(*this));
     addChild(channel = new SRChannel(*this));
+    addChild(station = new SRStation(*this));
     addChild(title = new SRTitle(*this));
     addChild(subtitle = new SRSubtitle(*this));
     addChild(description = new SRDescription(*this));
@@ -242,13 +362,28 @@ ScheduledRecording::ScheduledRecording() {
     addChild(endDate = new SREndDate(*this));
     addChild(category = new SRCategory(*this));
     addChild(recpriority = new SRRecPriority(*this));
+    addChild(recgroup = new SRRecGroup(*this));
+    addChild(seriesid = new SRSeriesid(*this));
+    addChild(programid = new SRProgramid(*this));
 
     m_pginfo = NULL;
 }
 
-void ScheduledRecording::fromProgramInfo(ProgramInfo* proginfo)
+void ScheduledRecording::load(QSqlDatabase *db)
 {
+    if (getRecordID())
+        ConfigurationGroup::load(db);
+}
+
+void ScheduledRecording::fromProgramInfo(ProgramInfo* proginfo, 
+                                         QSqlDatabase *db)
+{
+    id->setValue(0);
+    id->setUnchanged();
+    type->clearSelections();
+    type->addNormalSelections();
     channel->setValue(proginfo->chanid);
+    station->setValue(proginfo->chansign);
     title->setValue(proginfo->title);
     subtitle->setValue(proginfo->subtitle);
     description->setValue(proginfo->description);
@@ -257,238 +392,55 @@ void ScheduledRecording::fromProgramInfo(ProgramInfo* proginfo)
     endTime->setValue(proginfo->endts.time());
     endDate->setValue(proginfo->endts.date());
     category->setValue(proginfo->category);
-    recpriority->setValue(proginfo->recpriority.toInt());
+    seriesid->setValue(proginfo->seriesid);
+    programid->setValue(proginfo->programid);
+
+    profile->fillSelections(db);
+    recgroup->fillSelections(db);
+    dupin->setValue(0);
+    dupmethod->setValue(0);
     autoexpire->setValue(gContext->GetNumSetting("AutoExpireDefault", 0));
+    maxepisodes->setValue(0);
+    maxnewest->setValue(0);
+    startoffset->setValue(0);
+    endoffset->setValue(0);
+    recpriority->setValue(0);
+    recgroup->setValue("Default");
 }
 
-void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
-                                                 list<ProgramInfo*>& proglist) {
-     QString query = QString(
-"SELECT DISTINCT channel.chanid, channel.sourceid, "
-"program.starttime, program.endtime, "
-"program.title, program.subtitle, program.description, "
-"channel.channum, channel.callsign, channel.name, "
-"oldrecorded.starttime IS NOT NULL AS oldrecduplicate, program.category, "
-"record.recpriority, record.recorddups, "
-"recorded.starttime IS NOT NULL as recduplicate, record.type, "
-"record.recordid, recordoverride.type, "
-"program.starttime - INTERVAL record.preroll minute, "
-"program.endtime + INTERVAL record.postroll minute, "
-"program.previouslyshown "
-"FROM record "
-" INNER JOIN channel ON (channel.chanid = program.chanid) "
-" INNER JOIN program ON (program.title = record.title) "
-" LEFT JOIN oldrecorded ON "
-"  ( "
-"    oldrecorded.title IS NOT NULL AND oldrecorded.title <> '' AND program.title = oldrecorded.title "
-"     AND "
-"    oldrecorded.subtitle IS NOT NULL AND oldrecorded.subtitle <> '' AND program.subtitle = oldrecorded.subtitle "
-"     AND "
-"    oldrecorded.description IS NOT NULL AND oldrecorded.description <> '' AND program.description = oldrecorded.description"
-"  ) "
-" LEFT JOIN recorded ON "
-"  ( "
-"    recorded.title IS NOT NULL AND recorded.title <> '' AND program.title = recorded.title "
-"     AND "
-"    recorded.subtitle IS NOT NULL AND recorded.subtitle <> '' AND program.subtitle = recorded.subtitle "
-"     AND "
-"    recorded.description IS NOT NULL AND recorded.description <> '' AND program.description = recorded.description"
-"  ) "
-" LEFT JOIN recordoverride ON "
-"  ( "
-"    record.recordid = recordoverride.recordid "
-"    AND program.chanid = recordoverride.chanid "
-"    AND program.starttime = recordoverride.starttime "
-"    AND program.endtime = recordoverride.endtime "
-"    AND program.title = recordoverride.title "
-"    AND program.subtitle = recordoverride.subtitle "
-"    AND program.description = recordoverride.description "
-"  ) "
-"WHERE "
-"((record.type = %1) " // allrecord
-" OR "
-" ((record.chanid = program.chanid) " // channel matches
-"  AND "
-"  ((record.type = %2) " // channelrecord
-"   OR"
-"   ((TIME_TO_SEC(record.starttime) = TIME_TO_SEC(program.starttime)) " // timeslot matches
-"    AND "
-"    ((record.type = %3) " // timeslotrecord
-"     OR"
-"     ((DAYOFWEEK(record.startdate) = DAYOFWEEK(program.starttime) "
-"      AND "
-"      ((record.type = %4) " // weekslotrecord
-"       OR"
-"       ((TO_DAYS(record.startdate) = TO_DAYS(program.starttime)) " // date matches
-"        AND "
-"        (TIME_TO_SEC(record.endtime) = TIME_TO_SEC(program.endtime)) "
-"        AND "
-"        (TO_DAYS(record.enddate) = TO_DAYS(program.endtime)) "
-"        )"
-"       )"
-"      )"
-"     )"
-"    )"
-"   )"
-"  )"
-" )"
-");")
-        .arg(kAllRecord)
-        .arg(kChannelRecord)
-        .arg(kTimeslotRecord)
-        .arg(kWeekslotRecord);
-
-     QSqlQuery result = db->exec(query);
-     QDateTime now = QDateTime::currentDateTime();
-
-     if (result.isActive() && result.numRowsAffected() > 0)
-     {
-         while (result.next()) {
-             ProgramInfo *proginfo = new ProgramInfo;
-             proginfo->recording = true;
-             proginfo->recstatus = rsWillRecord;
-             proginfo->chanid = result.value(0).toString();
-             proginfo->sourceid = result.value(1).toInt();
-             proginfo->startts = result.value(2).toDateTime();
-             proginfo->endts = result.value(3).toDateTime();
-             proginfo->title = QString::fromUtf8(result.value(4).toString());
-             proginfo->subtitle = QString::fromUtf8(result.value(5).toString());
-             proginfo->description = QString::fromUtf8(result.value(6).toString());
-             proginfo->chanstr = result.value(7).toString();
-             proginfo->chansign = result.value(8).toString();
-             proginfo->channame = QString::fromUtf8(result.value(9).toString());
-             proginfo->category = QString::fromUtf8(result.value(11).toString());
-             proginfo->recpriority = result.value(12).toString();
-             proginfo->recdups = RecordingDupsType(result.value(13).toInt());
-             proginfo->rectype = RecordingType(result.value(15).toInt());
-             proginfo->recordid = result.value(16).toInt();
-             proginfo->override = result.value(17).toInt();
-
-             proginfo->recstartts = result.value(18).toDateTime();
-             proginfo->recendts = result.value(19).toDateTime();
-
-             if (proginfo->recstartts >= proginfo->recendts)
-             {
-                 // pre/post-roll are invalid so ignore
-                 proginfo->recstartts = proginfo->startts;
-                 proginfo->recendts = proginfo->endts;
-             }
-
-             proginfo->repeat = result.value(20).toInt();
-
-             if (proginfo->override == 2)
-             {
-                 proginfo->recording = false;
-                 proginfo->recstatus = rsManualOverride;
-             }
-             else if (proginfo->rectype != kSingleRecord &&
-                      proginfo->override != 1 &&
-                      proginfo->recdups != kRecordDupsAlways)
-             {
-                 if (proginfo->recdups == kRecordDupsIfDeleted)
-                 {
-                     if (result.value(14).toInt())
-                     {
-                         proginfo->recording = false;
-                         proginfo->recstatus = rsCurrentRecording;
-                     }
-                 }
-                 else
-                 {
-                     if (result.value(10).toInt())
-                     {
-                         proginfo->recording = false;
-                         proginfo->recstatus = rsPreviousRecording;
-                     }
-                 }
-             }
-
-             // would save many queries to create and populate a
-             // ScheduledRecording and put it in the proginfo at the
-             // same time, since it will be loaded later anyway with
-             // multiple queries
-
-             if (proginfo->title == QString::null)
-                 proginfo->title = "";
-             if (proginfo->subtitle == QString::null)
-                 proginfo->subtitle = "";
-             if (proginfo->description == QString::null)
-                 proginfo->description = "";
-             if (proginfo->category == QString::null)
-                 proginfo->category = "";
-
-             if (proginfo->recendts < now)
-                 delete proginfo;
-             else 
-                 proglist.push_back(proginfo);
-         }
-    }
-    else if (!result.isActive())
-        MythContext::DBError("findAllProgramsToRecord", result);
-}
-
-void ScheduledRecording::findAllScheduledPrograms(QSqlDatabase* db,
-                                                  list<ProgramInfo*>& proglist)
+void ScheduledRecording::makeOverride(ProgramInfo* proginfo)
 {
-    QString temptime, tempdate;
-    QString query = QString("SELECT record.chanid, record.starttime, "
-"record.startdate, record.endtime, record.enddate, record.title, "
-"record.subtitle, record.description, record.recpriority, record.type, "
-"channel.name, record.recordid FROM record "
-"LEFT JOIN channel ON (channel.chanid = record.chanid) "
-"ORDER BY title ASC;");
+    if (type->getValue().toInt() == kOverrideRecord ||
+        type->getValue().toInt() == kDontRecord)
+        return;
 
-    QSqlQuery result = db->exec(query);
+    id->setValue(0);
+    id->setUnchanged();
+    type->clearSelections();
+    type->addOverrideSelections();
+    channel->setValue(proginfo->chanid);
+    station->setValue(proginfo->chansign);
+    title->setValue(proginfo->title);
+    subtitle->setValue(proginfo->subtitle);
+    description->setValue(proginfo->description);
+    startTime->setValue(proginfo->startts.time());
+    startDate->setValue(proginfo->startts.date());
+    endTime->setValue(proginfo->endts.time());
+    endDate->setValue(proginfo->endts.date());
+    category->setValue(proginfo->category);
+    seriesid->setValue(proginfo->seriesid);
+    programid->setValue(proginfo->programid);
 
-    if (result.isActive() && result.numRowsAffected() > 0)
-        while (result.next()) 
-        {
-            ProgramInfo *proginfo = new ProgramInfo;
-            proginfo->chanid = result.value(0).toString();
-            proginfo->rectype = RecordingType(result.value(9).toInt());
-            proginfo->recordid = result.value(11).toInt();
-
-            if (proginfo->rectype == kSingleRecord || 
-                proginfo->rectype == kTimeslotRecord ||
-                proginfo->rectype == kWeekslotRecord) 
-            {
-                temptime = result.value(1).toString();
-                tempdate = result.value(2).toString();
-                proginfo->startts = QDateTime::fromString(tempdate+":"+temptime,
-                                                          Qt::ISODate);
-                temptime = result.value(3).toString();
-                tempdate = result.value(4).toString();
-                proginfo->endts = QDateTime::fromString(tempdate+":"+temptime,
-                                                        Qt::ISODate);
-            }
-            else 
-            {
-                // put currentDateTime() in time fields to prevent
-                // Invalid date/time warnings later
-                proginfo->startts = QDateTime::currentDateTime();
-                proginfo->startts.setTime(QTime(0,0));
-                proginfo->endts = QDateTime::currentDateTime();
-                proginfo->endts.setTime(QTime(0,0));
-            }
-
-            proginfo->title = QString::fromUtf8(result.value(5).toString());
-            if (proginfo->rectype == kSingleRecord)
-                proginfo->subtitle = QString::fromUtf8(result.value(6).toString());
-            proginfo->description = QString::fromUtf8(result.value(7).toString());
-            proginfo->recpriority = result.value(8).toString();
-            proginfo->channame = QString::fromUtf8(result.value(10).toString());
-            proginfo->recstartts = proginfo->startts;
-            proginfo->recendts = proginfo->endts;
-
-            if (proginfo->title == QString::null)
-                proginfo->title = "";
-            if (proginfo->subtitle == QString::null)
-                proginfo->subtitle = "";
-            if (proginfo->description == QString::null)
-                proginfo->description = "";
-
-            proglist.push_back(proginfo);
-        }
+    profile->setChanged();
+    dupin->setChanged();
+    dupmethod->setChanged();
+    autoexpire->setChanged();
+    maxepisodes->setChanged();
+    maxnewest->setChanged();
+    startoffset->setChanged();
+    endoffset->setChanged();
+    recpriority->setChanged();
+    recgroup->setChanged();
 }
 
 bool ScheduledRecording::loadByProgram(QSqlDatabase* db,
@@ -509,6 +461,7 @@ bool ScheduledRecording::loadByProgram(QSqlDatabase* db,
 
     // prevent the SQL from breaking if chanid is null
     QString chanid = proginfo->chanid;
+    QString chansign = proginfo->chansign;
     if (!chanid || chanid == "")
          chanid = "0";
 
@@ -518,23 +471,28 @@ bool ScheduledRecording::loadByProgram(QSqlDatabase* db,
 "SELECT "
 "recordid "
 "FROM record "
+"LEFT JOIN channel ON (record.chanid = channel.chanid) "
 "WHERE "
 "record.title = \"%1\" "
 "AND "
-"((record.type = %2) " // allrecord
+"((record.type = %2 " // allrecord
+"OR record.type = %3) " // findonerecord
 " OR "
-" ((record.chanid = %3) " // channel matches
+" ((record.chanid = %4 " // channel matches
+" OR (callsign <> '' AND callsign = '%5')) "
 "  AND "
-"  ((record.type = %4) " // channelrecord
+"  ((record.type = %6) " // channelrecord
 "   OR"
-"   (((TIME_TO_SEC(record.starttime) = TIME_TO_SEC('%5')) " // timeslot matches
+"   (((TIME_TO_SEC(record.starttime) = TIME_TO_SEC('%7')) " // timeslot matches
 "      AND "
-"     (TIME_TO_SEC(record.endtime) = TIME_TO_SEC('%6')) "
+"     (TIME_TO_SEC(record.endtime) = TIME_TO_SEC('%8')) "
 "    )"
 "    AND ")
         .arg(sqltitle.utf8())
         .arg(kAllRecord)
+        .arg(kFindOneRecord)
         .arg(chanid)
+        .arg(chansign)
         .arg(kChannelRecord)
         .arg(proginfo->startts.time().toString(Qt::ISODate))
         .arg(proginfo->endts.time().toString(Qt::ISODate));
@@ -571,7 +529,7 @@ query += QString(
             loadByID(db, result.value(0).toInt());
             return true;
         } else {
-            fromProgramInfo(proginfo);
+            fromProgramInfo(proginfo, db);
         }
     } else {
         MythContext::DBError("loadByProgram", result);
@@ -590,7 +548,7 @@ RecordingType ScheduledRecording::getRecordingType(void) const {
 }
 
 void ScheduledRecording::setRecordingType(RecordingType newType) {
-    type->setValue((int)newType);
+    type->setValue(type->getValueIndex(QString::number(newType)));
 }
 
 bool ScheduledRecording::GetAutoExpire(void) const {
@@ -618,10 +576,18 @@ void ScheduledRecording::save(QSqlDatabase* db) {
 }
 
 void ScheduledRecording::remove(QSqlDatabase* db) {
-    QString query = QString("DELETE FROM record WHERE recordid = %1").arg(getRecordID());
+    if (!getRecordID())
+        return;
+
+    QString query = QString("DELETE FROM record WHERE recordid = %1")
+                            .arg(getRecordID());
     db->exec(query);
-    query = QString("DELETE FROM recordoverride WHERE recordid = %1").arg(getRecordID());
+    query = QString("DELETE FROM recordoverride WHERE recordid = %1")
+                    .arg(getRecordID());
     db->exec(query);
+    query = QString("UPDATE recordid FROM recorded SET recordid = NULL "
+                    "WHERE recordid = %1")
+                    .arg(getRecordID());
 }
 
 void ScheduledRecording::signalChange(QSqlDatabase* db) {
@@ -673,9 +639,15 @@ bool ScheduledRecording::hasChanged(QSqlDatabase* db) {
 void ScheduledRecording::doneRecording(QSqlDatabase* db, 
                                        const ProgramInfo& proginfo) 
 {
-    if (getRecordingType() == kSingleRecord)
+    if (getRecordingType() == kFindOneRecord)
         remove(db);
 
+    addHistory(db, proginfo);
+}
+
+void ScheduledRecording::addHistory(QSqlDatabase* db, 
+                                    const ProgramInfo& proginfo) 
+{
     QString sqltitle = proginfo.title;
     QString sqlsubtitle = proginfo.subtitle;
     QString sqldescription = proginfo.description;
@@ -686,22 +658,25 @@ void ScheduledRecording::doneRecording(QSqlDatabase* db,
     sqldescription.replace(QRegExp("\""), QString("\\\""));
     sqlcategory.replace(QRegExp("\""), QString("\\\""));
 
-    QString query = QString("INSERT INTO oldrecorded (chanid,starttime,endtime,title,"
-                            "subtitle,description,category) "
+    QString query = QString("INSERT INTO oldrecorded (chanid,starttime,"
+                            "endtime,title,subtitle,description,category,"
+                            "seriesid,programid) "
                             "VALUES(%1,\"%2\",\"%3\",\"%4\",\"%5\",\"%6\","
-                            "\"%7\");")
+                            "\"%7\",\"%8\",\"%9\");")
         .arg(proginfo.chanid)
         .arg(proginfo.recstartts.toString(Qt::ISODate))
         .arg(proginfo.recendts.toString(Qt::ISODate))
         .arg(sqltitle.utf8()) 
         .arg(sqlsubtitle.utf8())
         .arg(sqldescription.utf8())
-        .arg(sqlcategory.utf8());
+        .arg(sqlcategory.utf8())
+        .arg(proginfo.seriesid)
+        .arg(proginfo.programid);
 
     QSqlQuery result = db->exec(query);
     if (!result.isActive())
     {
-        MythContext::DBError("doneRecording", result);
+        MythContext::DBError("addHistory", result);
     }
     else
     {
@@ -760,32 +735,96 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
     f->setLineWidth((int)(4 * hmult));
     vbox->addWidget(f);    
 
-    vbox->addWidget(type->configWidget(this, dialog));
+    typeWidget = type->configWidget(this, dialog);
+    vbox->addWidget(typeWidget);
 
     QHBoxLayout* hbox = new QHBoxLayout(vbox, (int)(20 * wmult));
     QVBoxLayout* vbox1 = new QVBoxLayout(hbox, (int)(10 * wmult));
     QVBoxLayout* vbox2 = new QVBoxLayout(hbox, (int)(10 * wmult));
 
-    vbox1->addWidget(profile->configWidget(this, dialog));
-    vbox1->addWidget(recpriority->configWidget(this, dialog));
-    vbox1->addWidget(autoexpire->configWidget(this, dialog));
+    profileWidget = profile->configWidget(this, dialog);
+    vbox1->addWidget(profileWidget);
+    recpriorityWidget = recpriority->configWidget(this, dialog);
+    vbox1->addWidget(recpriorityWidget);
+    autoexpireWidget = autoexpire->configWidget(this, dialog);
+    vbox1->addWidget(autoexpireWidget);
+    maxepisodesWidget = maxepisodes->configWidget(this, dialog);
+    vbox1->addWidget(maxepisodesWidget);
+    maxnewestWidget = maxnewest->configWidget(this, dialog);
+    vbox1->addWidget(maxnewestWidget);
 
-    vbox2->addWidget(recorddups->configWidget(this, dialog));
-    vbox2->addWidget(preroll->configWidget(this, dialog));
-    vbox2->addWidget(postroll->configWidget(this, dialog));
+    recgroupWidget = recgroup->configWidget(this, dialog);
+    vbox2->addWidget(recgroupWidget);
+    startoffsetWidget = startoffset->configWidget(this, dialog);
+    vbox2->addWidget(startoffsetWidget);
+    endoffsetWidget = endoffset->configWidget(this, dialog);
+    vbox2->addWidget(endoffsetWidget);
+    dupmethodWidget = dupmethod->configWidget(this, dialog);
+    vbox2->addWidget(dupmethodWidget);
+    dupinWidget = dupin->configWidget(this, dialog);
+    vbox2->addWidget(dupinWidget);
 
-    vbox->addWidget(maxepisodes->configWidget(this, dialog));
-    vbox->addWidget(maxnewest->configWidget(this, dialog));
-
-    MythPushButton *button = new MythPushButton(tr("See a list of all up-coming"
-                                                   " episodes/playtimes."),
+    hbox = new QHBoxLayout(vbox, (int)(20 * wmult));
+    proglistButton = new MythPushButton(tr("List upcoming episodes"),
                                                 dialog);
-    connect(button, SIGNAL(clicked()), this, SLOT(runProgList()));
-    vbox->addWidget(button);
+    hbox->addWidget(proglistButton);
+    hbox->addStretch( 42 );
+    MythPushButton *cancel = new MythPushButton(tr("&Cancel"), dialog);
+    hbox->addWidget(cancel);
+    hbox->addSpacing( 6 );
+    MythPushButton *finish = new MythPushButton(tr("&Finish"), dialog);
+    hbox->addWidget(finish);
+
+    connect(type, SIGNAL(valueChanged(const QString&)), this,
+            SLOT(setAvailableOptions(const QString&)));
+    connect(maxepisodes, SIGNAL(valueChanged(int)), this,
+            SLOT(setAvailableOptions(int)));
+    connect(proglistButton, SIGNAL(clicked()), this, SLOT(runProgList()));
+    connect(cancel, SIGNAL(clicked()), dialog, SLOT(reject()));
+    connect(finish, SIGNAL(clicked()), dialog, SLOT(accept()));
+
+    setAvailableOptions();
+    type->setFocus();
 
     return dialog;
 }
 
+void ScheduledRecording::setAvailableOptions(void)
+{
+    bool multiEpisode = true;
+    bool isScheduled = true;
+    bool maxIsSet = false;
+
+    switch(type->getValue().toInt())
+    {
+        case kNotRecording:
+        case kDontRecord:
+                isScheduled = false;
+                break;
+
+        case kSingleRecord:
+        case kFindOneRecord:
+        case kOverrideRecord:
+                multiEpisode = false;
+                break;
+    }
+
+    if (maxepisodes->getValue().toInt() > 0)
+        maxIsSet = true;
+
+    profileWidget->setEnabled(isScheduled);
+    recpriorityWidget->setEnabled(isScheduled);
+    autoexpireWidget->setEnabled(isScheduled);
+    recgroupWidget->setEnabled(isScheduled);
+    startoffsetWidget->setEnabled(isScheduled);
+    endoffsetWidget->setEnabled(isScheduled);
+
+    maxepisodesWidget->setEnabled(isScheduled && multiEpisode);
+    maxnewestWidget->setEnabled(isScheduled && multiEpisode && maxIsSet);
+    dupinWidget->setEnabled(isScheduled && multiEpisode);
+    dupmethodWidget->setEnabled(isScheduled && multiEpisode);
+}
+
 void ScheduledRecording::runProgList(void)
 {
     ProgLister *pl = new ProgLister(plTitle, title->getValue(),
@@ -793,6 +832,7 @@ void ScheduledRecording::runProgList(void)
                                     gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
+    proglistButton->setFocus();
 }
 
 void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting) {
@@ -811,6 +851,9 @@ void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting
             case kAllRecord:
                 label = QString("%1").arg(sr.title->getValue());
                 break;
+            case kFindOneRecord:
+                label = QString("%1").arg(sr.title->getValue());
+                break;
             case kChannelRecord:
                 label = QString("%1 on channel %2")
                     .arg(sr.title->getValue())
@@ -827,6 +870,8 @@ void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting
                     .arg(sr.endTime->timeValue().toString());
                 break;
             case kSingleRecord:
+            case kOverrideRecord:
+            case kDontRecord:
                 label = QString("%1 on channel %2 (%3 %4 - %5)")
                     .arg(sr.title->getValue())
                     .arg(sr.channel->getSelectionLabel())
@@ -876,8 +921,12 @@ void ScheduledRecording::setEnd(const QDateTime& end) {
     endDate->setValue(end.date());
 }
 
-void ScheduledRecording::setRecPriority(const QString& newrecpriority) {
-    recpriority->setValue(newrecpriority.toInt());
+void ScheduledRecording::setRecPriority(int newrecpriority) {
+    recpriority->setValue(newrecpriority);
+}
+
+void ScheduledRecording::setRecGroup(const QString& newrecgroup) {
+    recgroup->setValue(newrecgroup);
 }
 
 QString ScheduledRecording::getProfileName(void) const {
