@@ -297,92 +297,37 @@ void TTFFont::merge_text(OSDSurface *surface, Raster_Map * rmap, int offset_x,
                          int offset_y, int xstart, int ystart, int width, 
                          int height, int color, int alphamod)
 {
-    int                 x, y;
-    unsigned char      *ptr, *src;
-    unsigned char       a, *destalpha;
+    unsigned char * asrc, * ydst, * udst, * vdst, * adst;
 
-    unsigned char *usrc, *vsrc;
- 
-    int offset; 
-
-    if (height + ystart > surface->height)
-        height = surface->height - ystart - 1;
-    if (width + xstart > surface->width)
-        width = surface->width - xstart - 1;
-
-    bool transdest = false; 
-    int newalpha = 0;
-
-    int realstarty = ystart;
-    int realstartx = xstart;    
-
-    if (realstarty < 0)
-        realstarty = 0;
-    if (realstartx < 0)
-        realstartx = 0;
-
-    QRect drawRect(realstartx, realstarty, width, height);
-
-    surface->AddRect(drawRect);
-
-    for (y = 0; y < height; y++)
+    if (xstart < 0)
     {
-        if (y + ystart < 0)
-            continue;
-
-	ptr = (unsigned char *)rmap->bitmap + offset_x + 
-              ((y + offset_y) * rmap->cols);
-
-	for (x = 0; x < width; x++, ptr++)
-	{
-            if (x + xstart < 0)
-                continue;
-
-	    if ((a = *ptr) > 0)
-	    {
-                a = ((a * alphamod) + 0x80) >> 8;
-                offset = (y + ystart) * surface->width + (x + xstart);
-
-                destalpha = surface->alpha + offset;
-		src = surface->y + offset;
-
-                newalpha = a;
-
-                if (*destalpha != 0)
-                {
-                    transdest = false;
-                    newalpha = surface->pow_lut[a][*destalpha];
-                    *src = blendColorsAlpha(color, *src, newalpha);
-                    *destalpha = *destalpha + ((a * (255 - *destalpha)) / 255);
-                }
-                else
-                {
-                    transdest = true;
-                    *src = color;
-                    *destalpha = a;
-                }
-
-		if (color > 0 & a >= 230)
-                {
-                    offset = ((y + ystart + 1) / 2) * (surface->width / 2) +
-                              (x + xstart + 1) / 2;
-                    usrc = surface->u + offset; 
-                    vsrc = surface->v + offset; 
-
-                    if (!transdest)
-                    {
-                        *usrc = blendColorsAlpha(128, *usrc, newalpha);
-                        *vsrc = blendColorsAlpha(128, *vsrc, newalpha);
-                    }
-                    else
-                    {
-                        *usrc = 128; 
-                        *vsrc = 128;
-                    }
-                }
-	    }
-        }
+        width += xstart;
+        offset_x -= xstart;
+        xstart = 0;
     }
+    if (ystart < 0)
+    {
+        height += ystart;
+        offset_y -= ystart;
+        ystart = 0;
+    }
+    if (height + ystart > surface->height)
+        height = surface->height - ystart;
+    if (width + xstart > surface->width)
+        width = surface->width - xstart;
+
+    QRect drawRect(xstart, ystart, width, height);
+    surface->AddRect(drawRect);
+    
+    asrc = rmap->bitmap + rmap->cols * offset_y + offset_x;
+    ydst = surface->y + surface->width * ystart + xstart;
+    adst = surface->alpha + surface->width * ystart + xstart;
+    udst = surface->u + (surface->width >>1) * (ystart >> 1) + (xstart >> 1);
+    vdst = surface->v + (surface->width >>1) * (ystart >> 1) + (xstart >> 1);
+    (surface->blendcolorfunc) (color, 128, 128, asrc, rmap->width, ydst, udst,
+                               vdst, adst, surface->width, width, height,
+                               alphamod, 1, surface->rec_lut,
+                               surface->pow_lut);
 }
 
 void TTFFont::DrawString(OSDSurface *surface, int x, int y, 
