@@ -13,7 +13,7 @@
 
 #include <qobject.h>
 
-class QSqlDatabase;
+class MythSqlDatabase;
 class QDateTime;
 class OSD;
 class RemoteEncoder;
@@ -23,13 +23,15 @@ class RingBuffer;
 class ProgramInfo;
 class MythDialog;
 class UDPNotify;
+class OSDListTreeType;
+class OSDGenericTree;
 
 class TV : public QObject
 {
     Q_OBJECT
   public:
-    TV(QSqlDatabase *db);
-   ~TV(void);
+    TV(void);
+   ~TV();
 
     static void InitKeys(void);
 
@@ -42,7 +44,7 @@ class TV : public QObject
     int GetLastRecorderNum(void) { return lastRecorderNum; }
     int PlayFromRecorder(int recordernum);
 
-    void AskAllowRecording(const QString &message, int timeuntil);
+    void AskAllowRecording(const QStringList &messages, int timeuntil);
 
     // next two functions only work on recorded programs.
     int Playback(ProgramInfo *rcinfo);
@@ -54,6 +56,7 @@ class TV : public QObject
     bool IsPlaying(void) { return StateIsPlaying(GetState()); }
     bool IsRecording(void) { return StateIsRecording(GetState()); }
     bool IsMenuRunning(void) { return menurunning; }
+    bool IsSwitchingCards(void) { return switchingCards; }
 
     void GetNextProgram(RemoteEncoder *enc, int direction,
                         QMap<QString, QString> &infoMap);
@@ -61,13 +64,15 @@ class TV : public QObject
                         QString &title, QString &subtitle,
                         QString &desc, QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
 
     void GetChannelInfo(RemoteEncoder *enc, QMap<QString, QString> &infoMap);
     void GetChannelInfo(RemoteEncoder *enc, QString &title, QString &subtitle,
                         QString &desc, QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
 
     // for the guidegrid to use
     void EmbedOutput(WId wid, int x, int y, int w, int h);
@@ -93,6 +98,9 @@ class TV : public QObject
     void UnMute(void);
     void KeyRepeatOK(void);
     void BrowseEndTimer(void) { BrowseEnd(false); }
+    void SleepEndTimer(void);
+    void TreeMenuEntered(OSDListTreeType *tree, OSDGenericTree *item);
+    void TreeMenuSelected(OSDListTreeType *tree, OSDGenericTree *item);
 
   protected:
     void doLoadMenu(void);
@@ -115,7 +123,7 @@ class TV : public QObject
 
     void ChangeVolume(bool up);
     void ToggleMute(void);
-    void ToggleLetterbox(void);
+    void ToggleLetterbox(int letterboxMode = -1);
     void ChangeContrast(bool up, bool recorder);
     void ChangeBrightness(bool up, bool recorder);
     void ChangeColour(bool up, bool recorder);
@@ -128,7 +136,12 @@ class TV : public QObject
 
     void ToggleInputs(void); 
 
+    void SwitchCards(void);
+
+    void ToggleSleepTimer(void);
+
     void DoInfo(void);
+    void DoPlay(void);
     void DoPause(void);
     bool UpdatePosOSD(float time, const QString &mesg);
     void DoSeek(float time, const QString &mesg);
@@ -138,9 +151,14 @@ class TV : public QObject
     float StopFFRew(void);
     void ChangeFFRew(int direction);
     void RepeatFFRew(void);
+    void DoToggleCC(int mode);
     void DoSkipCommercials(int direction);
+    void DoEditMode(void);
 
     void DoQueueTranscode(void);  
+
+    void SetAutoCommercialSkip(int skipMode = 0);
+    void SetManualZoom(bool zoomON = false);
  
     void ToggleOSD(void); 
     void UpdateOSD(void);
@@ -164,6 +182,8 @@ class TV : public QObject
     void ToggleActiveWindow(void);
     void SwapPIP(void);
 
+    void ToggleAutoExpire(void);
+
     void BrowseStart(void);
     void BrowseEnd(bool change);
     void BrowseDispInfo(int direction);
@@ -174,8 +194,8 @@ class TV : public QObject
     void DoToggleRecPictureAttribute(void);
     void DoChangePictureAttribute(int control, bool up, bool rec);
 
-    void DoProgramMenu(void);
-    void ProgramMenuAction(int result);
+    void BuildOSDTreeMenu(void);
+    void ShowOSDTreeMenu(void);
 
     int osd_display_time;
 
@@ -242,9 +262,11 @@ class TV : public QObject
     QString browsechanid;
     QString browsestarttime;
 
+    bool zoomMode;
+
     ProgramInfo *playbackinfo;
 
-    QSqlDatabase *m_db;
+    MythSqlDatabase *m_db;
 
     VolumeControl *volumeControl;
 
@@ -284,6 +306,17 @@ class TV : public QObject
     int lastCCDir;
     bool showBufferedWarnings;
     int bufferedChannelThreshold;
+
+    bool MuteIndividualChannels;
+
+    bool switchingCards;
+
+    OSDGenericTree *treeMenu;
+
+    int sleep_index;
+    QTimer *sleepTimer;
+
+    char vbimode;
 };
 
 #endif
