@@ -23,7 +23,7 @@ using namespace std;
 #include "hdtvrecorder.h"
 #include "NuppelVideoPlayer.h"
 #include "channel.h"
-#include "commercial_skip.h"
+#include "mythdbcon.h"
 
 #ifdef USING_IVTV
 #include "mpegrecorder.h"
@@ -46,6 +46,7 @@ TVRec::TVRec(int capturecardnum)
     db_conn = NULL;
     channel = NULL;
     rbuffer = NULL;
+    encode = static_cast<pthread_t>(0);
     nvr = NULL;
     readthreadSock = NULL;
     readthreadlive = false;
@@ -64,11 +65,12 @@ TVRec::TVRec(int capturecardnum)
     QString chanorder = gContext->GetSetting("ChannelOrdering", "channum + 0");
 
     audiosamplerate = -1;
+    skip_btaudio = false;
 
     QString inputname, startchannel;
 
     GetDevices(capturecardnum, videodev, vbidev, audiodev, audiosamplerate,
-               inputname, startchannel, cardtype, dvb_options);
+               inputname, startchannel, cardtype, dvb_options, skip_btaudio);
 
     if (cardtype == "DVB")
     {
@@ -81,6 +83,8 @@ TVRec::TVRec(int capturecardnum)
             channel->SwitchToInput(inputname, startchannel);
         channel->SetChannelOrdering(chanorder);
         // don't close this channel, otherwise we cannot read data
+        if (dvb_options.dvb_on_demand) 
+            ((DVBChannel *)channel)->CloseDVB();
 #else
         VERBOSE(VB_IMPORTANT, "ERROR: DVB Card configured, "
                               "but no DVB support compiled in!");
@@ -91,7 +95,7 @@ TVRec::TVRec(int capturecardnum)
     }
     else // "V4L" or "MPEG", ie, analog TV, or "HDTV"
     {
-        Channel *achannel = new Channel(this, videodev);
+        Channel *achannel = new Channel(this, videodev, (cardtype == "HDTV"));
         channel = achannel; // here for SetFormat()->RetrieveInputChannels()
         channel->Open();
         achannel->SetFormat(gContext->GetSetting("TVFormat"));
@@ -356,7 +360,7 @@ void TVRec::FinishedRecording(void)
     pthread_mutex_lock(&db_lock);
     MythContext::KickDatabase(db_conn);
 
-    curRecording->FinishedRecording(db_conn);
+    curRecording->FinishedRecording(db_conn, prematurelystopped);
     pthread_mutex_unlock(&db_lock);
 }
 
@@ -482,26 +486,63 @@ void TVRec::HandleStateChange(void)
         // Determine whether to automatically run the transcoder or not
         autoTranscode = profile.byName("autotranscode")->getValue().toInt();
 
+        bool error = false;
+
         SetupRecorder(profile);
-        nvr->SetRecording(curRecording);
-        nvr->SetDB(db_conn, &db_lock);
         if (channel != NULL)
-            nvr->ChannelNameChanged(channel->GetCurrentName());
+        {
+            // If there is a tuner make sure this is a valid station
+            // hdtv will block for a long time if we try to get
+            // mpeg ts packets from outer space.
+            bool success = channel->Open();
+            if (success)
+            {
+                success = channel->CheckSignalFull();
+                if (!success)
+                {
+                    VERBOSE(VB_IMPORTANT, "Signal level too low?");
+                    error = true;
+                }
+                channel->Close();
+            } 
+            else
+                error = true;
+        }
 
-        SetVideoFiltersForChannel(channel, channel->GetCurrentName());
-        pthread_create(&encode, NULL, SpawnEncode, nvr);
+        if (!error)
+        {
+            nvr->SetRecording(curRecording);
+            nvr->SetDB(db_conn, &db_lock);
+            if (channel != NULL)
+                nvr->ChannelNameChanged(channel->GetCurrentName());
 
-        while (!nvr->IsRecording() && !nvr->IsErrored())
-            usleep(50);
+            SetVideoFiltersForChannel(channel, channel->GetCurrentName());
+            if (channel->Open())
+            {
+                channel->SetBrightness();
+                channel->SetContrast();
+                channel->SetColour();
+                channel->SetHue();
+                channel->Close();
+            }
+            pthread_create(&encode, NULL, SpawnEncode, nvr);
+
+            while (!nvr->IsRecording() && !nvr->IsErrored())
+                usleep(50);
+        } 
+        else
+            VERBOSE(VB_IMPORTANT, "Tuning Error -- aborting recording");
 
         if (nvr->IsRecording())
-        {
+	{
             // evil.
             channel->SetFd(nvr->GetVideoFd());
             frameRate = nvr->GetFrameRate();
         }
         else
         {
+            if (nvr->IsErrored())
+                prematurelystopped = true;
             FinishedRecording();
             killRecordingFile = true;
             closeRecorder = true;
@@ -559,11 +600,16 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
 
         nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
 
+        nvr->SetOption("dvb_on_demand", dvb_options.dvb_on_demand);
         nvr->SetOption("swfilter", dvb_options.swfilter);
         nvr->SetOption("recordts", dvb_options.recordts);
         nvr->SetOption("wait_for_seqstart", dvb_options.wait_for_seqstart);
         nvr->SetOption("dmx_buf_size", dvb_options.dmx_buf_size);
         nvr->SetOption("pkt_buf_size", dvb_options.pkt_buf_size);
+        nvr->SetOption("signal_monitor_interval", 
+                       gContext->GetNumSetting("DVBMonitorInterval", 0));
+        nvr->SetOption("expire_data_days",
+                       gContext->GetNumSetting("DVBMonitorRetention", 3));
         nvr->Initialize();
 #endif
         return;
@@ -575,6 +621,7 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
 
     nvr->SetRingBuffer(rbuffer);
 
+    nvr->SetOption("skipbtaudio", skip_btaudio);
     nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
  
     nvr->Initialize();
@@ -609,7 +656,9 @@ void TVRec::TeardownRecorder(bool killFile)
         if (prevRecording && !killFile)
             nvr->GetBlankFrameMap(blank_frame_map);
 
-        pthread_join(encode, NULL);
+        if (encode)
+            pthread_join(encode, NULL);
+        encode = static_cast<pthread_t>(0);
         delete nvr;
     }
     nvr = NULL;
@@ -652,7 +701,8 @@ void TVRec::TeardownRecorder(bool killFile)
 
         if (!prematurelystopped)
         {
-            if (gContext->GetNumSetting("AutoCommercialFlag", 1))
+            if ((gContext->GetNumSetting("AutoCommercialFlag", 1)) &&
+                (prevRecording->chancommfree == 0))
                 FlagCommercials();
 
             if (autoTranscode)
@@ -682,18 +732,10 @@ char *TVRec::GetScreenGrab(ProgramInfo *pginfo, const QString &filename,
 
     QString name = QString("screen%1%2").arg(getpid()).arg(rand());
 
-    QSqlDatabase *screendb = QSqlDatabase::addDatabase("QMYSQL3", name);
-    if (!screendb)
-    {
-        cerr << "Couldn't initialize mysql connection for comm thread\n";
+    MythSqlDatabase *screendb = new MythSqlDatabase(name);
+
+    if (!screendb || !screendb->isOpen())
         return NULL;
-    }
-    if (!gContext->OpenDatabase(screendb))
-    {
-        cerr << "Couldn't open database connection for comm thread\n";
-        QSqlDatabase::removeDatabase(name);
-        return NULL;
-    }
 
     NuppelVideoPlayer *nupvidplay = new NuppelVideoPlayer(screendb, pginfo);
     nupvidplay->SetRingBuffer(tmprbuf);
@@ -704,9 +746,7 @@ char *TVRec::GetScreenGrab(ProgramInfo *pginfo, const QString &filename,
 
     delete nupvidplay;
     delete tmprbuf;
-
-    screendb->close(); 
-    QSqlDatabase::removeDatabase(name);
+    delete screendb;
 
     return retbuf;
 }
@@ -782,21 +822,15 @@ void TVRec::RunTV(void)
             int timeuntil = QDateTime::currentDateTime()
                                 .secsTo(recordPendingStart);
 
-            QString message = QString("MythTV wants to record \"")
-                               + pendingRecording->title;
-
-            if (gContext->GetNumSetting("DisplayChanNum") != 0)
-                message += QString("\" on ")
-                        + pendingRecording->channame
-                        + " [" +  pendingRecording->chansign + "]";
-            else
-                message += "\" on Channel " + pendingRecording->chanstr;
-
-            message += " in %d seconds.  Do you want to:";
-
             QString query = QString("ASK_RECORDING %1 %2").arg(m_capturecardnum)
-                                                        .arg(timeuntil);
-            MythEvent me(query, message);
+                                                          .arg(timeuntil);
+            QStringList messages;
+            messages << pendingRecording->title
+                    << pendingRecording->chanstr
+                    << pendingRecording->chansign
+                    << pendingRecording->channame;
+            
+            MythEvent me(query, messages);
 
             gContext->dispatch(me);
         }
@@ -846,7 +880,8 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
                            QString &desc, QString &category, 
                            QString &starttime, QString &endtime, 
                            QString &callsign, QString &iconpath, 
-                           QString &channelname, QString &chanid)
+                           QString &channelname, QString &chanid,
+                           QString &seriesid, QString &programid)
 {
     title = "";
     subtitle = "";
@@ -858,6 +893,8 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
     iconpath = "";
     channelname = "";
     chanid = "";
+    seriesid = "";
+    programid = "";
 
     if (!db_conn)
         return;
@@ -879,50 +916,35 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
  
     QString thequery = QString("SELECT starttime,endtime,title,subtitle,"
                                "description,category,callsign,icon,"
-                               "channel.chanid "
+                               "channel.chanid, seriesid, programid "
                                "FROM program,channel,capturecard,cardinput "
                                "WHERE channel.channum = \"%1\" "
                                "AND starttime < %2 AND endtime > %3 AND "
                                "program.chanid = channel.chanid AND "
                                "channel.sourceid = cardinput.sourceid AND "
-                               "cardinput.inputname = \"%4\" AND "
                                "cardinput.cardid = capturecard.cardid AND "
-                               "capturecard.cardid = \"%5\" AND "
-                               "capturecard.hostname = \"%6\";")
+                               "capturecard.cardid = \"%4\" AND "
+                               "capturecard.hostname = \"%5\";")
                                .arg(channelname).arg(curtimestr).arg(curtimestr)
-                               .arg(channelinput).arg(m_capturecardnum)
-                               .arg(gContext->GetHostName());
+                               .arg(m_capturecardnum).arg(gContext->GetHostName());
 
     QSqlQuery query = db_conn->exec(thequery);
 
-    QString test;
     if (query.isActive() && query.numRowsAffected() > 0)
     {
         query.next();
 
         starttime = query.value(0).toString();
         endtime = query.value(1).toString();
-        test = query.value(2).toString();
-        if (test != QString::null)
-            title = QString::fromUtf8(test);
-        test = query.value(3).toString();
-        if (test != QString::null)
-            subtitle = QString::fromUtf8(test);
-        test = query.value(4).toString();
-        if (test != QString::null)
-            desc = QString::fromUtf8(test);
-        test = query.value(5).toString();
-        if (test != QString::null)
-            category = QString::fromUtf8(test);
-        test = query.value(6).toString();
-        if (test != QString::null)
-            callsign = test;
-        test = query.value(7).toString();
-        if (test != QString::null)
-            iconpath = test;
-        test = query.value(8).toString();
-        if (test != QString::null)
-            chanid = test;
+        title = QString::fromUtf8(query.value(2).toString());
+        subtitle = QString::fromUtf8(query.value(3).toString());
+        desc = QString::fromUtf8(query.value(4).toString());
+        category = QString::fromUtf8(query.value(5).toString());
+        callsign = query.value(6).toString();
+        iconpath = query.value(7).toString();
+        chanid = query.value(8).toString();
+        seriesid = query.value(9).toString();
+        programid = query.value(10).toString();
     }
     else
     {
@@ -932,30 +954,22 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
                                    "FROM channel,capturecard,cardinput "
                                    "WHERE channel.channum = \"%1\" AND "
                                    "channel.sourceid = cardinput.sourceid AND "
-                                   "cardinput.inputname = \"%2\" AND "
                                    "cardinput.cardid = capturecard.cardid AND "
-                                   "capturecard.cardid = \"%3\" AND "
-                                   "capturecard.hostname = \"%4\";")
+                                   "capturecard.cardid = \"%2\" AND "
+                                   "capturecard.hostname = \"%3\";")
                                    .arg(channelname)
-                                   .arg(channelinput).arg(m_capturecardnum)
+                                   .arg(m_capturecardnum)
                                    .arg(gContext->GetHostName());
 
         QSqlQuery query = db_conn->exec(thequery);
 
-        QString test;
         if (query.isActive() && query.numRowsAffected() > 0)
         {
             query.next();
 
-            test = query.value(0).toString();
-            if (test != QString::null)
-                callsign = test;
-            test = query.value(1).toString();
-            if (test != QString::null)
-                iconpath = test;
-            test = query.value(2).toString();
-            if (test != QString::null)
-                chanid = test;
+            callsign = query.value(0).toString();
+            iconpath = query.value(1).toString();
+            chanid = query.value(2).toString();
         }
      }
 
@@ -999,7 +1013,7 @@ void TVRec::DisconnectDB(void)
 void TVRec::GetDevices(int cardnum, QString &video, QString &vbi, 
                        QString &audio, int &rate, QString &defaultinput,
                        QString &startchan, QString &type, 
-                       dvb_options_t &dvb_opts)
+                       dvb_options_t &dvb_opts, bool &skip_bt)
 {
     video = "";
     vbi = "";
@@ -1016,7 +1030,7 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
                                "audioratelimit,defaultinput,cardtype,"
                                "dvb_swfilter, dvb_recordts,"
                                "dvb_wait_for_seqstart,dvb_dmx_buf_size,"
-                               "dvb_pkt_buf_size "
+                               "dvb_pkt_buf_size, skipbtaudio, dvb_on_demand "
                                "FROM capturecard WHERE cardid = %1;")
                               .arg(cardnum);
 
@@ -1058,6 +1072,9 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
         dvb_opts.wait_for_seqstart = query.value(8).toInt();
         dvb_opts.dmx_buf_size = query.value(9).toInt();
         dvb_opts.pkt_buf_size = query.value(10).toInt();
+
+        skip_bt = query.value(11).toInt();
+        dvb_opts.dvb_on_demand = query.value(12).toInt();
     }
 
     thequery = QString("SELECT if(startchan!='', startchan, '3') "
@@ -1085,18 +1102,21 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
 bool TVRec::CheckChannel(QString name)
 {
     QSqlDatabase* dummy1;
-    pthread_mutex_t dummy2;
-    return CheckChannel(channel, name, dummy1, dummy2);
+    pthread_mutex_t* dummy2;
+    QString dummyID;
+    return CheckChannel(channel, name, dummy1, dummy2, dummyID);
 }
 
 bool TVRec::CheckChannel(ChannelBase *chan, const QString &channum, 
-                         QSqlDatabase *&a_db_conn, pthread_mutex_t &a_db_lock)
+                         QSqlDatabase *&a_db_conn, pthread_mutex_t *&a_db_lock, QString& inputName)
 {
     if (!db_conn)
         return true;
 
+    inputName = "";
+    
     a_db_conn = db_conn;
-    a_db_lock = db_lock;
+    a_db_lock = &db_lock;
 
     pthread_mutex_lock(&db_lock);
     MythContext::KickDatabase(db_conn);
@@ -1126,7 +1146,46 @@ bool TVRec::CheckChannel(ChannelBase *chan, const QString &channum,
         pthread_mutex_unlock(&db_lock);
         return true;
     }
+    VERBOSE( VB_CHANNEL, QString("Failed to find channel(%1) on current input (%2) of card (%3).")
+                         .arg(channum).arg(channelinput).arg(m_capturecardnum) );
 
+
+    // We didn't find it on the current input let's widen the search
+    thequery = QString("SELECT channel.chanid, cardinput.inputname FROM "
+                       "channel,capturecard,cardinput "
+                       "WHERE channel.channum = \"%1\" AND "
+                       "channel.sourceid = cardinput.sourceid AND "
+                       "cardinput.cardid = capturecard.cardid AND "
+                       "capturecard.cardid = \"%2\" AND "
+                       "capturecard.hostname = \"%3\";")
+                       .arg(channum)
+                       .arg(m_capturecardnum)
+                       .arg(gContext->GetHostName());
+
+    query = db_conn->exec(thequery);
+
+    if (!query.isActive())
+        MythContext::DBError("checkchannel", query);
+    else if (query.numRowsAffected() > 0)
+    {
+        
+        query.next();
+        QString test = query.value(1).toString();
+        if (test != QString::null)
+            inputName = QString::fromUtf8(test);
+
+        VERBOSE( VB_CHANNEL, QString("Found channel(%1) on another input (%2) of card (%3).")
+                             .arg(channum).arg(inputName).arg(m_capturecardnum) );
+
+        pthread_mutex_unlock(&db_lock);
+        return true;
+    }
+
+    VERBOSE( VB_CHANNEL, QString("Failed to find channel(%1) on any input of card (%2).")
+                         .arg(channum).arg(m_capturecardnum) );
+
+                                                                  
+    
     thequery = "SELECT NULL FROM channel;";
     query = db_conn->exec(thequery);
 
@@ -1239,9 +1298,6 @@ bool TVRec::SetVideoFiltersForChannel(ChannelBase *chan, const QString &channum)
 
         videoFilters = query.value(0).toString();
 
-        if (videoFilters == QString::null) 
-            videoFilters = "";
-
         if (nvr != NULL)
         {
             nvr->SetVideoFilters(videoFilters);
@@ -1404,12 +1460,11 @@ void TVRec::DoGetNextChannel(QString &channum, QString channelinput,
                                "channel,capturecard,cardinput "
                                "WHERE channel.channum = \"%2\" AND "
                                "channel.sourceid = cardinput.sourceid AND "
-                               "cardinput.inputname = \"%3\" AND "
                                "cardinput.cardid = capturecard.cardid AND "
-                               "capturecard.cardid = \"%4\" AND "
-                               "capturecard.hostname = \"%5\";")
+                               "capturecard.cardid = \"%3\" AND "
+                               "capturecard.hostname = \"%4\";")
                                .arg(channelorder).arg(channum)
-                               .arg(channelinput).arg(cardid)
+                               .arg(cardid)
                                .arg(gContext->GetHostName());
 
     QSqlQuery query = db_conn->exec(thequery);
@@ -1431,11 +1486,10 @@ void TVRec::DoGetNextChannel(QString &channum, QString channelinput,
 
         thequery = QString("SELECT %1 FROM channel,capturecard,cardinput "
                            "WHERE channel.sourceid = cardinput.sourceid AND "
-                           "cardinput.inputname = \"%2\" AND "
                            "cardinput.cardid = capturecard.cardid AND "
-                           "capturecard.cardid = \"%3\" AND "
-                           "capturecard.hostname = \"%4\" ORDER BY %5 "
-                           "LIMIT 1;").arg(channelorder).arg(channelinput)
+                           "capturecard.cardid = \"%2\" AND "
+                           "capturecard.hostname = \"%3\" ORDER BY %4 "
+                           "LIMIT 1;").arg(channelorder)
                            .arg(cardid).arg(gContext->GetHostName())
                            .arg(channelorder);
        
@@ -1478,12 +1532,10 @@ void TVRec::DoGetNextChannel(QString &channum, QString channelinput,
         comp = "=";
     }
 
-    QString wherepart = QString("channel.sourceid = cardinput.sourceid AND "
-                                "cardinput.inputname = \"%1\" AND "
-                                "cardinput.cardid = capturecard.cardid AND "
-                                "capturecard.cardid = \"%2\" AND "
-                                "capturecard.hostname = \"%3\" ")
-                                .arg(channelinput).arg(cardid)
+    QString wherepart = QString("cardinput.cardid = capturecard.cardid AND "
+                                "capturecard.cardid = \"%1\" AND "
+                                "capturecard.hostname = \"%2\" ")
+                                .arg(cardid)
                                 .arg(gContext->GetHostName());
 
     thequery = QString("SELECT channel.channum, channel.chanid "
@@ -1649,10 +1701,19 @@ void TVRec::PauseRecorder(void)
 
 void TVRec::ToggleInputs(void)
 {
+    if (!nvr || !channel || !rbuffer)
+        return;
+
     nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
+    if (!rbuffer)
+    {
+        UnpauseRingBuffer();
+        return;
+    }
+
     rbuffer->Reset();
 
     channel->ToggleInputs();
@@ -1665,10 +1726,19 @@ void TVRec::ToggleInputs(void)
 
 void TVRec::ChangeChannel(int channeldirection)
 {
+    if (!nvr || !channel || !rbuffer)
+        return;
+
     nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
+    if (!rbuffer)
+    {
+        UnpauseRingBuffer();
+        return;
+    }
+
     rbuffer->Reset();
 
     if (channeldirection == CHANNEL_DIRECTION_FAVORITE)
@@ -1786,10 +1856,19 @@ int TVRec::ChangeHue(bool direction)
 
 void TVRec::SetChannel(QString name)
 {
+    if (!nvr || !channel || !rbuffer)
+        return;
+
     nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
+    if (!rbuffer)
+    {
+        UnpauseRingBuffer();
+        return;
+    }
+
     rbuffer->Reset();
 
     QString chan = name.stripWhiteSpace();
@@ -1809,7 +1888,8 @@ void TVRec::GetNextProgram(int direction,
                         QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid)
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid)
 {
     QString querystr;
     QString nextchannum = channelname;
@@ -1818,7 +1898,7 @@ void TVRec::GetNextProgram(int direction,
 
     querystr = QString("SELECT title, subtitle, description, category, "
                        "starttime, endtime, callsign, icon, channum, "
-                       "program.chanid "
+                       "program.chanid, seriesid, programid "
                        "FROM program, channel "
                        "WHERE program.chanid = channel.chanid ");
 
@@ -1871,26 +1951,18 @@ void TVRec::GetNextProgram(int direction,
     {
         if (sqlquery.next())
         {
-            QString test;
-
-            test = sqlquery.value(0).toString();
-            if (test != QString::null)
-                title = QString::fromUtf8(test);
-            test = sqlquery.value(1).toString();
-            if (test != QString::null)
-                subtitle = QString::fromUtf8(test);
-            test = sqlquery.value(2).toString();
-            if (test != QString::null)
-                desc = QString::fromUtf8(test);
-            test = sqlquery.value(3).toString();
-            if (test != QString::null)
-                category = QString::fromUtf8(test);
+            title = QString::fromUtf8(sqlquery.value(0).toString());
+            subtitle = QString::fromUtf8(sqlquery.value(1).toString());
+            desc = QString::fromUtf8(sqlquery.value(2).toString());
+            category = QString::fromUtf8(sqlquery.value(3).toString());
             starttime =  sqlquery.value(4).toString();
             endtime = sqlquery.value(5).toString();
             callsign = sqlquery.value(6).toString();
             iconpath = sqlquery.value(7).toString();
             channelname = sqlquery.value(8).toString();
             chanid = sqlquery.value(9).toString();
+            seriesid = sqlquery.value(10).toString();
+            programid = sqlquery.value(11).toString();
         }
     }
     else
@@ -1913,10 +1985,12 @@ void TVRec::GetNextProgram(int direction,
 void TVRec::GetChannelInfo(QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid)
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid)
 {
     GetChannelInfo(channel, title, subtitle, desc, category, starttime,
-                   endtime, callsign, iconpath, channelname, chanid);
+                   endtime, callsign, iconpath, channelname, chanid,
+                   seriesid, programid);
 }
 
 void TVRec::GetInputName(QString &inputname)
@@ -1926,31 +2000,31 @@ void TVRec::GetInputName(QString &inputname)
 
 void TVRec::UnpauseRingBuffer(void)
 {
-    if (!rbuffer)
-        return;
-
-    rbuffer->StartReads();
+    if (rbuffer)
+        rbuffer->StartReads();
     readthreadLock.unlock();
 }
 
 void TVRec::PauseClearRingBuffer(void)
 {
+    readthreadLock.lock();
+
     if (!rbuffer)
         return;
 
     rbuffer->StopReads();
-    readthreadLock.lock();
 }
 
 long long TVRec::SeekRingBuffer(long long curpos, long long pos, int whence)
 {
-    if (!rbuffer)
-        return -1;
-    if (!readthreadlive)
-        return -1;
-
     PauseClearRingBuffer();
 
+    if (!rbuffer || !readthreadlive)
+    {
+        UnpauseRingBuffer();
+        return -1;
+    }
+
     if (whence == SEEK_CUR)
     {
         long long realpos = rbuffer->GetTotalReadPosition();
@@ -1991,14 +2065,18 @@ void TVRec::SetReadThreadSock(QSocket *sock)
 
 int TVRec::RequestRingBufferBlock(int size)
 {
-    if (!readthreadlive || !rbuffer)
-        return -1;
-
     char buffer[256001];
     int tot = 0;
     int ret = 0;
 
     readthreadLock.lock();
+
+    if (!readthreadlive || !rbuffer)
+    {
+        readthreadLock.unlock();
+        return -1;
+    }
+
     while (tot < size && !rbuffer->GetStopReads() && readthreadlive)
     {
         int request = size - tot;
@@ -2037,18 +2115,10 @@ void TVRec::DoFlagCommercialsThread(void)
 
     QString name = QString("commercial%1%2").arg(getpid()).arg(rand());
 
-    QSqlDatabase *commthread_db = QSqlDatabase::addDatabase("QMYSQL3", name);
-    if (!commthread_db)
-    {
-        cerr << "Couldn't initialize mysql connection for comm thread\n";
+    MythSqlDatabase *commthread_db = new MythSqlDatabase(name);
+
+    if (!commthread_db || !commthread_db->isOpen())
         return;
-    }
-    if (!gContext->OpenDatabase(commthread_db))
-    {
-        cerr << "Couldn't open database connection for comm thread\n";
-        QSqlDatabase::removeDatabase(name);
-        return;
-    }
 
     nice(19);
 
@@ -2073,9 +2143,7 @@ void TVRec::DoFlagCommercialsThread(void)
     delete nvp;
     delete tmprbuf;
     delete program_info;
-
-    commthread_db->close();
-    QSqlDatabase::removeDatabase(name);
+    delete commthread_db;
 }
 
 void *TVRec::FlagCommercialsThread(void *param)
