@@ -37,6 +37,7 @@ typedef struct _dvb_options_t
     int wait_for_seqstart;
     int dmx_buf_size;
     int pkt_buf_size;
+    bool dvb_on_demand;
 } dvb_options_t;
 
 class TVRec
@@ -69,7 +70,7 @@ class TVRec
     bool IsRecording(void) { return StateIsRecording(internalState); }
 
     bool CheckChannel(ChannelBase *chan, const QString &channum, 
-                      QSqlDatabase *& a_db_conn, pthread_mutex_t &a_db_lock); 
+                      QSqlDatabase *& a_db_conn, pthread_mutex_t *&a_db_lock, QString& inputID); 
     void SetChannelValue(QString &field_name,int value, ChannelBase *chan,
                          const QString &channum);
     int GetChannelValue(const QString &channel_field, ChannelBase *chan, 
@@ -115,11 +116,13 @@ class TVRec
                         QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime, 
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
     void GetChannelInfo(QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime, 
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
     void GetInputName(QString &inputname);
 
     QSocket *GetReadThreadSocket(void);
@@ -132,6 +135,8 @@ class TVRec
 
     bool isParsingCommercials(ProgramInfo *pginfo);
 
+    int GetCaptureCardNum(void) { return m_capturecardnum; }
+
  protected:
     void RunTV(void);
     static void *EventThread(void *param);
@@ -146,11 +151,12 @@ class TVRec
     void GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle, 
                         QString &desc, QString &category, QString &starttime, 
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
 
     void GetDevices(int cardnum, QString &video, QString &vbi, QString &audio,
                     int &rate, QString &defaultinput, QString &startchannel,
-                    QString &type, dvb_options_t &dvb_opts);
+                    QString &type, dvb_options_t &dvb_opts, bool &skip_bt);
 
     void ConnectDB(int cardnum);
     void DisconnectDB(void);
@@ -202,6 +208,7 @@ class TVRec
  
     QString videodev, vbidev, audiodev, cardtype;
     int audiosamplerate;
+    bool skip_btaudio;
 
     bool inoverrecord;
     int overrecordseconds;
