@@ -22,10 +22,14 @@ class VideoOutputDirectfb: public VideoOutput
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
-    void Show(void);
+    void Show(FrameScanType);
 
     void InputChanged(int width, int height, float aspect);
-    void AspectChanged(float aspect);
+	void AspectChanged(float aspect);
+    void Zoom(int direction);
+
+	float GetDisplayAspect(void);
+	void VideoOutputDirectfb::MoveResize();
 
     //void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
     //void StopEmbedding(void);
@@ -39,6 +43,8 @@ class VideoOutputDirectfb: public VideoOutput
                       FilterChain *filterList,
                       NuppelVideoPlayer *pipPlayer);
 
+    int ChangePictureAttribute(int attributeType, int newValue);
+
   private:
 
 	QObject *widget;
@@ -50,12 +56,30 @@ class VideoOutputDirectfb: public VideoOutput
     VideoFrame *scratchFrame;
     VideoFrame pauseFrame;
 
-    void Exit(void);
-    bool CreateDirectfbBuffers(void);
+    bool CreateDirectfbBuffers(DFBSurfaceDescription);
     void DeleteDirectfbBuffers(void);
+    bool InitOsdSurface(OSD *osd);
 };
 
 DFBEnumerationResult LayerCallback(unsigned int id,
   DFBDisplayLayerDescription desc, void *data);
 
+static inline void * memcpy_pic(unsigned char * dst, unsigned char * src, int bytesPerLine, int height, int dstStride, int srcStride)
+{
+        int i;
+        void *retval=dst;
+
+        if(dstStride == srcStride) memcpy(dst, src, srcStride*height);
+        else
+        {
+                for(i=0; i<height; i++)
+                {
+                        memcpy(dst, src, bytesPerLine);
+                        src+= srcStride;
+                        dst+= dstStride;
+                }
+        }
+
+        return retval;
+}
 #endif
