@@ -87,6 +87,12 @@ void VideoOutputDX::AspectChanged(float aspect)
     MoveResize();
 }
 
+void VideoOutputDX::Zoom(int direction)
+{
+    VideoOutput::Zoom(direction);
+    MoveResize();
+}
+
 void VideoOutputDX::MoveResize()
 {
     VideoOutput::MoveResize();
@@ -323,7 +329,7 @@ void VideoOutputDX::PrepareFrame(VideoFrame *buffer)
         VERBOSE(VB_ALL, "Could not lock surface!");    
 }
 
-void VideoOutputDX::Show()
+void VideoOutputDX::Show(FrameScanType )
 {
     HRESULT dxresult;
 
