@@ -16,10 +16,11 @@ class VideoOutputVIA : public VideoOutput
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
-    void Show(void);
+    void Show(FrameScanType );
 
     void InputChanged(int width, int height, float aspect);
     void AspectChanged(float aspect);
+    void Zoom(int direction);
 
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
     void StopEmbedding(void);
