@@ -115,6 +115,16 @@ void VideoOutputXv::AspectChanged(float aspect)
     pthread_mutex_unlock(&lock);
 }
 
+void VideoOutputXv::Zoom(int direction)
+{
+    pthread_mutex_lock(&lock);
+
+    VideoOutput::Zoom(direction);
+    MoveResize();
+
+    pthread_mutex_unlock(&lock);
+}
+
 void VideoOutputXv::InputChanged(int width, int height, float aspect)
 {
     pthread_mutex_lock(&lock);
@@ -189,7 +199,6 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
     unsigned int p_version, p_release, p_request_base, p_event_base, 
                  p_error_base;
     int p_num_adaptors;
-    int w_mm, h_mm;
     bool usingXinerama;
     int event_base, error_base;
 
@@ -216,7 +225,25 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
         (XineramaQueryExtension(data->XJ_disp, &event_base, &error_base) &&
          XineramaIsActive(data->XJ_disp));
     if (w_mm == 0 || h_mm == 0 || usingXinerama)
+    {
+        w_mm = (int)(300 * XJ_aspect);
+        h_mm = 300;
         data->display_aspect = XJ_aspect;
+    }
+    else if (gContext->GetNumSetting("GuiSizeForTV", 0))
+    {
+        int w = DisplayWidth(data->XJ_disp, XJ_screen_num);
+        int h = DisplayHeight(data->XJ_disp, XJ_screen_num);
+        int gui_w = gContext->GetNumSetting("GuiWidth", w);
+        int gui_h = gContext->GetNumSetting("GuiHeight", h);
+
+		if (gui_w)
+            w_mm = w_mm * gui_w / w;
+		if (gui_h)
+            h_mm = h_mm * gui_h / h;
+
+        data->display_aspect = (float)w_mm/h_mm;
+    }
     else
         data->display_aspect = (float)w_mm/h_mm;
 
@@ -779,7 +806,7 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer)
     }
 }
 
-void VideoOutputXv::Show()
+void VideoOutputXv::Show(FrameScanType )
 {
     pthread_mutex_lock(&lock);
     XSync(data->XJ_disp, False);
