@@ -14,9 +14,10 @@ class VideoOutputXvMC : public VideoOutput
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
-    void Show(void);
+    void Show(FrameScanType scan);
 
     void InputChanged(int width, int height, float aspect);
+    void Zoom(int direction);
     void AspectChanged(float aspect);
 
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
@@ -37,6 +38,8 @@ class VideoOutputXvMC : public VideoOutput
 
     int ChangePictureAttribute(int attributeType, int newValue);
 
+    bool hasIDCTAcceleration() const;
+
   private:
     void Exit(void);
     bool CreateXvMCBuffers(void);
@@ -59,6 +62,7 @@ class VideoOutputXvMC : public VideoOutput
     pthread_mutex_t lock;
 
     int colorkey;
+    int chroma;
 };
 
 #endif
