@@ -81,15 +81,9 @@ VideoOutput::VideoOutput()
 
     needrepaint = false;
 
-    for (int i = 0; i < 256; i++) 
-        cropTbl[i + MAX_NEG_CROP] = i;
-    for (int i = 0; i < MAX_NEG_CROP; i++) 
-    {
-        cropTbl[i] = 0;
-        cropTbl[i + MAX_NEG_CROP + 256] = 255;
-    }
-
-    cm = cropTbl + MAX_NEG_CROP;
+    ZoomedIn = 0;
+    ZoomedUp = 0;
+    ZoomedRight = 0;
 
     piptmpbuf = NULL;
     pipscontext = NULL;
@@ -101,6 +95,9 @@ VideoOutput::VideoOutput()
     desired_pipw = 160;
 
     allowpreviewepg = true;
+
+    w_mm = 400;
+    h_mm = 300;
 }
 
 VideoOutput::~VideoOutput()
@@ -166,11 +163,42 @@ bool VideoOutput::Init(int width, int height, float aspect, WId winid,
     hue = gContext->GetNumSettingOnHost("PlaybackHue", 
                                         gContext->GetHostName(), 0);
 
+    int asp_override = gContext->GetNumSetting("AspectOverride", 0);
+
+    if (asp_override > 0) 
+        AspectOverride(asp_override);
+
     embedding = false;
 
     return true;
 }
 
+void VideoOutput::AspectOverride(int override)
+{
+    switch(override)
+    {
+        default:
+        case 0:
+           break;
+        case 1:
+           XJ_aspect = (16.0 / 9);
+           letterbox = 0;
+           break;
+        case 2:
+           XJ_aspect = (4.0 / 3);
+           letterbox = 1;
+           break;
+        case 3:
+           XJ_aspect = (16.0 / 9);
+           letterbox = 2;
+           break;
+        case 4:
+           XJ_aspect = (4.0 / 3);
+           letterbox = 3;
+           break;
+    }
+}
+
 void VideoOutput::AspectChanged(float aspect)
 {
     XJ_aspect = aspect;
@@ -274,8 +302,8 @@ void VideoOutput::MoveResize(void)
     if (img_vscanf > 0) 
     {
         // Veritcal overscan. Move the Y start point in original image.
-        imgy = (int)ceil(XJ_height * img_vscanf);
-        imgh = (int)ceil(XJ_height * (1 - 2 * img_vscanf));
+        imgy = (int)floor(0.5 + XJ_height * img_vscanf);
+        imgh = (int)floor(0.5 + XJ_height * (1 - 2 * img_vscanf));
 
         // If there is an offset, apply it now that we have a room.
         // To move the image down, move the start point up.
@@ -299,8 +327,8 @@ void VideoOutput::MoveResize(void)
     if (img_hscanf > 0) 
     {
         // Horizontal overscan. Move the X start point in original image.
-        imgx = (int)ceil(XJ_width * img_hscanf);
-        imgw = (int)ceil(XJ_width * (1 - 2 * img_hscanf));
+        imgx = (int)floor(0.5 + XJ_width * img_hscanf);
+        imgw = (int)floor(0.5 + XJ_width * (1 - 2 * img_hscanf));
         if (xoff > 0) 
         {
             if (xoff > imgx) 
@@ -318,11 +346,11 @@ void VideoOutput::MoveResize(void)
     float vscanf, hscanf;
     if (img_vscanf < 0) 
     {
-        // Veritcal underscan. Move the starting Y point in the display window.
+        // Vertical underscan. Move the starting Y point in the display window.
         // Use the abolute value of scan factor.
         vscanf = fabs(img_vscanf);
-        dispyoff = (int)ceil(disph * vscanf);
-        disphoff = (int)ceil(disph * (1 - 2 * vscanf));
+        dispyoff = (int)floor(0.5 + disph * vscanf) + dispy;
+        disphoff = (int)floor(0.5 + disph * (1 - 2 * vscanf));
         // Now offset the image within the extra blank space created by
         // underscanning.
         // To move the image down, increase the Y offset inside the display
@@ -345,8 +373,8 @@ void VideoOutput::MoveResize(void)
     if (img_hscanf < 0) 
     {
         hscanf = fabs(img_hscanf);
-        dispxoff = (int)ceil(dispw * hscanf);
-        dispwoff = (int)ceil(dispw * (1 - 2 * hscanf));
+        dispxoff = (int)floor(0.5 + dispw * hscanf) + dispx;
+        dispwoff = (int)floor(0.5 + dispw * (1 - 2 * hscanf));
         if (xoff > 0) 
         {
             if (xoff > dispxoff) 
@@ -362,14 +390,9 @@ void VideoOutput::MoveResize(void)
         }
     }
 
-    // Manage aspect ratio and letterbox settings.  XJ_aspect is the
-    // (desired) video aspect ratio.  GetDisplayAspect provides the
-    // actual display aspect ratio, in mm not pixels.  In addition,
-    // letterbox == 2 indicates a zoom mode (large overscan).  Do no
-    // conversion between x and y sizes in pixels, as pixels are not
-    // necessarily square.
-    // n.b. assumes aspect < 1.4 is 4:3 and > 1.4 is 16:9; other
-    // aspect ratios wil cause incorrect results
+    // Manage aspect ratio and letterbox settings.  Code should take into
+    // account the aspect ratios of both the video as well as the actual
+    // screen to allow proper letterboxing to take place.
     
     //cout << "XJ aspect " << XJ_aspect << endl;
     //cout << "Display aspect " << GetDisplayAspect() << endl;
@@ -380,38 +403,32 @@ void VideoOutput::MoveResize(void)
     {
         // Video is 4:3
         if (letterbox == -1)
-            letterbox = 0;
-        if (GetDisplayAspect() <= 1.4)
-        {
-            // Display is 4:3 - do nothing
-        }
-        else 
-        {
-            // Display is 16:9 - pillarbox it
-            // image in center, 3/4 of the overall width
-            dispxoff += (dispwoff/8);
-            dispwoff = dispwoff*3/4;
-        }
+            letterbox = kLetterbox_4_3;
     }
-    else 
+    else
     {
         // Video is 16:9
         if (letterbox == -1)
-            letterbox = 1;
-        if (GetDisplayAspect() <= 1.4)
-        {
-            // Display is 4:3 -- letterbox it
-            // image in center, 3/4 of the overall height
-            dispyoff += (disphoff/8);
-            disphoff = disphoff*3/4;
-        }
-        else
-        {
-            // Display is 16:9 - do nothing
-        }
+            letterbox = kLetterbox_16_9;
     }
 
-    if (letterbox > 1) 
+    if (GetDisplayAspect() > XJ_aspect)
+    {
+        float pixNeeded = (h_mm * XJ_aspect) * ((float)dispwoff / w_mm);
+
+        dispxoff += (dispwoff - (int)pixNeeded) / 2;
+        dispwoff = (int)pixNeeded;
+    }
+    else
+    {
+        float pixNeeded = (w_mm / XJ_aspect) * ((float)disphoff / h_mm);
+
+        dispyoff += (disphoff - (int)pixNeeded) / 2;
+        disphoff = (int)pixNeeded;
+    }
+
+    if ((letterbox == kLetterbox_4_3_Zoom) ||
+        (letterbox == kLetterbox_16_9_Zoom))
     {
         // Zoom mode
         //printf("Before zoom: %dx%d%+d%+d\n", dispwoff, disphoff,
@@ -423,21 +440,116 @@ void VideoOutput::MoveResize(void)
         disphoff = disphoff*4/3;
     }
 
+    if (letterbox == kLetterbox_16_9_Stretch)
+    {
+        //Stretch mode
+        //Should be used to eliminate side bars on 4:3 material
+        //encoded to 16:9. Basically crop a 16:9 to 4:3
+        //printf("Before zoom: %dx%d%+d%+d\n", dispwoff, disphoff,
+        //dispxoff, dispyoff);
+        dispxoff -= (dispwoff/6); // 1/6 of original is 1/8 of new
+        dispwoff = dispwoff*4/3;
+    }
+
+    if (ZoomedIn)
+    {
+        int oldDW = dispwoff;
+        int oldDH = disphoff;
+        dispwoff = (int)(dispwoff * (1 + (ZoomedIn * .01)));
+        disphoff = (int)(disphoff * (1 + (ZoomedIn * .01)));
+
+        dispxoff += (oldDW - dispwoff) / 2;
+        dispyoff += (oldDH - disphoff) / 2;
+    }
+
+    if (ZoomedUp)
+    {
+        dispyoff = (int)(dispyoff * (1 - (ZoomedUp * .01)));
+    }
+
+    if (ZoomedRight)
+    {
+        dispxoff = (int)(dispxoff * (1 - (ZoomedRight * .01)));
+    }
+
     //printf("After: %dx%d%+d%+d\n", dispwoff, disphoff, dispxoff, 
     //dispyoff);
 
+ 
+    VERBOSE(VB_PLAYBACK,
+            QString("Image size. dispxoff %1, dispyoff: %2, dispwoff: %3, "
+                    "disphoff: %4")
+            .arg(dispxoff).arg(dispyoff).arg(dispwoff).arg(disphoff));
+
+    VERBOSE(VB_PLAYBACK,
+            QString("Image size. imgx %1, imgy: %2, imgw: %3, imgh: %4")
+           .arg(imgx).arg(imgy).arg(imgw).arg(imgh));
+
     DrawUnusedRects();
 }
 
-void VideoOutput::ToggleLetterbox(void)
+void VideoOutput::Zoom(int direction)
 {
-    if (++letterbox > 3)
-        letterbox = 0;
+    switch (direction)
+    {
+        case kZoomHome:
+                ZoomedIn = 0;
+                ZoomedUp = 0;
+                ZoomedRight = 0;
+                break;
+        case kZoomIn:
+                if (ZoomedIn < 200)
+                    ZoomedIn += 10;
+                else
+                    ZoomedIn = 0;
+                break;
+        case kZoomOut:
+                if (ZoomedIn > -50)
+                    ZoomedIn -= 10;
+                else
+                    ZoomedIn = 0;
+                break;
+        case kZoomUp:
+                if (ZoomedUp < 90)
+                    ZoomedUp += 10;
+                break;
+        case kZoomDown:
+                if (ZoomedUp > -90)
+                    ZoomedUp -= 10;
+                break;
+        case kZoomLeft:
+                if (ZoomedRight < 90)
+                    ZoomedRight += 10;
+                break;
+        case kZoomRight:
+                if (ZoomedRight > -90)
+                    ZoomedRight -= 10;
+                break;
+    }
+}
+
+void VideoOutput::ToggleLetterbox(int letterboxMode)
+{
+    if (letterboxMode == kLetterbox_Toggle)
+    {
+        if (++letterbox >= kLetterbox_END)
+            letterbox = 0;
+    }
+    else
+    {
+        letterbox = letterboxMode;
+    }
 
     switch (letterbox) 
     {
-        default: case 0: case 2: AspectChanged(4.0 / 3); break;
-        case 1: case 3: AspectChanged(16.0 / 9); break;
+        default:
+        case kLetterbox_4_3:
+        case kLetterbox_4_3_Zoom:      AspectChanged(4.0 / 3);
+                                       break;
+        case kLetterbox_16_9:
+        case kLetterbox_16_9_Zoom:
+        case kLetterbox_16_9_Stretch:  AspectChanged(16.0 / 9);
+                                       break;
     }
 }
 
@@ -793,9 +905,17 @@ int VideoOutput::DisplayOSD(VideoFrame *frame, OSD *osd, int stride)
 {
     int retval = -1;
 
+    //struct timeval one, two, three, four;
+    //struct timeval done = {0, 0}, dtwo = {0, 0};
+
     if (osd)
     {
+        //gettimeofday(&one, NULL);
         OSDSurface *surface = osd->Display();
+        //gettimeofday(&two, NULL);
+        //timersub(&two, &one, &done);
+
+        //gettimeofday(&three, NULL);
         if (surface)
         {
             switch (frame->codec)
@@ -832,14 +952,20 @@ int VideoOutput::DisplayOSD(VideoFrame *frame, OSD *osd, int stride)
             }
             retval = surface->Changed() ? 1 : 0;
         }
+        //gettimeofday(&four, NULL);
+        //timersub(&four, &three, &dtwo);
     }
 
+    //cout << done.tv_usec << " " << dtwo.tv_usec << endl;
+
     return retval;
 }
- 
+
 void VideoOutput::BlendSurfaceToYV12(OSDSurface *surface, unsigned char *yuvptr,
                                      int stride)
 {
+    blendtoyv12_8_fun blender = blendtoyv12_8_init(surface);
+
     unsigned char *uptrdest = yuvptr + surface->width * surface->height;
     unsigned char *vptrdest = uptrdest + surface->width * surface->height / 4;
 
@@ -871,36 +997,64 @@ void VideoOutput::BlendSurfaceToYV12(OSDSurface *surface, unsigned char *yuvptr,
             dest = yuvptr + yoffset + startcol;
             alpha = surface->alpha + yoffset + startcol;
 
-            usrc = surface->u + yoffset / 4 + startcol / 2;
-            udest = uptrdest + yoffset / 4 + startcol / 2;
-
-            vsrc = surface->v + yoffset / 4 + startcol / 2;
-            vdest = vptrdest + yoffset / 4 + startcol / 2;
-
             for (int x = startcol; x <= endcol; x++)
             {
-                if (*alpha == 0)
-                    goto blendimageyv12end;
-
-                *dest = blendColorsAlpha(*src, *dest, *alpha);
-
-                if ((y % 2 == 0) && (x % 2 == 0))
+                if (x + 8 >= endcol)
                 {
-                    *udest = blendColorsAlpha(*usrc, *udest, *alpha);
-                    *vdest = blendColorsAlpha(*vsrc, *vdest, *alpha);
+                    if (*alpha != 0)
+                        *dest = blendColorsAlpha(*src, *dest, *alpha);
+                    src++;
+                    dest++;
+                    alpha++;
                 }
-
-blendimageyv12end:
-                if ((y % 2 == 0) && (x % 2 == 0))
+                else
                 {
-                    usrc++;
-                    udest++;
-                    vsrc++;
-                    vdest++;
+                    blender(src, dest, alpha, false);
+                    src += 8;
+                    dest += 8;
+                    alpha += 8;
+                    x += 7;
+                }
+            }
+
+            alpha = surface->alpha + yoffset + startcol;
+
+            if (y % 2 == 0)
+            {
+                usrc = surface->u + yoffset / 4 + startcol / 2;
+                udest = uptrdest + yoffset / 4 + startcol / 2;
+
+                vsrc = surface->v + yoffset / 4 + startcol / 2;
+                vdest = vptrdest + yoffset / 4 + startcol / 2;
+
+                for (int x = startcol; x <= endcol; x += 2)
+                {
+                    alpha = surface->alpha + yoffset + x;
+
+                    if (x + 16 >= endcol)
+                    {
+                        if (*alpha != 0)
+                        {
+                            *udest = blendColorsAlpha(*usrc, *udest, *alpha);
+                            *vdest = blendColorsAlpha(*vsrc, *vdest, *alpha);
+                        }
+
+                        usrc++;
+                        udest++;
+                        vsrc++;
+                        vdest++;
+                    }
+                    else
+                    {
+                        blender(usrc, udest, alpha, true);
+                        blender(vsrc, vdest, alpha, true);
+                        usrc += 8;
+                        udest += 8;
+                        vsrc += 8;
+                        vdest += 8;
+                        x += 14;
+                    }
                 }
-                src++;
-                dest++;
-                alpha++;
             }
         }
     }
@@ -918,6 +1072,9 @@ void VideoOutput::BlendSurfaceToI44(OSDSurface *surface, unsigned char *yuvptr,
     if (stride < 0)
         stride = XJ_width;
 
+    dithertoia44_8_fun ditherer = dithertoia44_8_init(surface);
+    dither8_context *dcontext = init_dithertoia44_8_context(ifirst);
+
     memset(yuvptr, 0x0, stride * XJ_height);
 
     QMemArray<QRect> rects = surface->usedRegions.rects();
@@ -956,57 +1113,36 @@ void VideoOutput::BlendSurfaceToI44(OSDSurface *surface, unsigned char *yuvptr,
 
             for (int x = startcol; x <= endcol; x++)
             {
-                if (*alpha == 0)
+                if (x + 8 >= endcol)
                 {
-                    *dest = 0;
-                    goto blendimagei44end;
+                    if (*alpha != 0)
+                    {
+                        grey = *src + ((dmp[(x & (DM_WIDTH - 1))] << 2) >> 4);
+                        grey = (grey - (grey >> 4)) >> 4;
+
+                        *dest = (((*alpha >> 4) << ashift) & amask) |
+                                (((grey) << ishift) & imask);
+                    }
+                    else
+                        *dest = 0;
+
+                    src++;
+                    dest++;
+                    alpha++;
+                }
+                else
+                {
+                    ditherer(src, dest, alpha, dmp, x, dcontext);
+                    src += 8;
+                    dest += 8;
+                    alpha += 8;
+                    x += 7;
                 }
-
-#if 0
-                grey = *src >> 4; 
-#endif
-                grey = *src + ((dmp[(x & (DM_WIDTH - 1))] << 2) >> 4);
-                grey = (grey - (grey >> 4)) >> 4;
-
-                *dest = (((*alpha >> 4) << ashift) & amask) |
-                        (((grey) << ishift) & imask);
-
-blendimagei44end:
-                src++;
-                dest++;
-                alpha++;
             }
         }
     }
-}
 
-#define SCALEBITS 10
-#define ONE_HALF  (1 << (SCALEBITS - 1))
-#define FIX(x)    ((int) ((x) * (1<<SCALEBITS) + 0.5))
-
-#define YUV_TO_RGB1(cb1, cr1)\
-{\
-    cb = ((int)cb1) - 128;\
-    cr = ((int)cr1) - 128;\
-    r_add = FIX(1.40200) * cr + ONE_HALF;\
-    g_add = - FIX(0.34414) * cb - FIX(0.71414) * cr + ONE_HALF;\
-    b_add = FIX(1.77200) * cb + ONE_HALF;\
-}
-
-#define YUV_TO_RGB2(r, g, b, y1)\
-{\
-    y0 = ((int)y1) << SCALEBITS;\
-    r = cm[(y0 + r_add) >> SCALEBITS];\
-    g = cm[(y0 + g_add) >> SCALEBITS];\
-    b = cm[(y0 + b_add) >> SCALEBITS];\
-}
-
-#define RGBA_OUT(d, r, g, b, a)\
-{\
-    ((unsigned int *)(d))[0] =  ((a) << 24) | \
-                                ((r) << 16) | \
-                                ((g) << 8) | \
-                                (b);\
+    delete_dithertoia44_8_context(dcontext);
 }
 
 void VideoOutput::BlendSurfaceToARGB(OSDSurface *surface, 
@@ -1015,6 +1151,9 @@ void VideoOutput::BlendSurfaceToARGB(OSDSurface *surface,
     if (stride < 0)
         stride = XJ_width;
 
+    blendtoargb_8_fun blender = blendtoargb_8_init(surface);
+    unsigned char *cm = surface->cm;
+
     memset(argbptr, 0x0, stride * XJ_height);
 
     QMemArray<QRect> rects = surface->usedRegions.rects();
@@ -1029,7 +1168,7 @@ void VideoOutput::BlendSurfaceToARGB(OSDSurface *surface,
         endcol = drawRect.right();
         endline = drawRect.bottom();
 
-        unsigned char *src, *usrc, *vsrc;
+        unsigned char *src, *usrcbase, *vsrcbase, *usrc, *vsrc;
         unsigned char *dest;
         unsigned char *alpha;
 
@@ -1048,28 +1187,34 @@ void VideoOutput::BlendSurfaceToARGB(OSDSurface *surface,
             dest = argbptr + destyoffset + startcol * 4;
             alpha = surface->alpha + yoffset + startcol;
 
-            usrc = surface->u + yoffset / 4 + startcol / 2;
-            vsrc = surface->v + yoffset / 4 + startcol / 2;
+            usrcbase = surface->u + (y / 2) * (surface->width / 2);
+            vsrcbase = surface->v + (y / 2) * (surface->width / 2);
 
             for (int x = startcol; x <= endcol; x++)
             {
-                if (*alpha == 0)
-                    goto blendimageargbend;
+                usrc = usrcbase + x / 2;
+                vsrc = vsrcbase + x / 2;
 
-                YUV_TO_RGB1(*usrc, *vsrc);
-                YUV_TO_RGB2(r, g, b, *src);
-                RGBA_OUT(dest, r, g, b, *alpha);
-
-blendimageargbend:
-                src++;
-                alpha++;
-                dest += 4;
-
-                if ((y % 2 == 0) && (x % 2 == 0))
+                if (x + 8 >= endcol)
                 {
-                    usrc++;
-                    vsrc++;
+                    if (*alpha != 0)
+                    {
+                        YUV_TO_RGB1(*usrc, *vsrc);
+                        YUV_TO_RGB2(r, g, b, *src);
+                        RGBA_OUT(dest, r, g, b, *alpha);
+                    }
+                    src++;
+                    alpha++;
+                    dest += 4;
                 }
+                else
+                {
+                    blender(surface, src, usrc, vsrc, alpha, dest);
+                    src += 8;
+                    dest += 32;
+                    alpha += 8;
+                    x += 7;
+                } 
             }
         }
     }
