@@ -80,11 +80,13 @@ class EncoderLink
                         QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
     void GetChannelInfo(QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid);
+                        QString &channelname, QString &chanid,
+                        QString &seriesid, QString &programid);
     void GetInputName(QString &inputname);
 
     void SetReadThreadSock(QSocket *rsock);
