@@ -19,6 +19,7 @@ using namespace std;
 #include "mainserver.h"
 #include "encoderlink.h"
 #include "remoteutil.h"
+#include "housekeeper.h"
 
 #include "libmythtv/programinfo.h"
 #include "libmyth/mythcontext.h"
@@ -30,6 +31,7 @@ Scheduler *sched = NULL;
 Transcoder *trans = NULL;
 QString pidfile;
 QString lockfile_location;
+HouseKeeper *housekeeping = NULL;
 
 bool setupTVs(bool ismaster)
 {
@@ -37,6 +39,77 @@ bool setupTVs(bool ismaster)
 
     QSqlQuery query;
 
+    if (ismaster)
+    {
+        // Hack to make sure record.station gets set if the user
+        // downgrades to a prior version and creates new entries
+        // without it.
+        if (!query.exec("UPDATE channel SET callsign=chanid "
+                        "WHERE callsign IS NULL OR callsign='';"))
+            MythContext::DBError("Updating channel callsign",
+                                 query.lastQuery());
+
+        if (query.exec("SELECT MIN(chanid) FROM channel;"))
+        {
+            query.first();
+            int min_chanid = query.value(0).toInt();
+            if (!query.exec(QString("UPDATE record SET chanid = %1 "
+                                    "WHERE chanid IS NULL;").arg(min_chanid)))
+                MythContext::DBError("Updating record chanid",
+                                     query.lastQuery());
+        }
+        else
+            MythContext::DBError("Querying minimum chanid",
+                                 query.lastQuery());
+
+        QSqlQuery records_without_station("SELECT record.chanid,"
+                " channel.callsign FROM record LEFT JOIN channel"
+                " ON record.chanid = channel.chanid WHERE record.station='';");
+        if (records_without_station.first())
+        {
+            QSqlQuery update_record;
+            update_record.prepare("UPDATE record SET station = :CALLSIGN"
+                    " WHERE chanid = :CHANID;");
+            do
+            {
+                update_record.bindValue(":CALLSIGN",
+                        records_without_station.value(1));
+                update_record.bindValue(":CHANID",
+                        records_without_station.value(0));
+                if (!update_record.exec())
+                {
+                    MythContext::DBError("Updating record station",
+                            update_record.lastQuery());
+                }
+            } while (records_without_station.next());
+        }
+
+        QSqlQuery overrides_without_station("SELECT recordoverride.recordid,"
+                " channel.callsign FROM recordoverride LEFT JOIN channel"
+                " ON recordoverride.chanid = channel.chanid"
+                " WHERE recordoverride.station='';");
+        if (overrides_without_station.first())
+        {
+            QSqlQuery update_override;
+            update_override.prepare("UPDATE recordoverride"
+                    " SET station = :CALLSIGN,"
+                    " starttime=starttime, endtime=endtime"
+                    " WHERE recordid = :RECORDID;");
+            do
+            {
+                update_override.bindValue(":CALLSIGN",
+                        overrides_without_station.value(1));
+                update_override.bindValue(":RECORDID",
+                        overrides_without_station.value(0));
+                if (!update_override.exec())
+                {
+                    MythContext::DBError("Updating recordoverride station",
+                            update_override.lastQuery());
+                }
+            } while (overrides_without_station.next());
+        }
+    }
+
     query.exec("SELECT cardid,hostname FROM capturecard ORDER BY cardid;");
 
     if (query.isActive() && query.numRowsAffected())
@@ -48,9 +121,13 @@ bool setupTVs(bool ismaster)
 
             if (host.isNull() || host.isEmpty())
             {
-                cerr << "One of your capturecard entries does not have a "
-                     << "hostname.\n  Please run setup and confirm all of the "
-                     << "capture cards.\n";
+                QString msg = "One of your capturecard entries does not have a "
+                              "hostname.\n  Please run setup and confirm all "
+                              "of the capture cards.\n";
+
+                cerr << msg;
+                gContext->LogEntry("mythbackend", LP_CRITICAL,
+                                   "Problem with capture cards", msg);
                 exit(-1);
             }
 
@@ -85,6 +162,9 @@ bool setupTVs(bool ismaster)
     {
         cerr << "ERROR: no capture cards are defined in the database.\n";
         cerr << "Perhaps you should read the installation instructions?\n";
+        gContext->LogEntry("mythbackend", LP_CRITICAL,
+                           "No capture cards are defined", 
+                           "Please run the setup program.");
         return false;
     }
 
@@ -120,8 +200,9 @@ int main(int argc, char **argv)
 
     bool daemonize = false;
     bool printsched = false;
+    bool testsched = false;
+    bool nosched = false;
     bool printexpire = false;
-    bool noAutoShutdown = false;
     for (int argpos = 1; argpos < a.argc(); ++argpos)
     {
         if (!strcmp(a.argv()[argpos],"-l") ||
@@ -164,12 +245,6 @@ int main(int argc, char **argv)
             daemonize = true;
 
         }
-        else if (!strcmp(a.argv()[argpos],"-n") ||
-                 !strcmp(a.argv()[argpos],"--noautoshutdown"))
-        {
-            noAutoShutdown = true;
-            VERBOSE(VB_ALL, "Auto shutdown procedure disabled by commandline"); 
-        }
         else if (!strcmp(a.argv()[argpos],"-v") ||
                  !strcmp(a.argv()[argpos],"--verbose"))
         {
@@ -239,6 +314,16 @@ int main(int argc, char **argv)
                             print_verbose_messages |= VB_NETWORK;
                             verboseString += " " + *it;
                         }
+                        else if(!strcmp(*it,"commflag"))
+                        {
+                            print_verbose_messages |= VB_COMMFLAG;
+                            verboseString += " " + *it;
+                        }
+                        else if(!strcmp(*it,"audio"))
+                        {
+                            print_verbose_messages |= VB_AUDIO;
+                            verboseString += " " + *it;
+                        }
                         else
                         {
                             cerr << "Unknown argument for -v/--verbose: "
@@ -257,6 +342,14 @@ int main(int argc, char **argv)
         {
             printsched = true;
         } 
+        else if (!strcmp(a.argv()[argpos],"--testsched"))
+        {
+            testsched = true;
+        } 
+        else if (!strcmp(a.argv()[argpos],"--nosched"))
+        {
+            nosched = true;
+        } 
         else if (!strcmp(a.argv()[argpos],"--printexpire"))
         {
             printexpire = true;
@@ -276,12 +369,10 @@ int main(int argc, char **argv)
                     "-p or --pidfile filename       Write PID of mythbackend " <<
                                                     "to filename" << endl <<
                     "-d or --daemon                 Runs mythbackend as a daemon" << endl <<
-                    "-n or --noautoshutdonw         Blocks the auto shutdown routine, so that the" << endl <<
-                    "                               backend will stay alive"<<endl <<
                     "-v or --verbose debug-level    Prints more information" << endl <<
                     "                               Accepts any combination (separated by comma)" << endl << 
-                    "                               of all,none,quiet,record,playback," << endl <<
-                    "                               channel,osd,file,schedule,network" << endl <<
+                    "                               of all,none,quiet,record,playback,channel," << endl <<
+                    "                               osd,file,schedule,network,commflag,audio" << endl <<
                     "--printexpire                  List of auto-expire programs" << endl <<
                     "--printsched                   Upcoming scheduled programs" << endl <<
                     "--version                      Version information" << endl;
@@ -364,6 +455,13 @@ int main(int argc, char **argv)
         return -1;
     }
 
+    QSqlDatabase *hkthread = QSqlDatabase::addDatabase("QMYSQL3", "HKDB");
+    if (!hkthread)
+    {
+        cerr << "Couldn't connect to database\n";
+        return -1;
+    }
+
     QSqlDatabase *transthread = QSqlDatabase::addDatabase("QMYSQL3", "TRANSDB");
     if (!transthread)
     {
@@ -380,6 +478,7 @@ int main(int argc, char **argv)
 
     if (!gContext->OpenDatabase(db) || !gContext->OpenDatabase(subthread) ||
         !gContext->OpenDatabase(expthread) ||
+        !gContext->OpenDatabase(hkthread) ||
         !gContext->OpenDatabase(transthread) ||
         !gContext->OpenDatabase(msdb))
     {
@@ -389,10 +488,10 @@ int main(int argc, char **argv)
 
     UpgradeTVDatabaseSchema();
 
-    if (printsched)
+    if (printsched || testsched)
     {
         sched = new Scheduler(false, &tvList, db);
-        if (gContext->ConnectToMasterServer())
+        if (!testsched && gContext->ConnectToMasterServer())
         {
             cout << "Retrieving Schedule from Master backend.\n";
             sched->FillRecordListFromMaster();
@@ -402,10 +501,11 @@ int main(int argc, char **argv)
             cout << "Calculating Schedule from database.\n" <<
                     "Inputs, Card IDs, and Conflict info may be invalid "
                     "if you have multiple tuners.\n";
-            sched->FillRecordLists(false);
+            sched->FillRecordLists();
         }
 
-        sched->PrintList();
+        print_verbose_messages |= VB_SCHEDULE;
+        sched->PrintList(true);
         cleanup();
         exit(0);
     }
@@ -438,24 +538,35 @@ int main(int argc, char **argv)
     if (masterip == myip)
     {
         cerr << "Starting up as the master server.\n";
+        gContext->LogEntry("mythbackend", LP_INFO,
+                           "MythBackend started as master server", "");
         ismaster = true;
+
+        if (nosched)
+            cerr << "******** The Scheduler has been DISABLED with "
+                    "the --nosched option ********\n";
     }
     else
     {
         cerr << "Running as a slave backend.\n";
+        gContext->LogEntry("mythbackend", LP_INFO,
+                           "MythBackend started as a slave backend", "");
     }
  
     bool runsched = setupTVs(ismaster);
 
-    if (ismaster && runsched)
+    if (ismaster && runsched && !nosched)
     {
         QSqlDatabase *scdb = QSqlDatabase::database("SUBDB");
-        sched = new Scheduler(true, &tvList, scdb, noAutoShutdown);
+        sched = new Scheduler(true, &tvList, scdb);
     }
 
     QSqlDatabase *expdb = QSqlDatabase::database("EXPDB");
     expirer = new AutoExpire(true, ismaster, expdb);
 
+    QSqlDatabase *hkdb = QSqlDatabase::database("HKDB");
+    housekeeping = new HouseKeeper(true, ismaster, hkdb);
+
     QSqlDatabase *trandb = QSqlDatabase::database("TRANSDB");
     trans = new Transcoder(&tvList, trandb);
 
@@ -489,8 +600,8 @@ int main(int argc, char **argv)
 
     if (ismaster)
     {
-        QString WOLslaveBackends;
-        WOLslaveBackends = gContext->GetSetting("WOLslaveBackendsCommand","");
+        QString WOLslaveBackends
+            = gContext->GetSetting("WOLslaveBackendsCommand","");
         if (!WOLslaveBackends.isEmpty())
         {
             VERBOSE(VB_ALL, "Waking slave Backends now.");
@@ -502,6 +613,7 @@ int main(int argc, char **argv)
 
     // delete trans;
 
+    gContext->LogEntry("mythbackend", LP_INFO, "MythBackend exiting", "");
     cleanup();
 
     return 0;
