@@ -257,6 +257,10 @@ void MainServer::ProcessRequest(QSocket *sock)
     {
         HandleDeleteRecording(listline, pbs);
     }
+    else if (command == "REACTIVATE_RECORDING")
+    {
+        HandleReactivateRecording(listline, pbs);
+    }
     else if (command == "FORGET_RECORDING")
     {
         HandleForgetRecording(listline, pbs);
@@ -271,15 +275,16 @@ void MainServer::ProcessRequest(QSocket *sock)
     }
     else if (command == "QUERY_GETCONFLICTING")
     {
-        if (tokens.size() != 2)
-            VERBOSE(VB_ALL, "Bad QUERY_GETCONFLICTING");
-        else
-            HandleGetConflictingRecordings(listline, tokens[1], pbs);
+        HandleGetConflictingRecordings(listline, pbs);
     }
     else if (command == "GET_FREE_RECORDER")
     {
         HandleGetFreeRecorder(pbs);
     }
+    else if (command == "GET_NEXT_FREE_RECORDER")
+    {
+        HandleGetNextFreeRecorder(listline, pbs);
+    }
     else if (command == "QUERY_RECORDER")
     {
         if (tokens.size() != 2)
@@ -350,9 +355,15 @@ void MainServer::ProcessRequest(QSocket *sock)
             VERBOSE(VB_ALL, "Bad SHUTDOWN_NOW query");
         else if (!ismaster)
         {
-            VERBOSE(VB_ALL, "Going down now as of Mainserver request!");
             QString halt_cmd = listline[1];
-            system(halt_cmd.ascii());
+            if (!halt_cmd.isEmpty())
+            {
+                VERBOSE(VB_ALL, "Going down now as of Mainserver request!");
+                system(halt_cmd.ascii());
+            }
+            else
+                VERBOSE(VB_ALL,
+                        "WARNING: Recieved an empty SHUTDOWN_NOW query!");
         }
     }
     else if (command == "BACKEND_MESSAGE")
@@ -362,6 +373,11 @@ void MainServer::ProcessRequest(QSocket *sock)
         MythEvent me(message, extra);
         gContext->dispatch(me);
     }
+    else if (command == "REFRESH_BACKEND")
+    {
+        VERBOSE(VB_ALL,"Reloading backend settings");
+        HandleBackendRefresh(sock);
+    }
     else
         VERBOSE(VB_ALL, "Unknown command: " + command);
 }
@@ -410,7 +426,7 @@ void MainServer::customEvent(QCustomEvent *e)
 
         broadcast = "BACKEND_MESSAGE";
         broadcast << me->Message();
-        broadcast << me->ExtraData();
+        broadcast += me->ExtraDataList();
         sendstuff = true;
     }
 
@@ -506,14 +522,16 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
 
             int dsp_status, soundcardcaps;
             QString audiodevice, audiooutputdevice, querytext;
+            QString cardtype;
 
-            querytext = QString("SELECT audiodevice FROM capturecard "
+            querytext = QString("SELECT audiodevice,cardtype FROM capturecard "
                                 "WHERE cardid=%1;").arg(recnum);
             QSqlQuery query = m_db->exec(querytext);
             if (query.isActive() && query.numRowsAffected())
             {
                 query.next();
                 audiodevice = query.value(0).toString();
+                cardtype = query.value(1).toString();
             }
 
             query = m_db->exec("SELECT data FROM settings WHERE "
@@ -526,7 +544,8 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
 
             dblock.unlock();
 
-            if (audiodevice.right(4) == audiooutputdevice.right(4)) //they match
+            if (audiodevice.right(4) == audiooutputdevice.right(4) &&
+                (cardtype == "V4L" || cardtype == "MJPEG")) //they match
             {
                 int dsp_fd = open(audiodevice, O_RDWR | O_NONBLOCK);
                 if (dsp_fd != -1)
@@ -609,16 +628,24 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
     dblock.lock();
     MythContext::KickDatabase(m_db);
 
-    thequery = "SELECT recorded.chanid,starttime,endtime,title,subtitle,"
-               "description,hostname,channum,name,callsign,commflagged,cutlist,"
-               "autoexpire,editing,bookmark,category FROM recorded "
+    thequery = "SELECT recorded.chanid,recorded.starttime,recorded.endtime,"
+               "recorded.title,recorded.subtitle,recorded.description,"
+               "recorded.hostname,channum,name,callsign,commflagged,cutlist,"
+               "recorded.autoexpire,editing,bookmark,recorded.category,"
+               "recorded.recgroup,record.dupin,record.dupmethod,"
+               "record.recordid,outputfilters,"
+               "recorded.seriesid,recorded.programid "
+               "FROM recorded "
+               "LEFT JOIN record ON recorded.recordid = record.recordid "
                "LEFT JOIN channel ON recorded.chanid = channel.chanid "
-               "ORDER BY starttime";
+               "ORDER BY recorded.starttime";
 
     if (type == "Delete")
         thequery += " DESC";
     thequery += ";";
 
+    
+    
     QSqlQuery query = m_db->exec(thequery);
 
     QStringList outputlist;
@@ -644,19 +671,17 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
             proginfo->description = QString::fromUtf8(query.value(5).toString());
             proginfo->hostname = query.value(6).toString();
 
+            proginfo->dupin = RecordingDupInType(query.value(17).toInt());
+            proginfo->dupmethod = RecordingDupMethodType(query.value(18).toInt());
+            proginfo->recordid = query.value(19).toInt();
+            proginfo->chanOutputFilters = query.value(20).toString();
+            proginfo->seriesid = query.value(21).toString();
+            proginfo->programid = query.value(22).toString();
+
             if (proginfo->hostname.isEmpty() || proginfo->hostname.isNull())
                 proginfo->hostname = gContext->GetHostName();
 
-            proginfo->conflicting = false;
-
-            if (proginfo->title == QString::null)
-                proginfo->title = "";
-            if (proginfo->subtitle == QString::null)
-                proginfo->subtitle = "";
-            if (proginfo->description == QString::null)
-                proginfo->description = "";
-
-            if (!query.value(7).toString().isNull())
+            if (!query.value(7).toString().isEmpty())
             {
                 proginfo->chanstr = query.value(7).toString();
                 proginfo->channame = QString::fromUtf8(query.value(8).toString());
@@ -680,9 +705,9 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
 
             proginfo->programflags = flags;
 
-            QString test = query.value(15).toString();
-            if (test != QString::null) 
-               proginfo->category = QString::fromUtf8(test);
+            proginfo->category = QString::fromUtf8(query.value(15).toString());
+
+            proginfo->recgroup = query.value(16).toString();
 
             QString lpath = proginfo->GetRecordFilename(fileprefix);
             PlaybackSock *slave = NULL;
@@ -906,7 +931,8 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             {
                 (*encoderList)[num]->StopRecording();
                 pginfo->recstatus = rsStopped;
-                m_sched->AddToDontRecord(pginfo);
+                if (m_sched)
+                    m_sched->UpdateRecStatus(pginfo);
             }
 
             if (pbssock)
@@ -942,7 +968,8 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             if (ismaster)
             {
                 pginfo->recstatus = rsStopped;
-                m_sched->AddToDontRecord(pginfo);
+                if (m_sched)
+                    m_sched->UpdateRecStatus(pginfo);
             }
         }
     }
@@ -1064,7 +1091,8 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
            {
                (*encoderList)[num]->StopRecording();
                pginfo->recstatus = rsDeleted;
-               m_sched->AddToDontRecord(pginfo);
+               if (m_sched)
+                   m_sched->UpdateRecStatus(pginfo);
            }
 
            if (pbssock)
@@ -1100,7 +1128,8 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             if (ismaster)
             {
                 pginfo->recstatus = rsDeleted;
-                m_sched->AddToDontRecord(pginfo);
+                if (m_sched)
+                    m_sched->UpdateRecStatus(pginfo);
             }
         }
     }
@@ -1181,6 +1210,25 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
     delete pginfo;
 }
 
+void MainServer::HandleReactivateRecording(QStringList &slist, PlaybackSock *pbs)
+{
+    ProgramInfo pginfo;
+    pginfo.FromStringList(slist, 1);
+
+    QStringList result;
+    if (m_sched)
+        result = QString::number(m_sched->ReactivateRecording(&pginfo));
+    else
+        result = QString::number(0);
+
+    if (pbs)
+    {
+        QSocket *pbssock = pbs->getSocket();
+        if (pbssock)
+            SendResponse(pbssock, result);
+    }
+}
+
 void MainServer::HandleForgetRecording(QStringList &slist, PlaybackSock *pbs)
 {
     ProgramInfo *pginfo = new ProgramInfo();
@@ -1300,7 +1348,13 @@ void MainServer::HandleGetPendingRecordings(PlaybackSock *pbs)
 
     QStringList strList;
 
-    m_sched->getAllPending(&strList);
+    if (m_sched)
+        m_sched->getAllPending(strList);
+    else
+    {
+        strList << QString::number(0);
+        strList << QString::number(0);
+    }
 
     SendResponse(pbssock, strList);
 }
@@ -1311,25 +1365,28 @@ void MainServer::HandleGetScheduledRecordings(PlaybackSock *pbs)
 
     QStringList strList;
 
-    m_sched->getAllScheduled(&strList);
+    if (m_sched)
+        m_sched->getAllScheduled(strList);
+    else
+        strList << QString::number(0);
 
     SendResponse(pbssock, strList);
 }
 
 void MainServer::HandleGetConflictingRecordings(QStringList &slist,
-                                                QString purge,
                                                 PlaybackSock *pbs)
 {
     QSocket *pbssock = pbs->getSocket();
 
-    bool removenonplaying = purge.toInt();
-
     ProgramInfo *pginfo = new ProgramInfo();
     pginfo->FromStringList(slist, 1);
 
     QStringList strlist;
 
-    m_sched->getConflicting(pginfo, removenonplaying, &strlist);
+    if (m_sched)
+        m_sched->getConflicting(pginfo, strlist);
+    else
+        strlist << QString::number(0);
 
     SendResponse(pbssock, strlist);
 
@@ -1518,6 +1575,101 @@ void MainServer::HandleGetFreeRecorder(PlaybackSock *pbs)
     SendResponse(pbssock, strlist);
 }
 
+void MainServer::HandleGetNextFreeRecorder(QStringList &slist, 
+                                           PlaybackSock *pbs)
+{
+    QSocket *pbssock = pbs->getSocket();
+    QString pbshost = pbs->getHostname();
+
+    QStringList strlist;
+    int retval = -1;
+    int currrec = slist[1].toInt();
+
+    EncoderLink *encoder = NULL;
+    QString enchost;
+
+    VERBOSE(VB_ALL, QString("Getting next free recorder : %1").arg(currrec));
+
+    // find current recorder
+    QMap<int, EncoderLink *>::Iterator iter, curr = encoderList->find(currrec);
+
+    if (curr != encoderList->end())
+    {
+        // cycle through all recorders
+        for (iter = curr;;)
+        {
+            EncoderLink *elink;
+
+            // last item? go back
+            if (++iter == encoderList->end())
+            {
+                iter = encoderList->begin();
+            }
+
+            elink = iter.data();
+
+            if (elink->isLocal())
+                enchost = gContext->GetHostName();
+            else
+                enchost = elink->getHostname();
+
+            if ((enchost == pbshost) &&
+                (elink->isConnected()) &&
+                (!elink->IsBusy()) &&
+                (!elink->isTunerLocked()))
+            {
+                encoder = elink;
+                retval = iter.key(); 
+                break;
+            }
+            if ((retval == -1) &&
+                (elink->isConnected()) &&
+                (!elink->IsBusy()) &&
+                (!elink->isTunerLocked()))
+            {
+                encoder = elink;
+                retval = iter.key();
+            }
+
+            // cycled right through? no more available recorders
+            if (iter == curr) 
+                break;
+        }
+    } 
+    else 
+    {
+        HandleGetFreeRecorder(pbs);
+        return;
+    }
+
+
+    strlist << QString::number(retval);
+        
+    if (encoder)
+    {
+        if (encoder->isLocal())
+        {
+            strlist << gContext->GetSetting("BackendServerIP");
+            strlist << gContext->GetSetting("BackendServerPort");
+        }
+        else
+        {
+            strlist << gContext->GetSettingOnHost("BackendServerIP", 
+                                                  encoder->getHostname(),
+                                                  "nohostname");
+            strlist << gContext->GetSettingOnHost("BackendServerPort", 
+                                                  encoder->getHostname(), "-1");
+        }
+    }
+    else
+    {
+        strlist << "nohost";
+        strlist << "-1";
+    }
+
+    SendResponse(pbssock, strlist);
+}
+
 void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
                                      PlaybackSock *pbs)
 {
@@ -1718,10 +1870,12 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
 
         QString title = "", subtitle = "", desc = "", category = "";
         QString endtime = "", callsign = "", iconpath = "";
+        QString seriesid = "", programid = "";
 
         enc->GetNextProgram(direction,
                             title, subtitle, desc, category, starttime,
-                            endtime, callsign, iconpath, channelname, chanid);
+                            endtime, callsign, iconpath, channelname, chanid,
+                            seriesid, programid);
 
         if (title == "")
             title = " ";
@@ -1743,6 +1897,10 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
             channelname = " ";
         if (chanid == "")
             chanid = " ";
+        if (seriesid == "")
+            seriesid = " ";
+        if (programid == "")
+            programid = " ";
 
         retlist << title;
         retlist << subtitle;
@@ -1754,15 +1912,18 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
         retlist << iconpath;
         retlist << channelname;
         retlist << chanid;
+        retlist << seriesid;
+        retlist << programid;
     }
     else if (command == "GET_PROGRAM_INFO")
     {
         QString title = "", subtitle = "", desc = "", category = "";
         QString starttime = "", endtime = "", callsign = "", iconpath = "";
-        QString channelname = "", chanid = "";
+        QString channelname = "", chanid = "", seriesid = "", programid = "";
 
         enc->GetChannelInfo(title, subtitle, desc, category, starttime,
-                            endtime, callsign, iconpath, channelname, chanid);
+                            endtime, callsign, iconpath, channelname, chanid,
+                            seriesid, programid);
 
         if (title == "")
             title = " ";
@@ -1784,6 +1945,10 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
             channelname = " ";
         if (chanid == "")
             chanid = " ";
+        if (seriesid == "")
+            seriesid = " ";
+        if (programid == "")
+            programid = " ";
 
         retlist << title;
         retlist << subtitle;
@@ -1795,6 +1960,8 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
         retlist << iconpath;
         retlist << channelname;
         retlist << chanid;
+        retlist << seriesid;
+        retlist << programid;
     }
     else if (command == "GET_INPUT_NAME")
     {
@@ -2134,13 +2301,32 @@ void MainServer::HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs)
     if (qurl.host() != "")
         filename = LocalFilePath(qurl);
 
+    EncoderLink *elink = NULL;
+    QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
+
+    for (; iter != encoderList->end(); ++iter)
+    {
+        if (iter.data()->isLocal())
+            elink = iter.data();
+    }
+
+    if (elink == NULL)
+    {
+        VERBOSE(VB_ALL, QString("Couldn't find backend for: %1 : %2")
+                               .arg(pginfo->title).arg(pginfo->subtitle));
+        QStringList outputlist = "BAD";
+        SendResponse(pbssock, outputlist);
+        delete pginfo;
+        return;
+    }
+
     int len = 0;
     int width = 0, height = 0;
-
-    EncoderLink *elink = encoderList->begin().data();
+    int secondsin = gContext->GetNumSetting("PreviewPixmapOffset", 64);
 
     unsigned char *data = (unsigned char *)elink->GetScreenGrab(pginfo, 
-                                                                filename, 64,
+                                                                filename, 
+                                                                secondsin,
                                                                 len, width,
                                                                 height);
 
@@ -2153,13 +2339,27 @@ void MainServer::HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs)
         filename += ".png";
         img.save(filename.ascii(), "PNG");
 
-        delete [] data;
+        QStringList retlist = "OK";
+        SendResponse(pbssock, retlist);    
     }
+    else
+    {
+        VERBOSE(VB_ALL, QString("Not enough video to make thumbnail"));
+
+        QStringList outputlist = "BAD";
+        SendResponse(pbssock, outputlist);
+    } 
+
+    delete [] data;
+    delete pginfo;
+}
+
+void MainServer::HandleBackendRefresh(QSocket *socket)
+{
+    gContext->RefreshBackendConfig();
 
     QStringList retlist = "OK";
-    SendResponse(pbssock, retlist);    
-
-    delete pginfo;
+    SendResponse(socket, retlist);    
 }
 
 void MainServer::endConnection(QSocket *socket)
@@ -2462,6 +2662,89 @@ void MainServer::ShutSlaveBackendsDown(QString &haltcmd)
     }
 }
 
+#if USING_DVB
+void MainServer::PrintDVBStatus(QTextStream& os)
+{
+    dblock.lock();
+
+    QString querytext;
+
+    bool doneAnything = false;
+    
+    os << "\r\n  <div class=\"content\">\r\n" <<
+        "    <h2>DVB Signal Information</h2>\r\n" <<
+        "    Details of DVB error statistics for last 48 hours:<br />\r\n";
+
+    QString outerqry =
+        "SELECT starttime,endtime FROM recorded "
+        "WHERE starttime >= DATE_SUB(NOW(), INTERVAL 48 HOUR) "
+        "ORDER BY starttime;";
+
+    QSqlQuery oquery = m_db->exec(outerqry);
+
+    if (oquery.isActive() && oquery.numRowsAffected())
+    {
+        querytext = QString("SELECT cardid,"
+                            "max(fe_ss),min(fe_ss),avg(fe_ss),"
+                            "max(fe_snr),min(fe_snr),avg(fe_snr),"
+                            "max(fe_ber),min(fe_ber),avg(fe_ber),"
+                            "max(fe_unc),min(fe_unc),avg(fe_unc),"
+                            "max(myth_cont),max(myth_over),max(myth_pkts) "
+                            "FROM dvb_signal_quality "
+                            "WHERE sampletime BETWEEN ? AND ? "
+                            "GROUP BY cardid");
+
+        QSqlQuery query = m_db->exec(NULL);
+        query.prepare(querytext);
+        
+        while (oquery.next())
+        {
+            QDateTime t_start = oquery.value(0).toDateTime();
+            QDateTime t_end = oquery.value(1).toDateTime();
+
+            query.bindValue(0, t_start);
+            query.bindValue(1, t_end);
+
+            if (!query.exec())
+                cout << query.lastError().databaseText() << "\r\n" 
+                     << query.lastError().driverText() << "\r\n";
+            
+            if (query.isActive() && query.numRowsAffected())
+            {
+                os << "    <br />Recording period from " 
+                   << t_start.toString() << 
+                    " to " << t_end.toString() <<
+                    "<br />\n";
+                
+                while (query.next())
+                {
+                    os << "    Encoder " << query.value(0).toInt() <<
+                        " Min SNR: " << query.value(5).toInt() <<
+                        " Avg SNR: " << query.value(6).toInt() <<
+                        " Min BER: " << query.value(8).toInt() <<
+                        " Avg BER: " << query.value(9).toInt() <<
+                        " Cont Errs: " << query.value(13).toInt() <<
+                        " Overflows: " << query.value(14).toInt() <<
+                        "<br />\r\n";
+
+                    doneAnything = true;
+                }
+            }
+        }
+    }
+
+    if (!doneAnything)
+    {
+        os << "    <br />There is no DVB signal quality data available to "
+            "display.<br />\r\n";
+    }
+
+    dblock.unlock();
+
+    os << "  </div>\r\n";
+}
+#endif
+
 void MainServer::PrintStatus(QSocket *socket)
 {
     QTextStream os(socket);
@@ -2575,6 +2858,7 @@ void MainServer::PrintStatus(QSocket *socket)
 
     // encoder information ---------------------
     QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
+    int numencoders = 0;
     for (; iter != encoderList->end(); ++iter)
     {
         EncoderLink *elink = iter.data();
@@ -2585,6 +2869,7 @@ void MainServer::PrintStatus(QSocket *socket)
         {
             os << "    Encoder " << elink->getCardId() << " is local on "
                << gContext->GetHostName();
+            numencoders++;
         }
         else
         {
@@ -2595,6 +2880,8 @@ void MainServer::PrintStatus(QSocket *socket)
                 os << " (currently not connected).<br />";
                 continue;
             }
+            else
+                numencoders++;
         }
 
         TVState encstate = elink->GetState();
@@ -2605,8 +2892,9 @@ void MainServer::PrintStatus(QSocket *socket)
             if (bIsLocal)
             {
                 QString title, callsign, dummy;
-                elink->GetChannelInfo(title, dummy, dummy, dummy, dummy, dummy,
-                                      callsign, dummy, dummy, dummy);
+                elink->GetChannelInfo(title, dummy, dummy, dummy, dummy,
+                                      dummy, callsign, dummy, dummy,
+                                      dummy, dummy, dummy);
                 os << ": '" << title << "' on "  << callsign;
             }
             os << ".";
@@ -2643,7 +2931,8 @@ void MainServer::PrintStatus(QSocket *socket)
 
     // upcoming shows ---------------------
     list<ProgramInfo *> recordingList;
-    m_sched->getAllPending(&recordingList);
+    if (m_sched)
+        m_sched->getAllPending(&recordingList);
 
     unsigned int iNum = 10;
     if (recordingList.size() < iNum) 
@@ -2662,7 +2951,7 @@ void MainServer::PrintStatus(QSocket *socket)
        for (unsigned int i = 0; (iter != recordingList.end()) && i < iNum; 
             iter++, i++)
        {
-           if (!(*iter)->recording ||     // bad entry, don't show as upcoming
+           if ((*iter)->recstatus > rsWillRecord ||     // bad entry, don't show as upcoming
                ((*iter)->recstartts) < QDateTime::currentDateTime()) // rec
            {                       
                i--;
@@ -2715,8 +3004,10 @@ void MainServer::PrintStatus(QSocket *socket)
                   << ((*iter)->recstartts).addSecs(-preRollSeconds)
                                           .toString(shortdateformat) << " "
                   << ((*iter)->recstartts).addSecs(-preRollSeconds)
-                                          .toString(timeformat) << " - "
-                  << (*iter)->channame << " - " << qstrTitle << "<br />"
+                                          .toString(timeformat) << " - ";
+               if (numencoders > 1)
+                   os << "Encoder " << (*iter)->cardid << " - ";
+               os << (*iter)->channame << " - " << qstrTitle << "<br />"
                   << "<span><strong>" << qstrTitle << "</strong> ("
                   << (*iter)->startts.toString(timeformat) << "-"
                   << (*iter)->endts.toString(timeformat) << ")<br />"
@@ -2792,10 +3083,12 @@ void MainServer::PrintStatus(QSocket *socket)
     dblock.unlock();
 
     QString mfdLastRunStart, mfdLastRunEnd, mfdLastRunStatus;
+    QString DataDirectMessage;
 
     mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
     mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
     mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
+    DataDirectMessage = gContext->GetSetting("DataDirectMessage");
 
     os << "    Last mythfilldatabase run started on " << mfdLastRunStart
        << " and ";
@@ -2827,7 +3120,16 @@ void MainServer::PrintStatus(QSocket *socket)
         os << "    There's <strong>no guide data</strong> available! "
            << "Have you run mythfilldatabase?";
 
-    os << "\r\n  </div>\r\n\r\n</body>\r\n</html>\r\n";
+    if (!DataDirectMessage.isNull())
+        os << "<br />\r\nDataDirect Status: " << DataDirectMessage;
+
+    os << "\r\n  </div>\r\n";
+
+#if USING_DVB
+    PrintDVBStatus(os);
+#endif
+
+    os << "\r\n</body>\r\n</html>\r\n";
 
     while (recordingList.size() > 0)
     {
