@@ -60,12 +60,13 @@ class MainServer : public QObject
     void HandleDeleteRecording(QStringList &slist, PlaybackSock *pbs);
     void DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs);
     void HandleForgetRecording(QStringList &slist, PlaybackSock *pbs);
+    void HandleReactivateRecording(QStringList &slist, PlaybackSock *pbs);
     void HandleQueryFreeSpace(PlaybackSock *pbs);
     void HandleQueryCheckFile(QStringList &slist, PlaybackSock *pbs);
     void HandleGetPendingRecordings(PlaybackSock *pbs);
     void HandleGetScheduledRecordings(PlaybackSock *pbs);
-    void HandleGetConflictingRecordings(QStringList &slist, QString purge, 
-                                        PlaybackSock *pbs);
+    void HandleGetConflictingRecordings(QStringList &slist, PlaybackSock *pbs);
+    void HandleGetNextFreeRecorder(QStringList &slist, PlaybackSock *pbs);
     void HandleGetFreeRecorder(PlaybackSock *pbs);
     void HandleRecorderQuery(QStringList &slist, QStringList &commands,
                              PlaybackSock *pbs);
@@ -85,11 +86,14 @@ class MainServer : public QObject
     void HandleLockTuner(PlaybackSock *pbs);
     void HandleFreeTuner(int cardid, PlaybackSock *pbs);
     void HandleVersion(QSocket *socket, QString version);
+    void HandleBackendRefresh(QSocket *socket);
 
     void SendResponse(QSocket *pbs, QStringList &commands);
 
     void getFreeSpace(int &total, int &used);
 
+    void PrintDVBStatus(QTextStream &os);
+
     PlaybackSock *getSlaveByHostname(QString &hostname);
     PlaybackSock *getPlaybackBySock(QSocket *socket);
     FileTransfer *getFileTransferByID(int id);
