@@ -192,8 +192,7 @@ void startManualSchedule(void)
 
 void startTV(void)
 {
-    QSqlDatabase *db = QSqlDatabase::database();
-    TV *tv = new TV(db);
+    TV *tv = new TV();
 
     QTime timer;
     timer.start();
@@ -261,12 +260,20 @@ void startTV(void)
             qApp->lock();
         }
 
-        if (tv->WantsToQuit())
+        if (tv->WantsToQuit() && !tv->IsSwitchingCards())
             quitAll = true;
         else
         {
-            tryRecorder = true;
-            tryTV = false;
+            if (tv->IsSwitchingCards())
+            {
+                tryRecorder = false;
+                tryTV = true;
+            }
+            else
+            {
+                tryRecorder = true;
+                tryTV = false;
+            }
         }
     }
 
@@ -276,7 +283,7 @@ void startTV(void)
 void showStatus(void)
 {
     QString mfdLastRunStart, mfdLastRunEnd, mfdLastRunStatus, Status;
-    QString querytext;
+    QString querytext, DataDirectMessage;
     int DaysOfData;
     QDateTime qdtNow, GuideDataThrough;
     QSqlDatabase *db = QSqlDatabase::database();
@@ -296,6 +303,7 @@ void showStatus(void)
     mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
     mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
     mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
+    DataDirectMessage = gContext->GetSetting("DataDirectMessage");
 
     Status = QObject::tr("Myth version:") + " " + MYTH_BINARY_VERSION + "\n";
 
@@ -347,6 +355,13 @@ void showStatus(void)
         Status += QObject::tr("WARNING: is mythfilldatabase running?");
     }
 
+    if (!DataDirectMessage.isNull())
+    {
+        Status += "\n";
+        Status += QObject::tr("DataDirect Status: ");
+        Status += DataDirectMessage;
+    }
+
     DialogBox *status_dialog = new DialogBox(gContext->GetMainWindow(), Status);
     status_dialog->AddButton(QObject::tr("OK"));
     status_dialog->exec();
@@ -420,6 +435,8 @@ void TVMenuCallback(void *data, QString &selection)
         MainGeneralSettings mainsettings;
         mainsettings.exec(QSqlDatabase::database());
         menu->ReloadExitKey();
+        QStringList strlist = QString("REFRESH_BACKEND");
+        gContext->SendReceiveStringList(strlist);
     } 
     else if (sel == "settings playback") 
     {
@@ -494,7 +511,8 @@ void haltnow()
 {
     QString halt_cmd = gContext->GetSetting("HaltCommand", 
                                             "sudo /sbin/halt -p");
-    system(halt_cmd.ascii());
+    if (!halt_cmd.isEmpty())
+        system(halt_cmd.ascii());
 }
 
 int RunMenu(QString themedir)
@@ -586,8 +604,7 @@ QString RandTheme(QString &themename, QSqlDatabase *db)
 int internal_play_media(const char *mrl) 
 {
     int res = -1;
-    QSqlDatabase *db = QSqlDatabase::database();
-    TV *tv = new TV(db);
+    TV *tv = new TV();
 
     tv->Init();
     ProgramInfo *pginfo = new ProgramInfo();
@@ -741,6 +758,16 @@ int main(int argc, char **argv)
                             print_verbose_messages |= VB_NETWORK;
                             verboseString += " " + *it;
                         }
+                        else if(!strcmp(*it,"commflag"))
+                        {
+                            print_verbose_messages |= VB_COMMFLAG;
+                            verboseString += " " + *it;
+                        }
+                        else if(!strcmp(*it,"audio"))
+                        {
+                            print_verbose_messages |= VB_AUDIO;
+                            verboseString += " " + *it;
+                        }
                         else
                         {
                             cerr << "Unknown argument for -v/--verbose: "
@@ -773,8 +800,8 @@ int main(int argc, char **argv)
                     "-l or --logfile filename       Writes STDERR and STDOUT messages to filename" << endl <<
                     "-v or --verbose debug-level    Prints more information" << endl <<
                     "                               Accepts any combination (separated by comma)" << endl << 
-                    "                               of all,none,quiet,record,playback," << endl <<
-                    "                               channel,osd,file,schedule,network" << endl <<
+                    "                               of all,none,quiet,record,playback,channel," << endl <<
+                    "                               osd,file,schedule,network,commflag,audio" << endl <<
                     "--version                      Version information" << endl <<
                     "<plugin>                       Initialize and run this plugin" << endl;
             return -1;
@@ -804,6 +831,12 @@ int main(int argc, char **argv)
     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
         cerr << "Unable to ignore SIGPIPE\n";
 
+    QString fileprefix = QDir::homeDirPath() + "/.mythtv";
+
+    QDir dir(fileprefix);
+    if (!dir.exists())
+        dir.mkdir(fileprefix);
+
     gContext = new MythContext(MYTH_BINARY_VERSION);
 
     QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
@@ -816,7 +849,37 @@ int main(int argc, char **argv)
     if (!gContext->OpenDatabase(db))
     {
         printf("couldn't open db\n");
-        return -1;
+        int pause;
+        if ((pause = gContext->GetNumSetting("WOLsqlReconnectWaitTime", 0)) > 0)
+        {
+            int acttry = 1;
+            int retries = gContext->GetNumSetting("WOLsqlConnectRetry", 5);
+
+            QString WOLsqlCommand = gContext->GetSetting("WOLsqlCommand", 
+                                                   "echo \'WOLsqlServerCommand "
+                                                   "not set\nPlease do so in "
+                                                   "your mysql.txt!\'");
+
+            if (!WOLsqlCommand.isEmpty())
+            {
+                while (acttry <= retries && !gContext->OpenDatabase(db))
+                {
+                    printf("Trying to wakeup SQLserver (Try %d of %d)\n",
+                           acttry, retries);
+                    system(WOLsqlCommand);
+                    sleep(pause);
+                    ++acttry;
+                }
+            }
+            
+            if (WOLsqlCommand.isEmpty() || (acttry > retries))
+            {
+                printf("Sorry, couldn't open db\nExiting.\n");
+                return -1;
+            }
+        }
+        else
+            return -1;
     }
 
     UpgradeTVDatabaseSchema();
@@ -848,6 +911,17 @@ int main(int argc, char **argv)
 
     gContext->LoadQtConfig();
 
+    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
+    if (switchMode) 
+    {
+        // xrandr to gui size
+        char buf[128];
+        int screenwidth = gContext->GetNumSetting("GuiVidModeWidth", 0);
+        int screenheight = gContext->GetNumSetting("GuiVidModeHeight", 0);
+        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
+        system(buf);
+    }
+
     MythMainWindow *mainWindow = new MythMainWindow();
     mainWindow->Show();
     mainWindow->Init();
@@ -924,6 +998,7 @@ int main(int argc, char **argv)
     }
 #endif
 
+    delete mainWindow;
     delete gContext;
     return exitstatus;
 }
