@@ -45,7 +45,11 @@ class PlaybackBox : public MythDialog
     void stopSelected();
     void deleteSelected();
     void expireSelected();
+    void showMenu();
     void showActionsSelected();
+    void showRecGroupChanger();
+    void showRecGroupChooser();
+    void showRecGroupPasswordChanger();
 
     void doPlay();
 
@@ -53,6 +57,8 @@ class PlaybackBox : public MythDialog
     void doStop();
     void noStop();
 
+    void doEditScheduled();
+
     void askDelete();
     void doDelete();
     void doDeleteForgetHistory();
@@ -67,6 +73,13 @@ class PlaybackBox : public MythDialog
 
     void setUpdateFreeSpace() { updateFreeSpace = true; }
 
+    void chooseComboBoxChanged(void);
+    void chooseSetViewGroup(void);
+    void changeComboBoxChanged(void);
+    void changeSetRecGroup(void);
+    void changeRecGroupPassword();
+    void changeOldPasswordChanged(const QString &newText);
+    void doBeginTranscoding();
   protected:
     void paintEvent(QPaintEvent *);
     void keyPressEvent(QKeyEvent *e);
@@ -84,6 +97,8 @@ class PlaybackBox : public MythDialog
     void expire(ProgramInfo *);
     void showActions(ProgramInfo *);
 
+    bool haveGroupInfoSet;
+    
     bool skipUpdate;
     bool pageDowner;
     bool connected;
@@ -100,6 +115,7 @@ class PlaybackBox : public MythDialog
     int skipNum;
     int skipCnt;
     int listCount;
+    int countInGroup;
     bool inTitle;
     bool playingVideo;
     bool leftRight;
@@ -131,6 +147,9 @@ class PlaybackBox : public MythDialog
 
     void showIconHelp();
 
+    QString getRecGroupPassword(QString recGroup);
+    void fillRecGroupPasswordCache(void);
+
     QTimer *timer;
     NuppelVideoPlayer *nvp;
     RingBuffer *rbuffer;
@@ -156,7 +175,6 @@ class PlaybackBox : public MythDialog
     bool graphicPopup;
     bool playbackPreview;
     bool generatePreviewPixmap;
-    bool displayChanNum;
     QString dateformat;
     QString timeformat;
 
@@ -166,7 +184,9 @@ class PlaybackBox : public MythDialog
     void updateShowTitles(QPainter *);
     void updateInfo(QPainter *);
     void updateUsage(QPainter *);
-  
+    void updateProgramInfo(QPainter *p, QRect& pr, QPixmap& pix);
+    void updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix);
+    
     QString showDateFormat;
     QString showTimeFormat;
 
@@ -197,6 +217,31 @@ class PlaybackBox : public MythDialog
     QTimer *freeSpaceTimer;
     int freeSpaceTotal;
     int freeSpaceUsed;
+
+    QString groupDisplayName;
+    QString recGroup;
+    QString recGroupPassword;
+    QString curGroupPassword;
+    QMap <QString, QString> recGroupType;
+    QMap <QString, QString> recGroupPwCache;
+
+    MythPopupBox *choosePopup;
+    MythListBox *chooseListBox;
+    MythComboBox *chooseComboBox;
+    MythLineEdit *chooseLineEdit;
+    MythLineEdit *chooseOldPassword;
+    MythLineEdit *chooseNewPassword;
+    MythPushButton *chooseOkButton;
+    MythPushButton *chooseDeleteButton;
+    QString chooseGroupPassword;
+    bool groupnameAsAllProg;
+    QPixmap *previewPixmap;
+    ProgramInfo *previewProgramInfo;
+    int listOrder;
+    int overrectime;
+    int underrectime;
+
+    bool playingSomething;
 };
 
 #endif
