@@ -61,13 +61,13 @@ ProgramRecPriorityInfo& ProgramRecPriorityInfo::operator=(const ProgramInfo &oth
     spread = other.spread;
     startCol = other.startCol;
 
-    conflicting = other.conflicting;
-    recording = other.recording;
-    override = other.override;
     recstatus = other.recstatus;
     recordid = other.recordid;
     rectype = other.rectype;
-    recdups = other.recdups;
+    dupin = other.dupin;
+    dupmethod = other.dupmethod;
+    recgroup = other.recgroup;
+    chancommfree = other.chancommfree;
 
     sourceid = other.sourceid;
     inputid = other.inputid;
@@ -75,6 +75,9 @@ ProgramRecPriorityInfo& ProgramRecPriorityInfo::operator=(const ProgramInfo &oth
     schedulerid = other.schedulerid;
     recpriority = other.recpriority;
 
+    seriesid = other.seriesid;
+    programid = other.programid;
+
     return(*this);
 }
 
@@ -88,6 +91,9 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
     bgTransBackup = NULL;
     pageDowner = false;
 
+    channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
+    longChannelFormat = gContext->GetSetting("LongChannelFormat", "<num> <name>");
+
     listCount = 0;
     dataCount = 0;
 
@@ -412,12 +418,15 @@ void ProgramRecPriority::edit(void)
 
     if (rec)
     {
+        int recid = 0;
+
         if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
             (rec->GetProgramRecordingStatus(db) > kAllRecord))
         {
             ScheduledRecording record;
             record.loadByProgram(db, rec);
             record.exec(db);
+            recid = record.getRecordID();
         }
         else
         {
@@ -430,7 +439,10 @@ void ProgramRecPriority::edit(void)
         // We need to refetch the recording priority values since the Advanced
         // Recording Options page could've been used to change them 
         QString thequery;
-        int recid = rec->GetScheduledRecording(db)->getRecordID();
+
+        if (!recid)
+            recid = rec->getRecordID(db);
+
         thequery = QString(
                    "SELECT recpriority, type FROM record WHERE recordid = %1;")
                            .arg(recid);
@@ -440,7 +452,7 @@ void ProgramRecPriority::edit(void)
             if (query.numRowsAffected() > 0)
             {
                 query.next();
-                QString recPriority = query.value(0).toString();
+                int recPriority = query.value(0).toInt();
                 int rectype = query.value(1).toInt();
 
                 int cnt;
@@ -453,12 +465,15 @@ void ProgramRecPriority::edit(void)
                      cnt++, ++it);
                 progInfo = &(it.data());
            
-                int rtRecPriors[5];
+                int rtRecPriors[8];
                 rtRecPriors[0] = gContext->GetNumSetting("SingleRecordRecPrior", 0);
                 rtRecPriors[1] = gContext->GetNumSetting("TimeslotRecordRecPrior", 0);
                 rtRecPriors[2] = gContext->GetNumSetting("ChannelRecordRecPrior", 0);
                 rtRecPriors[3] = gContext->GetNumSetting("AllRecordRecPrior", 0);
                 rtRecPriors[4] = gContext->GetNumSetting("WeekslotRecordRecPrior", 0);
+                rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
+                rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
+                rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
 
                 // set the recording priorities of that program 
                 progInfo->recpriority = recPriority;
@@ -483,7 +498,19 @@ void ProgramRecPriority::edit(void)
                      cnt++, ++it);
                 programData.remove(it);
                 SortList();
-                inList = inData = 0;
+                delete curitem;
+                curitem = NULL;
+                dataCount--;
+
+                if (cnt >= dataCount)
+                    cnt = dataCount - 1;
+                if (dataCount <= listsize || cnt <= listsize / 2)
+                    inData = 0;
+                else if (cnt >= dataCount - listsize + listsize / 2)
+                    inData = dataCount - listsize;
+                else
+                    inData = cnt - listsize / 2;
+                inList = cnt - inData;
             }
         else
             MythContext::DBError("Get new recording priority query", query);
@@ -505,10 +532,10 @@ void ProgramRecPriority::changeRecPriority(int howMuch)
     progInfo = &(it.data());
 
     // inc/dec recording priority
-    tempRecPriority = progInfo->recpriority.toInt() + howMuch;
+    tempRecPriority = progInfo->recpriority + howMuch;
     if(tempRecPriority > -100 && tempRecPriority < 100) 
     {
-        progInfo->recpriority = QString::number(tempRecPriority);
+        progInfo->recpriority = tempRecPriority;
 
         // order may change if sorting by recording priority, so resort
         if (sortType == byRecPriority)
@@ -537,7 +564,7 @@ void ProgramRecPriority::saveRecPriority(void)
 
 void ProgramRecPriority::FillList(void)
 {
-    int cnt = 999, rtRecPriors[5];
+    int cnt = 999, rtRecPriors[8];
     vector<ProgramInfo *> recordinglist;
 
     programData.clear();
@@ -569,6 +596,9 @@ void ProgramRecPriority::FillList(void)
     rtRecPriors[2] = gContext->GetNumSetting("ChannelRecordRecPriority", 0);
     rtRecPriors[3] = gContext->GetNumSetting("AllRecordRecPriority", 0);
     rtRecPriors[4] = gContext->GetNumSetting("WeekslotRecordRecPriority", 0);
+    rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
+    rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+    rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
     
     // get channel recording priorities and recording types associated with each
     // program from db
@@ -605,7 +635,8 @@ void ProgramRecPriority::FillList(void)
             // does so the keys will match 
             QDateTime startts; 
             if (recType == kSingleRecord || recType == kTimeslotRecord ||
-                recType == kWeekslotRecord)
+                recType == kWeekslotRecord || recType == kOverrideRecord ||
+                recType == kDontRecord)
                 startts = QDateTime::fromString(tempDate + ":" + tempTime,
                                                 Qt::ISODate);
             else
@@ -662,10 +693,10 @@ class programRecPrioritySort
     public:
         bool operator()(const RecPriorityInfo a, const RecPriorityInfo b) 
         {
-            int finalA = a.prog->recpriority.toInt() + 
+            int finalA = a.prog->recpriority + 
                          a.prog->channelRecPriority +
                          a.prog->recTypeRecPriority;
-            int finalB = b.prog->recpriority.toInt() + 
+            int finalB = b.prog->recpriority + 
                          b.prog->channelRecPriority +
                          b.prog->recTypeRecPriority;
 
@@ -776,13 +807,16 @@ void ProgramRecPriority::updateList(QPainter *p)
                     {
                         ProgramRecPriorityInfo *progInfo = &(it.data());
 
-                        int progRecPriority = progInfo->recpriority.toInt();
+                        int progRecPriority = progInfo->recpriority;
                         int finalRecPriority = progRecPriority + 
                                         progInfo->channelRecPriority +
                                         progInfo->recTypeRecPriority;
         
                         QString tempSubTitle = progInfo->title;
-                        if ((progInfo->subtitle).stripWhiteSpace().length() > 0)
+                        if ((progInfo->rectype == kSingleRecord ||
+                             progInfo->rectype == kOverrideRecord ||
+                             progInfo->rectype == kDontRecord) &&
+                            (progInfo->subtitle).stripWhiteSpace().length() > 0)
                             tempSubTitle = tempSubTitle + " - \"" + 
                                            progInfo->subtitle + "\"";
 
@@ -860,7 +894,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
         int progRecPriority, chanrecpriority, rectyperecpriority, finalRecPriority;
         RecordingType rectype; 
 
-        progRecPriority = curitem->recpriority.toInt();
+        progRecPriority = curitem->recpriority;
         chanrecpriority = curitem->channelRecPriority;
         rectyperecpriority = curitem->recTypeRecPriority;
         finalRecPriority = progRecPriority + chanrecpriority + rectyperecpriority;
@@ -893,6 +927,9 @@ void ProgramRecPriority::updateInfo(QPainter *p)
                     case kSingleRecord:
                         text = tr("Recording just this showing");
                         break;
+                    case kOverrideRecord:
+                        text = tr("Recording this showing with override options");
+                        break;
                     case kWeekslotRecord:
                         text = tr("Recording every week");
                         break;
@@ -905,6 +942,12 @@ void ProgramRecPriority::updateInfo(QPainter *p)
                     case kAllRecord:
                         text = tr("Recording all showings");
                         break;
+                    case kFindOneRecord:
+                        text = tr("Recording one showing of this program");
+                        break;
+                    case kDontRecord:
+                        text = tr("Manually not recording this showing");
+                        break;
                     case kNotRecording:
                         text = tr("Not recording this showing");
                         break;
@@ -929,10 +972,18 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("channel");
             if (type) {
-                if(curitem->channame != "")
-                    type->SetText(curitem->channame);
+                if(rectype != kAllRecord && rectype != kFindOneRecord)
+                    type->SetText(curitem->ChannelText(channelFormat));
                 else
-                    type->SetText("Any");
+                    type->SetText(tr("Any"));
+            }
+
+            type = (UITextType *)container->GetType("longchannel");
+            if (type) {
+                if(rectype != kAllRecord && rectype != kFindOneRecord)
+                    type->SetText(curitem->ChannelText(longChannelFormat));
+                else
+                    type->SetText(tr("Any"));
             }
 
             type = (UITextType *)container->GetType("channelrecpriority");
@@ -950,10 +1001,10 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("recpriority");
             if (type) {
-                if(curitem->recpriority.toInt() >= 0)
-                    type->SetText("+"+curitem->recpriority);
+                if(curitem->recpriority >= 0)
+                    type->SetText("+"+QString::number(curitem->recpriority));
                 else
-                    type->SetText(curitem->recpriority);
+                    type->SetText(QString::number(curitem->recpriority));
             }
 
             type = (UITextType *)container->GetType("recpriorityB");
