@@ -29,7 +29,7 @@ ViewScheduled::ViewScheduled(QSqlDatabase *ldb, MythMainWindow *parent,
 
     dateformat = gContext->GetSetting("ShortDateFormat", "M/d");
     timeformat = gContext->GetSetting("TimeFormat", "h:mm AP");
-    displayChanNum = gContext->GetNumSetting("DisplayChanNum");
+    channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
     showAll = !gContext->GetNumSetting("ViewSchedShowLevel", 0);
 
     fullRect = QRect(0, 0, size().width(), size().height());
@@ -37,6 +37,7 @@ ViewScheduled::ViewScheduled(QSqlDatabase *ldb, MythMainWindow *parent,
     infoRect = QRect(0, 0, 0, 0);
     conflictRect = QRect(0, 0, 0, 0);
     showLevelRect = QRect(0, 0, 0, 0);
+    recStatusRect = QRect(0, 0, 0, 0);
 
     theme = new XMLParse();
     theme->SetWMult(wmult);
@@ -149,6 +150,8 @@ void ViewScheduled::parseContainer(QDomElement &element)
         conflictRect = area;
     if (name.lower() == "showlevel_info")
         showLevelRect = area;
+    if (name.lower() == "status_info")
+        recStatusRect = area;
 }
 
 void ViewScheduled::updateBackground(void)
@@ -180,6 +183,8 @@ void ViewScheduled::paintEvent(QPaintEvent *e)
         updateConflict(&p);
     if (r.intersects(showLevelRect))
         updateShowLevel(&p);
+    if (r.intersects(recStatusRect))
+        updateRecStatus(&p);
 }
 
 void ViewScheduled::cursorDown(bool page)
@@ -212,7 +217,8 @@ void ViewScheduled::FillList(void)
     for (i = schedList.begin(); i != schedList.end(); i++)
     {
         ProgramInfo *p = *i;
-        if ((p->recording || showAll || p->recstatus > rsOtherShowing) &&
+        if ((p->recstatus <= rsWillRecord || 
+             showAll || p->recstatus > rsEarlierShowing) &&
             p->recendts >= now)
             schedListFilter.push_back(p);
         else
@@ -270,15 +276,11 @@ void ViewScheduled::updateList(QPainter *p)
 
                 QString temp;
 
-                if (displayChanNum)
-                    temp = p->chansign;
-                else
-                    temp = p->chanstr;
-                ltype->SetItemText(i, 1, temp);
-
                 temp = (p->recstartts).toString(dateformat);
                 temp += " " + (p->recstartts).toString(timeformat);
-                ltype->SetItemText(i, 2, temp);
+                ltype->SetItemText(i, 1, temp);
+
+                ltype->SetItemText(i, 2, p->ChannelText(channelFormat));
 
                 temp = p->title;
                 if ((p->subtitle).stripWhiteSpace().length() > 0)
@@ -293,9 +295,10 @@ void ViewScheduled::updateList(QPainter *p)
 
                 if (p->recstatus == rsRecording)
                     ltype->EnableForcedFont(i, "recording");
-                else if (p->conflicting)
+                else if (p->recstatus == rsConflict)
                     ltype->EnableForcedFont(i, "conflictingrecording");
-                else if (!p->recording || p->recstatus < rsRecording)
+                else if (p->recstatus > rsWillRecord || 
+                         p->recstatus < rsRecording)
                     ltype->EnableForcedFont(i, "disabledrecording");
             }
         }
@@ -424,6 +427,40 @@ void ViewScheduled::updateInfo(QPainter *p)
     p->drawPixmap(pr.topLeft(), pix);
 }
 
+void ViewScheduled::updateRecStatus(QPainter *p)
+{
+    QRect pr = recStatusRect;
+    QPixmap pix(pr.size());
+    pix.fill(this, pr.topLeft());
+    QPainter tmp(&pix);
+    QMap<QString, QString> infoMap;
+
+    LayerSet *container = theme->GetSet("status_info");
+    if (container)
+    {
+        ProgramInfo p;
+        if (recList.getSelected(p))
+        {
+            QSqlDatabase *m_db = QSqlDatabase::database();
+            p.ToMap(m_db, infoMap);
+            container->ClearAllText();
+            container->SetText(infoMap);
+        }
+    }
+
+    if (container)
+    {
+        container->Draw(&tmp, 4, 0);
+        container->Draw(&tmp, 5, 0);
+        container->Draw(&tmp, 6, 0);
+        container->Draw(&tmp, 7, 0);
+        container->Draw(&tmp, 8, 0);
+    }
+
+    tmp.end();
+    p->drawPixmap(pr.topLeft(), pix);
+}
+
 void ViewScheduled::edit()
 {
     ProgramInfo curItem;
