@@ -43,6 +43,7 @@ class ViewScheduled : public MythDialog
     void updateConflict(QPainter *);
     void updateShowLevel(QPainter *);
     void updateInfo(QPainter *);
+    void updateRecStatus(QPainter *);
 
     void LoadWindow(QDomElement &);
     void parseContainer(QDomElement &);
@@ -56,12 +57,13 @@ class ViewScheduled : public MythDialog
     bool conflictBool;
     QString dateformat;
     QString timeformat;
-    bool displayChanNum;
+    QString channelFormat;
 
     QRect listRect;
     QRect infoRect;
     QRect conflictRect;
     QRect showLevelRect;
+    QRect recStatusRect;
     QRect fullRect;
 
     int listsize;
